/*******************************************************************************
 * Licensed Materials - Property of IBM Copyright IBM Corp. 2012. All
 * Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
(function(){
dojo.provide("rmps.ui.ActionsTable");

dojo.require("jazz.ui.StyledTable");
dojo.require("jazz.ui.Toolbar");
dojo.require("jazz.ui.toolbar.Button");

/*
 * A table API provides extensiblity on Jazz Stlyed Table to help us to define row, action and cell creation descriptors 
*/
dojo.declare("rmps.ui.ActionsTable", jazz.ui.StyledTable, {
	actionDescriptors: null,
	messages: null,
	showHover: true,
	showHeaders: true,
	
	/*Used to get the required information for a given row */
	_rowCache: null,
	
	constructor: function(){
		this._rowCache = new rmps.ui.ActionsTable._RowsCache();
	},
	
	buildRendering: function(){
		this.inherited(arguments);
		dojo.addClass(this.domNode, "rmps-ui-actions-table");
		this.domNode.style.width= "100%";
		this.domNode.style.overflow= "hidden";
	},
	
	postCreate: function(){
		if(this.actionDescriptors&& this.actionDescriptors.length>0){
			var width= this.getActionsColumnWidth();
			var difference= width/this.columns.length;
			for(var index = 0, column; column = this.columns[index]; index++) {
				column.width= parseInt(column.width.replace("%",""))- difference +"%"; 
			}
			this.columns.push({	label: this.messages.actions,width: width+"%"});	
		}else{
			this.actionDescriptor = [];		
		}
		
		if(!this.showHeaders){
			this._headerRow.style.display= "none";
			this._globalHeader.style.display = "none";
			this._globalLineBottom.style.display = "none";
			this._globalLineTop.style.display = "none";
			this._headerLineTop.style.display = "none";
		}
		this.inherited(arguments);
	},
	
	postMixInProperties: function(){
		dojo.requireLocalization("rmps.ui", "ActionsTableMessages");
		this.messages = dojo.i18n.getLocalization("rmps.ui", "ActionsTableMessages");
		this.inherited(arguments);
	},
	
	fillTable: function(tableData){
		this.clearBody();
		var tbody= this.tbody;
		for(var index = 0, dataEntry; dataEntry = tableData[index]; index++) {
			this.addRow(dataEntry);
		}
	},
	
	createCells: function(tableRow, dataEntry){
		var length= this.columns.length;
		if(this.actionDescriptors&& this.actionDescriptors.length){
			length--;	//If actions column is defined, then make sure not to loop through it
		}
		
		var rowInformationDescriptor= {
			fieldsGetter: {},
			actionsGetter: {}
		};
		
		for(var index=0; index<length; index++){
			var descriptor= this.columns[index];
			var cell = dojo.create("td", null, tableRow);
			descriptor.creator(cell, tableRow, dataEntry, descriptor);
			var fieldName= descriptor.fieldName;
			if(!fieldName){
				fieldName= "cell"+index;
			}
			
			rowInformationDescriptor.fieldsGetter[fieldName]= dojo.hitch(this, function(cell){
				return cell;
			},cell);
		}
		
		if(!this.actionDescriptors){
			rowInformationDescriptor.dataEntry = dataEntry;
			this._rowCache.add(tableRow, rowInformationDescriptor);
			return;
		}
		
		var actionsCell = dojo.create("td", null, tableRow);
		var simpleToolbar = new jazz.ui.Toolbar();
		
		simpleToolbar.connect(this, "uninitialize", function() {
			simpleToolbar.destroyRecursive();
		});
		
		rowInformationDescriptor.getActions= dojo.hitch(this, function(simpleToolbar){
			return simpleToolbar;
		},simpleToolbar);
		actionsCell.appendChild(simpleToolbar.domNode);
		
		if (this.actionDescriptors) {
			for (var index = 0, actionDescriptor; actionDescriptor = this.actionDescriptors[index]; index++) {
				var buttonArgs = actionDescriptor.args;
				buttonArgs.onClick = dojo.hitch(this, function(currentActionDescriptor, currentDataEntry, currentRow, currentActionsCell){
					currentActionDescriptor.actionCallback(currentDataEntry, currentRow, currentActionsCell);
				}, actionDescriptor, dataEntry, tableRow, actionsCell);
						
				var button= new jazz.ui.toolbar.Button(buttonArgs);
								
				var actionId= actionDescriptor.id;
				if(!actionId){
					actionId= "action"+index;
				}
			
				rowInformationDescriptor.actionsGetter[actionId]= dojo.hitch(this,function(button){
					return button;
				},button);
				
				var onCreateCallback= actionDescriptor.onCreateCallback;
				if(onCreateCallback){
					onCreateCallback(actionDescriptor, button, tableRow, dataEntry);
					delete actionDescriptor.onCreateCallback;
				}
				
				simpleToolbar.addChild(button);
			}
		}
		rowInformationDescriptor.dataEntry = dataEntry;
		this._rowCache.add(tableRow, rowInformationDescriptor);
		
		simpleToolbar.domNode.align= "right";
		simpleToolbar.startup();
	},
	
	getActionDescriptorById: function(id){
		for (var index = 0, actionDescriptor; actionDescriptor = this.actionDescriptors[index]; index++) {
			if(actionDescriptor.id=== id){
				return actionDescriptor;
			}
		}
	},
	
	getRowInfomation: function(row){
		return this._rowCache.item(row);
	},
	
	getRowsInformation: function(){
		return this._rowCache;
	},
    
    /*It could be overridden to increase/reduce the action column width */
    getActionsColumnWidth: function(){
    	var number= this.actionDescriptors.length;
    	if(number=== 0){
    		return number;
    	}
    	return (number*3) + (Math.round(number/2));
    },
    
    /*If it is true, this means that the actions width will be increased to make sure that it will be in one line */
    enableDynamicActionsWidth: function(){
    	return true; /*by default */
    },
	
	addRow: function(dataEntry){
		var row = dojo.create("tr");
		this.tbody.appendChild(row);
		this.createCells(row, dataEntry);
		return row;
	},
	
	removeRow: function(row){
		this.tbody.deleteRow(dojo.indexOf(this.tbody.rows, row));	
		this._rowCache.remove(row);
	},
	
	updateRowCell: function(row, fieldId, dataEntry){
		var column= this._getCellColumnDataByFieldId(fieldId);
		if(!column){
			console.error("Invalid fieldId was used for updating row");
			return false;
		}
		var cell= row.cells[column.index];
		if(cell){ 
			dojo.empty(cell); 
			column.creator(cell, row, dataEntry, column);
		}
		return true;
	},
	
	getRowIndex: function(row){
		return dojo.indexOf(this.tbody.rows, row);
	},
	
	getRowByIndex: function(index){
		return this.tbody.rows[index];
	},
	
	getRowsCount: function(){
		return this.tbody.rows.length;
	},
	
	getLastRow:function(){
		return this.getRowByIndex(this.tbody.rows.length-1);
	},
	
	moveRowDown: function(row){
		var rows= this.tbody.rows;
		var rowIndex= this.getRowIndex(row);
		if(rowIndex=== rows.length-1){
			//Can't move the last row down
			return false;
		}
		
		var nextRowIndex= rowIndex+1;
		var nextRow= rows[nextRowIndex];
		this.tbody.deleteRow(nextRowIndex);
		this.tbody.insertBefore(nextRow, row);
		return true; 
	},
	
	moveRowUp: function(row){
		var rows= this.tbody.rows;
		var rowIndex= this.getRowIndex(row);
		if(rowIndex<1){
			//Can't move the first row up
			return;
		}
		
		var previousRowIndex= rowIndex-1;
		var previousRow= rows[previousRowIndex];
		this.tbody.deleteRow(rowIndex);
		this.tbody.insertBefore(row, previousRow); 
	},
	
	uninitialize: function(){
		this._rowCache.clear();
		this.inherited(arguments);
	},
	
	_getCellColumnDataByFieldId: function(fieldName){
		for(var index = 0, column; column = this.columns[index]; index++) {
			if(column.fieldName===fieldName){
				return dojo.mixin({index:index},column); 
			}
		}
		return null;
	}
});

dojo.declare("rmps.ui.ActionsTable._RowsCache", null, {
	_rowsInformation: null,
	
	constructor: function(){
		this._rowsInformation= [];
	},
	
	add: function(row, rowInformation){
		this._rowsInformation.push(new rmps.ui.ActionsTable._RowCache(row, rowInformation));
	},
	
	remove: function(row){
		for(var index = 0, rowInformation; rowInformation = this._rowsInformation[index]; index++) {
			if(rowInformation.getRow()=== row){
				return this._rowsInformation.splice(index, 1);
			}
		}
	},
	
	item: function(row){
		for(var index = 0, rowInformation; rowInformation = this._rowsInformation[index]; index++) {
			if(rowInformation.getRow()=== row){
				return rowInformation;
			}
		}
	},
	
	length: function(){
		return this._rowsInformation.length;
	},
	
	clear: function(){
		this._rowsInformation= [];
	}
});

dojo.declare("rmps.ui.ActionsTable._RowCache", null, {
	_row: null,
	_fieldsGetter: null,
	_actionsGetter: null,
	_dataEntry: null,
	
	constructor: function(row, rowInformation){
		this._row= row;
		if(!rowInformation){
			return;
		}
		this._actionsGetter= rowInformation.actionsGetter||{};
		this._fieldsGetter= rowInformation.fieldsGetter||{};
		this._dataEntry = rowInformation.dataEntry;
	},
	
	getRow: function(){
		return this._row;	
	},
	
	getActionWidget: function(action){
		var getter= this._actionsGetter[action];
		return getter?getter():null;
	},
	
	getFieldWidget: function(action){
		var getter= this._fieldsGetter[action];
		return getter?getter():null;
	},
	
	getDataEntry: function() {
		return this._dataEntry;
	}
});

})();
