dojo.provide('ibm_ilog.diagram.GraphElement');

dojo.require('ibm_ilog.diagram.base');
dojo.require('ibm_ilog.diagram.Selectable');
dojo.require('ibm_ilog.diagram.templating');


(function() {

dojo.declare("ibm_ilog.diagram.GraphElement", [dojox.gfx.Group, ibm_ilog.diagram.Selectable, ibm_ilog.diagram.templating._DiagramTemplated], {
  //
	// summary:
  //		An element that can be contained in a Graph.
	//
    
	_connections: null,
	
	//
	//	_IBMDiagramType: String
	//		Graph element type, used to distinguish basic graph element types: node, subgraph, link. 
	//		Used to quickly index maps based on graph element types.
	//
	_IBMDiagramType: undefined,
	
	_isIBMDiagramGraphElement: true,
	
	toString: function() {
    	return this.declaredClass.split(".").pop()+" "+this.getId()+(this.getLabel?(": "+this.getLabel()):"");
    },
    constructor: function() {
        var id = ibm_ilog.diagram._IdGenerator.nextId(this).toString();
        this.getId = function() {
            return id;
        };
        this._connections = [];
        // do not add _labelH to _connections as it is the result of a dojo.connect,
		// while this._connections will be disposed via shape.disconnect. See comments
		// in addConnection.
		this._labelH = dojo.connect(this,"setLabel",this,function(){
        	dojo.publish('/ibm_diagram/GraphElement/labelChanged', [this]);
        });
    },
    dispose: function(options) {
    	//	summary:
			//		Dispose this GraphElement.
			//	description:
			//		This function should be called when the GraphElement is completely deleted, that is, when it is removed
			//		from its parent graph and it will never be used again.
			//	options:
    	//		options.noRecurse:
			//			Do not recurse children
    	this.beforeDisposing(this);
    	dojo.publish('/ibm_diagram/GraphElement/beforeDisposing',[this]);
    	this._dispose();
    	this.afterDisposing(this);
    	dojo.publish('/ibm_diagram/GraphElement/afterDisposing',[this]);
    	//free connections
    	this.disconnectAll();
    },
    beforeDisposing: function(graphElement) {
    	// summary:
        //		This function is called before disposing the GraphElement
    },
    afterDisposing: function(graphElement) {
    	// summary:
        //		This function is called after disposing the GraphElement
    	// description:
        //		This function free all possible listeners
    	
    	// free possible listeners
        this._disposeBoundCountext();
    },
    connect: function() {
    	// summary:
        //		override gfx.connect to add the handle to the internal list
    	var conn = this.inherited(arguments);
    	this._connections.push(conn);
    	return conn;
    },
    addConnection: function(connection){
    	// summary:
        //		this method should be called on every dojo.connect done to the graph element that
    	//		should be disconnected on dispose.
    	this._connections.push(connection);
		
		// PR: you should NOT called this method for dojo.connect handler. Indeed, the
		// disconnectAll() method calls this.disconnect() which does not always relies on dojo.disconnect,
		// depending on the renderer. In general: you should not mix dojo.connect/disconnect with shape.connect/disconnect.
		//
		// ==> This method should be removed.
    },
    disconnectAll: function() {
    	// summary:
        //		This function is disconnect every dojo.connect attached to this element
    	dojo.forEach(this._connections,dojo.hitch(this,function(conn){
    		this.disconnect(conn);
    		delete conn;
    	}));
		dojo.disconnect(this._labelH);
    	this._connections = [];
    },
    _dispose: function(graphElement) {
    	// summary:
        //		this function should be overriden by any GraphElement subclass that needs any specific disposing action.
    }
    
});

ibm_ilog.diagram.GraphElement.byShape = function(shape) {
	// summary:
	//		Returns the GraphElement that contains the specified shape, or null.
	
	for(var p = shape; p && p.getParent; p = p.getParent())
		if(p instanceof ibm_ilog.diagram.GraphElement)
			return p;
	return null;
};

})();
