dojo.provide('ibm_ilog.graphlayout.ILinkConnectionBoxProvider');

///////////////////////////////////////////////////////////////////////////////
// IBM ILOG Dojo Diagrammer 1.0
///////////////////////////////////////////////////////////////////////////////

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

dojo.require('ibm_ilog.graphlayout.internalutil.GraphModelUtil');
dojo.require('ibm_ilog.graphlayout.internalutil.TranslateUtil');
dojo.require('ibm_ilog.graphlayout.INodeBoxProvider');

dojo.declare('ibm_ilog.graphlayout.ILinkConnectionBoxProvider', ibm_ilog.graphlayout.INodeBoxProvider, 
{
  // summary:
  //		An interface used to customize the computation of the connection
  //		points of the links. 
  // description:
  //		An interface used to customize the computation of the connection
  //		points of the links. 
  //		<p>
  //		This interface is useful when a layout algorithm that supports this
  //		customization must place the connection points of the links at a
  //		certain distance inside or outside the bounding box of the nodes.
  //		This interface also allows a tangential "translation" of the
  //		connection points separately for each side of each node.  
  //		</p>
  //		<p>
  //		The following code is an example of the implementation of the method
  //		<code> ILinkConnectionBoxProvider.getBox()</code>. In this example,
  //		the connection points are at a distance of 10 outside the bounding
  //		box of the nodes. Therefore, the method computes the bounding box of
  //		the node and expands it with a value of 10 in all four directions:  
  //		</p>
  //		<p>
  //		</p>
  //		<pre>
  //		
  //		
  //		
  //		function getBox(graphModel, node) {
  //		  var bbox = graphModel.boundingBox(node);
  //		  bbox.x -= 10;
  //		  bbox.y -= 10;
  //		  bbox.width += 20;
  //		  bbox.height += 20;
  //		  return bbox;
  //		}
  //		
  //		</pre>
  //		In the second code example, the connection points on the left and
  //		right side of the nodes are shifted to the top by a fixed amount of
  //		10. (Of course, this amount could be variable dependent on specific
  //		conditions, for example, the height of a label below a node):  
  //		<pre>
  //		
  //		
  //		
  //		function getTangentialOffset(graphModel, node, nodeSide) {
  //		  switch (nodeSide) {
  //		    case ibm&#95;ilog.graphlayout.Direction.LEFT:
  //		    case ibm&#95;ilog.graphlayout.Direction.RIGHT:
  //		      return -10;
  //		    default:
  //		      return 0;
  //		  }
  //		}
  //		
  //		</pre>
  
  getBox : function(graphModel, node)
  {
    // summary:
    //		Returns the rectangle on which the connection points of the links
    //		connected to 'node' can be placed. 
    // description:
    //		Returns the rectangle on which the connection points of the links
    //		connected to <code>node</code> can be placed. 
    //		The rectangle must be in the actual coordinate system of the input
    //		node.  
    //		<p>
    //		Note that when routing intergraph links, the coordinate system of
    //		the node is not necessarily the same as the coordinate system of the
    //		link.   
    //		</p>
    // graphModel: ibm_ilog.graphlayout.AbstractGraphLayoutModel
    //		The graph model to which the node belongs.
    // node: Object
    //		The node.
    // returns:
    //		The rectangle on which the connection points of the links can be
    //		placed.
    
  },
  
  getTangentialOffset : function(graphModel, node, nodeSide)
  {
    // summary:
    //		Returns the tangential offset for the connection points of a link on
    //		the specified side of the specified node. 
    // description:
    //		Returns the tangential offset for the connection points of a link on
    //		the specified side of the specified node. 
    //		The offset must be in the current coordinate system of the input
    //		node.  
    //		<p>
    //		By default, some layout algorithms place the links connected to one
    //		side of a node symmetrically with respect to the middle of the side.
    //		You can use this method to "shift" the links tangentially as
    //		follows:  
    //		<ul>
    //		<li> On the "top" side of a node, the connection points are
    //		translated to the right if the offset is positive and to the left if
    //		the offset is negative.</li>  
    //		<li> On the "bottom" side of a node, the connection points are
    //		translated to the right if the offset is positive and to the left if
    //		the offset is negative.</li>  
    //		<li> On the "left" side of a node, the connection points are
    //		translated to the bottom if the offset is positive and to the top if
    //		the offset is negative.</li>  
    //		<li> On the "right" side of a node, the connection points are
    //		translated to the bottom if the offset is positive and to the top if
    //		the offset is negative.</li>  
    //		</ul>
    //		This method must return <code>0</code> if no offset is necessary on
    //		the specified node and side.  
    //		</p>
    //		<p>
    //		Note that when routing intergraph links, the coordinate system of
    //		the node is not necessarily the same as the coordinate system of the
    //		link.    
    //		</p>
    // graphModel: ibm_ilog.graphlayout.AbstractGraphLayoutModel
    //		The graph model to which the node belongs.
    // node: Object
    //		The node.
    // nodeSide: int
    //		The side of the node. The values are defined in '
    //		ibm_ilog.graphlayout.Direction'. Valid values are
    //		'Top',
    //		'Bottom', 'Left', and 'Right'. .
    // returns:
    //		The tangential offset for the connection points of a link on the
    //		specified side of the specified node or '0' if no offset
    //		is required.
    
  }
  
});
