dojo.provide("ibm_ilog.graphlayout.LightVersion");

dojo.mixin(ibm_ilog.graphlayout.LightVersion, {
	// summary:
	//		Loads a "light" version of the ibm_graphlayout module used only to run graph layouts
	//		on a web server or in a web worker.
	// example:
	//		dojo.require('ibm_ilog.graphlayout.LightVersion').load();
	
	load : function(classesToLoad, classesNotToLoad) {
		// summary:
		//		Loads a "light" version of the ibm_graphlayout module used only to run graph layouts
		//		on a web server or in a web worker.
		// description:
		//		This function prevents some classes in the <code>ibm&#95;graphlayout</code> module
		//		from being loaded when they are not needed.
		//		<p>
		//		This is useful when an application knows that it will only execute
		//		graph layout algorithms on a remote web server
		//		(using the <code>ibm&#95;diagram.graphlayout.ServerSideLayout</code> class)
		//		or in a Web Worker thread
		//		(using the <code>ibm&#95;diagram.graphlayout.WebWorkerLayout</code> class).
		//		</p>
		//		<p>
		//		In these cases, only the public classes of the graph layout modules are necessary 
		//		(for example, the <code>ibm&#95;graphlayout.tree.TreeLayout</code> class) to configure the layout
		//		on the client, but the helper classes containing the actual implementation of the layout
		//		(like <code>ibm&#95;graphlayout.tree.TGraph</code>, etc) do not need to be loaded.
		//		</p>
		//		<p>
		//		This function must be called <strong>before</strong> all other <code>ibm&#95;graphlayout</code> requires,
		//		so it is recommended to call it immediately after the <code>dojo.require</code> call as shown in the
		//		example.
		//		</p>
		//		<p>
		//		<strong>Note</strong>: Once this function has been called, it is not possible any more
		//		to execute a graph layout on the client. If your application may execute graph layout
		//		both on the client and on a server/in a web worker, do not call this function.
		//		It is never necessary to call this function: its purpose is only to reduce the
		//		download size of an application that does server-side or web worker layout only.
		//		</p>
		// classesToLoad: Array
		//		Specifies a set of classes that must actually be loaded. By default, the load()
		//		function uses a predefined set of classes that will work in most cases, but in
		//		some cases you may need to force additional classes to be loaded.
		//		You can do this by passing the class names in the classesToLoad array.
		//		This is an advanced option. You should normally just call load().
		// classesNotToLoad: Array
		//		Specifies the set of classes that must not be loaded. By default, the load()
		//		function uses a predefined set of classes that will work in most cases, but
		//		you may want to specify the classes not to be loaded yourself.
		//		Note that this argument will completely replace the default classes not to be loaded.
		//		This is an advanced option. You should normally just call load().
		// example:
		//		dojo.require('ibm_ilog.graphlayout.LightVersion').load();
		
		if(dojo.config.isDebug)
			console.log("preparing to load light version of ibm_graphlayout");
		
		classesNotToLoad = classesNotToLoad || ibm_ilog.graphlayout.LightVersion.defaultClassesNotToLoad;
		for(var i = 0; i < classesNotToLoad.length; i++){
			var className = classesNotToLoad[i];
			if(className && (!classesToLoad || dojo.indexOf(classesToLoad, className) < 0)){
				dojo.provide(className);
				dojo.declare(className, null, {});
			}
		}
	},
	
	// defaultClassesNotToLoad: Array
	//		The default set of classes that should not be loaded in the light version of ibm_ilog.graphlayout.
	defaultClassesNotToLoad : [
//"ibm_ilog.graphlayout.AbstractGraphLayoutModel",
//"ibm_ilog.graphlayout.DefaultLayoutProvider",
//"ibm_ilog.graphlayout.Direction",
//"ibm_ilog.graphlayout.GraphLayout",
"ibm_ilog.graphlayout.GraphLayoutModelEvent",
"ibm_ilog.graphlayout.GraphLayoutReport",
"ibm_ilog.graphlayout.GraphLayoutUtil",
//"ibm_ilog.graphlayout.IIterator",
"ibm_ilog.graphlayout.ILayoutProvider",
"ibm_ilog.graphlayout.ILinkConnectionBoxProvider",
"ibm_ilog.graphlayout.INodeBoxProvider",
"ibm_ilog.graphlayout.INodeSideFilter",
"ibm_ilog.graphlayout.ISubgraphCorrection",
//"ibm_ilog.graphlayout.MultipleLayout",
"ibm_ilog.graphlayout.MultipleLayoutReport",
//"ibm_ilog.graphlayout.RLayoutModes",
//"ibm_ilog.graphlayout.RecursiveLayout",
"ibm_ilog.graphlayout.RecursiveLayoutProvider",
"ibm_ilog.graphlayout.RecursiveLayoutReport",
//"ibm_ilog.graphlayout.RecursiveMultipleLayout",
"ibm_ilog.graphlayout.SplineLinkFilter",
//"ibm_ilog.graphlayout.circular.CircularClusterId",
//"ibm_ilog.graphlayout.circular.CircularClusterName",
//"ibm_ilog.graphlayout.circular.CircularClusterNumber",
//"ibm_ilog.graphlayout.circular.CircularLayout",
"ibm_ilog.graphlayout.circular.clayout.AbstractGraph",
"ibm_ilog.graphlayout.circular.clayout.AbstractGraphElement",
"ibm_ilog.graphlayout.circular.clayout.AbstractGraphicNode",
"ibm_ilog.graphlayout.circular.clayout.AbstractLink",
"ibm_ilog.graphlayout.circular.clayout.AbstractNode",
"ibm_ilog.graphlayout.circular.clayout.CLANode",
"ibm_ilog.graphlayout.circular.clayout.CLAutoClustering",
"ibm_ilog.graphlayout.circular.clayout.CLCluster",
"ibm_ilog.graphlayout.circular.clayout.CLCrossingReduction",
"ibm_ilog.graphlayout.circular.clayout.CLGraphOfClusters",
"ibm_ilog.graphlayout.circular.clayout.CLLink",
"ibm_ilog.graphlayout.circular.clayout.CLNode",
"ibm_ilog.graphlayout.circular.clayout.CircularLayoutOfSubTrees",
// Defect #763
// Do not exclude, required by CircularLayout.add/set/removeClusterId
// "ibm_ilog.graphlayout.circular.clayout.ClusterIdCollection",
"ibm_ilog.graphlayout.circular.clayout.ClusterLayoutAlgo",
"ibm_ilog.graphlayout.circular.clayout.ClustersConnexion",
"ibm_ilog.graphlayout.circular.clayout.GraphOfClusters",
"ibm_ilog.graphlayout.circular.clayout.GrapherCluster",
"ibm_ilog.graphlayout.circular.clayout.GrapherClusterCollection",
"ibm_ilog.graphlayout.circular.galg.BiconnAlgorithm",
"ibm_ilog.graphlayout.circular.galg.BiconnBlock",
"ibm_ilog.graphlayout.circular.galg.BiconnModel",
"ibm_ilog.graphlayout.circular.galg.BiconnPartModel",
"ibm_ilog.graphlayout.circular.galg.ClusteringGraph",
"ibm_ilog.graphlayout.circular.galg.DefaultBiconnModel",
"ibm_ilog.graphlayout.circular.galg.DefaultSimpleGraphModel",
"ibm_ilog.graphlayout.circular.galg.DefaultSimpleTraversableGraphModel",
"ibm_ilog.graphlayout.circular.galg.KLAlgorithm",
"ibm_ilog.graphlayout.circular.galg.SimpleGraphModel",
"ibm_ilog.graphlayout.circular.galg.SimpleGraphUtil",
"ibm_ilog.graphlayout.circular.galg.SimpleTraversableGraphModel",
"ibm_ilog.graphlayout.circular.galg.SimpleTraversableGraphPartModel",
"ibm_ilog.graphlayout.forcedirected.FMMAlgorithm",
//"ibm_ilog.graphlayout.forcedirected.ForceDirectedLayout",
"ibm_ilog.graphlayout.forcedirected.ForceDirectedLayoutReport",
"ibm_ilog.graphlayout.forcedirected.GraphCoarseningAlgo",
"ibm_ilog.graphlayout.forcedirected.MultilevelGraphPartitioningAlgo",
"ibm_ilog.graphlayout.forcedirected.MultipoleULELayout",
"ibm_ilog.graphlayout.forcedirected.RegularizationAlgo",
"ibm_ilog.graphlayout.forcedirected.SolarSystemNode",
//"ibm_ilog.graphlayout.forcedirected.ULEAlgorithm",
//"ibm_ilog.graphlayout.grid.GridLayout",
"ibm_ilog.graphlayout.hierarchical.AnchorPointOptimization",
"ibm_ilog.graphlayout.hierarchical.BuildLevelsAlgorithm",
"ibm_ilog.graphlayout.hierarchical.BuildRelPosNetsAlgorithm",
"ibm_ilog.graphlayout.hierarchical.CalcConnectors",
"ibm_ilog.graphlayout.hierarchical.CalcCrossings",
"ibm_ilog.graphlayout.hierarchical.CalcLevelingAlgorithm",
//"ibm_ilog.graphlayout.hierarchical.ConstraintManager",
"ibm_ilog.graphlayout.hierarchical.CrossRedSelfLinkAlgorithm",
"ibm_ilog.graphlayout.hierarchical.CrossRedSwimLaneAlgorithm",
"ibm_ilog.graphlayout.hierarchical.CrossingReductionAlgorithm",
"ibm_ilog.graphlayout.hierarchical.CrossingReductionSwapAlgorithm",
"ibm_ilog.graphlayout.hierarchical.EastWestExpansionAlgorithm",
"ibm_ilog.graphlayout.hierarchical.EastWestLinkRemovalAlgorithm",
"ibm_ilog.graphlayout.hierarchical.EastWestLinkRoutingAlgorithm",
"ibm_ilog.graphlayout.hierarchical.ForkPointAlgorithm",
"ibm_ilog.graphlayout.hierarchical.HBooleanNodeMarker",
"ibm_ilog.graphlayout.hierarchical.HConstraintSort",
"ibm_ilog.graphlayout.hierarchical.HEnum",
"ibm_ilog.graphlayout.hierarchical.HFloatNodeMarker",
"ibm_ilog.graphlayout.hierarchical.HGraph",
"ibm_ilog.graphlayout.hierarchical.HGraphAlgorithm",
"ibm_ilog.graphlayout.hierarchical.HGraphIterator",
"ibm_ilog.graphlayout.hierarchical.HGraphMember",
"ibm_ilog.graphlayout.hierarchical.HGraphMemberList",
"ibm_ilog.graphlayout.hierarchical.HIntLevelMarker",
"ibm_ilog.graphlayout.hierarchical.HIntNodeMarker",
"ibm_ilog.graphlayout.hierarchical.HLevel",
"ibm_ilog.graphlayout.hierarchical.HLevelAlgApplicator",
"ibm_ilog.graphlayout.hierarchical.HLevelAlgorithm",
"ibm_ilog.graphlayout.hierarchical.HLevelIterator",
"ibm_ilog.graphlayout.hierarchical.HLevelSweepAlgorithm",
"ibm_ilog.graphlayout.hierarchical.HLink",
"ibm_ilog.graphlayout.hierarchical.HLinkIterator",
"ibm_ilog.graphlayout.hierarchical.HNode",
"ibm_ilog.graphlayout.hierarchical.HNodeIterator",
"ibm_ilog.graphlayout.hierarchical.HNodeSelfLinkInfo",
"ibm_ilog.graphlayout.hierarchical.HNodeSort",
"ibm_ilog.graphlayout.hierarchical.HPercentages",
"ibm_ilog.graphlayout.hierarchical.HPortCache",
"ibm_ilog.graphlayout.hierarchical.HSegment",
"ibm_ilog.graphlayout.hierarchical.HSegmentBufferedIterator",
"ibm_ilog.graphlayout.hierarchical.HSegmentCombineIterator",
"ibm_ilog.graphlayout.hierarchical.HSegmentFilterIterator",
"ibm_ilog.graphlayout.hierarchical.HSegmentIterator",
"ibm_ilog.graphlayout.hierarchical.HSegmentSetMarker",
"ibm_ilog.graphlayout.hierarchical.HSegmentSort",
"ibm_ilog.graphlayout.hierarchical.HSwimLane",
//"ibm_ilog.graphlayout.hierarchical.HierarchicalConstraint",
//"ibm_ilog.graphlayout.hierarchical.HierarchicalExtremityConstraint",
//"ibm_ilog.graphlayout.hierarchical.HierarchicalGroupSpreadConstraint",
//"ibm_ilog.graphlayout.hierarchical.HierarchicalLayout",
//"ibm_ilog.graphlayout.hierarchical.HierarchicalLevelRangeConstraint",
//"ibm_ilog.graphlayout.hierarchical.HierarchicalNodeGroup",
//"ibm_ilog.graphlayout.hierarchical.HierarchicalRelativeConstraint",
//"ibm_ilog.graphlayout.hierarchical.HierarchicalRelativeLevelConstraint",
//"ibm_ilog.graphlayout.hierarchical.HierarchicalRelativePositionConstraint",
//"ibm_ilog.graphlayout.hierarchical.HierarchicalSameLevelConstraint",
//"ibm_ilog.graphlayout.hierarchical.HierarchicalSideBySideConstraint",
//"ibm_ilog.graphlayout.hierarchical.HierarchicalSwimLaneConstraint",
"ibm_ilog.graphlayout.hierarchical.IndexSet",
"ibm_ilog.graphlayout.hierarchical.IntEnumeration",
"ibm_ilog.graphlayout.hierarchical.LabelingUtil",
"ibm_ilog.graphlayout.hierarchical.LinkSegmentOptim",
"ibm_ilog.graphlayout.hierarchical.NodePlacementAlgorithm",
"ibm_ilog.graphlayout.hierarchical.OptimizeLinksAlgorithm",
"ibm_ilog.graphlayout.hierarchical.OrthLinkRepositionAlgorithm",
"ibm_ilog.graphlayout.hierarchical.OrthRoutingAlgorithm",
"ibm_ilog.graphlayout.hierarchical.OrthRoutingLevelAlgorithm",
"ibm_ilog.graphlayout.hierarchical.PolyLinkRepositionAlgorithm",
"ibm_ilog.graphlayout.hierarchical.RecCrossingReductionAlgorithm",
"ibm_ilog.graphlayout.hierarchical.SelfLinkAlgorithm",
"ibm_ilog.graphlayout.hierarchical.SortAdjByCoordAlgorithm",
"ibm_ilog.graphlayout.hierarchical.SortAdjacenciesAlgorithm",
"ibm_ilog.graphlayout.hierarchical.StraightenChainAlgorithm",
"ibm_ilog.graphlayout.hierarchical.SwimLanePlacementAlgorithm",
"ibm_ilog.graphlayout.hierarchical.cgraphbase.HTCBaseChildGraphIterator",
"ibm_ilog.graphlayout.hierarchical.cgraphbase.HTCBaseGraph",
"ibm_ilog.graphlayout.hierarchical.cgraphbase.HTCBaseIGAdjEdgeIterator",
"ibm_ilog.graphlayout.hierarchical.cgraphbase.HTCBaseIGEdgeIterator",
"ibm_ilog.graphlayout.hierarchical.cgraphbase.HTCBaseIGInEdgeIterator",
"ibm_ilog.graphlayout.hierarchical.cgraphbase.HTCBaseIGOutEdgeIterator",
"ibm_ilog.graphlayout.hierarchical.cgraphbase.HTCBaseNode",
"ibm_ilog.graphlayout.hierarchical.graphbase.Algorithm",
"ibm_ilog.graphlayout.hierarchical.graphbase.HTBaseAdjEdgeIterator",
"ibm_ilog.graphlayout.hierarchical.graphbase.HTBaseEdge",
"ibm_ilog.graphlayout.hierarchical.graphbase.HTBaseEdgeIterator",
"ibm_ilog.graphlayout.hierarchical.graphbase.HTBaseGraph",
"ibm_ilog.graphlayout.hierarchical.graphbase.HTBaseGraphMember",
"ibm_ilog.graphlayout.hierarchical.graphbase.HTBaseGraphMemberList",
"ibm_ilog.graphlayout.hierarchical.graphbase.HTBaseGraphMemberListIterator",
"ibm_ilog.graphlayout.hierarchical.graphbase.HTBaseInEdgeIterator",
"ibm_ilog.graphlayout.hierarchical.graphbase.HTBaseNode",
"ibm_ilog.graphlayout.hierarchical.graphbase.HTBaseNodeIterator",
"ibm_ilog.graphlayout.hierarchical.graphbase.HTBaseOutEdgeIterator",
// Defect #763
// Do not exclude, required by GridLayout.setLayoutMode/setNodeOrdering
// which require fields of GridLayout which are initialized
// using fields of these 3 internal classes.
// "ibm_ilog.graphlayout.hierarchical.leveling.Algorithm",
"ibm_ilog.graphlayout.hierarchical.leveling.HLVBlock",
"ibm_ilog.graphlayout.hierarchical.leveling.HLVEdge",
"ibm_ilog.graphlayout.hierarchical.leveling.HLVEdgeIterator",
"ibm_ilog.graphlayout.hierarchical.leveling.HLVGraph",
"ibm_ilog.graphlayout.hierarchical.leveling.HLVNode",
"ibm_ilog.graphlayout.hierarchical.leveling.HLVNodeIterator",
"ibm_ilog.graphlayout.hierarchical.makeacyclic.Algorithm",
"ibm_ilog.graphlayout.hierarchical.makeacyclic.HMAEdge",
"ibm_ilog.graphlayout.hierarchical.makeacyclic.HMAEdgeIterator",
"ibm_ilog.graphlayout.hierarchical.makeacyclic.HMAGraph",
"ibm_ilog.graphlayout.hierarchical.makeacyclic.HMANode",
"ibm_ilog.graphlayout.hierarchical.makeacyclic.HMANodeIterator",
"ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring",
"ibm_ilog.graphlayout.hierarchical.relpositioning.HRPGraph",
"ibm_ilog.graphlayout.hierarchical.relpositioning.HRPNode",
"ibm_ilog.graphlayout.hierarchical.relpositioning.HRPNodeIterator",
"ibm_ilog.graphlayout.hierarchical.relpositioning.HRPOrdering",
"ibm_ilog.graphlayout.hierarchical.relpositioning.HRPSolving",
"ibm_ilog.graphlayout.hierarchical.relpositioning.HRPTransform",
//"ibm_ilog.graphlayout.internalutil.AlignmentOptions",
"ibm_ilog.graphlayout.internalutil.ArrayStableSort",
"ibm_ilog.graphlayout.internalutil.BaseSubgraphData",
"ibm_ilog.graphlayout.internalutil.ConnectedComponentAdapter",
"ibm_ilog.graphlayout.internalutil.ConnectedComponentCollectionAdapter",
//"ibm_ilog.graphlayout.internalutil.ConnectionPointModes",
//"ibm_ilog.graphlayout.internalutil.ConnectorStyles",
//"ibm_ilog.graphlayout.internalutil.DefaultParameterContainer",
"ibm_ilog.graphlayout.internalutil.Dimension",
"ibm_ilog.graphlayout.internalutil.Event",
"ibm_ilog.graphlayout.internalutil.FixedPoint",
"ibm_ilog.graphlayout.internalutil.GraphLayoutData",
"ibm_ilog.graphlayout.internalutil.GraphLayoutRecUtil",
"ibm_ilog.graphlayout.internalutil.GraphModelDFS",
"ibm_ilog.graphlayout.internalutil.GrowableArray",
//"ibm_ilog.graphlayout.internalutil.HashTable",
"ibm_ilog.graphlayout.internalutil.IFixedNode",
"ibm_ilog.graphlayout.internalutil.IParameterContainer",
"ibm_ilog.graphlayout.internalutil.KeyDataInt",
//"ibm_ilog.graphlayout.internalutil.LayoutParametersUtil",
"ibm_ilog.graphlayout.internalutil.LayoutUtil",
//"ibm_ilog.graphlayout.internalutil.LinkStyles",
"ibm_ilog.graphlayout.internalutil.ListOfIntegers",
//"ibm_ilog.graphlayout.internalutil.LogResUtil",
//"ibm_ilog.graphlayout.internalutil.MovementModes",
"ibm_ilog.graphlayout.internalutil.MultiAndSelfLinkUtil",
"ibm_ilog.graphlayout.internalutil.NestedData",
"ibm_ilog.graphlayout.internalutil.PercCompleteController",
//"ibm_ilog.graphlayout.internalutil.Point",
"ibm_ilog.graphlayout.internalutil.PositionData",
"ibm_ilog.graphlayout.internalutil.PriorityQueue",
"ibm_ilog.graphlayout.internalutil.QuickSort",
//"ibm_ilog.graphlayout.internalutil.RandomUtil",
//"ibm_ilog.graphlayout.internalutil.Rectangle",
"ibm_ilog.graphlayout.internalutil.ReshapeLinkUtil",
//"ibm_ilog.graphlayout.internalutil.ReshapeModes",
//"ibm_ilog.graphlayout.internalutil.SplineRouter",
//"ibm_ilog.graphlayout.internalutil.SubgraphData",
//"ibm_ilog.graphlayout.internalutil.TopologicalData",
//"ibm_ilog.graphlayout.internalutil.TranslateUtil",
"ibm_ilog.graphlayout.internalutil.genericgrapher.AbstractGraph",
"ibm_ilog.graphlayout.internalutil.genericgrapher.AbstractGraphAdapter",
"ibm_ilog.graphlayout.internalutil.genericgrapher.AbstractGraphElement",
"ibm_ilog.graphlayout.internalutil.genericgrapher.AbstractLink",
"ibm_ilog.graphlayout.internalutil.genericgrapher.AbstractNode",
"ibm_ilog.graphlayout.internalutil.genericquadtree.GenericApplyObject",
"ibm_ilog.graphlayout.internalutil.genericquadtree.GenericIndexedSet",
"ibm_ilog.graphlayout.internalutil.genericquadtree.GenericQuadtree",
"ibm_ilog.graphlayout.internalutil.genericquadtree.IGenericQuadtreeObject",
"ibm_ilog.graphlayout.internalutil.rowcolumngrid.DataMatrix",
// Defect #763
// Do not exclude, required by GridLayout.setLayoutMode/setNodeOrdering
// which require fields of GridLayout which are initialized
// using fields of these 3 internal classes.
// "ibm_ilog.graphlayout.internalutil.rowcolumngrid.Graph",
// "ibm_ilog.graphlayout.internalutil.rowcolumngrid.GraphOnMatrix",
// "ibm_ilog.graphlayout.internalutil.rowcolumngrid.GraphOnRowsOrColumns",
"ibm_ilog.graphlayout.internalutil.rowcolumngrid.NodeOnMatrix",
"ibm_ilog.graphlayout.internalutil.rowcolumngrid.Row",
"ibm_ilog.graphlayout.internalutil.rowcolumngrid.RowsCollection",
"ibm_ilog.graphlayout.longlink.ILongLinkTerminationPointFilter",
"ibm_ilog.graphlayout.longlink.LLCrossPoint",
"ibm_ilog.graphlayout.longlink.LLCrossRedAlgorithm",
"ibm_ilog.graphlayout.longlink.LLDirectRouteAlgorithm",
"ibm_ilog.graphlayout.longlink.LLFallbackRouteAlgorithm",
"ibm_ilog.graphlayout.longlink.LLGraph",
"ibm_ilog.graphlayout.longlink.LLGrid",
"ibm_ilog.graphlayout.longlink.LLGridLine",
"ibm_ilog.graphlayout.longlink.LLGridSegment",
"ibm_ilog.graphlayout.longlink.LLIncrementalData",
"ibm_ilog.graphlayout.longlink.LLIncrementalLink",
"ibm_ilog.graphlayout.longlink.LLIncrementalNode",
"ibm_ilog.graphlayout.longlink.LLLink",
"ibm_ilog.graphlayout.longlink.LLLinkRouteAlgorithm",
"ibm_ilog.graphlayout.longlink.LLTerminationCandAlgorithm",
"ibm_ilog.graphlayout.longlink.LLTerminationPoint",
//"ibm_ilog.graphlayout.longlink.LongLinkLayout",
"ibm_ilog.graphlayout.longlink.LongLinkLayoutReport",
//"ibm_ilog.graphlayout.nls.messages",
//"ibm_ilog.graphlayout.random.RandomLayout",
//"ibm_ilog.graphlayout.shortlink.ShortLinkLayout",
"ibm_ilog.graphlayout.shortlink._BestNeighborOptimization",
"ibm_ilog.graphlayout.shortlink._CombinatorialOptimization",
"ibm_ilog.graphlayout.shortlink._CostComputer",
"ibm_ilog.graphlayout.shortlink._DirectIncidentLinksRefiner",
"ibm_ilog.graphlayout.shortlink._DirectIncidentLinksSorter",
//"ibm_ilog.graphlayout.shortlink._DirectLinkShapeProducer",
"ibm_ilog.graphlayout.shortlink._IncidentLinksRefiner",
"ibm_ilog.graphlayout.shortlink._IncidentLinksSorter",
"ibm_ilog.graphlayout.shortlink._LinkLayoutBestNeighborOptimization",
"ibm_ilog.graphlayout.shortlink._LinkShape",
//"ibm_ilog.graphlayout.shortlink._LinkShapeProducer",
//"ibm_ilog.graphlayout.shortlink._LinkShapeType",
//"ibm_ilog.graphlayout.shortlink._MultiLinkBundleLayouter",
//"ibm_ilog.graphlayout.shortlink._NodeSideLayouter",
"ibm_ilog.graphlayout.shortlink._OrthoWith4BendsPositionArrays",
"ibm_ilog.graphlayout.shortlink._OrthoWithout4BendsPositionArrays",
"ibm_ilog.graphlayout.shortlink._OrthogonalIncidentLinksRefiner",
"ibm_ilog.graphlayout.shortlink._OrthogonalIncidentLinksSorter",
//"ibm_ilog.graphlayout.shortlink._OrthogonalLinkShapeProducer",
"ibm_ilog.graphlayout.shortlink._SLLinkData",
"ibm_ilog.graphlayout.shortlink._SLMultiLinkData",
"ibm_ilog.graphlayout.shortlink._SLNodeData",
"ibm_ilog.graphlayout.shortlink._SLNodeOrLinkData",
"ibm_ilog.graphlayout.shortlink._SLNodeSide",
//"ibm_ilog.graphlayout.shortlink._ShortLinkAlgorithm",
//"ibm_ilog.graphlayout.tree.TAngleSpacingModes",
"ibm_ilog.graphlayout.tree.TAutoTipOver",
"ibm_ilog.graphlayout.tree.TBalloon",
//"ibm_ilog.graphlayout.tree.TBalloonRadiusModes",
"ibm_ilog.graphlayout.tree.TGraph",
"ibm_ilog.graphlayout.tree.TGraphAlgorithm",
//"ibm_ilog.graphlayout.tree.TLayoutModes",
//"ibm_ilog.graphlayout.tree.TLinkParameters",
"ibm_ilog.graphlayout.tree.TNode",
"ibm_ilog.graphlayout.tree.TNodeIterator",
//"ibm_ilog.graphlayout.tree.TNodeParameters",
"ibm_ilog.graphlayout.tree.TPath",
"ibm_ilog.graphlayout.tree.TRadial",
"ibm_ilog.graphlayout.tree.TSpanTree",
//"ibm_ilog.graphlayout.tree.TreeLayout",
null
	]
});
