/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.diagrams.render.RenderRoot");

dojo.require("rmpw.common.policy.Host");
dojo.require("rmpw.common.transform.InvalidationController");
dojo.require("rmpw.diagrams.render.util.Workarounds");
dojo.require("rmpw.diagrams.render.Viewport");
dojo.require("rmpw.diagrams.render.graph.BaseGraphConfigurator");
dojo.require("dojo.i18n");

dojo.requireLocalization("rmps.graphics.ui", "DiagramEditorMessages");

(function(){
	var messages = dojo.i18n.getLocalization("rmps.graphics.ui", "DiagramEditorMessages");
	
	dojo.declare("rmpw.diagrams.render.RenderRoot", rmpw.common.policy.Host , {
		
		diagram: null, 
		 
		constructor: function(args) {
			args = args || {};
		    
			this._metadataMap = {}; // add a meta data root where policy data will be stored
			
			this._constructing = true;
		    this.configuration = args.configuration;
		    this.container = args.node;
		    
		    // respond to parent.viewer in role of ViewMapping hierarchy root
		    this.viewer = this; // TODO (MG): temporary shim for mapping/policy host refactor 
		    this.root = this;
		    this.level = 0;
		    
			this.allowAnimation = this.configuration.allowAnimation ? true : this.configuration.allowAnimation === undefined ? true : false;
			this._subScribeAnimation = dojo.subscribe('gef/diagram/animation', dojo.hitch(this, function(allow) {
				this.allowAnimation = allow;
			}));
			
			if (this.configuration.graphConfigurator) {
				this.graphConfigurator = new this.configuration.graphConfigurator();
			}
			else {
				// install the default
				this.graphConfigurator = new rmpw.diagrams.render.graph.BaseGraphConfigurator();
			}
			
		    this.surface = dojox.gfx.createSurface(this.container, 1, 1);
			this.surface.whenLoaded(dojo.hitch(this, this._whenLoaded));
			
			delete this._constructing;
		},
		
		_whenLoaded: function(surface) {
            // create viewport
			this.rearFeedbackLayer = surface.createGroup();
			this.viewport = new rmpw.diagrams.render.Viewport(surface, this.container);
			this.feedbackLayer = surface.createGroup();
			this.attachListeners();
			
			this.configure( this.configuration );
		},
		
	    configure: function( config ){
			if( config.store ){
                if(config.feed) {
                    var diagramStore = new config.store(this);
                    diagramStore.fetch(config.feed, config.interpretation, config.tooling);
                }
             }
		},
		
		destroy: function() {
			this.inherited(arguments);
			
			this.clear();
			dojo.unsubscribe(this._subScribeAnimation);
			
			if (this._invalidationController) {
				this._invalidationController.destroy();
			}
		},
		
		setDiagram: function(diagram, diagramWidget) {
			this.diagram = diagram;
			diagram.setParent(this);
			
			// hide diagram until first refresh
			if (!dojo.isOffscreen) {
				this._hideBeforeRefresh(diagram, diagramWidget);
			}
		},
		
		_hideBeforeRefresh: function(diagram, diagramWidget) {
			if (diagramWidget && diagramWidget.outline && diagramWidget.outline.setDiagram) {
				diagramWidget.outline.setDiagram(diagram);
			}
			if (diagram.policies.graphElement && diagram.children.length >= 1 &&
					this.diagram.rootShapes && this.diagram.rootShapes.length > 0) {
				if (diagram.children.length > 1 && this.diagram.policies.graphLayout && this.diagram.policies.graphLayout.hideBeforeLayout &&
						diagramWidget && diagramWidget.showMessage) {
					this.diagram.policies.graphLayout.hideBeforeLayout(diagram, diagramWidget);
				} else {
					diagramWidget.showMessage(messages.loading);
					this.diagram.rootShapes[0].rawNode.style.display = 'none';
					var graphManager = this.diagram.policies.graphElement.getGraphManager();
					this._connectContainerRefresh = dojo.connect(graphManager, "onRefreshComplete", dojo.hitch(this, function() {
						diagramWidget.hideMessage();
						// make sure diagram is centered in viewport
						var box = this.diagram.policies.graphContainer.getChildrenBox();
						if (box && box.x!==0 && box.y!==0) {
							var bnds = this.diagram.policies.graphElement.getBounds();
							var x = ((bnds.width > box.width) ? (bnds.width - box.width)/2 : 10) - box.x;
							var y = ((bnds.height > box.height) ? (bnds.height - box.height)/2 : 10) - box.y;
							diagramWidget.outline.pan(x,y);
						}
						dojo.disconnect(this._connectContainerRefresh);
						this.diagram.rootShapes[0].rawNode.style.display = '';
					}));
				}
			}
		},
	
	
		attachListeners: function() {
			this.listeners = [];
			this.listeners.push(dojo.connect(this.viewport, "setZoom", this, "refreshViewport"));
		},
		
		detachListeners: function() {
			dojo.forEach(this.listeners, dojo.disconnect);
		},
		
		clear: function() {
			this.feedbackLayer.clear();
			this.viewport.contents.clear();
			this.rearFeedbackLayer.clear();
	
			if(this.diagram) {
				this.diagram.destroy();
				this.diagram = null;
			}
			
			this.viewport.reset();
		},
		
		getInvalidationController: function() {
			if(!this._invalidationController) {
				this._invalidationController = new rmpw.common.transform.InvalidationController(this);
			}
			return this._invalidationController;
		},
		
		getCanvasSize: function() {
			if (this.canvasSize) {
				return this.canvasSize;	
			}
			else {
				var dim = this.surface.getDimensions();
				return {w: dim.width, h: dim.height};
			}
		},
		
		resize: function(s){
			// TODO: just call updateViewport, or rename it resize
			if (this.canvasSize === undefined || (s.w !== this.canvasSize.w || s.h !== this.canvasSize.h)) {
				this.canvasSize = {w: s.w, h: s.h};
				if( s.w && s.h )
				{
				   this.surface.setDimensions( s.w, s.h );
				}
			}
			
			// this.updateViewport();
		},
		
		/*
		 * Refresh the surface dimensions if the size of the diagram has increased.
		 */
		refreshViewport: function() {
			var surfaceDim = this.surface.getDimensions();
			
			var box = null;
			if (this.bbox) {
				box = this.bbox;
			}
			else {
				box = rmpw.diagrams.render.util.Workarounds.getTransformedBoundingBox(this.surface);
			}
			
			var xExtent = box.x + box.width + 100;
			var yExtent = box.y + box.height + 100;
			if (xExtent > surfaceDim.width || yExtent > surfaceDim.height) {
				var canvasSize = this.getCanvasSize() || {h: 0, w: 0};
				
				this.surface.setDimensions(Math.max(canvasSize.w, Math.max(surfaceDim.width, xExtent)), 
						Math.max(canvasSize.h, Math.max(surfaceDim.height, yExtent)));
			}
		},
		
		updateViewport: function(bbox) {
			this.bbox = bbox;
			this.refreshViewport();
		},
		
		invalidate: function() {
			// indication that the contents of the diagram may have changed, needed to implement ViewMapping contract
		}
	});
}());