/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.doc = document;
dojo.isIE = true; // required for text handling
dojo.isOffscreen = true; // helps runtime detect scenarios for offscreen rendering

dojo._getJazzBasedFileName = function(moduleName) {
	var syms = dojo._getModuleSymbols(moduleName);
	var bundleID = syms[0];
	var fileName = '';
	for (var i=1, seg; seg=syms[i]; i++) {
		fileName += seg;
		if (i != syms.length - 1) {
			fileName += '/';
		}
	}
	fileName +=  '.js';
	return {bundleID: bundleID, fileName: fileName};
};

dojo._loadModule = dojo.require = function(/*String*/moduleName, /*Boolean?*/omitModuleCheck){
	var d = dojo;
	
	//Check if it is already loaded.
	var module = d._loadedModules[moduleName];
	if(module){
		return module;
	}

	var info = d._getJazzBasedFileName(moduleName);
	var fileName = info.fileName;
	var bundleID = info.bundleID;
	
	try {
		if (javaScriptFileLoader.shouldSkipModule(bundleID, fileName)) {
			d._loadedModules[moduleName] = {}; // set empty module
		}
		else {
			var ok = javaScriptFileLoader.evaluate(rhinoContext, bundleID, fileName);
			if(!ok && !omitModuleCheck){
				throw new Error("File loader return false: Could not load '" + fileName + "'; last tried '" + bundleID + "/" + fileName + "'");
			}
		}
	}
	catch (e) {
		throw new Error("Exception: Could not load '" + fileName + "'; last tried '" + bundleID + "/" + fileName + "'");
	}	

	// check that the symbol was defined
	// Don't bother if we're doing xdomain (asynchronous) loading.
	if(!omitModuleCheck && !d._isXDomain){
		// pass in false so we can give better error
		module = d._loadedModules[moduleName];
		if(!module){
			throw new Error("omitModuleCheck: Could not load '" + fileName + "'; last tried '" + bundleID + "/" + fileName + "'");
		}
	}
	return module;
};

dojo.experimental = dojo.provide;

dojo.requireLocalization = function(/*String*/moduleName, /*String*/bundleName, /*String?*/locale, /*String?*/availableFlatLocales){
	var d = dojo;
	
	//Check if it is already loaded.
	var targetLocale = locale ? locale.toLowerCase() : dojo.locale;
	if(targetLocale == "root"){
		targetLocale = "ROOT";
	}
	
	var bundlePackage = moduleName + ".nls." + bundleName;
	var bundle = d._loadedModules[bundlePackage];
	if(bundle){
		return bundle;
	}
	
	var info = d._getJazzBasedFileName(bundlePackage);
	var fileName = info.fileName;
	var bundleID = info.bundleID;
	
	bundle = dojo["provide"](bundlePackage);
	
	// STUB out localization code for server side execution.
	// TODO - do we need localization for server side execution?
	dojo.require("dojo.i18n");
	
	try {
		var contents = javaScriptFileLoader.load(bundleID, fileName);
		if (contents) {
			contents = d._scopePrefix + contents + d._scopeSuffix;
			var value = d["eval"](contents);
			
			bundle[dojo.locale] = value;
		}
	}
	catch (e) {
		throw e;
	}
	
	return bundle;
};

initializeHttpArguments = function(args) {
	var firstParam = true;
	var fullURL = args.url;
	
	// add stream context and project ID
	if (baselineURL != null) {
		args.content["rmps.context"] = baselineURL;
	}
	if (projectID != null) {
		args.content.projectId = projectID;
	}
	if (changeset != null) {
		args.content.changeset = changeset;
	}
	
	for (var paramName in args.content) {
		var param = args.content[paramName];
		if (dojo.isString(param)) { 
			if (httpServiceWrapper.isURI(param)) {
				param = httpServiceWrapper.decode(param); // ensure URI is decoded
			}
			param = encodeURIComponent(param);
		}
		
		fullURL += (firstParam ? "?" : "&") + paramName + "=" + param;
		firstParam = false;
	}
	
	var headerKeys = [];
	var headerValues = [];
	var i=0;
	for (var key in args.headers) {
		headerKeys[i] = key;
		headerValues[i] = args.headers[key];
		i++;
	}
	
	return {fullURL: fullURL, headerKeys: headerKeys, headerValues: headerValues};
};

// Redirect xhrGet to java code which will call the services directly
var jazzClient = dojo.getObject('jazz.client', true);
jazzClient.xhrGet = dojo.xhrGet = function(args) {
	var xhrParms = initializeHttpArguments(args);
	var result = httpServiceWrapper.xhrGet(xhrParms.fullURL, xhrParms.headerKeys, xhrParms.headerValues);
	if (result) {
		args.load(result.content, result.headers);
	}
};

jazzClient.xhrPut = dojo.xhrPut = function(args) {
	var xhrParms = initializeHttpArguments(args);
	var result = httpServiceWrapper.xhrPut(xhrParms.fullURL, xhrParms.headerKeys, xhrParms.headerValues);
	if (result) {
		args.load(result, result.headers);
	}
};

jazzClient.xhrPost = dojo.xhrPost = function(args) {
	var xhrParms = initializeHttpArguments(args);
	var result = httpServiceWrapper.xhrPost(xhrParms.fullURL, xhrParms.headerKeys, xhrParms.headerValues);
	if (result) {
		args.load(result.content, result.headers);
	}
};

var SPARQL = new Object();
SPARQL.select = function(select, where) {
	var javaObject = null;
	dojo.xhrGet({
		url: net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL + "/modelvalidation/select",
		handleAs: 'json',
		content: {
			selectClause: select,
			whereClause: where
		},
		load: function(data) {
			javaObject = dojox.json.ref.fromJson(data);
		},
		error: function(error) {
			console.error(error);
		}
	});
	
	return javaObject;
};

dojo.require("dojox.json.ref");


