dojo.provide('ibm_ilog.diagram._base.diagext');

if(dojo.config.diagramForMobile === undefined){
	// If dojo.config.diagramForMobile is not defined explicitly,
	// try to detect automatically if we are on mobile.
	// Note that this auto-detection can always be disabled by setting
	// dojo.config.diagramForMobile explicitly to either true or false.
	if(dojox.mobile && dojox.mobile.View){
		// If we are using dojox.mobile then we are probably on mobile.
		dojo.config.diagramForMobile = true;
	} else {
		// Test some well-known user agent names:
		var nua = navigator.userAgent.toLowerCase();
		if((nua.indexOf('iphone') >= 0) ||
		   (nua.indexOf('ipod') >= 0) ||
		   (nua.indexOf('ipad') >= 0) ||
		   (nua.indexOf('android') >= 0) ||
		   (nua.indexOf('blackberry') >= 0)){
			dojo.config.diagramForMobile = true;
		}
	}
}
dojo.config._notDiagramForMobile = !dojo.config.diagramForMobile;

dojo.require("dojox.gfx.shape");
dojo.require('dojox.collections.Dictionary');
dojo.require('ibm_ilog.diagram.util.Geometry');

//
// TransformChain: intended to be used on Node/Link via dojo.extend
//

ibm_ilog.diagram.TransformChain = {
	// summary:
	//		Provides additional methods on GFX shapes to compute transforms.
	// tags:
	//		private

    getTransformToContainer: function(/*dojox.gfx.Group*/container) {
        // summary:
        //        Gets the transform from the parent coordinate system
        //        to the specified container coordinate system.
        // container: dojox.gfx.Group:
        //        the target container.
        
        if (container === null) {
            //TODO: ErrorReporter
            throw {
                'name': 'ArgumentNullException',
                message: "Null parameter: container"
            };
        }
        var m = dojox.gfx.matrix;
        if (container === this) {
            return m.identity;
        }
        var t1 = m.identity;
        // does not take into account shape transform
        var p = this.getParent();
        // TODO: no need to go up til the surface. Should stop to 
        // 1st common ancestor. check perf gain ?
        if (p !== null) {
            t1 = p._getRealMatrix() || m.identity;
        }
        var t2 = container._getRealMatrix() || m.identity;
        var t = m.multiply(t2.inverse(), t1);
        return t;
    },
    
    getShapeToContainerTransform: function(container) {
        // summary:
        //        Gets the transform from this shape coordinate system
        //        to the specified container coordinate system.
        // description:
        //        This method takes into account the shape transform.
        // container: dojox.gfx.Group:
        //        the target container.
        if (container === null) {
            //TODO: ErrorReporter
            throw {
                'name': 'ArgumentNullException',
                message: "Null parameter: container"
            };
        }
        var m = dojox.gfx.matrix;
        if (container === this) {
            return m.identity;
        }
        // TODO: no need to go up til the surface. Should stop to 
        // 1st common ancestor. check perf gain ? 
        // t1: transform from surface -> shape
        var t1 = this._getRealMatrix() || m.identity;
        // t2: transform from surface -> container
        var t2 = container._getRealMatrix() || m.identity;
        return m.multiply(t2.inverse(), t1);
    }
};
dojo.extend(dojo.version.major > 1 || (dojo.version.major == 1 && dojo.version.minor >= 6) ? dojox.gfx.shape.Shape : dojox.gfx.Shape, ibm_ilog.diagram.TransformChain);

//
// BoundingBox fix
//

(function(){
	var g = ibm_ilog.diagram.util.Geometry;
	var eq = g.equalRect;

	
ibm_ilog.diagram.BoundingBoxFix = {
	// summary:
	//		Fixes/provides additional methods on GFX groups for bounding box computations.
	// tags:
	//		private
	
    _computeBoundingBox: function() {
    	var bb = null;
    	var cs = this.children;
    	var ncs = this.children.length;
    	var c;
        for (var i = 0; i < ncs; ++i) {
        	var c = cs[i];
            var cbb = c.getBoundingBox();
            if (!cbb) {
            	continue;
            }
            var ct = c.getTransform();
            if (ct && !ct.isIdentity()) {
                cbb = ct.transformRectangle(cbb);
            }
            bb = bb ? g.addRect(bb, cbb) : cbb;
        }

    	this._bbCache = bb;
        if (bb) {
            var t = this.getTransform() || dojox.gfx.matrix.identity;
            if (t) 
                bb = t.transformRectangle(bb);
        }
    	this._lbbCache = bb;
    },

    _prepareBoundingBoxCache: function() {
		var oldbb = this._bbCache;
		var oldlbb = this._lbbCache;
		
		if(!oldbb || !this._bbCacheValid) {
	    	this._computeBoundingBox();
	    	this._bbCacheValid = true;
		} 
	},
	
	getBoundingBox: function() {
        // summary:
        //        Gets the bounding box of this dojox.gfx.Group. Returns null if the group is empty.
        // description:
        //        This method returns the global children bounding box without
        //        taken into account the transform of this Group.
		this._prepareBoundingBoxCache();

		return this._bbCache;
    },
    
    _onChanged: function(source,reason) {
    	this._bbCacheValid = false;
    	// 	[av]
    	// 	this method became an "inlined inherited" at:
    	//	- Node.js
    	//	- Graph.js
    },
    
    getLocalBoundingBox: function() {
        // summary:
        //        Gets the bounding box of this dojox.gfx.Group in the local coordinate system.
        // description:
        //        This method returns the global children bounding box 
        //        taken into account the transform of this Group.        
		this._prepareBoundingBoxCache();

		return this._lbbCache;
    }
};

dojo.extend(dojox.gfx.Group, ibm_ilog.diagram.BoundingBoxFix);

})();


