/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.diagrams.edit.actionBindings.Menu");

dojo.require("rmpw.diagrams.edit.actionBindings.Button");

/**
 * Defines a drop down menu
 * 
 * @author Steve Shaw
 */
dojo.declare("rmpw.diagrams.edit.actionBindings.Menu", rmpw.diagrams.edit.actionBindings.Button, {
	
	/*
	 * either returns a Menu object that contains enabled only items or will return a 
	 * button if only one action binding exists in the menu after excluding disabled items.
	 */
	_filterDisabled: function(targetMapping) {
		var actionBindings = this.getActionBindings();
		var enabledBindings = [];
		
		// only return action bindings that are enabled
		for (var i=0, ab; ab=actionBindings[i]; i++) {
			var abEnabled = ab._filterDisabled(targetMapping);
			if (abEnabled) {
				enabledBindings.push(ab);
			}
		}
		
		if (enabledBindings.length === 1 && enabledBindings[0] instanceof rmpw.diagrams.edit.actionBindings.Button) {
			return enabledBindings[0];
		}
		else if (enabledBindings.length > 1) {
			var actionBinding = new rmpw.diagrams.edit.actionBindings.Menu(this.host);
			actionBinding.setIconURL(this.getIconURL());
			actionBinding.setDescription(this.getDescription());
			actionBinding.setLabel(this.getLabel());
			actionBinding.setIconClass(this.getIconClass());
			actionBinding._dropDownActionBindings = enabledBindings;
			
			return actionBinding;
		}
		
		return null; // no bindings enabled 
	},
	
	setActionBindings: function() {
		this._dropDownActionBindings = arguments;
	},
	
	getActionBindings: function() {
		return this._dropDownActionBindings;
	}
	
});
