/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.diagrams.edit.actions.AddElement");

dojo.require("rmpw.diagrams.edit.actions.Action");
dojo.require("rmpw.dsv.editor.form.ui.CreateResource");

/**
 *Add class 
 * 
 * @author Steve Shaw
 */
dojo.declare("rmpw.diagrams.edit.actions.AddElement", rmpw.diagrams.edit.actions.Action, {
	
	setOwnerTrace: function(ownerTrace) {
		this._options.ownerTrace = ownerTrace;
	},
	
	setTypeId: function(typeId) {
		this._options.typeId = typeId;
	},
	
	setAddTypeId: function(addTypeId) {
		this._options.addTypeId = addTypeId;
	},
	
	suppressLabelIncrement: function(suppress) {
		this._options.suppress = suppress;
	},
	
	setRealtimeMode: function(realtime) {
		this._realtime = realtime;
	},
	
	getElementPosition: function(e) {
		var renderMapping = this.host.renderMapping;
		
		// try to get creation point from context action bar, else use action button position
		var controller = renderMapping.root.policies.contextActionBarScope;
		var offset = renderMapping.root.viewport.getAbsoluteCanvasOffset();
		var p = controller.getCreationPoint() || {x: e.pageX - offset.x, y: e.pageY - offset.y};
		var relpos = new rmpw.diagrams.render.util.Point(p.x, p.y); 
		
		// find relative location
		var isShapeContainer = !this.host.renderMapping.policies.graphContainer || 
			!this.host.renderMapping.policies.graphContainer.isTopLevel();
		renderMapping.policies.graphContainer.translateToRelative(relpos);
		if (isShapeContainer) {
			if (relpos.x<10){ 
			    relpos.x=10;
			}
			if (relpos.y<22){ 
			    relpos.y=22;
			}
		}
		return relpos;
	},
	
	/*
	 * Gets the arguments passed into the command at execution time
	 */
	getExecutionArguments: function (e) {
		var editController = this.host;
		var relpos = this.getElementPosition(e);
		var ownerElement = editController.renderMapping.getSourceByPath(this._options.ownerTrace);
		
		return {
			ownerElement: ownerElement,
			x: relpos.x,
			y: relpos.y,
			suppressIncrement: this._options.suppress,
			viewer: editController.root,
			diagram: editController.renderMapping
		};
	},
	
	/*
	 * Gets the command to be execution by the action
	 */
	getCommand: function(e) {
		var commandPackage = this.host.root.getCommandPackage();
		
		var createElCmd = commandPackage.select({
			typeId: [this._options.typeId],
			addTypeId: this._options.addTypeId,
			commandId: "rmpw.common.command.AddElement"
		});
		
		return createElCmd;
	},
	
	run: function(e) {
		
		var editController = this.host;
		var relpos = this.getElementPosition(e);
		
		if (!this._realtime) {		
			this.waiting(e, true);
			this._connectAddChild = dojo.connect(this.host.renderMapping, "addChild", dojo.hitch(this, function(child) {
				dojo.disconnect(this._connectAddChild);
				delete this._connectAddChild;
				
				if (child.policies.graphNode) {
					child.policies.graphNode.setInitialPosition(relpos.x, relpos.y);
				}
				if (child.__editController.policies.selectable) {
					child.__editController.policies.selectable.selectOnUpdate();
				}
				
				child.enforcePolicies();
				this.waiting(e, false);
			}));
			
			// check if identifier is not generated - if not invoke creation dialog
			var command = this.getCommand(e);
			if (command && !command._options.generateIdentifier) {
				var dialog = new rmpw.dsv.editor.form.ui.CreateResource();
				var target = { rdf_type: [{rdf_about: command._options.addTypeId}] };
				dialog.createResource(command._options.defaultName, command._options.defaultName, "", null, this, "dmcodegen_defines", target);
				dojo.connect(dialog, "onCancel", dojo.hitch(this, function() {
					dojo.disconnect(this._connectAddChild);
					delete this._connectAddChild;
					this.waiting(e, false);
				}));
				var self = this;
				dojo.connect(dialog, "onSubmit", dojo.hitch(this, function(createdResource) {
					var root = e.viewer ? e.viewer : self.host.root;
					var commandStack = root.getCommandController().commandStack;
					var args = self.getExecutionArguments(e);
					args.semanticModel = createdResource;
					var bret = commandStack.execute(command, args);
					if (!bret) {
						dojo.disconnect(self._connectAddChild);
						delete self._connectAddChild;
						this.waiting(e, false);
					}
				}));
				
				return true;
			}
			else {
				var bret = this.inherited(arguments);
				if (!bret) {
					dojo.disconnect(this._connectAddChild);
					delete this._connectAddChild;
					this.waiting(e, false);
				}
			}
		} else {
			if (!this._pendingChildren) this._pendingChildren = [];
			this._pendingChildren.push(relpos);
			this.waiting(e, true);
			this.addCreationFeedback(editController);
			
			// action may run multiple times before children are created
			if (!this._connectAddChild) {
				this._connectAddChild = dojo.connect(this.host.renderMapping, "addChild", dojo.hitch(this, function(child) {
					// get location of child if it's a node
					var pt = this._pendingChildren.shift();
					if (this._pendingChildren.length == 0) {
						dojo.disconnect(this._connectAddChild);
						delete this._connectAddChild;
						delete this._pendingChildren;
					}
					
					if (child.policies.graphNode) {
						child.policies.graphNode.setInitialPosition(pt.x, pt.y);
					}
					if (child.__editController.policies.selectable) {
						child.__editController.policies.selectable.selectOnUpdate();
					}
					
					child.enforcePolicies();
					this.waiting(e, false);
					this.removeCreationFeedback();
				}));
			}
			
			var bret = this.inherited(arguments);
			if (!bret) {
				dojo.disconnect(this._connectAddChild);
				delete this._connectAddChild;
				delete this._pendingChildren;
				this.waiting(e, false);
				this.removeCreationFeedback();
			}
		}
	},
	
	addCreationFeedback: function(editController) {
		if (editController.renderMapping.policies.graphContainer) {
			// add feedback to show where shape will be created
			var feedback = this._createFeedback(editController);
			if (feedback) {
				if (!this.creationFeedback) {
					this.creationFeedback = [];
				}
				this.creationFeedback.push(feedback);
			}
		}
	},

	_createFeedback: function(editController) {
		if (editController.renderMapping.policies.graphContainer) {
			var pt = editController.root.policies.contextActionBarScope.createAt;
			if (pt) {
				var creationFeedback = editController.renderMapping.root.feedbackLayer.createGroup();
				var rect = creationFeedback.createRect( { x:pt.x,  y: pt.y, width: 8, height: 8 } );
				rect.setStroke({color: 'black', lineWidth: 1, lineStyle: "solid"});
				rect.rawNode.setAttribute("pointer-events", "none");
				rect.rawNode.style.shapeRendering = "crispEdges";
				return creationFeedback;				
			}
		}
		return null;
	},
	
	removeCreationFeedback: function(editController, pt) {
		if(this.creationFeedback){
			for (var i = 0; i < this.creationFeedback.length; i++) {
				this.creationFeedback[i].removeShape();
			}
			delete this.creationFeedback;	
		}
	}

	
});

