/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.sketch.behavior.ShapeController");

dojo.require("rmpw.diagrams.edit.EditController");

dojo.require("rmpw.diagrams.edit.tools.Focusable");
dojo.require("rmpw.diagrams.edit.tools.Resizable");
dojo.require("rmpw.diagrams.edit.tools.Moveable");
dojo.require("rmpw.diagrams.render.util.Point");

dojo.require("rmpw.sketch.serializer.SVGUtil");
dojo.require("dojox.gfx.path");

dojo.require( "rmpw.diagrams.edit.handles.HandleUtils" );
dojo.require("rmpw.sketch.utilities.ChildBounds");
dojo.require("rmpw.diagrams.edit.actionBindings.Button");
dojo.require("rmpw.diagrams.edit.actions.NativeCode");

dojo.require("rmpw.sketch.policies.TextEdit");
dojo.require("rmpw.sketch.actions.AddText");

dojo.requireLocalization("rmpw.sketch", "ToolStrings");
var strings = dojo.i18n.getLocalization("rmpw.sketch", "ToolStrings");

dojo.declare("rmpw.sketch.behavior.ShapeController", rmpw.diagrams.edit.EditController, {	
	
	position: rmpw.diagrams.edit.handles.HandleUtils,
	
	createActionBindings: function() {
		var actionBinding;
		
		actionBinding = this.createPolicy(rmpw.diagrams.edit.actionBindings.Button, "textButton");
		actionBinding.setIconURL( dojo.moduleUrl("rmpw.sketch", "icons/pal/text_pal16.png") );
		actionBinding.setDescription( strings.text_label );
		actionBinding.setAction(this.policies.text);
	},
	
	createActions: function() {
		var action;
		
		action = this.createPolicy(rmpw.sketch.actions.AddText, "text");
		
		/* 
		 * TODO: Acting strictly on groups at the moment ... needs some thought
		 * if working on primitives. 
		 */
		action = this.createPolicy(rmpw.diagrams.edit.actions.NativeCode, "changePosition");
		action.setRun(function(args) {
			var renderMapping = this.host.renderMapping;					
			var pt = new rmpw.diagrams.render.util.Point(args.x, args.y);
			var traces = renderMapping.traces;
			var owner = traces.transform.getOwner();
			var change = args.changeSet.createChange(owner);
			change.set("transform", "translate(" + pt.x + "," + pt.y + ")" );						
			
			var bounds = renderMapping.policies.graphNode.getBounds();
			
			if (renderMapping.sources.svgRoot.eClass && args.x != null && args.y != null){
				var mover = new rmpw.sketch.utilities.ChildBounds();
				bounds.x = args.x;
				bounds.y = args.y;
				mover.build( args.changeSet, renderMapping.sources.svgRoot, bounds );
			}
			
			return true;
		});
		
		/* 
		 * TODO: Acting strictly on groups at the moment ... needs some thought
		 * if working on primitives. 
		 */
		action = this.createPolicy(rmpw.diagrams.edit.actions.NativeCode, "changeSize");
		action.setRun(function(args) {
			var renderMapping = this.host.renderMapping;
			var pt = new rmpw.diagrams.render.util.Point(args.x, args.y);
			var traces = renderMapping.traces;
			var owner = traces.transform.getOwner();
			var change = args.changeSet.createChange(owner);
			
			var bounds = renderMapping.policies.graphNode.getBounds();
			
			if (renderMapping.sources.svgRoot.eClass && args.x != null && args.y != null){
				var mover = new rmpw.sketch.utilities.ChildBounds();
				bounds.x = args.x;
				bounds.y = args.y;
				bounds.width = args.width;
				bounds.height = args.height;
				mover.build( args.changeSet, renderMapping.sources.svgRoot, bounds );
			}
			
			/* TODO: transforms should work with scale rather than absolute dimensions */
			
			change.set("transform", "translate(" + pt.x + "," + pt.y + ") scale(" + args.width + "," + args.height + ")" );	
			
			return true;
		});
	},
	
	createTools: function() {
		var policy;

		if( this.root.__editMode ){
			policy = this.createPolicy( rmpw.diagrams.edit.tools.Focusable);

			policy = this.createPolicy( rmpw.diagrams.edit.tools.Resizable );
			policy.setOnChangeSize(this.policies.changeSize);
			
			policy = this.createPolicy( rmpw.diagrams.edit.tools.Moveable);
			policy.setOnChangePosition(this.policies.changePosition);
			
			policy = this.createPolicy(rmpw.sketch.policies.TextEdit);
			
			policy = this.createPolicy(rmpw.diagrams.edit.tools.ContextActionBar);
			
			policy.setActionBindings( this.policies.textButton );
		}	
		
		
    }
});