dojo.provide('ibm_ilog.graphlayout.circular.clayout.ClusterLayoutAlgo');

///////////////////////////////////////////////////////////////////////////////
// IBM ILOG Dojo Diagrammer 1.0
///////////////////////////////////////////////////////////////////////////////

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

dojo.require('ibm_ilog.graphlayout.AbstractGraphLayoutModel');
dojo.require('ibm_ilog.graphlayout.circular.clayout.CLCluster');
dojo.require('ibm_ilog.graphlayout.circular.clayout.CLLink');
dojo.require('ibm_ilog.graphlayout.circular.clayout.CLNode');
dojo.require('ibm_ilog.graphlayout.internalutil.GraphModelUtil');
dojo.require('ibm_ilog.graphlayout.internalutil.TranslateUtil');
dojo.require('ibm_ilog.graphlayout.internalutil.HashTable');
dojo.require('ibm_ilog.graphlayout.internalutil.Point');
dojo.require('ibm_ilog.graphlayout.GraphLayoutReport');

dojo.declare('ibm_ilog.graphlayout.circular.clayout.ClusterLayoutAlgo', null, 
{
  // summary:
  //		The main class for the Cluster Layout algorithm. 
  // description:
  //		The main class for the Cluster Layout algorithm. 
  //		<p>
  //		This algorithm handles a nested graph including intergraph links. It
  //		arranges the nodes of each subgraph on a circle, so that the number
  //		of link crossings is small. It respects the intergraph links and
  //		rotates the cluster so that the number of link crossings is small.  
  //		</p>
  //		<p>
  //		It assumes that all nodes are nearly square, all nodes are in
  //		subgraphs, but the subgraph nesting is only 1. Nodes that are inside
  //		subgraphs of subgraphs are not anymore handles. Each subgraph has a
  //		fixed position given as input to the layout.  
  //		</p>
  //		<p>
  //		Important: the cluster layout works only if the coordinates mode is
  //		view coordinates, or when the local transformer of each subgraph is
  //		only a translation without scale or skew. Otherwise, the layout is
  //		not able to determine the correct size of the nodes.   
  //		</p>
  // tags:
  //		private
  
  constructor : function(layout)
  {
    this._layout = layout;
  },
  
  _clusters: null,
  _layout: null,

  layout : function(redraw)
  {
    // summary:
    //		Computes the layout using the Cluster Layout algorithm. 
    // description:
    //		Computes the layout using the Cluster Layout algorithm. 
    //		To start the layout, call the method <code>
    //		ibm&#95;ilog.graphlayout.GraphLayout.performLayout()</code>.   
    // tags:
    //		private
    
    var model = this._layout.getGraphModel();
    var rootModel = model.getRoot();

    if (rootModel == null || !(rootModel instanceof ibm_ilog.graphlayout.AbstractGraphLayoutModel)) {
      rootModel = model;
    }
    var adapter = null;

    if (model instanceof ibm_ilog.graphlayout.AbstractGraphLayoutModel)
    {
      adapter = model;
      adapter.saveAndEnableViewCoordinates(rootModel);
    }

    try
    {
      this.collectData(model, rootModel);
      var offset = this._layout.getOffset();
      var maxNumberOfPerm = this._layout.getMaxNumberOfPermutations();
      var i;

      for (i = 0; i < this._clusters.length; i++)
        this._clusters[i].calcRadius(offset);

      for (i = 0; i < this._clusters.length; i++)
      {
        this._clusters[i].layout(maxNumberOfPerm, redraw, true);
        this._layout.increasePercentageComplete(Math.floor(i * 100 / this._clusters.length));
        this._layout.onLayoutStepPerformed(false, false);
      }

      for (i = 0; i < this._clusters.length; i++)
        this._clusters[i].cleanup();
      this._layout.increasePercentageComplete(100);
      this._layout.onLayoutStepPerformed(false, false);
      this._layout.getLayoutReport().setCode(ibm_ilog.graphlayout.GraphLayoutReport.LAYOUT_DONE);

      this._clusters = null;
    }
    finally
    {
      if (adapter != null) {
        adapter.restoreAndDisableViewCoordinates();
      }
    }
  },

  collectData : function(model, rootModel)
  {
    // summary:
    //		Build the internal data structures.
    // tags:
    //		private
    
    var nodeTable = new ibm_ilog.graphlayout.internalutil.HashTable();

    this._clusters = ibm_ilog.graphlayout.internalutil.TranslateUtil.newArray(model.getSubgraphsCount(), null);
    var i = 0;
    var e1 = new ibm_ilog.graphlayout.internalutil.ArrayIterator(model.getSubgraphs());
    var cluster;

    while (e1.hasNext())
    {
      var subgraph = e1.next();
      var submodel = rootModel.getGraphModel(subgraph);

      var rect = ibm_ilog.graphlayout.internalutil.GraphModelUtil.getObjectBounds(model, subgraph);
      cluster = new ibm_ilog.graphlayout.circular.clayout.CLCluster(submodel, true);
      cluster.setPosition1(new ibm_ilog.graphlayout.internalutil.Point(rect.x + 0.5 * rect.width, rect.y + 0.5 * rect.height));
      this._clusters[i] = cluster;
      var e2 = new ibm_ilog.graphlayout.internalutil.ArrayIterator(submodel.getNodes());

      while (e2.hasNext())
      {
        var obj = e2.next();
        var node = new ibm_ilog.graphlayout.circular.clayout.CLNode(obj, submodel);
        this._clusters[i].addNode(node);
        nodeTable.add(obj, node);
      }

      i++;
    }

    e1 = (new ibm_ilog.graphlayout.internalutil.ArrayIterator(model.getIntergraphLinks()));

    while (e1.hasNext())
    {
      var iobj = e1.next();
      var source = nodeTable.item(model.getFrom(iobj));
      var target = nodeTable.item(model.getTo(iobj));

      if (source != null && target != null)
      {
        new ibm_ilog.graphlayout.circular.clayout.CLLink(source, target);
      }
    }

    e1 = new ibm_ilog.graphlayout.internalutil.ArrayIterator(model.getSubgraphs());

    while (e1.hasNext())
    {
      var subg = e1.next();
      var subm = rootModel.getGraphModel(subg);
      var e3 = new ibm_ilog.graphlayout.internalutil.ArrayIterator(subm.getLinks());

      while (e3.hasNext())
      {
        var lobj = e3.next();
        var src = nodeTable.item(subm.getFrom(lobj));
        var tgt = nodeTable.item(subm.getTo(lobj));

        if (src != null && tgt != null)
        {
          new ibm_ilog.graphlayout.circular.clayout.CLLink(src, tgt);
        }
      }
    }

    nodeTable = null;
  },

  cleanData : function()
  {
    // summary:
    //		Cleans the internal data.
    // tags:
    //		private
    
    this._clusters = null;
  }

}
);
