/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.ui.clipboard.ClipboardTable");

dojo.require("jazz.ui.toolbar.Button");
dojo.require("jazz.ui.toolbar.TwoStateButton");
dojo.require("rmps.core.clipboard.Clipboard");
dojo.require("rmps.core.query.BasicQueryResult");
dojo.require("rmps.app.core.internal.util.QueryUtil");
dojo.require("rmps.app.ui.internal.search.SearchResults");

dojo.requireLocalization("rmps.ui.clipboard", "ClipboardWidgetMessages");

(function(){
	
var ResultSet = rmps.core.query.BasicQueryResult;
var QueryUtil = rmps.app.core.internal.util.QueryUtil;
var clipboard = rmps.core.clipboard.clipboard;

dojo.declare("rmps.ui.clipboard.ClipboardTable", rmps.app.ui.internal.search.SearchResults, {
	messages: dojo.i18n.getLocalization("rmps.ui.clipboard", "ClipboardWidgetMessages"),
	
	tableClass: "rmps.ui.clipboard._Table",
	readOnly: true,
	onSelect: null,
	isFullPage: false,
	onlyType: null,
	projectUri: null,
	disabledList: null,

	_selection: null,
	_updateDef: null,
	
	postCreate: function() {
		this.createToolbar = false;

		if (this.disabledList === null) {
			this.disabledList = [];
		}
		
		this.inherited(arguments);
		
		// Hide the header completely.
		dojo.style(this.headerContainerNode, {display : "none"});
	
		this._selection = [];		
		this._resultTable._onPageChange = dojo.hitch(this, this.onPageChange);
		
		if (!this.isFullPage) {
			// Set page size to 10
			this._resultTable.pagingControl.update(0, 10, null, null, null);
		}
		
		this._showPage(0);
	},
	
	refresh: function() {
		// recreate page
		if (this._updateDef) {
			this._updateDef.addCallback(this, function() {
				this.onPageChange(this.getQueryResult().query.page);				
			});
		}
	},
	
	getRowResourceUri: function(entry) {
		return entry;
	},
	
	getRowShortName: function(entry) {
		return null;
	},
	
	_showPage: function(page, preserveFilter) {
		this._updateDef = this._getResultSet(page);
		this._updateDef.addCallback(this, function(rs) {
			this.showResults(rs, preserveFilter, null);			
		});
	},
	
	_getResultSet: function(page) {
		var def = new dojo.Deferred();
		var cbdef = clipboard(this.projectUri);
		cbdef.addCallback(this, function(cb) {
			var pageSize = this._resultTable.pagingControl.pageSize;
			cb.getPageSet(page, pageSize, this.onlyType).addCallback(this, function(entries) {
				var rs = new ResultSet();
				dojo.mixin(rs, {
					id: "clipboard",
					entries: entries, 
					totalResults: cb.size,
					query: {
						page: page,
						pageSize: pageSize,
						limit: -1
					}
				});
				def.callback(rs);				
			});
		});
		cbdef.addErrback(function() {
			def.errback();
		});
		return def;
	},
	
	onPageChange: function(page) {
		this._showPage(page, true /* preserveFilter */);
	},
	


	_actionsHandler: function(result, rowObj, owned, resourceUri, presentationUri) {
		if (owned) {
			result.className = "actions";
			result.deferred = new dojo.Deferred();
			result.deferred.addCallback(dojo.hitch(this, function(info) {
				
				result.init = dojo.hitch(this, function(rowConfig, tableRow, rowIdx, columnConfig, cell, colIdx) {
					var toolbar = new jazz.ui.Toolbar();
					this.createRowActions(toolbar, rowObj, resourceUri, presentationUri,info.existsInCurrentContext);
					cell.style.width = ((toolbar.getChildren().length * 26) + 1) + "px";
					// use the Toolbar to connect so that disconnect is called on destroy
					toolbar.connect(cell, "onmousedown", dojo.hitch(dojo, dojo.stopEvent));
					cell.appendChild(toolbar.domNode);
					
					if (!info.existsInCurrentContext) {
						dojo.forEach(tableRow.children, function(td) {
							if (td != cell) {
								dojo.addClass(td,"dimmed")
							}
						});
					}
				});
				
				delete result.deferred;
			}));
			result.deferred.addErrback(function() {
				delete rowObj[propertyCache];
				delete result.deferred;
			});
		}
	},
	
	createRowActions: function(toolbar, rowObj, resourceUri, presentationUri,existsInCurrentContext) {
		if (existsInCurrentContext) {
			if (!this.readOnly) {
				this.createSelectionAction(toolbar, rowObj, resourceUri, presentationUri);
			}
			this.createPreviewAction(toolbar, rowObj, resourceUri, presentationUri);
			if (this.isFullPage) {
				this.createExplorerAction(toolbar, rowObj, resourceUri, presentationUri);	
			}
		}
		this.createRemoveAction(toolbar, rowObj, resourceUri, presentationUri);
	},
	
	createSelectionAction: function(toolbar, rowObj, resourceUri, presentationUri) {
		var selectbutton = new jazz.ui.toolbar.TwoStateButton({
			title: this.messages.select,
			titlePressed: this.messages.unselect,
			imgSrc: dojo.moduleUrl("rmps.ui", "graphics/icons/unchecked.png").toString(),
			imgSrcPressed: dojo.moduleUrl("rmps.ui", "graphics/icons/checked.png").toString(),
			onClick: dojo.hitch(this, function() {
				if (selectbutton.pressed) {
					this._selection.push(resourceUri);
				} else {
					var index = dojo.indexOf(this._selection, resourceUri);
					if (index !== -1) {
						this._selection.splice(index, 1);
					}
				}
				this.onSelect(this._selection);
				
			})
		});
		if (dojo.indexOf(this.disabledList, resourceUri)!==-1){
			selectbutton.attr("enabled", false);
		}
		toolbar.addChild(selectbutton);
	},
	
	createRemoveAction: function(toolbar, rowObj, resourceUri, presentationUri) {
		var removebutton = new jazz.ui.toolbar.Button({
			title: this.messages.remove,
			imgSrc: dojo.moduleUrl("rmps.ui", "graphics/icons/remove_grey_co.gif").toString(),
			onClick: dojo.hitch(this, function() {
				clipboard(this.projectUri).addCallback(this, function(cb) {
					cb.remove(resourceUri);
					
					// If in selection, remove.
					var index = dojo.indexOf(this._selection, resourceUri);
					if (index !== -1) {
						this._selection.splice(index, 1);
						this.onSelect(this._selection);
					}
					
					this.refresh();
				});
			})
		});
		toolbar.addChild(removebutton);
	},
	
	getSelection: function() {
		return this._selection;
	},
	
	createTable: function() {
		var tableConstructor = dojo.getObject(this.tableClass);

		return new tableConstructor({
			noEntriesLabel: this.messages.noResults,
			noFilterMatchesLabel: this.messages.noFilterMatches,
			pageSizeSelection: this.isFullPage 
		});
	},
	
	_handleProjectResult: function(deferreds, project, properties, baseline) {
		var resources = {};
		for (var resourceUri in project) {
			if (resourceUri) {
				var groupKey = this._getGroupKey(resourceUri);
				if (!resources[groupKey]) {
					resources[groupKey] = [];
				}
				resources[groupKey].push(resourceUri);
			}
		}

		for (var key in resources) {
			if (key) {
				dojo.when(
					this._getResourceInfo(key, resources[key], properties.concat(["creatorUri"]), baseline), // add creatorUri to make sure deleted elements are detected
					dojo.hitch(this, function(infos) {
						
						dojo.forEach(infos, function(info) {
							info.existsInCurrentContext = true;
							dojo.forEach(deferreds[info.uri], function(def) {
								def.callback(info);
							});
							delete deferreds[info.uri];
							delete project[info.uri];
							
							var index = resources[key].indexOf(info.uri);
							if (index != -1) {
								resources[key].splice(index,1);
							}
						});
						
						if (resources[key].length > 0) {
							dojo.when(
									this._getResourceInfo(key, resources[key], properties, ""),
									dojo.hitch(this, function(infos) {
										dojo.forEach(infos, function(info) {
											info.existsInCurrentContext = false;
											dojo.forEach(deferreds[info.uri], function(def) {
												def.callback(info);
											});
											delete deferreds[info.uri];
											delete project[info.uri];
										});
										this._handleError(project);
									}),
									dojo.hitch(this,function(error) {
										if (deferred.ioArgs && !dojo._isDocumentOk(deferred.ioArgs.xhr)) {
											rmps.core.error.createXhrError(deferred.ioArgs).dispatch();
										} else {
											rmps.core.error.createError(error).dispatch();
										}
										this._handleError(project);
									})
								);
						} else {
							this._handleError(project);
						}
					}),
					dojo.hitch(this,function(error) {
						if (deferred.ioArgs && !dojo._isDocumentOk(deferred.ioArgs.xhr)) {
							rmps.core.error.createXhrError(deferred.ioArgs).dispatch();
						} else {
							rmps.core.error.createError(error).dispatch();
						}
						this._handleError(project);
					})
				);
			}
		}
	}
});

dojo.declare("rmps.ui.clipboard._Table", rmps.ui.Table, {
	
	pageSizeSelection: false,
	
	_onPageChange: null,
	
	postCreate: function() {
		this.inherited(arguments);
		
		if (!this.pageSizeSelection) {
			// No page size selection
			this._itemsPerPage.style.display = "none";		
		}
	},
	
	onPreviousClick: function(e) {
		this._onPageChange(this.pagingControl.page);
		this.inherited(arguments);
	},
	
	onNextClick: function(e) {
		this._onPageChange(this.pagingControl.page);
		this.inherited(arguments);
	},
	
	_onPageSizeChange: function(e) {
		var pageSize = parseInt(this._pageSizeSelect.value, 10);
		this.pagingControl.update(null, pageSize, null, null);
		this._onPageChange(this.pagingControl.page);
	},
	
	_constructPageSizeDropDown: function() {
		if (!this.pageSizeSelection) {
			// No page size selection
			return;			
		}
		
		this.inherited(arguments);
	}
});

}());