/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.sketch.commands.shapes.CreateShapeAction");

dojo.require("rmpw.diagrams.edit.actions.Action");
dojo.require("rmpw.sketch.commands.shapes.CreateRectangle");
dojo.require("rmpw.sketch.commands.shapes.CreateCloud");
dojo.require("rmpw.sketch.commands.shapes.CreateBug");
dojo.require("rmpw.sketch.commands.shapes.CreateOval");
dojo.require("rmpw.sketch.commands.shapes.CreateActor");
dojo.require("rmpw.sketch.behavior.Scribbler");
dojo.require("rmpw.sketch.behavior.LineMaker");
dojo.require("rmpw.sketch.behavior.RectangleMaker");
dojo.require("rmpw.sketch.behavior.TextMaker");
dojo.require("rmpw.sketch.behavior.OvalMaker");

dojo.declare("rmpw.sketch.commands.shapes.CreateShapeAction", rmpw.diagrams.edit.actions.Action, {
	
	shapeCommand: null,
	
	constructor: function() {
		this.initShapeCommands();	
	},
	
	initShapeCommands: function(){
		
		this.shapeCommand = [];
		
		this.shapeCommand.Rectangle = rmpw.sketch.commands.shapes.CreateRectangle;
		this.shapeCommand.Cloud = rmpw.sketch.commands.shapes.CreateCloud;
		this.shapeCommand.Bug = rmpw.sketch.commands.shapes.CreateBug;
		this.shapeCommand.Oval = rmpw.sketch.commands.shapes.CreateOval;
		this.shapeCommand.Text = rmpw.sketch.commands.shapes.CreateText;
	},
	
	run: function(args){
		var editController = this.host;
		if(editController && editController.viewer && editController.viewer.getCommandController) {
			this.doRun(editController.viewer.getCommandController(), editController, args);
		}
	},
	
	doRun: function(controller, editController, args) {
		// parse args
		var relpos = editController.viewer.viewport.getSurfaceRelativeMousePosition(args.pageX, args.pageY);
		
		var commandArgs = {
				svgParent: editController.renderMapping.sources.svgRoot,
				shape: this._options.addTypeId,
				x: relpos.x,
				y: relpos.y,
				viewport: controller.viewer.viewport
		};
		
		switch( this._options.addTypeId ){
		
			case "Scribble":
			
				if(!this.scribbler){
					this.scribbler = new rmpw.sketch.behavior.Scribbler( controller );
				}
					
				this.scribbler.capture( commandArgs );
				
				break;
			
			case "Line":
				
				if(!this.lineMaker){
					this.lineMaker = new rmpw.sketch.behavior.LineMaker( controller );
				}
					
				this.lineMaker.capture( commandArgs );
				
				break;
				
			case "Rectangle":
				
				if(!this.rectMaker){
					this.rectMaker = new rmpw.sketch.behavior.RectangleMaker( controller );
				}
					
				this.rectMaker.capture( commandArgs );
				
				break;
				
			case "Oval":
				
				if(!this.ovalMaker){
					this.ovalMaker = new rmpw.sketch.behavior.OvalMaker( controller );
				}
					
				this.ovalMaker.capture( commandArgs );
				
				break;
				
				
			case "Cloud":
				
				if(!this.cloudMaker){
					this.cloudMaker = new rmpw.sketch.behavior.CloudMaker( controller );
				}
					
				this.cloudMaker.capture( commandArgs );
				
				break;
				
			case "Text":
				
				if(!this.textMaker){
					this.textMaker = new rmpw.sketch.behavior.TextMaker( controller );
				}
					
				this.textMaker.capture( commandArgs );
				
				break;
				
			case "TextBox":
				
				console.log( 'CREATE SHAPE ACTION - TEXTBOX' );
				
				if(!this.boxMaker){
					this.boxMaker = new rmpw.sketch.behavior.TextMaker( controller );
				}
					
				this.boxMaker.capture( commandArgs );
				
				break;
				
			
			default:
			
				if( this.shapeCommand[this._options.addTypeId] ){	
					controller.commandStack.execute( new this.shapeCommand[this._options.addTypeId](), commandArgs );
				}else {
					throw "Trying to call an unknown creation command";	
				}
				
				break;
		}		
	}	
});

