/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.core.internal.WebUiInitializers");

//
// Clients must simply require this class in their application.
//

dojo.require("jazz.core.RegistryFactory");
dojo.require("jazz.core.loader");

var EXT_PT = "com.ibm.xtools.rmps.web.app.webUiInitializers";
var E_INITIALIZER = "initializer";
var A_JS_CLASS = "jsClass";
var A_JS_MODULE = "jsModule";
var A_RESOURCE_BINDING = "resourceBinding";
var A_CLASS_BINDING = "classBinding";

var resourceBindings = null;
var classBindings = null;

var queue = [];

//
// Reads the server extension. All unbound initializers are executed
// immediately while all bound initializers are cached.
//
var init = function() {
	dojo.forEach(jazz.core.RegistryFactory.getRegistry().getExtensionPoint(EXT_PT).getExtensions(), function(extension) {
		dojo.forEach(extension.getConfigurationElements(), function(config) {
			if (E_INITIALIZER == config.getName()) {
				var className = config.getAttribute(A_JS_MODULE);
				
				var binding = config.getChildren()[0];
				
				if (binding) {
					var bindings;
					if (binding.getName() === A_RESOURCE_BINDING) {
						if (!resourceBindings) {
							resourceBindings = {};
						}
						bindings = resourceBindings[binding.getAttribute(A_JS_MODULE)];
						if (!bindings) {
							resourceBindings[binding.getAttribute(A_JS_MODULE)] = bindings = [];
							
						}
						bindings.push(className);
						
					} else if (binding.getName() === A_CLASS_BINDING) {
						if (!classBindings) {
							classBindings = {};
						}
						bindings = classBindings[binding.getAttribute(A_JS_CLASS)];
						if (!bindings) {
							classBindings[binding.getAttribute(A_JS_CLASS)] = bindings = [];
							
						}
						bindings.push(className);
					}	
				} else {
					queue.push(className);
				}
			}
		});
	});
};

var resourceBindingsHandle;
var classBindingsHandle;

var process = function(modules) {
	// load the queue
	while (modules[0]) {
		jazz.core.loader.load_sync(modules.splice(0, 1)[0]);
	}
};

var _handleResourceBinding = function(resourceName) {
	var initializers = resourceBindings[resourceName];
	if (initializers) {
		delete resourceBindings[resourceName];
		
		process(initializers);
		
		// check if we have any more bindings
		for (var i in resourceBindings) {
			return;
		}
		
		// no more bindings
		dojo.disconnect(resourceBindingsHandle);
		resourceBindings = resourceBindingsHandle = null;
	}
};

var _handleClassBinding = function(className) {
	if (dojo.isString(className)) {
		var initializers = classBindings[className];
		if (initializers) {
			delete classBindings[className];
			
			process(initializers);
			
			// check if we have any more bindings
			for (var i in classBindings) {
				return;
			}
			
			// no more bindings
			dojo.disconnect(classBindingsHandle);
			classBindings = classBindingsHandle = null;
		}
	}
};

var link = function() {
	var i;
	if (resourceBindings) {
		// Connect to dojo provides for resource bindings.
		// TMA: Connecting to dojo.provide causes loading errors. Using dojo.getObject until web UI initializers can be replaced.
		resourceBindingsHandle = dojo.connect(dojo, "getObject", this, _handleResourceBinding);
		
		// queue bindings for which their hook has already occurred
		for (i in resourceBindings) {
			if (dojo._hasResource[i]) {
				queue = queue.concat(resourceBindings[i]);
				delete resourceBindings[i];
			}
		}
	}
	
	if (classBindings) {
		// Connect to dojo setObject for class bindings.
		// Tried to connect to #declare but it caused loading errors.
		classBindingsHandle = dojo.connect(dojo, "setObject", this, _handleClassBinding);
		
		// queue bindings for which their hook has already occurred
		for (i in classBindings) {
			if (dojo.isFunction(dojo.getObject(i))) {
				queue = queue.concat(classBindings[i]);
				delete classBindings[i];
			}
		}
	}
	
	process(queue);
};

// START intialization
init();
dojo.addOnLoad(link);

})();