/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.dsv.editor.data.EcoreJsonStore");

dojo.require("dojox.json.ref");

/**
 * TODO: meant to eventually implement the dojo.data.api.Read interface
 */
dojo.declare("rmpw.dsv.editor.data.EcoreJsonStore", null, {

	/**
	 * Map of visualisation ID to the path within an instance model
	 * from its root to the root of the visualisation.
	 */
	queryMap: {
		SVG: "model[0].svg",
		UML2: "model[0].eAnnotations[0].contents[0]",
		TraceDiagram: "model"
	},
	
	// NOTE: statically declared because Layout.js creates a new data store every fetch.
	// TODO (MG): fix.
	loadedEditorCode: {},
	
	constructor: function(viewer, defaultVisualization, forceCodeReload) {
		this.viewer = viewer;
		this.defaultVisualization = defaultVisualization;
		this.forceCodeReload = forceCodeReload;
	},
	
	renderReady: function(editorURI, resourcePath) {
		var model;
		var modelIndex = {};
		
		var that = this;
		dojo.xhrGet({
			url: "resource/itemPath/Model" + resourcePath,
			handleAs: "text",
			load: function(data) {
				that._createDiagram(resourcePath, editorURI, dojox.json.ref.fromJson(data, {index: modelIndex}));
			},
			error: function(error) {
				console.error(error);
			}
		});	
	},
	
	fetch: function(resourcePath, visualizationId) {
		visualizationId = visualizationId || this.defaultVisualization;
		if(!resourcePath || resourcePath == '' || !visualizationId || visualizationId == '') {
			return;
		}		
		
		this.viewer.clear();
		
		// load editor code
		var codeReady;
		if(this.loadedEditorCode[visualizationId] && !this.forceCodeReload) {
			this.renderReady(visualizationId, resourcePath);
		} else {
			this.loadedEditorCode[visualizationId] = true;
			var that = this;
			dojo.xhrGet({
				url: "dsv/codegen",
				content: { render: visualizationId },
				handleAs: "text",
				load: function(data, ioArgs) {
					var requiredModulesHeader = ioArgs.xhr.getResponseHeader("required-modules") || [];
					var requiredModules = dojo.fromJson(requiredModulesHeader);
					jazz.core.loader.batch_load_async(requiredModules, function() {
						dojo.eval(data);
						that.renderReady(visualizationId, resourcePath);
					});
				},
				error:function(e) {
					console.error(e);
				}
			});

		}
	},
	
	createNew: function(resourcePath, visualizationId) {
		//TODO stub function ... 
	},
	
	save: function() {
		// TODO (MG): send the patch that describes changes made, rather than the whole resource
		// 			The whole resource should be used in an xhrPost() when saving a new resource 
		dojo.xhrPut({
			url: "resource/itemPath/Model" + this.viewer.resourceId,
			putData: dojox.json.ref.toJson(this.viewer.resource)
		});
	},
	
	deleteResource: function() {
		dojo.xhrDelete({
			url: "resource/itemPath/Model" + this.viewer.resourceId
		});
	},
	
	_createDiagram: function(resourcePath, visualizationId, model) {
		// initialise repository
		this.viewer.resource = model;			
		this.viewer.resourceId = resourcePath;

		var editorPackage = "rmpw.dsv.codegen." + visualizationId;

		// initialise domain package
		var allPackages = new Function("return new " + editorPackage + ".domain.AllPackages();")();
		// TODO (MG): support multiple packages
		var defaultPackage;
		for(var packageNames in allPackages.byName) {
			defaultPackage = packageNames;
			break;
		}
		this.viewer.commandPackage = allPackages.byName[defaultPackage];

		// create root shape
		var packageMetadata = new Function("return " + editorPackage + ".render._Metadata.prototype;")();
		var rootShape = new Function("return new " + editorPackage + ".render." + packageMetadata.rootShape + "();")();
		rootShape[packageMetadata.graphicsRootSetter](this.viewer.viewport.contents);
		
		// attach model root to root shape
		var modelRootPath = this.queryMap[visualizationId] || "model[0]";
		var modelRoot = new Function("model", "return " + modelRootPath + ";")(model);
		rootShape[packageMetadata.modelRootSetter](modelRoot);

		// create root behaviour controller
		var behaviorMetadata = new Function("return " + editorPackage + ".behavior._Metadata.prototype;")();
		var rootBehavior = new Function("rootShape", "return new " + editorPackage + ".behavior." + behaviorMetadata.rootController + "(rootShape);")(rootShape);
		
		// do a bunch of stuff that should be done by the behaviour controller itself...
		// TODO (MG): remove
		var viewer = this.viewer;
		rootBehavior._connectConstruction = dojo.connect(rootShape, "applyModelChanges", this, function() {
			rootBehavior.setParent(viewer);
			dojo.disconnect(rootBehavior._connectConstruction);
			delete rootBehavior._connectConstruction;
		});
		rootBehavior._connectDestroy = dojo.connect(rootShape, "destroy", this, function() {
			dojo.disconnect(rootBehavior._connectDestroy);
			rootBehavior.destroy();
		});		

		// hook for post diagram load initialisation
		this.viewer.setDiagram(rootShape);
	}
});