
/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.dsv.editor.data.KentonFileStore");
dojo.require("dojo.data.ItemFileReadStore");

dojo.declare("rmpw.dsv.editor.data.KentonFileStore", [dojo.data.ItemFileReadStore], {
	_fetchItems: function(	/* Object */ keywordArgs, 
						    /* Function */ findCallback, 
						    /* Function */ errorCallback){
	var self = this,
	filter = function(requestArgs, arrayOfItems){
		var items = [],i, key;
		if(requestArgs.query){
			var value,
		    ignoreCase = requestArgs.queryOptions ? requestArgs.queryOptions.ignoreCase : false;
			var regexpList = {};
			for(key in requestArgs.query){
				value = requestArgs.query[key];
				if(typeof value === "string"){
					regexpList[key] = dojo.data.util.filter.patternToRegExp(value, ignoreCase);
				}else if(value instanceof RegExp){
					regexpList[key] = value;
				}
			}
			for(i = 0; i < arrayOfItems.length; ++i){
				var match = true;
				var candidateItem = arrayOfItems[i];
				if(candidateItem === null){
					match = false;
				}else{
					for(key in requestArgs.query){
						value = requestArgs.query[key];
						if(!self._containsValue(candidateItem, key, value, regexpList[key])){
							match = false;
						}
					}
				}
				if(match){
					items.push(candidateItem);
				}
			}
			findCallback(items, requestArgs);
		}else{
			// We want a copy to pass back in case the parent wishes to sort the array. 
			// We shouldn't allow resort of the internal list, so that multiple callers 
			// can get lists and sort without affecting each other.  We also need to
			// filter out any null values that have been left as a result of deleteItem()
			// calls in ItemFileWriteStore.
			for(i = 0; i < arrayOfItems.length; ++i){
				var item = arrayOfItems[i];
				if(item !== null){
					items.push(item);
				}
			}
			findCallback(items, requestArgs);
		}
	};

	if(this._loadFinished){
		filter(keywordArgs, this._getItemsArray(keywordArgs.queryOptions));
	}else{
		if(this._jsonFileUrl){
			//If fetches come in before the loading has finished, but while
			//a load is in progress, we have to defer the fetching to be 
			//invoked in the callback.
			if(this._loadInProgress){
				this._queuedFetches.push({args: keywordArgs, filter: filter});
			}else{
				this._loadInProgress = true;
				var getArgs = {
						url: self._jsonFileUrl, 
						handleAs: "json-comment-optional",
						preventCache: this.urlPreventCache,
						failOk: this.failOk,
						headers: {
						"Accept": "application/rdf+json",
							"X-ibm-rmps-internal": "true"
						}
					};
				var getHandler = jazz.client.xhrGet(getArgs);
				getHandler.addCallback(function(data){
					try{
						self._getItemsFromLoadedData(data);
						self._loadFinished = true;
						self._loadInProgress = false;
						
						filter(keywordArgs, self._getItemsArray(keywordArgs.queryOptions));
						self._handleQueuedFetches();
					}catch(e){
						self._loadFinished = true;
						self._loadInProgress = false;
						errorCallback(e, keywordArgs);
					}
				});
				getHandler.addErrback(function(error){
					self._loadInProgress = false;
					errorCallback(error, keywordArgs);
				});
				}
			}
		  }
	}
}
);
