/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2011.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.editor.EditorRegistry");

dojo.require("jazz.core.loader");
dojo.require("jazz.util.array");

var entries = [];
var typeToEditorInput = {};

rmps.ui.editor.EditorRegistry = {
	
	createEditorInput: function(type, args) {
		var editorInputClass = typeToEditorInput[type];
		if (editorInputClass) {
			if (dojo.isString(editorInputClass)) {
				var classConstructor = dojo.getObject(editorInputClass);
				if (!classConstructor) {
					jazz.core.loader.load_sync(editorInputClass);
					classConstructor = dojo.getObject(editorInputClass);
				}
				return new classConstructor(args);
			} else {
				return new editorInputClass(args);
			}
		}
		return null;
	},
	
	getEditorClass: function(editorInput) {
		for (var i = 0; i < entries.length; ++i) {
			var entry = entries[i];
			if (entry.matcher(editorInput)) {
				if (dojo.isString(entry.editorClass)) {
					var classConstructor = dojo.getObject(entry.editorClass);
					if (!classConstructor) {
						jazz.core.loader.load_sync(entry.editorClass);
						classConstructor = dojo.getObject(entry.editorClass);
					}
					return classConstructor;
				} else {
					return entry.editorClass;
				}
			}
		}
		return null;
	},
	
	createEditor: function(editorInput) {
		var classConstructor = this.getEditorClass(editorInput);
		if (classConstructor) {
			return new classConstructor({editorInput: editorInput});
		}
		return null;
	},
	
	register: function(editorClass, matcher, priority) {
		if (isNaN(priority)) {
			priority = 0;
		}
		jazz.util.array.insertByPriority(entries, {
				p: priority,
				matcher: matcher,
				editorClass: editorClass
			});
	},
	
	registerEditorInput: function(editorInputClass, type) {
		if (typeToEditorInput[type]) {
			throw new Error("Editor input class '" + editorInputClass + "' is already registered for type '" + type + "'");
		}
		typeToEditorInput[type] = editorInputClass;
	}
};

})();