dojo.provide('ibm_ilog.diagram.editor.adorners.ComponentAdorner');

dojo.require('ibm_ilog.diagram.templates.editor_templates');
dojo.require('ibm_ilog.diagram.adorners.Adorner');
dojo.require('ibm_ilog.diagram.util.GraphUtil');

(function() {

	var g = ibm_ilog.diagram.util.Geometry,	Rect = g.Rect,	EmptyRect = g.EmptyRect, Size = g.Size, PointZero = g.PointZero, Point = g.Point;
	var m = dojox.gfx.matrix;

	var __invisibleLocation = Rect(ibm_ilog.diagram._invisibleLocation, ibm_ilog.diagram._invisibleLocation, ibm_ilog.diagram._invisibleSize, ibm_ilog.diagram._invisibleSize);

var cls = ibm_ilog.diagram.gfxDeclaration(dojo.declare('ibm_ilog.diagram.editor.adorners.ComponentAdorner', [ibm_ilog.diagram.adorners.Adorner], {

	//	summary:
	//		An adorner for adorning components of Graph elements. 
	//
		
	//
	// _component: dojox.gfx.Shape
	//
	_component: null,
	
	//
	//	_componentBounds: dojox.gfx.Rect
	//
	_componentBounds: null,
	
	
	setComponent: function( /*Shape|string|null*/ component ) {
		//
		// 	summary:
		//		Sets the attachPoint name of visual shape associated with the property. 
		//		If null the whole element shape is asummed.
		//
		if(component && dojo.isString(component)) {
			this._component = this[component];
		} else {
			this._component = component;
		}
	},

	getComponent: function() {
		//
		//	summary:
		//		Returns the attachPoint name of the visual shape associated with the property. 
		//
		return this._component;
	},
	
	_computeElementBoundsChanged: function () {
		//
		//	summary:
		//		Recomputes the geometric values published to the template (adds 
		//		property component bounds to the values already computed in the 
		//		superclass)   
		//
	  var c,bb;
	  // compute component bounds	  
	  if(this._component && this._required() ) {
			c = this._component;
			  
			bb = c.getBoundingBox();
			if(bb) { // gfx does not support this properly, some renderers may fail. Fallback to node BB
				var pm = this._diagram.getGraph()._getRealMatrix().inverse();
				var cm = c.getParent()._getRealMatrix() || m.identity;
				var t = m.multiply(pm,cm);
				this._componentBounds = t.transformRectangle(bb);
		  } else {
		  	this._componentBounds = null;
		  }
	  } else {
	  	this._componentBounds = null;
	  }
	  this.inherited(arguments);
	},
	
	_updateLayout: function() {
		//
		//	summary:
		//		Publish new geometric values (causes template bindings to be applied if changed).
		//
		
		this.startDTLBatch();

		this.inherited(arguments);
		
		// also publish component-related coords (fallback to element coords if not present)
		
		var bb = __invisibleLocation;
		if (this._adorned) {
			bb = this._viewport.contentToClient().transformRectangle(this._componentBounds || this._elementBounds);
		}
		this.setCompWidth(bb.width);
		this.setCompHeight(bb.height);
		this.setCompRight(bb.x + bb.width);
		this.setCompBottom(bb.y + bb.height);
		this.setCompLeft(bb.x);
		this.setCompTop(bb.y);
		this.setCompCenterX(bb.width / 2);
		this.setCompCenterY(bb.height / 2);
		
		this.endDTLBatch();
	}
	
}));

ibm_ilog.diagram.templating.declareBindableProperty(cls, "compLeft", null);
ibm_ilog.diagram.templating.declareBindableProperty(cls, "compTop", null);
ibm_ilog.diagram.templating.declareBindableProperty(cls, "compBottom", null);
ibm_ilog.diagram.templating.declareBindableProperty(cls, "compRight", null);
ibm_ilog.diagram.templating.declareBindableProperty(cls, "compWidth", null);
ibm_ilog.diagram.templating.declareBindableProperty(cls, "compHeight", null);
ibm_ilog.diagram.templating.declareBindableProperty(cls, "compCenterX", null);
ibm_ilog.diagram.templating.declareBindableProperty(cls, "compCenterY", null);

})();
