dojo.provide('ibm_ilog.diagram.editor.interactors.DeleteKeyInteractor');

dojo.require('ibm_ilog.diagram.interactors.KeyboardInteractorManager');
dojo.require('ibm_ilog.diagram.interactors.Interactor');
dojo.require('ibm_ilog.diagram.editor.undo.UndoActionList');
dojo.require('ibm_ilog.diagram.editor.undo.DeleteAction');
dojo.require('ibm_ilog.diagram.editor.EditingUtils');

dojo.require('ibm_ilog.diagram.util.Geometry');
dojo.require('ibm_ilog.diagram.util.GraphUtil');

(function(){
	var g = ibm_ilog.diagram.util.Geometry;
	var gu = ibm_ilog.diagram.util.GraphUtil;
	var m = dojox.gfx.matrix;
	
dojo.declare("ibm_ilog.diagram.editor.interactors.DeleteKeyInteractor", [ibm_ilog.diagram.interactors.Interactor], {
	//summary:
	//		This Interactor manages the graphElements deletion through the keyboard interaction.
	//		This Interactor have 2 connections: deleteDelete, deleteBackSpace. Both connections calls the same delete method.
	_diagram: null,
	_serializer: null,
	initialize: function (diagram) {
	// summary:
    //		initialize the new instance.
    // diagram: ibm_ilog.diagram.editor.DiagramEditor
    //		the associated diagram editor.		
		this._diagram = diagram;
		return this._initialize();
	},
	getKeyInteractorId: function(){
		// summary:
	    //		returns the Interactor Id
		return ibm_ilog.diagram.editor.interactors.DeleteKeyInteractor.KeyInteractorId;
	},
	getDefaultConnections: function () {
		// summary:
		//		returns the default event mapping
		
		return {
			deleteDelete: {
			hotKey: dojo.keys.DELETE,	
			connectTo: "doDelete"
			}, deleteBackSpace: {
				hotKey: dojo.keys.BACKSPACE,
				connectTo: "doDelete"
			}
		};
	},
	
	doDelete: function(){
		// summary:
		//		Deletes the selected elements and all the connected links. If none selected,
		//		removes the focused element and all connected links.
		var targetSet = this._buildTargetSet();
		return targetSet ? this._diagram.deleteGraphElements(targetSet) : false;
	},
	
	_buildTargetSet: function(){
		//
		//	summary:
		//		Builds the set of entities that are going to be affected by the delete action.
		//
		//	description:
		//		gets the selected object and iterate over them to add all the children and 
		//		connected nodes to to deletion target set.
		//		If no element is selected, the target set is created based on the focused object.
		var selection = this._diagram.getSelection();
		var s = selection.fastGet().toArray();
		var targetSet = null;
		
		if(s.length == 0){
			var c = this._diagram.getFocusedElement();
			if(c){
				targetSet = [c];
			}else{
				return null;
			}
		}else{
			targetSet = s;
		}
		return targetSet;
	}
});

ibm_ilog.diagram.editor.interactors.DeleteKeyInteractor.KeyInteractorId = "Delete";
})();