dojo.provide('ibm_ilog.diagram.editor.interactors.LinkConnectionHandleInteractor');

dojo.require('ibm_ilog.diagram.interactors.DragInteractor');
dojo.require('ibm_ilog.diagram.editor.interactors.ConnectionTargetInteractor');

dojo.require('ibm_ilog.diagram.util.Batch');
dojo.require('ibm_ilog.diagram.util.Geometry');
dojo.require('ibm_ilog.diagram.Port');
dojo.require('ibm_ilog.diagram.Link');

dojo.require('ibm_ilog.diagram.editor.undo.UndoManager');
dojo.require('ibm_ilog.diagram.editor.undo.ReConnectAction');
dojo.require('ibm_ilog.diagram.editor.undo.ReparentingAction');
dojo.require('ibm_ilog.diagram.editor.undo.MultipleAction');
dojo.require('ibm_ilog.diagram.editor.undo.UndoActionList');

(function(){
	var g = ibm_ilog.diagram.util.Geometry;
	var m = dojox.gfx.matrix;
	var gu = ibm_ilog.diagram.util.GraphUtil;

dojo.declare("ibm_ilog.diagram.editor.interactors.LinkConnectionHandleInteractor", [ibm_ilog.diagram.interactors.DragInteractor], {

	//	_handle: /*ibm_ilog.diagram.editor.ConnectionHandle*/
	_handle:null,
	_automaticPortType: false,
	_target: null,
	_targetPosition: null,
	_targetInteractor: null,
	_allowFreeLinks: false,
	_link: null,
	_savedConnection: null,
	_oldLinkParent: null,
	
	initialize: function( /* ibm_ilog.diagram.editor.ConnectionHandle */ connectionHandle ) {
		this._handle = connectionHandle;
		this._viewport = connectionHandle.getViewport();
		this._targetInteractor = new ibm_ilog.diagram.editor.interactors.ConnectionTargetInteractor().initialize(this._handle._adorner._viewport._diagram,this);
		this._initialize();
		return this;
	},
	
	setTarget: function(target){
		// Summary:
		// 		sets the target to be binded with the link 
		this._target = target;
	},
	
	setPortPosition: function(position){
		// Summary:
		//		sets the port position where to bind the link
		this._targetPosition = position;
	},
	_getInitialEventSource: function() {
		return this._handle;
	},

    _dragPreStart: function() {
		this._allowFreeLinks = this._viewport.getDiagram().getAllowFreeLinks();
		this._handle.interactionBegun(this);
		this.inherited(arguments);
	},
	
	_dragMove: function(e) {
    	
		this.inherited(arguments);
		
    	var A = this._handle._adorner;
    	var V = A._viewport;
    	var portPosition = this._handle.getLocation();

		if( this._link == null ){
			this._link = A._adorned;
			this._targetPosition = null;
			var fixedNode,isStart;
			
			var port,point;
			if(portPosition == 0){
				port= this._link.getStartPort();
				point= this._link.getFallbackStartPoint();
				this._link.setStartPort(null);
				fixedNode = this._link.getEndNode();
				isStart = false;
			}else{
				port= this._link.getEndPort();
				point= this._link.getFallbackEndPoint();
				this._link.setEndPort(null);
				fixedNode = this._link.getStartNode();
				isStart = true;
			}
			
			//Save unused port data
			if(port){
				this._savedConnection = {
					auto: !port._isIBMDiagramBasicPort,
					position: port._isIBMDiagramBasicPort?port.getPosition():null,
					node: port.getOwner()	
				};
			}else{
				this._savedConnection = {
						point: dojo.clone(point)
				};
			}
			
			//remove unused Port
			if(port && port._isIBMDiagramBasicPort){
				port.getOwner().removePort(port);
			}
			
			//Set the top level parent
			this._oldLinkParent = this._link.getParent();
			V.getDiagram().getGraph().add(this._link);
			this._targetInteractor.setUp(fixedNode,isStart,this._allowFreeLinks);
			this._targetInteractor.activate();
		}
		
		var point = V.eventContentLocation(e);
		var t = this._link.getShapeToContainerTransform(V._diagram.getGraph().getParent()).inverse();
		point = t.transformPoint(point);
		
		if(portPosition == 0){
			point = this._displacePoint(point,this._link,true);
			this._link.setFallbackStartPoint(point);
		}else{
			point = this._displacePoint(point,this._link,false);
			this._link.setFallbackEndPoint(point);
		}
		
		this._link.validateLinkShape();
	},

	_dragEnd: function(e) {
		this.inherited(arguments);
		var A = this._handle._adorner;
		var V = A._viewport;
		var D = A._diagram;
		var portPosition = this._handle.getLocation();
		if( this._link != null ){
			if(this._target){
				var connection = {auto:this._automaticPortType,
								  position: this._targetPosition,
								  node: this._target};
				
				this._applyConnection(this._link,portPosition,connection,this._createPort);
				
				//reparent the link, with the lower common parent
				var start = this._link.getStartPort().getOwner();
				var end = this._link.getEndPort().getOwner();
				if(D.allowCreateLink(start,end)){
					var cParent = gu.lowestCommonAncestor(start,end);
					if(cParent!=this._oldLinkParent){
						cParent.add(this._link);
					}
					var action = this._createUndoReConnectionAction(D,this._link,cParent,this._oldLinkParent,portPosition,this._savedConnection,connection);
					D.onLinkUpdated(D,this._link,false,action);
					if(D._isIBMDiagramEditor){
						D.getUndoManager().addAction(this._createUndoAction(D,this._link,cParent,this._oldLinkParent,action));
					}
				}else{
					this._applyConnection(this._link,portPosition,this._savedConnection,this._createPort);
					//TODO add onMessage call
				}
			}else{
				if(!this._allowFreeLinks){
					this._applyConnection(this._link,portPosition,this._savedConnection,this._createPort);
					//TODO add onMessage call
				}else{
					var action = this._createUndoReConnectionAction(D,this._link,this._link.getParent(),this._oldLinkParent,portPosition,this._savedConnection,connection);
					D.onLinkUpdated(D,this._link,false,action);
					if(D._isIBMDiagramEditor){
						var connection = {point:dojo.clone(portPosition==0?this._link.getFallbackStartPoint():this._link.getFallbackEndPoint())};
						D.getUndoManager().addAction(this._createUndoReConnectionAction(D,this._link,this._link.getParent(),this._oldLinkParent,action));
					}
				}
			}
			
			this._savedPort = null;
			this._link =null;
			this._targetInteractor.deactivate();
		}
		this._handle.interactionEnded(this);
	},
	
	_displacePoint: function(point,link,isStart){
		// Added to fix requirement 642
		var p;
		if(!isStart){
			p = link._pathPoints[link._pathPoints.length-1];
		}else{
			p = link._pathPoints[0];
		}
		var dx = (point.x-p.x)<0?5:-5;
		var dy = (point.y-p.y)<0?5:-5;
		return {x: point.x+dx, y: point.y+dy};
	},
	
	_applyConnection: function(link,portPosition,connection,portFunction){
		var port;
		if(connection.node){
			port = portFunction.call(this,connection.auto,connection.position);
			connection.node.addPort(port);
		}
		if(portPosition == 0){
			if(port){
				link.setStartPort(port);
			}else{
				link.setFallbackStartPoint(dojo.clone(connection.point));
			}
		}else{
			if(port){
				link.setEndPort(port);
			}else{
				link.setFallbackEndPoint(dojo.clone(connection.point));
			}
		}
	},
	
	_createPort: function(automaticPortType,position){
		var port;
		if(automaticPortType || !position){
			port = new ibm_ilog.diagram.AutomaticPort();
		}else{
			port = new ibm_ilog.diagram.BasicPort();
			port.setPosition(position);
		}
		return port;
	},
	_createUndoReConnectionAction: function(diagram,link,newParent,oldParent,position,savedConnection,newConnection){
		var action = new ibm_ilog.diagram.editor.undo.ReConnectAction(diagram,link,position,savedConnection,newConnection,this._createPort,this._applyConnection);
		return action;
	},
	_createUndoAction: function(diagram,link,newParent,oldParent,action){
		
		if(oldParent!=newParent){
			var reParentAction = new ibm_ilog.diagram.editor.undo.ReparentingAction();
			reParentAction.setOldParentId(diagram.getUndoManager().getParentId(oldParent));
			reParentAction.setNewParentId(diagram.getUndoManager().getParentId(newParent));
			reParentAction.setModifiedElementId(link.getId());
			var mAction = new ibm_ilog.diagram.editor.undo.MultipleAction(ibm_ilog.diagram.editor.undo.UndoActionList.ReConnect);
			mAction.addAction(action);
			mAction.addAction(reParentAction);
			return mAction;
		}else{
			return action;
		}
	}
});

})();