dojo.provide('ibm_ilog.diagram.editor.interactors.ResizeKeyInteractor');

dojo.require('ibm_ilog.diagram.interactors.KeyboardInteractorManager');
dojo.require('ibm_ilog.diagram.interactors.Interactor');
dojo.require('ibm_ilog.diagram.editor.undo.SimpleAction');
dojo.require('ibm_ilog.diagram.editor.undo.UndoActionList');
dojo.require('ibm_ilog.diagram.util.Geometry');

dojo.declare("ibm_ilog.diagram.editor.interactors.ResizeKeyInteractor", [ibm_ilog.diagram.interactors.Interactor], {
	//summary:
	//		This Interactor manages the graphElements resizing through the keyboard interaction.
	//		This Interactor have 8 connections: reduceWidth, fineReduceWidth, increaseWidth, fineIncreaseWidth, 
	//		reduceHeight, fineReduceHeight, increaseHeight, fineIncreaseHeight.
	
	_diagram: null,
	_increment: null,
	_fineIncrement: null,
	initialize: function (diagram) {
	// summary:
    //		initialize the new instance.
    // diagram: ibm_ilog.diagram.widget.Diagram
    //		the associated diagram.		
		this._diagram = diagram;
		this.setIncrement(10);
		this.setFineIncrement(1);
		return this._initialize();
	},
	setIncrement: function(increment){
		// summary: 
		//		sets the resize increment
		// increment: integer
		//		the increment to be set
		this._increment = increment;
	},
	getIncrement: function(){
		// summary: 
		//		gets the resize increment
		return this._increment;
	},
	setFineIncrement: function(increment){
		// summary: 
		//		sets the resize fine increment
		// increment: integer
		//		the fine increment to be set
		this._fineIncrement = increment;
	},
	getFineIncrement: function(){
		// summary: 
		//		gets the resize fine increment
		return this._fineIncrement;
	},
	reduceHeight: function(e){
		// summary: 
		//		Reduces the focused element height from the right bottom corner
		this._resize({x:0,y:-this._increment});
	},
	increaseHeight: function(e){
		// summary: 
		//		Increases the focused element height from the right bottom corner
		this._resize({x:0,y:this._increment});
	},
	increaseWidth: function(e){
		// summary: 
		//		Increases the focused element width from the right bottom corner
		this._resize({x:this._increment,y:0});
	},
	reduceWidth: function(e){
		// summary: 
		//		Reduces the focused element width from the right bottom corner
		this._resize({x:-this._increment,y:0});
	},
	fineReduceHeight: function(e){
		// summary: 
		//		Finely reduces the focused element height from the right bottom corner
		this._resize({x:0,y:-this._fineIncrement});
	},
	fineIncreaseHeight: function(e){
		// summary: 
		//		Finely increases the focused element height from the right bottom corner
		this._resize({x:0,y:this._fineIncrement});
	},
	fineIncreaseWidth: function(e){
		// summary: 
		//		Finely increases the focused element width from the right bottom corner
		this._resize({x:this._fineIncrement,y:0});
	},
	fineReduceWidth: function(e){
		// summary: 
		//		Finely reduces the focused element width from the right bottom corner
		this._resize({x:-this._fineIncrement,y:0});
	},
	getKeyInteractorId: function(){
		// summary:
	    //		returns the Interactor Id
		return ibm_ilog.diagram.editor.interactors.ResizeKeyInteractor.KeyInteractorId;
	},
	getDefaultConnections: function () {
		// summary:
		//		returns the default event mapping
		
		return {
			reduceWidth: {
				hotKey: dojo.keys.LEFT_ARROW,
				connectTo: "reduceWidth",
				filter: this._buildInputFilter({ctrl:true,shift:false})
			}, fineReduceWidth: {
				hotKey: dojo.keys.LEFT_ARROW,
				connectTo: "fineReduceWidth",
				filter: this._buildInputFilter({ctrl:true,shift:true})
			}, increaseWidth: {
				hotKey: dojo.keys.RIGHT_ARROW,
				connectTo: "increaseWidth",
				filter: this._buildInputFilter({ctrl:true,shift:false})
			}, fineIncreaseWidth: {
				hotKey: dojo.keys.RIGHT_ARROW,
				connectTo: "fineIncreaseWidth",
				filter: this._buildInputFilter({ctrl:true,shift:true})
			}, reduceHeight: {
				hotKey: dojo.keys.UP_ARROW,
				connectTo: "reduceHeight",
				filter: this._buildInputFilter({ctrl:true,shift:false})
			}, fineReduceHeight: {
				hotKey: dojo.keys.UP_ARROW,
				connectTo: "fineReduceHeight",
				filter: this._buildInputFilter({ctrl:true,shift:true})
			}, increaseHeight: {
				hotKey: dojo.keys.DOWN_ARROW,
				connectTo: "increaseHeight",
				filter: this._buildInputFilter({ctrl:true,shift:false})
			}, fineIncreaseHeight: {
				hotKey: dojo.keys.DOWN_ARROW,
				connectTo: "fineIncreaseHeight",
				filter: this._buildInputFilter({ctrl:true,shift:true})
			}
		};
	},
	_resize: function(delta){
		var ge = this._diagram.getFocusedElement();
		if(ge && !ge._IBMDiagramLink){
			var startBB = ge.getBounds();
			var endBB = ibm_ilog.diagram.util.Geometry.cloneRect(startBB);
			var min = ge.minimumSize;
		
			endBB.width = Math.max(min.width,startBB.width + delta.x);
			endBB.height = Math.max(min.height,startBB.height + delta.y);
		
			ge.setBounds(endBB);
			this._diagram.getUndoManager().addAction(this._createUndoAction(ge, startBB, endBB));
		}
	},
	_createUndoAction: function(ge, startBB, endBB){
		var action = new ibm_ilog.diagram.editor.undo.SimpleAction(ibm_ilog.diagram.editor.undo.UndoActionList.Resize);
		action.setOldValue(startBB);
		action.setNewValue(endBB);
		action.setMethodOrProperty('setBounds');
		action.setModifiedElementId(ge.getId());
		return action;
	}
});

ibm_ilog.diagram.editor.interactors.ResizeKeyInteractor.KeyInteractorId = "Resize";