dojo.provide('ibm_ilog.diagram.editor.undo.DeleteAction');

dojo.require('ibm_ilog.diagram.editor.undo.Action');
dojo.require('ibm_ilog.diagram.editor.undo.UndoActionList');
dojo.require('ibm_ilog.diagram.editor.undo.MultipleAction');
dojo.require('ibm_ilog.diagram.editor.undo.ConnectAction');
(function(){
	var R = ibm_ilog.diagram.util.ErrorReporter;
	
dojo.declare("ibm_ilog.diagram.editor.undo.DeleteAction", [ibm_ilog.diagram.editor.undo.Action], {
	_serialized: null,
	_deletedNodes: null,
	_diagram: null,
	_deletedLinksAction: null,
	
	constructor:function(diagram){
		this.setLabel(ibm_ilog.diagram.editor.undo.UndoActionList.Delete);
		this._diagram = diagram;
		this._deletedLinksAction = new ibm_ilog.diagram.editor.undo.MultipleAction(ibm_ilog.diagram.editor.undo.UndoActionList.Delete);
		this.initialize(dojo.hitch(this,this._undoFunction),dojo.hitch(this,this._redoFunction));
	},
	setDeleted: function(targetSet){
		this._deletedNodes = {};
		var ts = [];
		for(var index in targetSet){
			var ge = targetSet[index];
			var parent = ge.getParent();
			if(ge._isIBMDiagramNode){
				this._deletedNodes[ge.getId()] = this._diagram.getUndoManager().getParentId(parent);
				ts.push(ge);
			}else{
				this._deletedLinksAction.addAction(new ibm_ilog.diagram.editor.undo.DeleteLinkAction(this._diagram,ge,parent));
			}
		}
		this._serialized = this._diagram._getRemoveSerializer().serialize(ts);
		this._deletedLinksAction.setUndoManager(this._diagram.getUndoManager());
	},
	_undoFunction: function(){
		var undoGroup = new ibm_ilog.diagram.editor.undo.MultipleAction("Dummy");
		dojo.forEach(this._serialized,function(item){
			var parent =  this.getUndoManager().getRegisteredParent(this._deletedNodes[item.id]); 
			this._diagram._getRemoveSerializer().deserialize([item],{graph:parent,undo:undoGroup});
		},this);
		this._deletedLinksAction.undo();
	},
	_redoFunction: function(){
		this._deletedLinksAction.redo();
		for(var elementId in this._deletedNodes){
			var ge = this.getUndoManager().getRegisteredGraphElement(elementId);
			ge.getParent().remove(ge);
			ge.dispose();
		}
	},
	_initializeSerializer: function(){
		this._serializer = new ibm_ilog.diagram.editor.DiagramSerializer(this._diagram,null,this);
	},
	onSerialize: function(graphElement,serializedData){
		
	},
	onDeserialize: function(serializedData,newGraphElement,container,undoAction){
		var um = this._diagram.getUndoManager();
		um.registerGraphElementReplacement(serializedData.id,newGraphElement.getId());
	}
});
	
dojo.declare("ibm_ilog.diagram.editor.undo.DeleteLinkAction", [ibm_ilog.diagram.editor.undo.ConnectAction], {
	undoFunction: function(){
		ibm_ilog.diagram.editor.undo.ConnectAction.prototype.redoFunction.apply(this, arguments);
	},
	redoFunction: function(){
		ibm_ilog.diagram.editor.undo.ConnectAction.prototype.undoFunction.apply(this, arguments);
	}
});

})();