dojo.provide('ibm_ilog.diagram.editor.undo.MultipleAction');

dojo.require('ibm_ilog.diagram.editor.undo.Action');
dojo.require('ibm_ilog.diagram.util.ErrorReporter');

(function(){
	var R = ibm_ilog.diagram.util.ErrorReporter;
	
	dojo.declare("ibm_ilog.diagram.editor.undo.MultipleAction", [ibm_ilog.diagram.editor.undo.Action], {
		//
		//	summary:
		//		this action groups a list of actions. It simple groups a set of actions in a single complex action.
		//
		_actionList:null,
		_reverseUndo: null,
		_reverseRedo: null,
	constructor:function(label){
		//
		//	summary:
		//		creates a  new instance, setting up the label
		//
		this._label = label;
		this.initialize(dojo.hitch(this,this._undoFunction),dojo.hitch(this,this._redoFunction));
		this._actionList = [];
		this._reverseUndo = true;
		this._reverseRedo = false;
	},
	addAction: function(action){
		//
		//	summary:
		//		adds an action to the list
		this._actionList.push(action);
	},
	_undoFunction: function(){
		var size = this._actionList.length;
		for(var index = 0;index<size;index++){
			var action = this._reverseUndo?this._actionList[size-index-1]:this._actionList[index];
			action.undo();
		}
	},
	_redoFunction: function(){
		var size = this._actionList.length;
		for(var index = 0;index<size;index++){
			var action = this._reverseRedo?this._actionList[size-index-1]:this._actionList[index];
			action.redo();
		}
	},
	setUndoManager: function(undoManager){
		//
		//	summary:
		//		sets the undo Manager to each individual action added to the list
		this._undoManager = undoManager;
		dojo.forEach(
				this._actionList,
			    function(item){
			        item.setUndoManager(undoManager);
			    }
			);
	},
	setReverseUndo:function(reverse){
		// summary:
		//		sets to apply the undo actions in reverse order
			this._reverseUndo = reverse;
	},
	getReverseUndo: function(){
		// summary:
		//		gets to apply the undo actions in reverse order
		return this._reverseUndo;
	},
	setReverseRedo:function(reverse){
		// summary:
		//		sets to apply the redo actions in reverse order
		this._reverseRedo = reverse;
	},
	getReverseRedo: function(){
		// summary:
		//		gets to apply the redo actions in reverse order
		return this._reverseRedo;
	}
	});
	
})();