dojo.provide('ibm_ilog.diagram.editor.undo.ReConnectAction');

dojo.require('ibm_ilog.diagram.editor.undo.UserCustomizedAction');
dojo.require('ibm_ilog.diagram.util.ErrorReporter');
dojo.require('ibm_ilog.diagram.editor.undo.UndoActionList');

(function(){
	var R = ibm_ilog.diagram.util.ErrorReporter;
	
	dojo.declare("ibm_ilog.diagram.editor.undo.ReConnectAction", [ibm_ilog.diagram.editor.undo.UserCustomizedAction], {
		//	
		// summary:
		//		this action undo/redo any ReConnect action.
		//		This action must be modified for the user if any user defined action is done on a ReConnect action.
		//
		_linkId: null,
		_applyConnectionFunction: null,
		_createPortFunction: null,
		_oldConnection: null,
		_newConnection: null,
		_portPosition: null,
	
	constructor:function(diagram,link,position,savedConnection,newConnection,createPort,applyConnection){
		
		this._label = ibm_ilog.diagram.editor.undo.UndoActionList.ReConnect;
		this._linkId = link.getId();
		this._portPosition = position;
		this._createPortFunction = dojo.hitch(this,createPort);
		this._applyConnectionFunction = dojo.hitch(this,applyConnection);
		
		savedConnection.nodeId = savedConnection.node?savedConnection.node.getId():null;
		savedConnection.node = null;
		this._oldConnection = savedConnection;
		
		newConnection.nodeId = newConnection.node?newConnection.node.getId():null;
		newConnection.node = null;
		this._newConnection = newConnection;
	},
	_clearPort: function(){
		var port;
		var link = this.getUndoManager().getRegisteredGraphElement(this._linkId);
		if(this._portPosition == 0){
			port = link.getStartPort();
			link.setStartPort(null);
		}else{
			port = link.getEndPort();
			link.setEndPort(null);
		}
		//remove Port
		if(port && port._isIBMDiagramBasicPort){
			port.getOwner().removePort(port);
		}
	},
	getLink: function(){
		// Summary:
		//		return the link modified (verifying replaces)
		return this.getUndoManager().getRegisteredGraphElement(this._linkId);
	},
	isStartModified: function(){
		// Summary:
		//		return true if the start of the link is the modified component.
		return this._portPosition == 0;
	},
	undoFunction: function(){
		var link = this.getLink();
		var node = this.getUndoManager().getRegisteredGraphElement(this._oldConnection.nodeId);
		this._oldConnection.nodeId = node.getId();
		this._clearPort.call(this);
		this._oldConnection.node = node;
		this._applyConnectionFunction.call(null,link,this._portPosition,this._oldConnection,this._createPortFunction);
		this._oldConnection.node = null;
		var D = this.getUndoManager().getDiagram();
		D.onLinkUpdated(D,link,false,this);
	},
	
	redoFunction: function(){
		var link = this.getLink();
		var node = this.getUndoManager().getRegisteredGraphElement(this._newConnection.nodeId);
		this._newConnection.nodeId = node?node.getId():null;
		this._clearPort.call(this);
		this._newConnection.node = node;
		this._applyConnectionFunction.call(null,link,this._portPosition,this._newConnection,this._createPortFunction);
		this._newConnection.node = null;
		var D = this.getUndoManager().getDiagram();
		D.onLinkUpdated(D,link,false,this);
	}
	});
	
})();