dojo.provide('ibm_ilog.diagram.editor.undo.UngroupAction');

dojo.require('ibm_ilog.diagram.editor.undo.UserCustomizedAction');
dojo.require('ibm_ilog.diagram.util.ErrorReporter');
dojo.require('ibm_ilog.diagram.editor.undo.UndoActionList');
dojo.require('ibm_ilog.diagram.editor.EditingUtils');
dojo.require('dojox.gfx.matrix');

(function(){
	var R = ibm_ilog.diagram.util.ErrorReporter;
	var eu = ibm_ilog.diagram.editor.EditingUtils;
	var m  = dojox.gfx.matrix;
	dojo.declare("ibm_ilog.diagram.editor.undo.UngroupAction", [ibm_ilog.diagram.editor.undo.UserCustomizedAction], {
		//	
		// summary:
		//		this action undo/redo any Ungroup action.
		//		This action must be modified for the user if any user defined action is done on a Ungroup action.
		//
		
	_elements: null,
	_subgraph: null,
	_subgraphTransform: null,
	_deleteAction: null,
	constructor:function(){
		//	
		// summary:
		//		creates a new instance, setting the corresponding label
		this._label = ibm_ilog.diagram.editor.undo.UndoActionList.Group;
	},
	setElements: function(elements){
		//	
		// summary:
		//		sets the elements list to be group
		this._elements = [];
		dojo.forEach(elements,function(ge){this._elements.push(ge.getId());},this);
	},
	getElements: function(){
		//	
		// summary:
		//		gets the elements list to be group
		return this._elements;// summary:
		//		creates the group undo action.
		// subgraph: ibm_ilog.diagram.Subgraph
		//		the created subgraph, that groups the elements.
		// elements: array of ibm_ilog.diagram.GraphElement
		//		the elements that have been group.
	},
	setSubgraphId: function(subgraphId){
		//	
		// summary:
		//		sets the subgraph where to group
		this._subgraph = subgraphId;
	},
	getSubgraphId: function(){
		//	
		// summary:
		//		gets the subgraph where to group
		return this._subgraph;
	},
	setSubgraphTransform: function(subgraphT){
		//	
		// summary:
		//		sets the subgraph where to group
		this._subgraphTransform = subgraphT;
	},
	getSubgraphTransform: function(){
		//	
		// summary:
		//		gets the subgraph where to group
		return this._subgraphTransform;
	},
	setDeleteAction: function(deleteAction){
		//	
		// summary:
		//		sets the DeleteAction
		this._deleteAction = deleteAction;
		//this._deleteAction.getUndoManager = dojo.hitch(function(){this.getUndoManager},this);
	},
	getDeleteAction: function(){
		//	
		// summary:
		//		gets the DeleteAction
		return this._deleteAction;
	},
	setUndoManager: function(undoManager){
		//
		//	summary:
		//		Sets the undo Manager
		this.inherited(arguments);
		this._deleteAction.setUndoManager(undoManager);
	},
	redoFunction: function(){
		var elements = this._getNewList();
		var D = this.getUndoManager().getDiagram();
		var sg = this.getUndoManager().getRegisteredGraphElement(this._subgraph);
		eu.groupReparent(D,elements,sg.getParent(),sg);
		this._deleteAction.redo();
	},
	undoFunction: function(){
		var newElements = this._getNewList();
		var D = this.getUndoManager().getDiagram();
		this._deleteAction.undo();
		var sg = this.getUndoManager().getRegisteredGraphElement(this._subgraph);
		sg.setTransform(m.clone(this._subgraphTransform));
		var lca = ibm_ilog.diagram.util.GraphUtil.lowestCommonAncestor.apply(this,newElements);
		eu.groupReparent(D,newElements,sg,lca);
	},
	_getNewList: function(){
		var newElements = [];
		var elements = this.getElements();
		dojo.forEach(elements, dojo.hitch(this, function(item){
			var newItem = this.getUndoManager().getRegisteredGraphElement(item);
			newElements.push(newItem);
		}));
		return newElements;
	}
	});
	
})();