dojo.provide('ibm_ilog.diagram.editor.undo.UserCustomizedAction');

dojo.require('ibm_ilog.diagram.editor.undo.Action');
dojo.require('ibm_ilog.diagram.util.ErrorReporter');

(function(){
	var R = ibm_ilog.diagram.util.ErrorReporter;
	
	dojo.declare("ibm_ilog.diagram.editor.undo.UserCustomizedAction", [ibm_ilog.diagram.editor.undo.Action], {
		//	
		// summary:
		//		this action is a simple action, that also called a userCustomized function on undo/redo.
		//		The functions onUndo and onRedo must be overwritten by subclasses
		//
		_redoUserFunction: null,
		_undoUserFunction: null,
		
	constructor:function(){
		//	
		// summary:
		//		creates a new instance, setting the corresponding label
		this.initialize(dojo.hitch(this,this._undoFunction),dojo.hitch(this,this._redoFunction));
	},
	setUndoUserFunction: function(undoUserFunction){
		//	
		// summary:
		//		sets the undoUser function. The set function is called when undo function. The undoUserFunction must be set every time the user adds custom code on drop action.
		this._undoUserFunction = undoUserFunction;
	},
	getUndoUserFunction: function(){
		//	
		// summary:
		//		gets the undoUser function. This function is called when undo function.
		return this._undoUserFunction;
	},
	setRedoUserFunction: function(redoUserFunction){
		//	
		// summary:
		//		sets the undoUser function. The set function is called after the standard undo function is done.
		this._redoUserFunction = redoUserFunction;
	},
	getRedoUserFunction: function(){
		//	
		// summary:
		//		gets the redoUser function.
		return this._redoUserFunction;
	},
	_undoFunction: function(){
		this.undoFunction();
		if(this._undoUserFunction){
			try{
				this._undoUserFunction();
			 }catch(e){
	            	R.error("UndoUserActionError",this.getLabel(),e);
	         }
		}
	},
	_redoFunction: function(){
		this.redoFunction();
		if(this._redoUserFunction){
			try{
				this._redoUserFunction();
			}catch(e){
				R.error("UndoUserActionError",this.getLabel(),e);
			}
		}
	},
	undoFunction: function(){
		//	
		// summary:
		//		this function must be overwritten by subclasses, and is called on undo
	},
	redoFunction: function(){
		//	
		// summary:
		//		this function must be overwritten by subclasses, and is called on redo
	}
	});
	
})();