/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.explorer.ExplorerSidebarWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("rmps.ui.explorer.ExplorerMain");
dojo.require("rmps.ui.EllipsisLabel");

dojo.requireLocalization("rmps.ui.explorer", "ExplorerMessages");
var messages = dojo.i18n.getLocalization("rmps.ui.explorer", "ExplorerMessages");

dojo.declare("rmps.ui.explorer.ExplorerSidebarWidget", [dijit._Widget, dijit._Templated], {

	messages: messages,
	templateString: dojo.cache("rmps.ui", "explorer/templates/ExplorerSidebarWidget.html"),
	widgetsInTemplate: true,

	// attach points
	_topContainer: null,
	_titleBarNode: null,
	_menuToolbar: null,
	_explorerMain: null, //Widget

	// props
	_titleBar: null,
	_toolBar: null,
	
	postCreate: function() {
		this.inherited(arguments);
		
		this._explorerMain = new rmps.ui.explorer.ExplorerMain({
			explorerID: this.explorerID
		});
		this.domNode.appendChild(this._explorerMain.domNode);
		this._explorerMain.domNode.style.overflow = "hidden";
		// Create the title bar
		this._titleBar = new rmps.ui.explorer.ExplorerSidebarWidget._TitleBar({
			defaultTitle: messages.explore,
			defaultState: dojo.hitch(this._explorerMain, this._explorerMain.getCurrentViewpointName)
		});
		this._titleBarNode.appendChild(this._titleBar.domNode);
		this._titleBar.startup();
		
		// Connect to explorer main's onRefresh to catch viewpoint updates
		dojo.connect(this._explorerMain, "onRefresh", dojo.hitch(this, function() {
			this._titleBar._setState(null);
		}));
		
		// Create the tool bar
		this._explorerMain.addToolbar(this._menuToolbar);		
	},
	
	startup: function() {
		this.inherited(arguments);
		this._explorerMain.startup();
	},
	
	destroy: function() {
		this._titleBar.destroyRecursive();
		this._toolBar.destroyRecursive();
		
		this.inherited(arguments);
	},
	
	resize: function(size) {
		if (size && size.h) {
			this.domNode.style.height = size.h + "px";
			var h = size.h - dojo.marginBox(this._topContainer).h;
			this._explorerMain.resize({h:h});
		}
		
		this._titleBar.resize(size);
	},
		
	layout: function() {
		this._titleBar.resize();
	}
});

// TODO: Copied from CommentingWidget. Break out into separate widget
dojo.declare("rmps.ui.explorer.ExplorerSidebarWidget._TitleBar", [dijit._Widget, dijit._Templated], {
	
	templateString: '<div data-dojo-attach-point="_title" class="rmps-ui-explorer-widget-title">' +
			'<div>' +
				'<nobr>' +
					'<div data-dojo-attach-point="_titleLabel" data-dojo-type="rmps.ui.EllipsisLabel" data-dojo-props="constrainWidth:true, \'class\':\'title dijitInline\'"></div>' +
					'<span style="font-weight: bold">:&nbsp;</span>' +
					'<span data-dojo-attach-point="_titleState" class="state"></span>' +
				'</nobr>' +
			'</div>' +
		'</div>',
	
	widgetsInTemplate: true,
	
	defaultTitle: null,
	defaultState: null,
	
	postCreate: function() {
		this._setTitle(null);
		this._setState(null);
	},
	
	resize: function(size) {
		this._titleLabel.resize(size);
		this.labelNew = dojo.create("a");
		this.labelNew.href = rmps.app.core.util.getShowViewpointUri(null);
		this.labelNew.text = this._titleLabel.label;
		this._titleLabel.target.removeChild(this._titleLabel.target.firstChild);
		this._titleLabel.target.appendChild(this.labelNew);
	},
	
	update: function(newTitle, newState) {
		if (newTitle !== undefined) {
			this._setTitle(newTitle);
		}
		if (newState !== undefined) {
			this._setState(newState);
		}
		this.resize();
	},
	
	_setTitle: function(title) {
		if (title) {
			this._titleLabel.setLabel(title);
			
		} else if (this.defaultTitle) {
			// default
			if (dojo.isFunction(this.defaultTitle)) {
				this._titleLabel.setLabel(this.defaultTitle());
			} else {
				this._titleLabel.setLabel(this.defaultTitle);
			}
			
		}
	},
	
	_setState: function(state) {
		if (state === null && this.defaultState) {
			// default
			if (dojo.isFunction(this.defaultState)) {
				state = this.defaultState();
				if (state instanceof dojo.Deferred) {
					state.addCallback(dojo.hitch(this, function(newstate) {
						state = newstate;
						this._setState2(state);
					}));
					return;
				}
			} else {
				state = this.defaultState;
			}
		}
		this._setState2(state);
	},
	
	_setState2: function(state) {
		if (state) {
			dojo.html._setNodeContent(this._titleState, dojo.doc.createTextNode(state), true);
			this._title.title = state;
		} else {
			dojo.html._emptyNode(this._titleState);
			this._title.title = "";
		}
	}
});


}());