// This file is what is executed by the Web Worker when graph layout is executed asynchronously
// using the WebWorkerLayout class.

importScripts("_dojo.js");

self.onmessage = function(e){
	var data = e.data;
	
	// On Dojo 1.7 we get an additional flag first, it is used in _dojo.js to load Dojo differently.
	if(!self.dojo && data == "_isDojo17"){
		this._isDojo17 = true;
		return;
	}
	
	// The first message we receive must be the parent directory of dojo:
	if(!self.dojo){
		_loadDojo(data);
		return;
	}
	
	// The second message we receive must be the path of the ibm_graphlayout module:
	if(!self.ibm_ilog){
		if(data && data.length > 0){
			var prefix = this._isDojo17 ? "../" : "";
			dojo.registerModulePath("ibm_ilog.graphlayout", prefix+data);
			// we also register the path of the ibm_ilog.diagram module,
			// since now ibm_graphlayout uses some classes from it (ClippingUtil.js)...
			dojo.registerModulePath("ibm_ilog.diagram", prefix+data.replace(/graphlayout/g, "diagram"));
		}
		dojo.require("ibm_ilog.graphlayout.RecursiveLayout");
		importScripts("_LayoutProvider.js");
		importScripts("_matrix.js");
		importScripts("_GraphAdapter.js");
		// these modules are required by ClippingUtil.js, but they are not really used during graph layout,
		// so let's just define them here so that Dojo does not try to load them (which would fail)...
		dojo.provide('ibm_ilog.diagram.util.ErrorReporter');
		dojo.provide('dojo.number');
		return;
	}
	
	// All subsequent messages are actual layout requests,
	// the data sent is the JSON graph description
	
	try {
		var graphData = dojo.fromJson(data);

		ibm_ilog.diagram.graphlayout.webworker._GraphAdapter.prepareGraph(graphData);

		var provider = new ibm_ilog.diagram.graphlayout.webworker._LayoutProvider();
		var layout = new ibm_ilog.graphlayout.RecursiveLayout(null, provider);
		var model = new ibm_ilog.diagram.graphlayout.webworker._GraphAdapter(graphData);
		layout.attach(model);

		layout.performLayout(true);

		ibm_ilog.diagram.graphlayout.webworker._GraphAdapter.cleanupGraph(graphData);

		self.postMessage(dojo.toJson(graphData));
	} catch(e){
		postMessage("{ _error: true, responseText: '" + e.message + "' }");
	}
};
