dojo.provide('ibm_ilog.graphlayout.hierarchical.BuildLevelsAlgorithm');

///////////////////////////////////////////////////////////////////////////////
// IBM ILOG Dojo Diagrammer 1.0
///////////////////////////////////////////////////////////////////////////////

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

dojo.require('ibm_ilog.graphlayout.hierarchical.HEnum');
dojo.require('ibm_ilog.graphlayout.hierarchical.HGraphAlgorithm');
dojo.require('ibm_ilog.graphlayout.hierarchical.HIntLevelMarker');
dojo.require('ibm_ilog.graphlayout.hierarchical.HPercentages');
dojo.require('ibm_ilog.graphlayout.internalutil.GraphModelUtil');
dojo.require('ibm_ilog.graphlayout.internalutil.TranslateUtil');

dojo.declare('ibm_ilog.graphlayout.hierarchical.BuildLevelsAlgorithm', ibm_ilog.graphlayout.hierarchical.HGraphAlgorithm, 
{
  // summary:
  //		BuildLevelsAlgorithm builds a proper hierarchy by actually reversing
  //		links and creating dummy nodes so that each segment has span 1. 
  // description:
  //		BuildLevelsAlgorithm builds a proper hierarchy by actually reversing
  //		links and creating dummy nodes so that each segment has span 1. 
  //		It presumes that level numbers are already calculated for all nodes.
  //		The algorithm creates the level lists in the graph.  
  // tags:
  //		private
  
  _levelNodesCount: null,

  constructor : function (a0)
  {
    if(arguments.length == 0)
      this._constructor_overload_ibm_ilog_graphlayout_hierarchical_BuildLevelsAlgorithm_0();
    else if(arguments.length == 1)
      this._constructor_overload_ibm_ilog_graphlayout_hierarchical_BuildLevelsAlgorithm_1(a0);
  },
  
  _constructor_overload_ibm_ilog_graphlayout_hierarchical_BuildLevelsAlgorithm_0 : function()
  {
  },
  
  _constructor_overload_ibm_ilog_graphlayout_hierarchical_BuildLevelsAlgorithm_1 : function(graph )
  {
    this.init(graph);

    this._levelNodesCount = null;
  },
  
  clean : function()
  {
    // summary:
    //		Cleans the object after usage. 
    // description:
    //		Cleans the object after usage. 
    //		It is not necessary to use this, but it speeds up garbage collection
    //		if we do it when we don't need the object anymore.
    // tags:
    //		private
    
    this.inherited("clean", arguments, []);
    if (this._levelNodesCount != null) {
      this._levelNodesCount.clean();
    }
    this._levelNodesCount = null;
  },

  getNumberOfNodes : function(level)
  {
    return this._levelNodesCount.get(level); // int
  },

  incrementNumberOfNodes : function(level)
  {
    this._levelNodesCount.add(level, 1);
  },

  run : function()
  {
    // summary:
    //		Starts the algorithm.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var numNodes = graph.getNumberOfNodes();
    var numLinks = graph.getNumberOfLinks();
    var points = 10 + 3 * numNodes + numLinks;
    if (graph.hasPortSides()) {
      points += numLinks;
    }
    this.getPercController().startStep(this.getPercentages().steps[ibm_ilog.graphlayout.hierarchical.HPercentages.STEP_BUILD_LEVELS], points, false);

    if (graph.hasPortSides() && graph.hasDummyFirstLevel() || graph.hasIntergraphLinkFirstLevel()) {
      this.shiftToHaveLevelZeroFree();
    }
    this.createDummyNodes();

    this.createIntergraphLinkDummyNodes();

    this.getGraph().createLevels(this.getMaxLevelNumber());

    this.fillLevels();

    this.restructureAdjacencies();

    if (graph.isIncrementalMode()) {
      this.handleDummyNodesForIncremental();
    }
    if (this.getGraph().getLastLevel().getNodesCount() == 0) {
      this.getGraph().removeLastLevel();
    }

    var iter = this.getGraph().getNodes();

    while (iter.hasNext())
    {
      iter.next().resolveNorthOrSouth();
      this.getPercController().addPoints(1);
      this.layoutStepPerformed();
    }
  },

  shiftToHaveLevelZeroFree : function()
  {
    // summary:
    //		If there is any node in level 0 that has a north port side link, we
    //		shift all nodes by one level, because we need the level zero free
    //		for the bypass dummy node.
    // tags:
    //		private
    
    var niter = this.getGraph().getNodes();
    var node;

    while (niter.hasNext())
    {
      node = niter.next();
      node.setLevelNumber(node.getLevelNumber() + 1);
    }
  },

  createDummyNodes : function()
  {
    // summary:
    //		Creates enough dummy nodes so that each segment has a span of 1 or
    //		-1.
    // tags:
    //		private
    
    var graph = this.getGraph();
    if (graph.getNumberOfNormalSegments() <= 0) {
      return;
    }
    var iter = graph.getNormalSegments();
    var lastSegment = graph.getLastSegment();
    var segment;

    while (iter.hasNext())
    {
      segment = iter.next();
      this.split(segment);
      if (segment == lastSegment) {
        break;
      }
      this.getPercController().addPoints(1);
      this.layoutStepPerformed();
    }
  },

  createIntergraphLinkDummyNodes : function()
  {
    // summary:
    //		Creates the dummy nodes for intergraph links.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var link;
    var iter = graph.getIntergraphLinks();

    while (iter.hasNext())
    {
      link = iter.next();
      var segment1 = link.getStartSegment();
      var segment2 = link.getEndSegment();
      this.splitIntergraphLink(segment1, true, true, 0);
      if (segment2 != segment1) {
        this.splitIntergraphLink(segment2, true, true, 0);
      }
    }

    var downIntergraphLinkLevel = this.getMaxLevelNumber();
    iter = graph.getTreeExternalIntergraphLinks();

    while (iter.hasNext())
    {
      link = iter.next();

      if (graph.getOwnedNode(link.getFrom()) != null)
      {
        this.splitIntergraphLink(link.getStartSegment(), true, false, downIntergraphLinkLevel);
      }
      else if (graph.getOwnedNode(link.getTo()) != null)
      {
        this.splitIntergraphLink(link.getEndSegment(), false, true, downIntergraphLinkLevel);
      }
    }
  },

  split : function (a0, a1, a2, a3, a4)
  {
    // summary:
    //		Splits the segment into dummy nodes and several segments, so that
    //		each segment has absolute span 1.
    // tags:
    //		private
    
    if(arguments.length == 1)
      return this.split_overload_1(a0);
    else if(arguments.length == 5)
      return this.split_overload_5(a0, a1, a2, a3, a4);
  },
  
  split_overload_1 : function(segment)
                                      
  {
    var fromLevelNumber;
    var toLevelNumber;
    var fromPortSide;
    var toPortSide;

    if (segment.isReversed())
    {
      fromLevelNumber = segment.getTo().getLevelNumber();
      toLevelNumber = segment.getFrom().getLevelNumber();
      fromPortSide = segment.getToPortSide();
      toPortSide = segment.getFromPortSide();
    }
    else
    {
      fromLevelNumber = segment.getFrom().getLevelNumber();
      toLevelNumber = segment.getTo().getLevelNumber();
      fromPortSide = segment.getFromPortSide();
      toPortSide = segment.getToPortSide();
    }

    this.split(segment, fromLevelNumber, toLevelNumber, fromPortSide, toPortSide);
  },

  splitIntergraphLink : function(segment, useFromSide, useToSide, downIntergraphLinkLevel)
  {
    // summary:
    //		Splits the segment into dummy nodes and several segments, so that
    //		each segment has absolute span 1. 
    // description:
    //		Splits the segment into dummy nodes and several segments, so that
    //		each segment has absolute span 1. 
    //		This variant is for intergraph links.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var fromNode;
    var toNode;
    var fromLevelNumber;
    var toLevelNumber;
    var fromPortSide;
    var toPortSide;

    if (segment.isReversed())
    {
      if (useFromSide)
      {
        fromNode = graph.getOwnedNode(segment.getTo());
        fromLevelNumber = fromNode.getLevelNumber();
        fromPortSide = segment.getToPortSide();

        if (fromNode != segment.getTo())
        {
          switch (segment.getOwnerLink().getDirection())
          {
            case ibm_ilog.graphlayout.hierarchical.HEnum.DOWN:
            case ibm_ilog.graphlayout.hierarchical.HEnum.DOWNUP:
              fromPortSide = ibm_ilog.graphlayout.hierarchical.HEnum.SOUTH;
              break;
            case ibm_ilog.graphlayout.hierarchical.HEnum.UP:
              fromPortSide = ibm_ilog.graphlayout.hierarchical.HEnum.NORTH;
              break;
          }
        }
      }
      else
      {
        switch (segment.getOwnerLink().getDirection())
        {
          case ibm_ilog.graphlayout.hierarchical.HEnum.DOWN:
          case ibm_ilog.graphlayout.hierarchical.HEnum.DOWNUP:
            
            fromLevelNumber = -1;
            fromPortSide = ibm_ilog.graphlayout.hierarchical.HEnum.SOUTH;
            break;
          case ibm_ilog.graphlayout.hierarchical.HEnum.UP:
          default:
            fromLevelNumber = downIntergraphLinkLevel + 1;
            fromPortSide = ibm_ilog.graphlayout.hierarchical.HEnum.NORTH;
            break;
        }
      }

      if (useToSide)
      {
        toNode = graph.getOwnedNode(segment.getFrom());
        toLevelNumber = toNode.getLevelNumber();
        toPortSide = segment.getFromPortSide();

        if (toNode != segment.getFrom())
        {
          switch (segment.getOwnerLink().getDirection())
          {
            case ibm_ilog.graphlayout.hierarchical.HEnum.DOWN:
              toPortSide = ibm_ilog.graphlayout.hierarchical.HEnum.NORTH;
              break;
            case ibm_ilog.graphlayout.hierarchical.HEnum.UP:
            case ibm_ilog.graphlayout.hierarchical.HEnum.DOWNUP:
              toPortSide = ibm_ilog.graphlayout.hierarchical.HEnum.SOUTH;
              break;
          }
        }
      }
      else
      {
        switch (segment.getOwnerLink().getDirection())
        {
          case ibm_ilog.graphlayout.hierarchical.HEnum.DOWN:
          default:
            
            toLevelNumber = downIntergraphLinkLevel + 1;
            toPortSide = ibm_ilog.graphlayout.hierarchical.HEnum.NORTH;
            break;
          case ibm_ilog.graphlayout.hierarchical.HEnum.UP:
          case ibm_ilog.graphlayout.hierarchical.HEnum.DOWNUP:
            toLevelNumber = -1;
            toPortSide = ibm_ilog.graphlayout.hierarchical.HEnum.SOUTH;
            break;
        }
      }
    }
    else
    {
      if (useFromSide)
      {
        fromNode = graph.getOwnedNode(segment.getFrom());
        fromLevelNumber = fromNode.getLevelNumber();
        fromPortSide = segment.getFromPortSide();

        if (fromNode != segment.getFrom())
        {
          switch (segment.getOwnerLink().getDirection())
          {
            case ibm_ilog.graphlayout.hierarchical.HEnum.DOWN:
            case ibm_ilog.graphlayout.hierarchical.HEnum.DOWNUP:
              fromPortSide = ibm_ilog.graphlayout.hierarchical.HEnum.SOUTH;
              break;
            case ibm_ilog.graphlayout.hierarchical.HEnum.UP:
              fromPortSide = ibm_ilog.graphlayout.hierarchical.HEnum.NORTH;
              break;
          }
        }
      }
      else
      {
        switch (segment.getOwnerLink().getDirection())
        {
          case ibm_ilog.graphlayout.hierarchical.HEnum.DOWN:
          case ibm_ilog.graphlayout.hierarchical.HEnum.DOWNUP:
            
            fromLevelNumber = -1;
            fromPortSide = ibm_ilog.graphlayout.hierarchical.HEnum.SOUTH;
            break;
          case ibm_ilog.graphlayout.hierarchical.HEnum.UP:
          default:
            fromLevelNumber = downIntergraphLinkLevel + 1;
            fromPortSide = ibm_ilog.graphlayout.hierarchical.HEnum.NORTH;
            break;
        }
      }

      if (useToSide)
      {
        toNode = graph.getOwnedNode(segment.getTo());
        toLevelNumber = toNode.getLevelNumber();
        toPortSide = segment.getToPortSide();

        if (toNode != segment.getTo())
        {
          switch (segment.getOwnerLink().getDirection())
          {
            case ibm_ilog.graphlayout.hierarchical.HEnum.DOWN:
              toPortSide = ibm_ilog.graphlayout.hierarchical.HEnum.NORTH;
              break;
            case ibm_ilog.graphlayout.hierarchical.HEnum.UP:
            case ibm_ilog.graphlayout.hierarchical.HEnum.DOWNUP:
              toPortSide = ibm_ilog.graphlayout.hierarchical.HEnum.SOUTH;
              break;
          }
        }
      }
      else
      {
        switch (segment.getOwnerLink().getDirection())
        {
          case ibm_ilog.graphlayout.hierarchical.HEnum.DOWN:
          default:
            
            toLevelNumber = downIntergraphLinkLevel + 1;
            toPortSide = ibm_ilog.graphlayout.hierarchical.HEnum.NORTH;
            break;
          case ibm_ilog.graphlayout.hierarchical.HEnum.UP:
          case ibm_ilog.graphlayout.hierarchical.HEnum.DOWNUP:
            toLevelNumber = -1;
            toPortSide = ibm_ilog.graphlayout.hierarchical.HEnum.SOUTH;
            break;
        }
      }
    }

    this.split(segment, fromLevelNumber, toLevelNumber, fromPortSide, toPortSide);
  },

  split_overload_5 : function(segment, fromLevelNumber, toLevelNumber, fromPortSide, toPortSide)
  {
    // summary:
    // tags:
    //		private
    
    var dummyNode;
    var link = segment.getOwnerLink();
    var thickness = link.getThickness();
    var graph = this.getGraph();

    if (fromLevelNumber == toLevelNumber)
    {
      var bypassLevel = fromLevelNumber + 1;
      if (fromPortSide == ibm_ilog.graphlayout.hierarchical.HEnum.NORTH || toPortSide == ibm_ilog.graphlayout.hierarchical.HEnum.NORTH) {
        bypassLevel = fromLevelNumber - 1;
      }
      if (fromPortSide == ibm_ilog.graphlayout.hierarchical.HEnum.NORTH && toPortSide == ibm_ilog.graphlayout.hierarchical.HEnum.SOUTH)
      {
        dummyNode = link.addDummyNodeAtGraph(segment, graph);
        this.makeBypassNode(dummyNode, fromLevelNumber + 1, thickness);
        dummyNode = link.addDummyNodeAtGraph(segment, graph);
        dummyNode.setLevelNumber(fromLevelNumber);
        dummyNode = link.addDummyNodeAtGraph(segment, graph);
        this.makeBypassNode(dummyNode, fromLevelNumber - 1, thickness);
      }
      else if (fromPortSide == ibm_ilog.graphlayout.hierarchical.HEnum.SOUTH && toPortSide == ibm_ilog.graphlayout.hierarchical.HEnum.NORTH)
      {
        dummyNode = link.addDummyNodeAtGraph(segment, graph);
        this.makeBypassNode(dummyNode, fromLevelNumber - 1, thickness);
        dummyNode = link.addDummyNodeAtGraph(segment, graph);
        dummyNode.setLevelNumber(fromLevelNumber);
        dummyNode = link.addDummyNodeAtGraph(segment, graph);
        this.makeBypassNode(dummyNode, fromLevelNumber + 1, thickness);
      }
      else
      {
        dummyNode = link.addDummyNodeAtGraph(segment, graph);
        this.makeBypassNode(dummyNode, bypassLevel, thickness);
      }
    }
    else if (fromLevelNumber < toLevelNumber)
    {
      if (toPortSide == ibm_ilog.graphlayout.hierarchical.HEnum.SOUTH)
      {
        dummyNode = link.addDummyNodeAtGraph(segment, graph);
        this.makeBypassNode(dummyNode, toLevelNumber + 1, thickness);
        dummyNode = link.addDummyNodeAtGraph(segment, graph);
        dummyNode.setLevelNumber(toLevelNumber);
      }

      for (var i = toLevelNumber - 1; i > fromLevelNumber; i--)
      {
        dummyNode = link.addDummyNodeAtGraph(segment, graph);
        dummyNode.setLevelNumber(i);
      }

      if (fromPortSide == ibm_ilog.graphlayout.hierarchical.HEnum.NORTH)
      {
        dummyNode = link.addDummyNodeAtGraph(segment, graph);
        dummyNode.setLevelNumber(fromLevelNumber);
        dummyNode = link.addDummyNodeAtGraph(segment, graph);
        this.makeBypassNode(dummyNode, fromLevelNumber - 1, thickness);
      }
    }
    else
    {
      if (toPortSide == ibm_ilog.graphlayout.hierarchical.HEnum.NORTH)
      {
        dummyNode = link.addDummyNodeAtGraph(segment, graph);
        this.makeBypassNode(dummyNode, toLevelNumber - 1, thickness);
        dummyNode = link.addDummyNodeAtGraph(segment, graph);
        dummyNode.setLevelNumber(toLevelNumber);
      }

      for (var j = toLevelNumber + 1; j < fromLevelNumber; j++)
      {
        dummyNode = link.addDummyNodeAtGraph(segment, graph);
        dummyNode.setLevelNumber(j);
      }

      if (fromPortSide == ibm_ilog.graphlayout.hierarchical.HEnum.SOUTH)
      {
        dummyNode = link.addDummyNodeAtGraph(segment, graph);
        dummyNode.setLevelNumber(fromLevelNumber);
        dummyNode = link.addDummyNodeAtGraph(segment, graph);
        this.makeBypassNode(dummyNode, fromLevelNumber + 1, thickness);
      }
    }
  },

  makeBypassNode : function(dummyNode, levelNumber, thickness)
  {
    // summary:
    //		Sets the special attributes of a bypass node.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var levelFlowDir = graph.getLevelFlow();
    var dsize = graph.getMinDistBetweenLinks(levelFlowDir);

    if (graph.hasOrthogonalLinks()) {
      dsize = dsize / 4.0;
    }
    else {
      dsize += thickness;
    }
    dummyNode.setLevelNumber(levelNumber);
    dummyNode.setSize(levelFlowDir, dsize);
  },

  getMaxLevelNumber : function()
  {
    // summary:
    //		Returns the maximal level number.
    // tags:
    //		private
    
    var maxLevelNumber = 0;
    var h;
    var nodeIter = this.getGraph().getNodes();

    while (nodeIter.hasNext())
    {
      h = nodeIter.next().getLevelNumber();
      if (h > maxLevelNumber) {
        maxLevelNumber = h;
      }
    }

    if (this.getGraph().hasIntergraphLinkLastLevel()) {
      maxLevelNumber++;
    }
    return maxLevelNumber; // int
  },

  fillLevels : function()
  {
    // summary:
    //		Fills the levels with the nodes.
    // tags:
    //		private
    
    this._levelNodesCount = new ibm_ilog.graphlayout.hierarchical.HIntLevelMarker(this.getGraph());

    var nodeIter = this.getGraph().getNodes();

    while (nodeIter.hasNext())
      this.incrementNumberOfNodes(nodeIter.next().getLevel());

    var levelIter = this.getGraph().getLevels();
    var level;

    while (levelIter.hasNext())
    {
      level = levelIter.next();
      level.setCapacity(this.getNumberOfNodes(level));
    }

    this._levelNodesCount.clean();
    this._levelNodesCount = null;

    nodeIter = this.getGraph().getNodes();
    var node;

    while (nodeIter.hasNext())
    {
      node = nodeIter.next();
      node.getLevel().add(node);
      this.getPercController().addPoints(1);
      this.layoutStepPerformed();
    }
  },

  handleDummyNodesForIncremental : function()
  {
    // summary:
    //		Handle the dummy nodes for incremental mode. 
    // description:
    //		Handle the dummy nodes for incremental mode. 
    //		Either mark the dummy nodes for incremental recalculation, or move
    //		them just to the crossing point where the old link shape used to
    //		cross the level. This avoids that the long links change their shape
    //		too much in incremental mode.
    // tags:
    //		private
    
    var levelIter = this.getGraph().getLevels();
    var level;

    while (levelIter.hasNext())
    {
      level = levelIter.next();
      level.handleDummyNodesForIncremental();
    }
  },

  restructureAdjacencies : function()
  {
    // summary:
    //		Restructures the adjacencies of the nodes so that all incoming
    //		segments come from nodes with lower level number, and all outgoing
    //		segments go to nodes with higher level number.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var segmentIter = graph.getAllSegments();
    var segment;

    while (segmentIter.hasNext())
    {
      segment = segmentIter.next();
      if (segment.getSpan() < 0) {
        segment.reverse();
      }
    }
  }

}
);
