dojo.provide('ibm_ilog.graphlayout.hierarchical.CalcConnectors');

///////////////////////////////////////////////////////////////////////////////
// IBM ILOG Dojo Diagrammer 1.0
///////////////////////////////////////////////////////////////////////////////

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

dojo.require('ibm_ilog.graphlayout.hierarchical.AnchorPointOptimization');
dojo.require('ibm_ilog.graphlayout.hierarchical.HEnum');
dojo.require('ibm_ilog.graphlayout.hierarchical.HGraphAlgorithm');
dojo.require('ibm_ilog.graphlayout.hierarchical.HPercentages');
dojo.require('ibm_ilog.graphlayout.hierarchical.HSegment');
dojo.require('ibm_ilog.graphlayout.hierarchical.HSegmentIterator');
dojo.require('ibm_ilog.graphlayout.hierarchical.HierarchicalLayout');
dojo.require('ibm_ilog.graphlayout.internalutil.GraphModelUtil');
dojo.require('ibm_ilog.graphlayout.internalutil.TranslateUtil');
dojo.require('ibm_ilog.graphlayout.internalutil.LayoutUtil');
dojo.require('ibm_ilog.graphlayout.internalutil.LogResUtil');

dojo.declare('ibm_ilog.graphlayout.hierarchical.CalcConnectors', ibm_ilog.graphlayout.hierarchical.HGraphAlgorithm, 
{
  // summary:
  //		CalcConnectors is a collection of utilities to calculate the
  //		connectors of the links at the nodes. 
  // description:
  //		CalcConnectors is a collection of utilities to calculate the
  //		connectors of the links at the nodes. 
  //		It assumes that the adjacencies are sorted at the nodes. This is
  //		technically not really an HGraphAlgorithm, as it does not have one
  //		main entry point called run(). Rather than that, there are the
  //		following entry points: calcCenteredConnectors - all links end at the
  //		center point of the border.  calcClippedConnectors  - all links end
  //		at different points on the border, so that the last segments of the
  //		links point to the center of the node. calcEvenSpacedConnectors - all
  //		links end at a different point of the border. These points are
  //		evently spaced around the border.   
  // tags:
  //		private
  
  _anchorOptimAlg: null,
  _levelFlowDirection: null,
  _edgeFlowDirection: null,
  _style: null,

  _thicknessFactor: null,

  _validNode: null,
  _segmentToValidNode: null,

  constructor : function(graph)
  {
    this.init(graph);
    this._anchorOptimAlg = new ibm_ilog.graphlayout.hierarchical.AnchorPointOptimization(graph);
    this._levelFlowDirection = graph.getLevelFlow();
    this._edgeFlowDirection = graph.getEdgeFlow();
    this._style = graph.getConnectorStyle();
  },
  
  clean : function()
  {
    // summary:
    //		Cleans the object after usage. 
    // description:
    //		Cleans the object after usage. 
    //		It is not necessary to use this, but it speeds up garbage collection
    //		if we do it when we don't need the object anymore.
    // tags:
    //		private
    
    this.inherited("clean", arguments, []);
    if (this._anchorOptimAlg != null) {
      this._anchorOptimAlg.clean();
    }
    this._anchorOptimAlg = null;
  },

  run : function()
  {
    // summary:
    //		Starts the algorithm.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var p = this.getPercentages().steps[ibm_ilog.graphlayout.hierarchical.HPercentages.STEP_CALC_CONNECTORS];
    var points = 2 * graph.getNumberOfNodes();
    this.getPercController().startStep(p, points, false);

    switch (this._style)
    {
      case ibm_ilog.graphlayout.hierarchical.HEnum.CENTERED_PINS:
        this.calcCenteredConnectors();
        break;
      case ibm_ilog.graphlayout.hierarchical.HEnum.CLIPPED_PINS:
        this.calcClippedConnectors();
        break;
      case ibm_ilog.graphlayout.hierarchical.HEnum.EVENLY_SPACED_PINS:
        this.calcEvenSpacedConnectors();
        break;
      default:
        ibm_ilog.graphlayout.internalutil.LogResUtil.logAndThrowRuntimeExc(ibm_ilog.graphlayout.hierarchical.HierarchicalLayout, "graphlayout.expert.message.6402E");
    }
  },

  fixEastWestPorts : function (a0, a1)
  {
    // summary:
    //		Runs the algorithm to resolve the east west ports. 
    // description:
    //		Runs the algorithm to resolve the east west ports. 
    //		This is called very last after all other positioning is done.
    // tags:
    //		private
    
    if(arguments.length == 0)
      return this.fixEastWestPorts_overload_0();
    else if(arguments.length == 2)
      return this.fixEastWestPorts_overload_2(a0, a1);
  },
  
  fixEastWestPorts_overload_0 : function()
                                          
  {
    var graph = this.getGraph();
    var numNodes = graph.getNumberOfNodes();
    var p = this.getPercentages().steps[ibm_ilog.graphlayout.hierarchical.HPercentages.STEP_FIX_EAST_WEST_PORTS];
    this.getPercController().startStep(p, 2 * numNodes, false);

    var iter = graph.getNodes();
    var node;
    var auxNode;

    while (iter.hasNext())
    {
      node = iter.next();
      auxNode = node.getEastPortAuxNode();
      if (auxNode != null) {
        this.fixEastWestPorts(node, auxNode);
      }
      auxNode = node.getWestPortAuxNode();
      if (auxNode != null) {
        this.fixEastWestPorts(node, auxNode);
      }
      this.getPercController().addPoints(1);
      this.layoutStepPerformed();
    }
  },

  fixEastWestPorts_overload_2 : function(node, auxNode)
  {
    // summary:
    //		Resolve the east west ports at this aux node.
    // tags:
    //		private
    
    switch (this._style)
    {
      case ibm_ilog.graphlayout.hierarchical.HEnum.CENTERED_PINS:
        this.calcCenteredSideConnectors(node, auxNode);
        break;
      case ibm_ilog.graphlayout.hierarchical.HEnum.CLIPPED_PINS:
        this.calcEvenSpacedSideConnectors(node, auxNode);
        break;
      case ibm_ilog.graphlayout.hierarchical.HEnum.EVENLY_SPACED_PINS:
        this.calcEvenSpacedSideConnectors(node, auxNode);
        break;
      default:
        ibm_ilog.graphlayout.internalutil.LogResUtil.logAndThrowRuntimeExc(ibm_ilog.graphlayout.hierarchical.HierarchicalLayout, "graphlayout.expert.message.6402E");
    }
  },

  calcCenteredConnectors : function (a0)
  {
    // summary:
    //		Calculates link connectors so that they are centered at the node
    //		border.
    // tags:
    //		private
    
    if(arguments.length == 0)
      return this.calcCenteredConnectors_overload_0();
    else if(arguments.length == 1)
      return this.calcCenteredConnectors_overload_1(a0);
  },
  
  calcCenteredConnectors_overload_0 : function()
                                                
  {
    this.calcNorthSouthPortConnectors();
    this.calcEastWestAuxPortNodeConnectors();

    var iter = this.getGraph().getNodes();

    while (iter.hasNext())
    {
      this.calcCenteredConnectors(iter.next());
      this.getPercController().addPoints(1);
      this.layoutStepPerformed();
    }
  },

  calcCenteredConnectors_overload_1 : function(node)
  {
    // summary:
    //		Calculates link connectors for this node, so that the links are
    //		centered on the border of the node.
    // tags:
    //		private
    
    var link;
    var iter;
    iter = node.getAllSegments(true, true, false, false);

    while (iter.hasNext())
    {
      link = iter.next().getOwnerLink();
      if (link.getFrom() == node) {
        this.calcCenteredConnectorsFromSide(link);
      }
      if (link.getTo() == node) {
        this.calcCenteredConnectorsToSide(link);
      }
    }
  },

  calcCenteredConnectorsFromSide : function(link)
  {
    // summary:
    //		Calculates the "from" link connector for this link, so that it is
    //		centered at the node border at the corresponding side.
    // tags:
    //		private
    
    if (!link.getFrom().isEastWestPortAuxNode())
    {
      if (link.isFromSideFixed()) {
        link.setFromCoordWhenFixed();
      }
      else if (!link.isFromPortNumberSpecified()) {
        this.calcCenteredConnectorsFromSideImpl(link, link.getFromPortSide());
      }
    }
  },

  calcCenteredConnectorsToSide : function(link)
  {
    // summary:
    //		Calculates the "to" link connector for this link, so that it is
    //		centered at the node border at the corresponding side.
    // tags:
    //		private
    
    if (!link.getTo().isEastWestPortAuxNode())
    {
      if (link.isToSideFixed()) {
        link.setToCoordWhenFixed();
      }
      else if (!link.isToPortNumberSpecified()) {
        this.calcCenteredConnectorsToSideImpl(link, link.getToPortSide());
      }
    }
  },

  calcCenteredConnectorsFromSideImpl : function(link, side)
  {
    // summary:
    //		Calculates the "from" link connector for this link, so that it is
    //		centered at the node border at the corresponding side.
    // tags:
    //		private
    
    var segment = link.getStartSegment();
    var node1 = segment.getFrom();
    var node2 = segment.getTo();
    var attachNode;
    var attachCoord;
    var otherCoord;
    var segDir = this.getSegmentDir(segment, side);
    var otherDir = 1 - segDir;

    if (segment.isReversed())
    {
      attachCoord = node2.getCoord(segDir);
      otherCoord = node1.getCoord(segDir);
      attachNode = node2;
    }
    else
    {
      attachCoord = node1.getCoord(segDir);
      otherCoord = node2.getCoord(segDir);
      attachNode = node1;
    }

    if (side == ibm_ilog.graphlayout.hierarchical.HEnum.EAST || side == ibm_ilog.graphlayout.hierarchical.HEnum.NORTH) {
      link.setFromCoord(segDir, attachCoord);
    }
    else if (side == ibm_ilog.graphlayout.hierarchical.HEnum.WEST || side == ibm_ilog.graphlayout.hierarchical.HEnum.SOUTH) {
      link.setFromCoord(segDir, attachCoord + attachNode.getSize(segDir));
    }
    else
    {
      if (attachCoord > otherCoord) {
        link.setFromCoord(segDir, attachCoord);
      }
      else {
        link.setFromCoord(segDir, attachCoord + attachNode.getSize(segDir));
      }
    }

    link.setFromCoord(otherDir, attachNode.getCenter(otherDir));
  },

  calcCenteredConnectorsToSideImpl : function(link, side)
  {
    // summary:
    //		Calculates the "to" link connector for this link, so that it is
    //		centered at the node border at the corresponding side.
    // tags:
    //		private
    
    var segment = link.getEndSegment();
    var node1 = segment.getTo();
    var node2 = segment.getFrom();
    var attachNode;
    var attachCoord;
    var otherCoord;
    var segDir = this.getSegmentDir(segment, side);
    var otherDir = 1 - segDir;

    if (segment.isReversed())
    {
      attachCoord = node2.getCoord(segDir);
      otherCoord = node1.getCoord(segDir);
      attachNode = node2;
    }
    else
    {
      attachCoord = node1.getCoord(segDir);
      otherCoord = node2.getCoord(segDir);
      attachNode = node1;
    }

    if (side == ibm_ilog.graphlayout.hierarchical.HEnum.EAST || side == ibm_ilog.graphlayout.hierarchical.HEnum.NORTH) {
      link.setToCoord(segDir, attachCoord);
    }
    else if (side == ibm_ilog.graphlayout.hierarchical.HEnum.WEST || side == ibm_ilog.graphlayout.hierarchical.HEnum.SOUTH) {
      link.setToCoord(segDir, attachCoord + attachNode.getSize(segDir));
    }
    else
    {
      if (attachCoord > otherCoord) {
        link.setToCoord(segDir, attachCoord);
      }
      else {
        link.setToCoord(segDir, attachCoord + attachNode.getSize(segDir));
      }
    }

    link.setToCoord(otherDir, attachNode.getCenter(otherDir));
  },

  calcCenteredSideConnectors : function(node, auxNode)
  {
    // summary:
    //		Calculates link connectors for this node for the links that are
    //		attached to the left or right side, so that the links are centered
    //		on the border of the node. 
    // description:
    //		Calculates link connectors for this node for the links that are
    //		attached to the left or right side, so that the links are centered
    //		on the border of the node. 
    //		At the time point when we do this, these links are not connected to
    //		the node but rather to an auxiliary node at the left or right side.
    //		We assume that the link connection points are already calculated for
    //		this auxiliary node. We correct the link bends and connection
    //		points, but actually
    //		don't care about the link from or to node. Because this is the very last of all steps, it doesn't
    //		matter that the "from" and "to" node data structures are messed up
    //		afterwards. Basically it converts this:         into axproximately
    //		this: | | |                           ||| --------       +-+-+      
    //		 --------  /// |        |     |     |              |        |/// |  
    //		     |     | aux |              |        |// |  node  |     |     | 
    //		            |  node  |/ |        |     | node|              |       
    //		|\ |        |     |     |              |        |\\ |        |     |
    //		    |              |        |\\\ --------       +-+-+        
    //		--------  \\\ | | |                           |||
    // tags:
    //		private
    
    if (node.getPortCache() == null) {
      return;
    }
    if (node.isDummyNode()) {
      return;
    }
    auxNode.correctPortNumbers();

    var rightSide = (node.getPositionInLevel() < auxNode.getPositionInLevel());
    var borderCoord = node.getCoord(this._levelFlowDirection);
    var centerCoord = node.getCenter(this._edgeFlowDirection);
    var baseCoord = node.getCoord(this._edgeFlowDirection);
    if (rightSide) {
      borderCoord += node.getSize(this._levelFlowDirection);
    }
    var portTable = ((rightSide ? node.getPortCache().getPortTable(ibm_ilog.graphlayout.hierarchical.HEnum.EAST) : node.getPortCache().getPortTable(ibm_ilog.graphlayout.hierarchical.HEnum.WEST)));
    var iter;

    this.calcAuxNodeWantedAttachCoord(auxNode);

    iter = auxNode.getAllSegmentsTo(true, true, false, false);

    while (iter.hasNext())
      this.routeToSegmentAtSideConnector(iter.next(), borderCoord, centerCoord, baseCoord, portTable);

    iter = auxNode.getAllSegmentsFrom(true, true, false, false);

    while (iter.hasNext())
      this.routeFromSegmentAtSideConnector(iter.next(), borderCoord, centerCoord, baseCoord, portTable);
  },

  getSegmentDir : function(segment, side)
  {
    // summary:
    //		Returns the index of the direction the segment flows to, if it is
    //		attached at the specified side of its end node.
    // tags:
    //		private
    
    if (side == ibm_ilog.graphlayout.hierarchical.HEnum.EAST || side == ibm_ilog.graphlayout.hierarchical.HEnum.WEST || side == ibm_ilog.graphlayout.hierarchical.HEnum.EAST_OR_WEST) {
      return this._levelFlowDirection; // int
    }
    else if (side == ibm_ilog.graphlayout.hierarchical.HEnum.NORTH || side == ibm_ilog.graphlayout.hierarchical.HEnum.SOUTH || side == ibm_ilog.graphlayout.hierarchical.HEnum.NORTH_OR_SOUTH) {
      return this._edgeFlowDirection; // int
    }
    else if (segment.getSpan() == 0) {
      return this._levelFlowDirection; // int
    }
    return this._edgeFlowDirection; // int
  },

  calcClippedConnectors : function()
  {
    // summary:
    //		Calculates link connectors so that they point to the node center but
    //		are spaced at the node border.
    // tags:
    //		private
    
    this.calcNorthSouthPortConnectors();
    this.calcEastWestAuxPortNodeConnectors();

    var iter = this.getGraph().getNodes();

    while (iter.hasNext())
    {
      this.calcNodeClippedConnectors(iter.next());
      this.getPercController().addPoints(1);
      this.layoutStepPerformed();
    }
  },

  calcNodeClippedConnectors : function(node)
  {
    // summary:
    //		
    // tags:
    //		private
    
    if (node.isDummyNode()) {
      return;
    }
    if (node.isEastWestPortAuxNode()) {
      return;
    }
    var iter = node.getAllSegmentsFrom(true, true, false, false);
    var segment;
    var link;

    while (iter.hasNext())
    {
      segment = iter.next();
      link = segment.getOwnerLink();
      if (link.getFrom() == node) {
        this.calcFromClippedConnector(link);
      }
      if (link.getTo() == node) {
        this.calcToClippedConnector(link);
      }
    }

    iter = node.getAllSegmentsTo(true, true, false, false);

    while (iter.hasNext())
    {
      segment = iter.next();
      link = segment.getOwnerLink();
      if (link.getFrom() == node) {
        this.calcFromClippedConnector(link);
      }
      if (link.getTo() == node) {
        this.calcToClippedConnector(link);
      }
    }
  },

  calcLinkClippedConnectors : function(link)
  {
    // summary:
    //		Calculates link connectors for this link, so that they point to the
    //		center of the node but are clipped on the border.  
    // tags:
    //		private
    
    this.calcFromClippedConnector(link);
    this.calcToClippedConnector(link);
  },

  calcClippedConnectorAtNode : function(link, node)
  {
    // summary:
    //		Calculates link connectors for this link at the incident node.
    // tags:
    //		private
    
    if (link.getFrom() == node) {
      this.calcFromClippedConnector(link);
    }
    if (link.getTo() == node) {
      this.calcToClippedConnector(link);
    }
  },

  calcFromClippedConnector : function(link)
  {
    // summary:
    //		Calculates link connectors for this link at the "from" side, so that
    //		they point to the center of the node but are clipped on the border.
    // tags:
    //		private
    
    if (!link.getFrom().isEastWestPortAuxNode())
    {
      if (link.isFromSideFixed()) {
        link.setFromCoordWhenFixed();
      }
      else if (!link.isFromPortNumberSpecified()) {
        this.calcFromClippedConnectorImpl(link);
      }
    }
  },

  calcToClippedConnector : function(link)
  {
    // summary:
    //		Calculates link connectors for this link at the "to" side, so that
    //		they point to the center of the node but are clipped on the border.
    // tags:
    //		private
    
    if (!link.getTo().isEastWestPortAuxNode())
    {
      if (link.isToSideFixed()) {
        link.setToCoordWhenFixed();
      }
      else if (!link.isToPortNumberSpecified()) {
        this.calcToClippedConnectorImpl(link);
      }
    }
  },

  calcFromClippedConnectorImpl : function(link)
  {
    // summary:
    //		Calculates link connectors for this link at the "from" side, so that
    //		they point to the center of the node but are clipped on the border.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var endNode;
    var otherNode;
    var endPoint = ibm_ilog.graphlayout.internalutil.TranslateUtil.newArray(2, 0);
    var otherPoint = ibm_ilog.graphlayout.internalutil.TranslateUtil.newArray(2, 0);
    var rect = ibm_ilog.graphlayout.internalutil.TranslateUtil.newArray(4, 0);
    var otherPortPoint;
    var otherPort;
    var segment = link.getStartSegment();

    if (segment.isReversed())
    {
      endNode = segment.getTo();
      otherNode = segment.getFrom();
    }
    else
    {
      endNode = segment.getFrom();
      otherNode = segment.getTo();
    }

    if (endNode.isInvalid()) {
      return;
    }
    while (otherNode.isInvalid())
    {
      segment = otherNode.getIntergraphOpposite(segment);
      otherNode = segment.getOpposite(otherNode);
    }

    if (segment.getFrom() == otherNode)
    {
      otherPort = segment.getFromPortNumber();
      otherPortPoint = segment.getFromPoint();
    }
    else
    {
      otherPort = segment.getToPortNumber();
      otherPortPoint = segment.getToPoint();
    }

    if (otherPort == ibm_ilog.graphlayout.hierarchical.HEnum.UNSPECIFIED)
    {
      otherPoint[0] = (otherNode.getCenter(0));
      otherPoint[1] = (otherNode.getCenter(1));
    }
    else
    {
      otherPoint[0] = otherPortPoint[0];
      otherPoint[1] = otherPortPoint[1];
    }

    if (otherNode.getOwnerGraph() != graph)
    {
      otherPoint[0] = graph.getTranslatedCoord(otherNode, otherPoint[0], 0);
      otherPoint[1] = graph.getTranslatedCoord(otherNode, otherPoint[1], 1);
    }

    endPoint[0] = (endNode.getCenter(0));
    endPoint[1] = (endNode.getCenter(1));
    rect[0] = (endNode.getX());
    rect[1] = (endNode.getX() + endNode.getWidth());
    rect[2] = (endNode.getY());
    rect[3] = (endNode.getY() + endNode.getHeight());
    ibm_ilog.graphlayout.internalutil.LayoutUtil.clip(endPoint, otherPoint, rect);
    link.setFromCoord(0, endPoint[0]);
    link.setFromCoord(1, endPoint[1]);
  },

  calcToClippedConnectorImpl : function(link)
  {
    // summary:
    //		Calculates link connectors for this link at the "to" side, so that
    //		they point to the center of the node but are clipped on the border.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var endNode;
    var otherNode;
    var endPoint = ibm_ilog.graphlayout.internalutil.TranslateUtil.newArray(2, 0);
    var otherPoint = ibm_ilog.graphlayout.internalutil.TranslateUtil.newArray(2, 0);
    var rect = ibm_ilog.graphlayout.internalutil.TranslateUtil.newArray(4, 0);
    var otherPortPoint;
    var otherPort;
    var segment = link.getEndSegment();

    if (segment.isReversed())
    {
      endNode = segment.getFrom();
      otherNode = segment.getTo();
    }
    else
    {
      endNode = segment.getTo();
      otherNode = segment.getFrom();
    }

    if (endNode.isInvalid()) {
      return;
    }
    while (otherNode.isInvalid())
    {
      segment = otherNode.getIntergraphOpposite(segment);
      otherNode = segment.getOpposite(otherNode);
    }

    if (segment.getFrom() == otherNode)
    {
      otherPort = segment.getFromPortNumber();
      otherPortPoint = segment.getFromPoint();
    }
    else
    {
      otherPort = segment.getToPortNumber();
      otherPortPoint = segment.getToPoint();
    }

    if (otherPort == ibm_ilog.graphlayout.hierarchical.HEnum.UNSPECIFIED)
    {
      otherPoint[0] = (otherNode.getCenter(0));
      otherPoint[1] = (otherNode.getCenter(1));
    }
    else
    {
      otherPoint[0] = otherPortPoint[0];
      otherPoint[1] = otherPortPoint[1];
    }

    if (otherNode.getOwnerGraph() != graph)
    {
      otherPoint[0] = graph.getTranslatedCoord(otherNode, otherPoint[0], 0);
      otherPoint[1] = graph.getTranslatedCoord(otherNode, otherPoint[1], 1);
    }

    endPoint[0] = (endNode.getCenter(0));
    endPoint[1] = (endNode.getCenter(1));
    rect[0] = (endNode.getX());
    rect[1] = (endNode.getX() + endNode.getWidth());
    rect[2] = (endNode.getY());
    rect[3] = (endNode.getY() + endNode.getHeight());
    ibm_ilog.graphlayout.internalutil.LayoutUtil.clip(endPoint, otherPoint, rect);
    link.setToCoord(0, endPoint[0]);
    link.setToCoord(1, endPoint[1]);
  },

  calcEvenSpacedConnectors : function (a0)
  {
    // summary:
    //		Calculates link connectors so that they are evenly spaced along the
    //		sides.
    // tags:
    //		private
    
    if(arguments.length == 0)
      return this.calcEvenSpacedConnectors_overload_0();
    else if(arguments.length == 1)
      return this.calcEvenSpacedConnectors_overload_1(a0);
  },
  
  calcEvenSpacedConnectors_overload_0 : function()
                                                  
  {
    this.calcNorthSouthPortConnectors();

    var iter = this.getGraph().getNodes();
    var node;

    while (iter.hasNext())
    {
      node = iter.next();
      if (node.isEastWestPortAuxNode()) {
        this.calcEastWestPortConnectors(node);
      }
      else {
        this.calcEvenSpacedConnectors(node);
      }
      this.getPercController().addPoints(1);
      this.layoutStepPerformed();
    }
  },

  calcEastWestAuxPortNodeConnectors : function()
  {
    // summary:
    //		Calculates link connectors for east-west auxiliary port nodes so
    //		that they are evenly spaced along the sides. 
    // description:
    //		Calculates link connectors for east-west auxiliary port nodes so
    //		that they are evenly spaced along the sides. 
    //		The connectors at these aux nodes MUST always be evenly spaced.
    // tags:
    //		private
    
    if (!this.getGraph().hasPortSides()) {
      return;
    }
    var iter = this.getGraph().getNodes();
    var node;

    while (iter.hasNext())
    {
      node = iter.next();

      if (node.isEastWestPortAuxNode())
      {
        this.calcEastWestPortConnectors(node);
      }
    }
  },

  calcEvenSpacedConnectors_overload_1 : function(node)
  {
    // summary:
    //		Calculates link connectors for this node, so that the links are
    //		evenly spaced on the border of the node.
    // tags:
    //		private
    
    if (node.isDummyNode()) {
      return;
    }
    var respectFixedSides = !node.isEastWestPortAuxNode();
    var size = node.getSize(this._levelFlowDirection);
    var baseCoord = node.getCoord(this._levelFlowDirection);
    var iter;
    var segment;
    var selfLinkInfo = node.getSelfLinkInfo();

    var delta;
    var actCoord;

    if (node.isEastWestPortAuxNode())
    {
      delta = this.calcMinDistUpdateThickness2(node, size, true, false, false);
      actCoord = baseCoord + delta;
    }
    else if (selfLinkInfo.hasSelfLoopOffsetsForEvenlySpaced(ibm_ilog.graphlayout.hierarchical.HEnum.SOUTH))
    {
      delta = selfLinkInfo.getMinDistBetweenEvenlySpacedSegments(ibm_ilog.graphlayout.hierarchical.HEnum.SOUTH);
      actCoord = baseCoord + selfLinkInfo.getStartOffsetRegularLinks(ibm_ilog.graphlayout.hierarchical.HEnum.SOUTH);
      this._thicknessFactor = selfLinkInfo.getThicknessFactorForEvenlySpacedSegments(ibm_ilog.graphlayout.hierarchical.HEnum.SOUTH);
    }
    else
    {
      delta = this.calcMinDistUpdateThickness2(node, size, true, false, false);
      actCoord = baseCoord + delta;
    }

    iter = node.getAllSortedSegmentsFrom();
    var borderCoord = node.getCoord(this._edgeFlowDirection) + node.getSize(this._edgeFlowDirection);
    var point;
    var thickness;

    while (iter.hasNext())
    {
      segment = iter.next();

      if (segment.isFromSideFixed() && respectFixedSides) {
        segment.setFromCoordWhenFixed();
      }
      else if (segment.getFromPortNumber() == ibm_ilog.graphlayout.hierarchical.HEnum.UNSPECIFIED)
      {
        thickness = this._thicknessFactor * segment.getOwnerLink().getThickness();
        point = segment.getFromPoint();
        point[this._levelFlowDirection] = actCoord + 0.5 * thickness;
        point[this._edgeFlowDirection] = borderCoord;
        actCoord += delta + thickness;
      }
    }

    if (node.isEastWestPortAuxNode())
    {
      delta = this.calcMinDistUpdateThickness2(node, size, false, true, false);
      actCoord = baseCoord + delta;
    }
    else if (selfLinkInfo.hasSelfLoopOffsetsForEvenlySpaced(ibm_ilog.graphlayout.hierarchical.HEnum.NORTH))
    {
      delta = selfLinkInfo.getMinDistBetweenEvenlySpacedSegments(ibm_ilog.graphlayout.hierarchical.HEnum.NORTH);
      actCoord = baseCoord + selfLinkInfo.getStartOffsetRegularLinks(ibm_ilog.graphlayout.hierarchical.HEnum.NORTH);
      this._thicknessFactor = selfLinkInfo.getThicknessFactorForEvenlySpacedSegments(ibm_ilog.graphlayout.hierarchical.HEnum.NORTH);
    }
    else
    {
      delta = this.calcMinDistUpdateThickness2(node, size, false, true, false);
      actCoord = baseCoord + delta;
    }

    iter = node.getAllSortedSegmentsTo();
    borderCoord = node.getCoord(this._edgeFlowDirection);

    while (iter.hasNext())
    {
      segment = iter.next();

      if (segment.isToSideFixed() && respectFixedSides) {
        segment.setToCoordWhenFixed();
      }
      else if (segment.getToPortNumber() == ibm_ilog.graphlayout.hierarchical.HEnum.UNSPECIFIED)
      {
        thickness = this._thicknessFactor * segment.getOwnerLink().getThickness();
        point = segment.getToPoint();
        point[this._levelFlowDirection] = actCoord + 0.5 * thickness;
        point[this._edgeFlowDirection] = borderCoord;
        actCoord += delta + thickness;
      }
    }
  },

  calcEvenSpacedSideConnectors : function(node, auxNode)
  {
    // summary:
    //		Calculates link connectors for this node for the links that are
    //		attached to the left or right side, so that the links are evenly
    //		spaced on the border of the node. 
    // description:
    //		Calculates link connectors for this node for the links that are
    //		attached to the left or right side, so that the links are evenly
    //		spaced on the border of the node. 
    //		At the time point when we do this, these links are not connected to
    //		the node but rather to an auxiliary node at the left or right side.
    //		We assume that the link connection points are already calculated for
    //		this auxiliary node. We correct the link bends and connection
    //		points, but actually
    //		don't care about the link from or to node. Because this is the very last of all steps, it doesn't
    //		matter that the "from" and "to" node data structures are messed up
    //		afterwards. Basically it converts this:         into this: | | |    
    //		 | | | --------       +-+-+                --------       | | | |   
    //		    |     |     |              |       
    //		|------' | | |        |     | aux |              |        |--------'
    //		| |  node  |     |     |              |  node 
    //		|----------' |        |     | node|              |        |----------. |        |     |     |              |        |--------. | |        |     |     |              |        |------. | | --------       +-+-+                --------       | | | | | |                               | | |
    // tags:
    //		private
    
    if (node.isDummyNode()) {
      return;
    }
    auxNode.correctPortNumbers();

    var rightSide = (node.getPositionInLevel() < auxNode.getPositionInLevel());
    var sideIndex = ((rightSide ? ibm_ilog.graphlayout.hierarchical.HEnum.EAST : ibm_ilog.graphlayout.hierarchical.HEnum.WEST));
    var selfLinkInfo = node.getSelfLinkInfo();

    var actCoord;
    var baseCoord = node.getCoord(this._edgeFlowDirection);
    var delta;
    var size;

    if (node.isEastWestPortAuxNode())
    {
      size = node.getSize(this._edgeFlowDirection);
      delta = this.calcMinDistUpdateThickness2(auxNode, size, true, true, false);
      actCoord = baseCoord + delta;
    }
    else if (selfLinkInfo.hasSelfLoopOffsetsForEvenlySpaced(sideIndex))
    {
      delta = selfLinkInfo.getMinDistBetweenEvenlySpacedSegments(sideIndex);
      actCoord = baseCoord + selfLinkInfo.getStartOffsetRegularLinks(sideIndex);
      this._thicknessFactor = selfLinkInfo.getThicknessFactorForEvenlySpacedSegments(sideIndex);
    }
    else
    {
      size = node.getSize(this._edgeFlowDirection);
      delta = this.calcMinDistUpdateThickness2(auxNode, size, true, true, false);
      actCoord = baseCoord + delta;
    }

    var borderCoord = node.getCoord(this._levelFlowDirection);
    if (rightSide) {
      borderCoord += node.getSize(this._levelFlowDirection);
    }
    var portTable = ((rightSide ? node.getPortCache().getPortTable(ibm_ilog.graphlayout.hierarchical.HEnum.EAST) : node.getPortCache().getPortTable(ibm_ilog.graphlayout.hierarchical.HEnum.WEST)));

    this.calcAuxNodeWantedAttachCoord(auxNode);

    var iter = ((rightSide ? auxNode.getAllSortedSegmentsTo() : auxNode.getAllSortedSegmentsToInReverseOrder()));
    var segment;
    var thickness;

    while (iter.hasNext())
    {
      segment = iter.next();
      thickness = this._thicknessFactor * segment.getOwnerLink().getThickness();
      if (this.routeToSegmentAtSideConnector(segment, borderCoord, actCoord + 0.5 * thickness, baseCoord, portTable)) {
        actCoord += delta + thickness;
      }
    }

    iter = ((rightSide ? auxNode.getAllSortedSegmentsFromInReverseOrder() : auxNode.getAllSortedSegmentsFrom()));

    while (iter.hasNext())
    {
      segment = iter.next();
      thickness = this._thicknessFactor * segment.getOwnerLink().getThickness();
      if (this.routeFromSegmentAtSideConnector(segment, borderCoord, actCoord + 0.5 * thickness, baseCoord, portTable)) {
        actCoord += delta + thickness;
      }
    }
  },

  calcMinDistUpdateThickness1 : function(node, size, segments)
  {
    // summary:
    //		Calculates the minimal distance between the border of segments, and
    //		the maximal effective thickness of the segments that
    //		don't have ports. 
    // description:
    //		Calculates the minimal distance between the border of segments, and the maximal effective thickness of the segments that don't
    //		have ports. 
    //		The maximal effective thickness is the actual thickness multiplied
    //		by the thickness factor. This factor ensures that the minimal
    //		distance never needs to be set to a negative value. The minimal
    //		distance is returned.  
    // tags:
    //		private
    
    var iter = new CalcConnectorsAnonClass_1(segments);
    return this.calcMinDistUpdateThickness3(node, size, iter, false); // Number
  },

  calcMinDistUpdateThickness2 : function(node, size, fromSide, toSide, ignorePorts)
  {
    // summary:
    //		Calculates the minimal distance between the border of segments, and
    //		the maximal effective thickness of the segments that
    //		don't have ports. 
    // description:
    //		Calculates the minimal distance between the border of segments, and the maximal effective thickness of the segments that don't
    //		have ports. 
    //		The maximal effective thickness is the actual thickness multiplied
    //		by the thickness factor. This factor ensures that the minimal
    //		distance never needs to be set to a negative value. The minimal
    //		distance is returned.  
    // tags:
    //		private
    
    var iter = ((fromSide ? ((toSide ? node.getAllSortedSegments() : node.getAllSortedSegmentsFrom())) : node.getAllSortedSegmentsTo()));
    return this.calcMinDistUpdateThickness3(node, size, iter, ignorePorts); // Number
  },

  calcMinDistUpdateThickness3 : function(node, size, iter, ignorePorts)
  {
    // summary:
    //		
    // tags:
    //		private
    
    var segment;
    var numberLinksWithoutPorts = 0;
    var portNumber;
    var fixedSide;
    var sumThickness = 0.0;
    var respectFixedSides = !node.isEastWestPortAuxNode();

    while (iter.hasNext())
    {
      segment = iter.next();

      if (segment.getFrom() == node)
      {
        portNumber = segment.getFromPortNumber();
        fixedSide = (respectFixedSides && segment.isFromSideFixed());
      }
      else
      {
        portNumber = segment.getToPortNumber();
        fixedSide = (respectFixedSides && segment.isToSideFixed());
      }

      if (!fixedSide && (portNumber == ibm_ilog.graphlayout.hierarchical.HEnum.UNSPECIFIED || ignorePorts))
      {
        numberLinksWithoutPorts++;
        sumThickness += segment.getOwnerLink().getThickness();
      }
    }

    this._thicknessFactor = 1;
    var delta = (size - sumThickness) / (numberLinksWithoutPorts + 1);

    if (delta < 0.0)
    {
      delta = 0;
      this._thicknessFactor = size / sumThickness;
    }

    return delta; // Number
  },

  routeFromSegmentAtSideConnector : function(segment, normalNodeBorderCoord, attachCoordForNormalLinks, baseCoordForPorts, portTable)
  {
    // summary:
    //		Routes a "from" segment that is incident to an east-west-port
    //		auxiliary node.  
    // segment: ibm_ilog.graphlayout.hierarchical.HSegment
    //		The segment.
    // normalNodeBorderCoord: Number
    //		The coordinate of the border of the real
    //		node that owns the east-west-aux node.
    // attachCoordForNormalLinks: Number
    //		The attach coordinate at the real node
    //		taken when the connection point is not a port and not a fixed end.
    // baseCoordForPorts: Number
    //		The top or left corner coordinate of the node.
    // portTable: Array
    //		The table containing the attach coordinates for ports.
    // returns:
    //		true, if 'attachCoordForNormalLinks' was taken.
    // tags:
    //		private
    
    var port = segment.getFromPortNumber();
    var isSideFixed = false;
    var normalLink = false;

    var auxNodeBorderCoord = segment.getRawFromCoord(this._edgeFlowDirection);
    var auxNodeAttachCoord = segment.getWantedAnchorCoord();
    var normalNodeAttachCoord;

    if (segment.isFromSideFixed())
    {
      segment.setFromCoordWhenFixed();
      normalNodeAttachCoord = segment.getRawFromCoord(this._edgeFlowDirection);
      isSideFixed = true;
    }
    else if (port == ibm_ilog.graphlayout.hierarchical.HEnum.UNSPECIFIED)
    {
      normalNodeAttachCoord = attachCoordForNormalLinks;
      normalLink = true;
    }
    else
    {
      normalNodeAttachCoord = baseCoordForPorts + portTable[port];
    }

    this.routeAtSideConnector(segment, true, normalNodeBorderCoord, normalNodeAttachCoord, auxNodeAttachCoord, auxNodeBorderCoord, isSideFixed);
    return normalLink; // Boolean
  },

  routeToSegmentAtSideConnector : function(segment, normalNodeBorderCoord, attachCoordForNormalLinks, baseCoordForPorts, portTable)
  {
    // summary:
    //		Routes a "to" segment that is incident to an east-west-port
    //		auxiliary node.  
    // segment: ibm_ilog.graphlayout.hierarchical.HSegment
    //		The segment.
    // normalNodeBorderCoord: Number
    //		The coordinate of the border of the real
    //		node that owns the east-west-aux node.
    // attachCoordForNormalLinks: Number
    //		The attach coordinate at the real node
    //		taken when the connection point is not a port and not a fixed end.
    // baseCoordForPorts: Number
    //		The top or left corner coordinate of the node.
    // portTable: Array
    //		The table containing the attach coordinates for ports.
    // returns:
    //		true, if 'attachCoordForNormalLinks' was taken.
    // tags:
    //		private
    
    var port = segment.getToPortNumber();
    var isSideFixed = false;
    var normalLink = false;

    var auxNodeBorderCoord = segment.getRawToCoord(this._edgeFlowDirection);
    var auxNodeAttachCoord = segment.getWantedAnchorCoord();
    var normalNodeAttachCoord;

    if (segment.isToSideFixed())
    {
      segment.setToCoordWhenFixed();
      normalNodeAttachCoord = segment.getRawToCoord(this._edgeFlowDirection);
;
      isSideFixed = true;
    }
    else if (port == ibm_ilog.graphlayout.hierarchical.HEnum.UNSPECIFIED)
    {
      normalNodeAttachCoord = attachCoordForNormalLinks;
      normalLink = true;
    }
    else
    {
      normalNodeAttachCoord = baseCoordForPorts + portTable[port];
    }

    this.routeAtSideConnector(segment, false, normalNodeBorderCoord, normalNodeAttachCoord, auxNodeAttachCoord, auxNodeBorderCoord, isSideFixed);
    return normalLink; // Boolean
  },

  routeAtSideConnector : function(segment, fromSide, x1, y1, x2, y2, isSideFixed)
  {
    // summary:
    //		Routes a segment that is incident to an east-west-port auxiliary
    //		node. 
    // description:
    //		Routes a segment that is incident to an east-west-port auxiliary
    //		node. 
    //		Assume top-to-bottom flow as example. For polyline links, it does
    //		the following: ------    -----                 ------ |      |  |
    //		aux |   |      | (x1,y1) | node |  | node|  goes into    | node |\ |
    //		     |  |     |               |      | \         (with one dummy
    //		node) ------    --+--                 ------   \ (x2,y2) |          
    //		 * |                             |  For orthogonal links, it does
    //		the following: ------    -----                 ------ |      |  |
    //		aux |               |      |  | node |  | node|  goes into    | node
    //		|---*    (with two dummy nodes) |      |  |     |      |      |   |
    //		------    --+--                 ------    * |   |
    // tags:
    //		private
    
    var graph = this.getGraph();
    var dummyNode1;
    var dummyNode2;
    var swapNode;
    var link = segment.getOwnerLink();
    var auxNode;
    var point;

    if (fromSide)
    {
      point = segment.getFromPoint();
      auxNode = segment.getFrom();
    }
    else
    {
      point = segment.getToPoint();
      auxNode = segment.getTo();
    }

    if (link.isOrthogonal())
    {
      var nextNode = this.getNextValidOrLeveledNode(auxNode, segment);
      if (!nextNode.isInvalid() && nextNode.isDummyNode() && nextNode.getLevelNumber() == -1) {
        nextNode.setCoord(this._levelFlowDirection, x2);
      }
      dummyNode1 = link.addDummyNodeAtGraph(segment, graph);
      dummyNode2 = link.addDummyNodeAtGraph(segment, graph);

      if (segment.isReversed() == fromSide)
      {
        swapNode = dummyNode1;
        dummyNode1 = dummyNode2;
        dummyNode2 = swapNode;
      }

      dummyNode2.setCoord(this._levelFlowDirection, x2);
      dummyNode2.setCoord(this._edgeFlowDirection, y1);
    }
    else {
      dummyNode1 = link.addDummyNodeAtGraph(segment, graph);
    }
    dummyNode1.setCoord(this._levelFlowDirection, x2);
    dummyNode1.setCoord(this._edgeFlowDirection, y2);

    if (!isSideFixed)
    {
      point[this._levelFlowDirection] = x1;
      point[this._edgeFlowDirection] = y1;
    }
  },

  calcNorthSouthPortConnectors : function (a0)
  {
    // summary:
    //		Calculates link connectors for all links that have a port
    //		specification.
    // tags:
    //		private
    
    if(arguments.length == 0)
      return this.calcNorthSouthPortConnectors_overload_0();
    else if(arguments.length == 1)
      return this.calcNorthSouthPortConnectors_overload_1(a0);
  },
  
  calcNorthSouthPortConnectors_overload_0 : function()
                                                      
  {
    var iter = this.getGraph().getNodes();

    while (iter.hasNext())
    {
      this.calcNorthSouthPortConnectors(iter.next());
      this.getPercController().addPoints(1);
      this.layoutStepPerformed();
    }
  },

  calcNorthSouthPortConnectors_overload_1 : function(node)
  {
    // summary:
    //		Calculates the side link connectors for all links of this node that
    //		have a port specification. 
    // description:
    //		Calculates the side link connectors for all links of this node that
    //		have a port specification. 
    //		This is for links attaching at north or south side. The links at
    //		east or west side are handled differently.
    // tags:
    //		private
    
    if (node.getPortCache() == null) {
      return;
    }
    if (node.isDummyNode()) {
      return;
    }
    if (node.isEastWestPortAuxNode()) {
      return;
    }

    var baseCoord = node.getCoord(this._levelFlowDirection);
    var port;
    var point;
    var borderCoord;
    var portTable;
    var iter;
    var segment;
    portTable = node.getPortCache().getPortTable(ibm_ilog.graphlayout.hierarchical.HEnum.SOUTH);

    if (portTable != null)
    {
      borderCoord = node.getCoord(this._edgeFlowDirection) + node.getSize(this._edgeFlowDirection);
      iter = node.getAllSegmentsFrom(true, true, false, false);

      while (iter.hasNext())
      {
        segment = iter.next();
        port = segment.getFromPortNumber();

        if (port != ibm_ilog.graphlayout.hierarchical.HEnum.UNSPECIFIED)
        {
          point = segment.getFromPoint();
          point[this._levelFlowDirection] = baseCoord + portTable[port];
          point[this._edgeFlowDirection] = borderCoord;
        }
      }
    }

    portTable = node.getPortCache().getPortTable(ibm_ilog.graphlayout.hierarchical.HEnum.NORTH);

    if (portTable != null)
    {
      borderCoord = node.getCoord(this._edgeFlowDirection);
      iter = node.getAllSegmentsTo(true, true, false, false);

      while (iter.hasNext())
      {
        segment = iter.next();
        port = segment.getToPortNumber();

        if (port != ibm_ilog.graphlayout.hierarchical.HEnum.UNSPECIFIED)
        {
          point = segment.getToPoint();
          point[this._levelFlowDirection] = baseCoord + portTable[port];
          point[this._edgeFlowDirection] = borderCoord;
        }
      }

      portTable = null;
    }
  },

  calcEastWestPortConnectors : function(node)
  {
    // summary:
    //		Calculates the side link connectors for all links of this node that
    //		have a port specification. 
    // description:
    //		Calculates the side link connectors for all links of this node that
    //		have a port specification. 
    //		This is for links attaching at east or west side of their original
    //		node. However, the input node here is the auxiliary node, and the
    //		links connect to this auxiliary node at the north or south side.
    // tags:
    //		private
    
    node.correctPortNumbers();

    if (node.getNumberOfPorts(ibm_ilog.graphlayout.hierarchical.HEnum.SOUTH) == 0 && node.getNumberOfPorts(ibm_ilog.graphlayout.hierarchical.HEnum.NORTH) == 0)
    {
      this.calcEvenSpacedConnectors(node);
      return;
    }

    var size = node.getSize(this._levelFlowDirection);
    var baseCoord = node.getCoord(this._levelFlowDirection);

    var delta = this.calcMinDistUpdateThickness2(node, size, true, true, true);

    var fromIter = node.getAllSortedSegmentsFrom();
    var toIter = node.getAllSortedSegmentsTo();
    var segment;
    var actCoord = baseCoord + delta;
    var northCoord = node.getCoord(this._edgeFlowDirection);
    var southCoord = northCoord + node.getSize(this._edgeFlowDirection);
    var point;
    var thickness;

    while (fromIter.hasNext() || toIter.hasNext())
    {
      if (fromIter.hasNext())
      {
        segment = fromIter.next();
        thickness = segment.getOwnerLink().getThickness();
        point = segment.getFromPoint();
        point[this._levelFlowDirection] = actCoord + 0.5 * thickness;
        point[this._edgeFlowDirection] = southCoord;
        actCoord += delta + thickness;
      }

      if (toIter.hasNext())
      {
        segment = toIter.next();
        thickness = segment.getOwnerLink().getThickness();
        point = segment.getToPoint();
        point[this._levelFlowDirection] = actCoord + 0.5 * thickness;
        point[this._edgeFlowDirection] = northCoord;
        actCoord += delta + thickness;
      }
    }
  },

  calcPortTables : function (a0)
  {
    // summary:
    //		Calculates the port table of all nodes.
    // tags:
    //		private
    
    if(arguments.length == 0)
      return this.calcPortTables_overload_0();
    else if(arguments.length == 1)
      return this.calcPortTables_overload_1(a0);
  },
  
  calcPortTables_overload_0 : function()
                                        
  {
    var iter = this.getGraph().getNodes();

    while (iter.hasNext())
    {
      this.calcPortTables(iter.next());
    }
  },

  calcPortTables_overload_1 : function(node)
  {
    // summary:
    //		Calculates the port tables of the node. 
    // description:
    //		Calculates the port tables of the node. 
    //		We can calculate this after east-west expansion, but before the self
    //		links are removed, since self-links may contain ports.
    // tags:
    //		private
    
    if (node.getPortCache() == null) {
      return;
    }
    if (node.isDummyNode()) {
      return;
    }
    if (node.isEastWestPortAuxNode()) {
      return;
    }
    node.correctPortNumbers();
    var hsize = node.getSize(this._levelFlowDirection);
    var vsize = node.getSize(this._edgeFlowDirection);
    var maxPort;
    var portTable;

    maxPort = node.getNumberOfPorts(ibm_ilog.graphlayout.hierarchical.HEnum.NORTH);

    if (maxPort >= 0)
    {
      portTable = ibm_ilog.graphlayout.internalutil.TranslateUtil.newArray(maxPort, 0);
      this.calcPortTable(node, portTable, 0, hsize, false, true);
      node.getPortCache().setPortTable(ibm_ilog.graphlayout.hierarchical.HEnum.NORTH, portTable);
    }

    maxPort = node.getNumberOfPorts(ibm_ilog.graphlayout.hierarchical.HEnum.SOUTH);

    if (maxPort >= 0)
    {
      portTable = ibm_ilog.graphlayout.internalutil.TranslateUtil.newArray(maxPort, 0);
      this.calcPortTable(node, portTable, 0, hsize, true, false);
      node.getPortCache().setPortTable(ibm_ilog.graphlayout.hierarchical.HEnum.SOUTH, portTable);
    }

    var auxNode = node.getEastPortAuxNode();

    if (auxNode != null)
    {
      auxNode.correctPortNumbers();
      maxPort = auxNode.getNumberOfPorts(ibm_ilog.graphlayout.hierarchical.HEnum.SOUTH);
      if (auxNode.getNumberOfPorts(ibm_ilog.graphlayout.hierarchical.HEnum.NORTH) > maxPort) {
        maxPort = auxNode.getNumberOfPorts(ibm_ilog.graphlayout.hierarchical.HEnum.NORTH);
      }
      if (node.getNumberOfPorts(ibm_ilog.graphlayout.hierarchical.HEnum.EAST) > maxPort) {
        maxPort = node.getNumberOfPorts(ibm_ilog.graphlayout.hierarchical.HEnum.EAST);
      }
      if (maxPort > 0)
      {
        portTable = ibm_ilog.graphlayout.internalutil.TranslateUtil.newArray(maxPort, 0);
        this.calcPortTable(auxNode, portTable, 0, vsize, true, true);
        node.getPortCache().setPortTable(ibm_ilog.graphlayout.hierarchical.HEnum.EAST, portTable);
      }
    }

    auxNode = node.getWestPortAuxNode();

    if (auxNode != null)
    {
      auxNode.correctPortNumbers();
      maxPort = auxNode.getNumberOfPorts(ibm_ilog.graphlayout.hierarchical.HEnum.SOUTH);
      if (auxNode.getNumberOfPorts(ibm_ilog.graphlayout.hierarchical.HEnum.NORTH) > maxPort) {
        maxPort = auxNode.getNumberOfPorts(ibm_ilog.graphlayout.hierarchical.HEnum.NORTH);
      }
      if (node.getNumberOfPorts(ibm_ilog.graphlayout.hierarchical.HEnum.WEST) > maxPort) {
        maxPort = node.getNumberOfPorts(ibm_ilog.graphlayout.hierarchical.HEnum.WEST);
      }
      if (maxPort > 0)
      {
        portTable = ibm_ilog.graphlayout.internalutil.TranslateUtil.newArray(maxPort, 0);
        this.calcPortTable(auxNode, portTable, 0, vsize, true, true);
        node.getPortCache().setPortTable(ibm_ilog.graphlayout.hierarchical.HEnum.WEST, portTable);
      }
    }
  },

  calcPortTable : function(node, portTable, baseCoord, size, fromSide, toSide)
  {
    // summary:
    //		Calculates the port table that indicates where each port is placed.
    // tags:
    //		private
    
    var iter = ((fromSide ? ((toSide ? node.getAllSegments(true, true, false, false) : node.getAllSegmentsFrom(true, true, false, false))) : node.getAllSegmentsTo(true, true, false, false)));
    var i;
    var maxPort = portTable.length;

    for (i = 0; i < maxPort; i++)
      portTable[i] = 0.0;

    var segment;
    var portNumber;

    while (iter.hasNext())
    {
      segment = iter.next();
      portNumber = ((segment.getFrom() == node ? segment.getFromPortNumber() : segment.getToPortNumber()));

      if (portNumber != ibm_ilog.graphlayout.hierarchical.HEnum.UNSPECIFIED)
      {
        portTable[portNumber] = Math.max(segment.getOwnerLink().getThickness(), portTable[portNumber]);
      }
    }

    var sumThickness = 0.0;

    for (i = 0; i < portTable.length; i++)
      sumThickness += portTable[i];

    var maxThickness = /*Number.MAX_VALUE*/(3.4028235E38);
    var delta = (size - sumThickness) / (maxPort + 1);

    if (delta < 0.0)
    {
      delta = size / (maxPort + 1);
      maxThickness = 0.0;
    }

    var actCoord = baseCoord + delta;
    var thickness;

    for (i = 0; i < maxPort; i++)
    {
      thickness = portTable[i];
      if (thickness > maxThickness) {
        thickness = maxThickness;
      }
      portTable[i] = actCoord + 0.5 * thickness;
      actCoord += delta + thickness;
    }
  },

  calcAuxNodeWantedAttachCoord : function(auxNode)
  {
    // summary:
    //		Calculates the wanted attach coordinates for the input east west
    //		auxiliary node and stores them into the wanted anchor coord field of
    //		each segment. 
    // description:
    //		Calculates the wanted attach coordinates for the input east west
    //		auxiliary node and stores them into the wanted anchor coord field of
    //		each segment. 
    //		The result is already optimized, that is, impossible settings that
    //		would violate the minimal distance are already eliminated.
    // tags:
    //		private
    
    var bounds = ibm_ilog.graphlayout.internalutil.TranslateUtil.newArray(2, 0);

    this.calcAuxNodeSegmentAttachCoordBounds(auxNode, bounds);

    var iter = auxNode.getAllSegmentsFrom(true, true, false, false);
    var segment;
    var numberOfFromSegments = 0;
    var numberOfToSegments = 0;
    var attachCoord;
    var hasPortNumbers = false;

    while (iter.hasNext())
    {
      segment = iter.next();
      if (segment.getFromPortNumber() != ibm_ilog.graphlayout.hierarchical.HEnum.UNSPECIFIED) {
        hasPortNumbers = true;
      }
      attachCoord = this.getAuxNodeSegmentAttachCoord(segment, bounds, true);
      segment.setWantedAnchorCoord(attachCoord);
      numberOfFromSegments++;
    }

    iter = auxNode.getAllSegmentsTo(true, true, false, false);

    while (iter.hasNext())
    {
      segment = iter.next();
      if (segment.getToPortNumber() != ibm_ilog.graphlayout.hierarchical.HEnum.UNSPECIFIED) {
        hasPortNumbers = true;
      }
      attachCoord = this.getAuxNodeSegmentAttachCoord(segment, bounds, false);
      segment.setWantedAnchorCoord(attachCoord);
      numberOfToSegments++;
    }

    if (hasPortNumbers)
    {
      this._anchorOptimAlg.initFromFromAndToSegments(auxNode.getAllSegmentsFrom(true, true, false, false), auxNode.getAllSegmentsTo(true, true, false, false),
        numberOfFromSegments + numberOfToSegments, true);
      this._anchorOptimAlg.run();
      this._anchorOptimAlg.initFromFromAndToSegments(auxNode.getAllSegmentsFrom(true, true, false, false), auxNode.getAllSegmentsTo(true, true, false, false),
        numberOfFromSegments + numberOfToSegments, false);
      this._anchorOptimAlg.run();
    }
    else
    {
      iter = auxNode.getAllSegmentsFrom(true, true, false, false);
      this._anchorOptimAlg.initFromSegments(iter, numberOfFromSegments, true);
      this._anchorOptimAlg.run();
      iter = auxNode.getAllSegmentsTo(true, true, false, false);
      this._anchorOptimAlg.initFromSegments(iter, numberOfToSegments, false);
      this._anchorOptimAlg.run();
    }
  },

  getAuxNodeSegmentAttachCoord : function(segment, bounds, fromSide)
  {
    // summary:
    //		Calculate the coordinate where an east or west segment should attach
    //		virtually the aux node.
    // tags:
    //		private
    
    var levelFlowDir = this._levelFlowDirection;
    var actCoord;
    var targetCoord;
    var auxNode;

    if (fromSide)
    {
      actCoord = segment.getRawFromCoord(levelFlowDir);
      auxNode = segment.getFrom();
    }
    else
    {
      actCoord = segment.getRawToCoord(levelFlowDir);
      auxNode = segment.getTo();
    }

    var link = segment.getOwnerLink();

    if (link.getFrom().getOwnerGraph() == link.getTo().getOwnerGraph() && link.getFrom().getLevelNumber() == link.getTo().getLevelNumber())
    {
      return actCoord; // Number
    }

    var nextNode = this.getNextValidOrLeveledNode(auxNode, segment);
    if (nextNode.isInvalid()) {
      return actCoord; // Number
    }
    targetCoord = this.getNextValidNodeAttachCoord();

    if (link.isOrthogonal() && nextNode.isDummyNode() && nextNode.getLevelNumber() == -1)
    {
      nextNode = this.getNextValidOrLeveledNode(nextNode, nextNode.getIntergraphOpposite(this._segmentToValidNode));
      if (nextNode.isInvalid()) {
        return actCoord; // Number
      }
      targetCoord = this.getNextValidNodeAttachCoord();
    }

    if (targetCoord >= bounds[0] && targetCoord <= bounds[1]) {
      return targetCoord; // Number
    }
    return actCoord; // Number
  },

  calcAuxNodeSegmentAttachCoordBounds : function(auxNode, bounds)
  {
    // summary:
    //		Calculates the minimal and maximal coordinate for link segments that
    //		start at the input aux node.
    // tags:
    //		private
    
    var dir = this._levelFlowDirection;
    var graph = this.getGraph();
    bounds[0] = -/*Number.MAX_VALUE*/(3.4028235E38);
    bounds[1] = /*Number.MAX_VALUE*/(3.4028235E38);
    var index = auxNode.getPositionInLevel();
    var level = auxNode.getLevel();
    var node;
    var additionalSpace;
    var dist;

    if (index > 0)
    {
      node = level.getNode(index - 1);

      if (node.isEastWestPortAuxNode()) {
        bounds[0] = 0.5 * (node.getCoord(dir) + node.getSize(dir) + auxNode.getCoord(dir) + graph.getMinDistBetweenLinks(dir));
      }
      else if (node.isDummyNode())
      {
        if (node.isInvalid())
        {
          bounds[0] = node.getDummyNodeLevelFlowCoord(dir) + graph.getMinDistBetweenLinks(dir);
        }
        else
        {
          bounds[0] = node.getCoord(dir) + node.getSize(dir) + graph.getMinDistBetweenLinks(dir);
        }
      }
      else
      {
        additionalSpace = node.getSelfLinkInfo().getAdditionalSpace(ibm_ilog.graphlayout.hierarchical.HEnum.EAST);
        dist = additionalSpace + ((additionalSpace > 0 ? graph.getMinDistBetweenLinks(dir) : graph.getMinDistBetweenNodeAndLink(dir)));
        bounds[0] = node.getCoord(dir) + node.getSize(dir) + dist;
      }
    }

    if (index < level.getNodesCount() - 1)
    {
      node = level.getNode(index + 1);

      if (node.isEastWestPortAuxNode()) {
        bounds[1] = 0.5 * (auxNode.getCoord(dir) + auxNode.getSize(dir) + node.getCoord(dir) - graph.getMinDistBetweenLinks(dir));
      }
      else if (node.isDummyNode())
      {
        if (node.isInvalid())
        {
          bounds[1] = node.getDummyNodeLevelFlowCoord(dir) - graph.getMinDistBetweenLinks(dir);
        }
        else
        {
          bounds[1] = node.getCoord(dir) - graph.getMinDistBetweenLinks(dir);
        }
      }
      else
      {
        additionalSpace = node.getSelfLinkInfo().getAdditionalSpace(ibm_ilog.graphlayout.hierarchical.HEnum.WEST);
        dist = additionalSpace + ((additionalSpace > 0 ? graph.getMinDistBetweenLinks(dir) : graph.getMinDistBetweenNodeAndLink(dir)));
        bounds[1] = node.getCoord(dir) - dist;
      }
    }
  },

  getNextValidOrLeveledNode : function(node, segment)
  {
    // summary:
    //		Returns the next valid node in the link, or a node that belongs to
    //		the level.
    // tags:
    //		private
    
    node = segment.getOpposite(node);

    while (node.isInvalid() && node.getLevelNumber() == -1)
    {
      segment = node.getIntergraphOpposite(segment);
      node = segment.getOpposite(node);
    }

    this._validNode = node;
    this._segmentToValidNode = segment;
    return node; // ibm_ilog.graphlayout.hierarchical.HNode
  },

  getNextValidNodeAttachCoord : function()
  {
    // summary:
    //		Returns the coordinate in level direction where the link attaches
    //		the next valid node.
    // tags:
    //		private
    
    var c;
    if (this._segmentToValidNode.getFrom() == this._validNode) {
      c = this._segmentToValidNode.getRawFromCoord(this._levelFlowDirection);
    }
    else {
      c = this._segmentToValidNode.getRawToCoord(this._levelFlowDirection);
    }
    var graph = this.getGraph();
    return graph.getTranslatedCoord(this._validNode, c, this._levelFlowDirection); // Number
  },

  calcSelfLinkConnectors : function(node)
  {
    // summary:
    //		Distributes the self-links at the node side border. 
    // description:
    //		Distributes the self-links at the node side border. 
    //		Note, we do this before the nodes have their final positions, but
    //		once again when the nodes have final positions.
    // tags:
    //		private
    
    if (node.isDummyNode()) {
      return;
    }
    if (node.isEastWestPortAuxNode()) {
      return;
    }
    if (node.getSelfLinkInfo().getSelfLinksCount() == 0) {
      return;
    }
    this.calcSelfLinkConnectorsAtNode(node);
  },

  calcSelfLinkConnectorsAtNode : function(node)
  {
    // summary:
    //		Calculates the self-link connectors. 
    // description:
    //		Calculates the self-link connectors. 
    //		For centered and clipped style, 1/3 of the corner side is reserved
    //		for self-links. The middle 1/3 is coceptually reserved for normal
    //		links, but its does not matter in this mode. For evenly spaced
    //		style, we find the area reserved for self-links in the self-link
    //		info.
    // tags:
    //		private
    
    this.calcSelfLinkConnectorsAtSide(node, ibm_ilog.graphlayout.hierarchical.HEnum.EAST, 0);
    this.calcSelfLinkConnectorsAtSide(node, ibm_ilog.graphlayout.hierarchical.HEnum.EAST, 1);
    this.calcSelfLinkConnectorsAtSide(node, ibm_ilog.graphlayout.hierarchical.HEnum.WEST, 0);
    this.calcSelfLinkConnectorsAtSide(node, ibm_ilog.graphlayout.hierarchical.HEnum.WEST, 1);
    this.calcSelfLinkConnectorsAtSide(node, ibm_ilog.graphlayout.hierarchical.HEnum.NORTH, 0);
    this.calcSelfLinkConnectorsAtSide(node, ibm_ilog.graphlayout.hierarchical.HEnum.NORTH, 1);
    this.calcSelfLinkConnectorsAtSide(node, ibm_ilog.graphlayout.hierarchical.HEnum.SOUTH, 0);
    this.calcSelfLinkConnectorsAtSide(node, ibm_ilog.graphlayout.hierarchical.HEnum.SOUTH, 1);
  },

  calcSelfLinkConnectorsAtSide : function(node, side, subzone)
  {
    // summary:
    //		Calculates the self-link connectors. 
    // description:
    //		Calculates the self-link connectors. 
    //		Works for all connector styles.
    // tags:
    //		private
    
    var graph = node.getOwnerGraph();
    var dir0 = ((side == ibm_ilog.graphlayout.hierarchical.HEnum.EAST || side == ibm_ilog.graphlayout.hierarchical.HEnum.WEST) ? graph.getEdgeFlow() : graph.getLevelFlow());
    var dir1 = 1 - dir0;
    var port;
    var point;
    var thickness;
    var portBaseCoord = node.getCoord(dir0);
    var baseCoord;
    var borderCoord = node.getCoord(dir1);
    if (side == ibm_ilog.graphlayout.hierarchical.HEnum.EAST || side == ibm_ilog.graphlayout.hierarchical.HEnum.SOUTH) {
      borderCoord += node.getSize(dir1);
    }
    var info = node.getSelfLinkInfo();
    var links = info.getSelfLinks2(side, subzone);
    if (links == null || links.length == 0) {
      return;
    }
    var segments = this.getSelfLinkSegmentArray(node, side, links);
    var delta;

    if (this._style == ibm_ilog.graphlayout.hierarchical.HEnum.EVENLY_SPACED_PINS && info.hasSelfLoopOffsetsForEvenlySpaced(side))
    {
      delta = info.getMinDistBetweenEvenlySpacedSegments(side);
      this._thicknessFactor = info.getThicknessFactorForEvenlySpacedSegments(side);
      baseCoord = node.getCoord(dir0);
      if (subzone == 1) {
        baseCoord += node.getSize(dir0) - info.getEndOffsetRegularLinks(side);
      }
    }
    else
    {
      var size = node.getSize(dir0) / 3;
      delta = this.calcMinDistUpdateThickness1(node, size, segments);

      baseCoord = node.getCoord(dir0);
      if (subzone == 1) {
        baseCoord += 2 * size;
      }
    }

    var portTable = node.getPortCache().getPortTable(side);
    var actCoord = baseCoord + delta;

    for (var i = 0; i < segments.length; i++)
    {
      var seg = segments[i];

      if (seg.getFrom() == node)
      {
        port = seg.getFromPortNumber();
        point = seg.getFromPoint();

        if (seg.isFromSideFixed())
        {
          seg.setFromCoordWhenFixed();
        }
        else if (port == ibm_ilog.graphlayout.hierarchical.HEnum.UNSPECIFIED)
        {
          thickness = this._thicknessFactor * seg.getOwnerLink().getThickness();
          point[dir0] = actCoord + 0.5 * thickness;
          point[dir1] = borderCoord;
          actCoord += delta + thickness;
        }
        else
        {
          point[dir0] = portBaseCoord + portTable[port];
          point[dir1] = borderCoord;
        }
      }
      else if (seg.getTo() == node)
      {
        port = seg.getToPortNumber();
        point = seg.getToPoint();

        if (seg.isToSideFixed())
        {
          seg.setToCoordWhenFixed();
        }
        else if (port == ibm_ilog.graphlayout.hierarchical.HEnum.UNSPECIFIED)
        {
          thickness = this._thicknessFactor * seg.getOwnerLink().getThickness();
          point[dir0] = actCoord + 0.5 * thickness;
          point[dir1] = borderCoord;
          actCoord += delta + thickness;
        }
        else
        {
          point[dir0] = portBaseCoord + portTable[port];
          point[dir1] = borderCoord;
        }
      }
    }
  },

  getSelfLinkSegmentArray : function(node, side, links)
  {
    // summary:
    //		Calculates the segment array for the links.
    // tags:
    //		private
    
    var info = node.getSelfLinkInfo();
    var sameSideNested = info.isSelfLinksSameSideNested();
    var segments = new Array();
    var seg;
    var link;
    var i;

    for (i = 0; i < links.length; i++)
    {
      link = links[i];

      if (link.getFromPortSide() == side)
      {
        seg = link.getStartSegment();
        segments.push(seg);
      }

      if (link.getToPortSide() == side)
      {
        if (link.getFromPortSide() == side && sameSideNested) {
          continue;
        }
        seg = link.getEndSegment();
        segments.push(seg);
      }
    }

    if (sameSideNested)
    {
      for (i = links.length - 1; i >= 0; i--)
      {
        link = links[i];

        if (link.getToPortSide() == side && link.getFromPortSide() == side)
        {
          seg = link.getEndSegment();
          segments.push(seg);
        }
      }
    }

    return ibm_ilog.graphlayout.internalutil.TranslateUtil.copyArrayInto(segments,
           ibm_ilog.graphlayout.internalutil.TranslateUtil.newArray(segments.length, null)); // Array
  },

  calcMinDistThicknessFactorAndOffsets : function(node, side)
  {
    // summary:
    //		Calculates the  offsets, thickness factor, min dist for evenly
    //		spaced links at the given side. 
    // description:
    //		Calculates the  offsets, thickness factor, min dist for evenly
    //		spaced links at the given side. 
    //		This is only called if there are any self links at that side.
    // tags:
    //		private
    
    if (node.isEastWestPortAuxNode()) {
      return;
    }
    if (node.isDummyNode()) {
      return;
    }
    var info = node.getSelfLinkInfo();
    var graph = node.getOwnerGraph();
    var dir = ((side == ibm_ilog.graphlayout.hierarchical.HEnum.EAST || side == ibm_ilog.graphlayout.hierarchical.HEnum.WEST) ? graph.getEdgeFlow() : graph.getLevelFlow());
    var size = node.getSize(dir);

    var linksZone0 = info.getSelfLinks2(side, 0);
    var linksZone1 = info.getSelfLinks2(side, 1);
    if ((linksZone0 == null || linksZone0.length == 0) && (linksZone1 == null || linksZone1.length == 0)) {
      return;
    }
    var segmentsZone0 = this.getSelfLinkSegmentArray(node, side, linksZone0);
    var segmentsZone1 = this.getSelfLinkSegmentArray(node, side, linksZone1);
    var allSegments = new Array();
    var i;

    for (i = 0; i < segmentsZone0.length; i++)
      allSegments.push(segmentsZone0[i]);
    var iter = null;
    var aux;

    switch (side)
    {
      case ibm_ilog.graphlayout.hierarchical.HEnum.EAST:
        aux = node.getEastPortAuxNode();
        if (aux != null) {
          iter = aux.getAllSegments(true, true, false, false);
        }
        break;
      case ibm_ilog.graphlayout.hierarchical.HEnum.WEST:
        aux = node.getWestPortAuxNode();
        if (aux != null) {
          iter = aux.getAllSegments(true, true, false, false);
        }
        break;
      case ibm_ilog.graphlayout.hierarchical.HEnum.NORTH:
        iter = node.getAllSegmentsTo(true, true, false, false);
        break;
      case ibm_ilog.graphlayout.hierarchical.HEnum.SOUTH:
        iter = node.getAllSegmentsFrom(true, true, false, false);
        break;
    }
    if (iter != null)
    {
      while (iter.hasNext())
      {
        allSegments.push(iter.next());
      }
    }

    for (i = 0; i < segmentsZone1.length; i++)
      allSegments.push(segmentsZone1[i]);

    var segments = ibm_ilog.graphlayout.internalutil.TranslateUtil.copyArrayInto(allSegments,
           ibm_ilog.graphlayout.internalutil.TranslateUtil.newArray(allSegments.length, null));
    var delta = this.calcMinDistUpdateThickness1(node, size, segments);
    info.setMinDistBetweenEvenlySpacedSegments(side, delta);
    info.setThicknessFactorForEvenlySpacedSegments(side, this._thicknessFactor);

    var thickness;
    var startOffset = delta;
    var segment;

    for (i = 0; i < segmentsZone0.length; i++)
    {
      segment = segmentsZone0[i];

      if (segment.getFromPortNumber() == ibm_ilog.graphlayout.hierarchical.HEnum.UNSPECIFIED && !segment.isFromSideFixed())
      {
        thickness = this._thicknessFactor * segment.getOwnerLink().getThickness();
        startOffset += delta + thickness;
      }
    }

    var endOffset = delta;

    for (i = 0; i < segmentsZone1.length; i++)
    {
      segment = segmentsZone1[i];

      if (segment.getFromPortNumber() == ibm_ilog.graphlayout.hierarchical.HEnum.UNSPECIFIED && !segment.isFromSideFixed())
      {
        thickness = this._thicknessFactor * segment.getOwnerLink().getThickness();
        endOffset += delta + thickness;
      }
    }

    info.setStartOffsetRegularLinks(side, startOffset);
    info.setEndOffsetRegularLinks(side, endOffset);
  }

}
);
/**
 * @private
 */
dojo.declare('CalcConnectorsAnonClass_1', ibm_ilog.graphlayout.hierarchical.HSegmentIterator,
{
  // tags:
  //		private
  
  i: null,
  __segments : null,

  constructor : function(input__segments)
  {
    
    this.__segments = input__segments;
    this.i = 0;
  },

  hasNext : function()
  {
    return this.i < this.__segments.length; // Boolean
  },

  next : function()
  {
    return this.__segments[this.i++]; // ibm_ilog.graphlayout.hierarchical.HSegment
  }

});

