dojo.provide('ibm_ilog.graphlayout.hierarchical.EastWestLinkRemovalAlgorithm');

///////////////////////////////////////////////////////////////////////////////
// IBM ILOG Dojo Diagrammer 1.0
///////////////////////////////////////////////////////////////////////////////

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

dojo.require('ibm_ilog.graphlayout.hierarchical.HEnum');
dojo.require('ibm_ilog.graphlayout.hierarchical.HLevelAlgorithm');
dojo.require('ibm_ilog.graphlayout.hierarchical.HNode');
dojo.require('ibm_ilog.graphlayout.internalutil.GraphModelUtil');
dojo.require('ibm_ilog.graphlayout.internalutil.TranslateUtil');

dojo.declare('ibm_ilog.graphlayout.hierarchical.EastWestLinkRemovalAlgorithm', ibm_ilog.graphlayout.hierarchical.HLevelAlgorithm, 
{
  // summary:
  //		EastWestLinkRemovalAlgorithm checks which links between neighbored
  //		nodes of the same level can be routed by the 
  //		EastWestLinkRoutingAlgorithm, and makes them east-west links. 
  // description:
  //		EastWestLinkRemovalAlgorithm checks which links between neighbored
  //		nodes of the same level can be routed by the 
  //		EastWestLinkRoutingAlgorithm, and makes them east-west links. 
  //		This removes these links from the adjacencies and from the main link
  //		list of the graph, because they would violate the proper hierarchy
  //		condition. So all algorithms working on the proper hierarchy
  //		don't see them anymore. This is called after crossing reduction before the nodes get their final positions. In fact it must be called after the converted intergraph links are removed, to avoid the a converted intergraph links becomes an east-west link. Example: &#95;&#95;    &#95;&#95;                    &#95;&#95;     &#95;&#95; Situation  |&#95;&#95;|  |&#95;&#95;|   should become: |&#95;&#95;|---|&#95;&#95;| after        \   / crossing      \ / reduction:     D that is, the lower bypass dummy node in the lower level that was introduced to have a proper hierarchy gets removed. This algorithm makes the link straight. Since both end nodes must belong to the same level, intergraph links are never east-west links.  
  // tags:
  //		private
  
  constructor : function()
  {
  },
  
  run : function()
  {
    // summary:
    //		Starts the algorithm.
    // tags:
    //		private
    
    this.unmarkLowerLevelNodes();
    var numEastWestLinks = this.detectEastWestLinks();
    if (numEastWestLinks == 0) {
      return;
    }
    this.removeMarkedDummyNodesAndMakeEastWestLinks(numEastWestLinks);
  },

  unmarkLowerLevelNodes : function()
  {
    // summary:
    //		Unmark all nodes at the lower level.
    // tags:
    //		private
    
    var iter = this.getLowerLevel().getNodes();
    var node;

    while (iter.hasNext())
    {
      node = iter.next();
      node.setMarker(0);
    }
  },

  detectEastWestLinks : function()
  {
    // summary:
    //		Detect all east-west links and assign the multiple link index to
    //		them. 
    // description:
    //		Detect all east-west links and assign the multiple link index to
    //		them. 
    //		It marks all dummy nodes in the lower level that belong to east-west
    //		links and returns the number of east-west links. This is also the
    //		number of marked dummy nodes in the lower level.
    // tags:
    //		private
    
    var numEastWestLinks = 0;
    var level = this.getUpperLevel();
    var nodeiter = level.getNodes();
    var node;
    var dummyNode;
    var segmentIter;
    var link;
    level.updateInfo();

    while (nodeiter.hasNext())
    {
      node = nodeiter.next();
      segmentIter = node.getSegmentsFrom();

      while (segmentIter.hasNext())
      {
        link = segmentIter.next().getOwnerLink();

        if (link.getFrom() == node && this.isEastWestLink(link))
        {
          dummyNode = this.getUniqueDummyNode(link);
          dummyNode.setMarker(1);
          numEastWestLinks++;
        }
      }
    }

    return numEastWestLinks; // int
  },

  removeMarkedDummyNodesAndMakeEastWestLinks : function(numEastWestLinks)
  {
    // summary:
    //		Remove the marked dummy nodes in the lower level and make their
    //		links to east-west links.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var level = this.getLowerLevel();
    var numNodes = level.getNodesCount();
    var src = level.getNodesField();
    var dst = ibm_ilog.graphlayout.internalutil.TranslateUtil.newArray(numNodes - numEastWestLinks, null);
    var j = 0;

    for (var i = 0; i < numNodes; i++)
    {
      if (src[i].getMarker() == 1)
      {
        src[i].setLevelNumber(-1);

        graph.makeEastWestLink(src[i].getOwnerLink());
      }
      else {
        dst[j++] = src[i];
      }
    }

    level.setNodesField(dst);
    level.updateInfo();
  },

  isEastWestLink : function(link)
  {
    // summary:
    //		Returns true if the link is an east-west link. 
    // description:
    //		Returns true if the link is an east-west link. 
    //		That means the link can be drawn straight between two neighbored
    //		nodes.
    // tags:
    //		private
    
    var from = link.getFrom();
    var toobj = link.getTo();

    if (from.getOwnerGraph() != toobj.getOwnerGraph()) {
      return false; // Boolean
    }
    if (from.getLevelNumber() != toobj.getLevelNumber()) {
      return false; // Boolean
    }
    var diff = (from.getPositionInLevel() - toobj.getPositionInLevel());
    if (diff != 1 && diff != -1) {
      return false; // Boolean
    }
    if (from.isEastWestPortAuxNode()) {
      return false; // Boolean
    }
    if (toobj.isEastWestPortAuxNode()) {
      return false; // Boolean
    }
    if (link.isFromPortNumberSpecified()) {
      return false; // Boolean
    }
    if (link.isToPortNumberSpecified()) {
      return false; // Boolean
    }

    var westNodeSide = ((diff < 0 ? link.getOrigFromPortSide() : link.getOrigToPortSide()));
    var eastNodeSide = ((diff < 0 ? link.getOrigToPortSide() : link.getOrigFromPortSide()));
    if (westNodeSide != ibm_ilog.graphlayout.hierarchical.HEnum.UNSPECIFIED && westNodeSide != ibm_ilog.graphlayout.hierarchical.HEnum.EAST &&
      westNodeSide != ibm_ilog.graphlayout.hierarchical.HEnum.EAST_OR_WEST) {
      return false; // Boolean
    }
    if (eastNodeSide != ibm_ilog.graphlayout.hierarchical.HEnum.UNSPECIFIED && eastNodeSide != ibm_ilog.graphlayout.hierarchical.HEnum.WEST &&
      eastNodeSide != ibm_ilog.graphlayout.hierarchical.HEnum.EAST_OR_WEST) {
      return false; // Boolean
    }
    return true; // Boolean
  },

  getUniqueDummyNode : function(link)
  {
    // summary:
    //		Returns the unique dummy node of the link, or null if there is no
    //		unique dummy node.
    // tags:
    //		private
    
    var segment = link.getStartSegment();
    var node = (segment.getFrom().isDummyNode() ? segment.getFrom() : segment.getTo());
    if (!node.isDummyNode()) {
      return null; // ibm_ilog.graphlayout.hierarchical.HNode
    }
    if (node.getOpposite(segment) != link.getEndSegment()) {
      return null; // ibm_ilog.graphlayout.hierarchical.HNode
    }
    return node; // ibm_ilog.graphlayout.hierarchical.HNode
  }

}
);
