dojo.provide('ibm_ilog.graphlayout.hierarchical.HierarchicalGroupSpreadConstraint');

///////////////////////////////////////////////////////////////////////////////
// IBM ILOG Dojo Diagrammer 1.0
///////////////////////////////////////////////////////////////////////////////

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

dojo.require('ibm_ilog.graphlayout.hierarchical.HierarchicalConstraint');
dojo.require('ibm_ilog.graphlayout.internalutil.GraphModelUtil');
dojo.require('ibm_ilog.graphlayout.internalutil.TranslateUtil');

dojo.declare('ibm_ilog.graphlayout.hierarchical.HierarchicalGroupSpreadConstraint', ibm_ilog.graphlayout.hierarchical.HierarchicalConstraint, 
{
  // summary:
  //		The class 'GroupSpreadConstraint' is  a constraint that
  //		forces a group of nodes to be placed no more than a  specified number
  //		of levels apart. 
  // description:
  //		The class <code>GroupSpreadConstraint</code> is  a constraint that
  //		forces a group of nodes to be placed no more than a  specified number
  //		of levels apart. 
  //		<p>
  //		Nodes are organized in horizontal or vertical levels numbered from 0
  //		to "n". If the link flow direction is from top to bottom, the nodes
  //		with level index 0 are placed in the topmost level, and the nodes
  //		with higher level indexes are placed in the levels below. If the link
  //		flow direction is from left to right, the nodes with level index 0
  //		are placed in the leftmost level, and the nodes with higher level
  //		indexes are placed in the levels farther to the right.  
  //		</p>
  //		<p>
  //		For instance, a constraint with spread size 2 forces the group to be
  //		placed such that the group node with smallest level index is no more
  //		than 2 levels apart from the group node with largest level index. All
  //		remaining nodes are placed in between. Assume that the group consists
  //		of the nodes A, B, and C. The group spread constraint is satisfied if
  //		the levels of A, B and C are 1, 2, 3. It is also satisfied if the
  //		levels of A, B and C are 10, 9, 8, or 10, 10, 12 or 10, 11, 11, or
  //		10, 10, 10. But it is not satisfied if the levels of A, B and C are
  //		10, 13, 12, because the highest node B is in this case more than 2
  //		levels apart from the lowest node A.  
  //		</p>
  //		<p>
  //		The constraint is evaluated, if the incremental mode is disabled. If
  //		the incremental mode is enabled, the constraint is only evaluated if
  //		the nodes of the group are marked for incremental recalculation,
  //		because otherwise, the incremental positions of the nodes are
  //		dominant.  
  //		</p>
  //		<p>
  //		Among the constraints, this kind has the highest priority. If several
  //		conflicting constraints are specified (such as two nodes must be
  //		placed no more than 2 number of levels apart, <i>but</i> in level 1
  //		and 10), the constraints with lower priority are ignored during
  //		layout.  
  //		</p>
  //		<p>
  //		The automatic conflict resolution can handle conflicting constraints.
  //		However, to speedup the layout, it is recommended to specify
  //		constraints in a way such that there are no conflicts.   
  //		</p>
  //		<p>
  //		The group spread constraint has the same priority as the same level
  //		constraint, because both cannot be conflicting. In fact, there are
  //		two ways of forcing two nodes to the same level: use the same level
  //		constraint, or use a group spread constraint of a group of two nodes
  //		with spread size 0.  
  //		</p>
  //		<p>
  //		In recursive layout mode (see <code>
  //		HierarchicalLayout.setRecursiveLayoutMode()</code>), all nodes of a 
  //		group spread constraint must belong to the same subgraph, and the
  //		constraint must be installed at the layout instance that is attached
  //		to this subgraph. It is not possible to have a group spread
  //		constraint containing nodes of different subgraphs.    
  //		</p>
  
  _group: null,
  _spreadSize: null,

  constructor : function (a0, a1)
  {
    if(arguments.length == 2)
      this._constructor_overload_ibm_ilog_graphlayout_hierarchical_HierarchicalGroupSpreadConstraint_2(a0, a1);
    else if(arguments.length == 1)
      this._constructor_overload_ibm_ilog_graphlayout_hierarchical_HierarchicalGroupSpreadConstraint_1(a0);
  },
  
  _constructor_overload_ibm_ilog_graphlayout_hierarchical_HierarchicalGroupSpreadConstraint_2 : function(groupArg , sizeArg )
  {
    this.setPriority(Number.MAX_VALUE);
    this._group = groupArg;
    this.setSpreadSize(sizeArg);
  },
  
  _constructor_overload_ibm_ilog_graphlayout_hierarchical_HierarchicalGroupSpreadConstraint_1 : function(source)
  {
    this._group = source._group;
    this._spreadSize = source._spreadSize;
  },
  
  copy : function()
  {
    // summary:
    //		Copies the constraint. 
    // description:
    //		Copies the constraint. 
    //		<p>
    //		</p>
    
    return HierarchicalGroupSpreadConstraint.createConstraint(this); // ibm_ilog.graphlayout.hierarchical.HierarchicalConstraint
  },

  getGroup : function()
  {
    // summary:
    //		Returns the group of this constraint.
    
    return this._group; // ibm_ilog.graphlayout.hierarchical.HierarchicalNodeGroup
  },

  getSpreadSize : function()
  {
    // summary:
    //		Returns the spread size of this constraint. 
    // description:
    //		Returns the spread size of this constraint.
    
    return this._spreadSize; // int
  },

  setSpreadSize : function(spreadSize)
  {
    // summary:
    //		Sets the spread size of the group. 
    // description:
    //		Sets the spread size of the group. 
    //		<p>
    //		 
    //		</p>
    // spreadSize: int
    //		The maximum number of levels two nodes of the group
    //		can be apart.
    
    if (spreadSize < 0) {
      this._spreadSize = /*Number.MAX_VALUE*/2147483647;
    }
    else {
      this._spreadSize = spreadSize;
    }
  },

  getFirstSubject : function()
  {
    // summary:
    //		Returns the "first" subject of the constraint. 
    // description:
    //		Returns the "first" subject of the constraint. 
    //		This means simply we can set it via <code>
    //		HierarchicalGroupSpreadConstraint.setFirstSubject()</code>.
    // tags:
    //		private
    
    return this._group; // Object
  },

  setFirstSubject : function(subject)
  {
    // summary:
    //		Sets the first subject of the constraint.
    // tags:
    //		private
    
    this._group = subject;
  },

  validate : function(model)
  {
    // summary:
    //		Returns <tt>true</tt> if the constraint is valid in the input graph
    //		model.
    // tags:
    //		private
    
    return this.validateGroup(model, this.getGroup()); // Boolean
  },

  validateForLayout : function(graph)
  {
    // summary:
    //		Sets the valid flag of the constraint during layout. 
    // description:
    //		Sets the valid flag of the constraint during layout. 
    //		The purpose is the same as validate, but this is used temporarily
    //		during layout while <code>
    //		HierarchicalGroupSpreadConstraint.validate()</code> is used to
    //		remove invalid constraints completely.
    // tags:
    //		private
    
    this.setValidForLayout(true);
    this.validateGroupForLayout(graph, this.getGroup());
  },

  actAfterAdd : function(manager)
  {
    // summary:
    //		Acts after the constraint was added to the constraint manager.
    // tags:
    //		private
    
    manager.addGroup(this.getGroup());
  }

}
);

dojo.mixin(ibm_ilog.graphlayout.hierarchical.HierarchicalGroupSpreadConstraint,
{
  serialVersionUID : null,
  _fields_init : function(c)
  {
    c = c || ibm_ilog.graphlayout.hierarchical.HierarchicalGroupSpreadConstraint;
    if(ibm_ilog.graphlayout.hierarchical.HierarchicalConstraint._fields_init)
      ibm_ilog.graphlayout.hierarchical.HierarchicalConstraint._fields_init(c);
    
    c.serialVersionUID = 4738965472489682855;
  }
  
}
);

(function(){ ibm_ilog.graphlayout.hierarchical.HierarchicalGroupSpreadConstraint._fields_init(); })();
