dojo.provide('ibm_ilog.graphlayout.hierarchical.OrthRoutingAlgorithm');

///////////////////////////////////////////////////////////////////////////////
// IBM ILOG Dojo Diagrammer 1.0
///////////////////////////////////////////////////////////////////////////////

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

dojo.require('ibm_ilog.graphlayout.hierarchical.HGraphAlgorithm');
dojo.require('ibm_ilog.graphlayout.hierarchical.HLevelAlgApplicator');
dojo.require('ibm_ilog.graphlayout.hierarchical.HPercentages');
dojo.require('ibm_ilog.graphlayout.hierarchical.HSegmentBufferedIterator');
dojo.require('ibm_ilog.graphlayout.hierarchical.OrthRoutingLevelAlgorithm');
dojo.require('ibm_ilog.graphlayout.internalutil.GraphModelUtil');
dojo.require('ibm_ilog.graphlayout.internalutil.TranslateUtil');

dojo.declare('ibm_ilog.graphlayout.hierarchical.OrthRoutingAlgorithm', ibm_ilog.graphlayout.hierarchical.HGraphAlgorithm, 
{
  // summary:
  //		OrthRoutingAlgorithm traverses the levels and performs orthogonal
  //		routing of the segments. 
  // description:
  //		OrthRoutingAlgorithm traverses the levels and performs orthogonal
  //		routing of the segments. 
  //		Note that afterwards, the graph is not anymore a proper hierarchy.
  //		Therefore, this step should always be the last one of the steps that
  //		use level sweeps.  
  // tags:
  //		private
  
  _levelAlg: null,

  constructor : function(graph)
  {
    this.init(graph);
    this._levelAlg = new ibm_ilog.graphlayout.hierarchical.OrthRoutingLevelAlgorithm(graph);
  },
  
  clean : function()
  {
    // summary:
    //		Clean the object after usage. 
    // description:
    //		Clean the object after usage. 
    //		It is not necessary to use this, but it speeds up garbage collection
    //		if we do it when we don't need the object anymore.
    // tags:
    //		private
    
    this.inherited("clean", arguments, []);
    if (this._levelAlg != null) {
      this._levelAlg.clean();
    }
    this._levelAlg = null;
  },

  run : function()
  {
    // summary:
    //		Starts the algorithm.
    // tags:
    //		private
    
    var graph = this.getGraph();
    this.getPercController().startStep(this.getPercentages().steps[ibm_ilog.graphlayout.hierarchical.HPercentages.STEP_ORTH_ROUTING], graph.getNumberOfLevels() - 1, false);

    if (graph.getNumberOfNormalSegments() == 0 && graph.getNumberOfIntergraphSegments() == 0) {
      return;
    }
    new ibm_ilog.graphlayout.hierarchical.HLevelAlgApplicator(graph, this._levelAlg, false).run();

    this._levelAlg.clean();
    this._levelAlg = null;

    this.createOrthogonalLinkShapes();
  },

  createOrthogonalLinkShapes : function()
  {
    // summary:
    //		Reorganizes the graph so that the links consist completely of
    //		orthogonal segments. 
    // description:
    //		Reorganizes the graph so that the links consist completely of
    //		orthogonal segments. 
    //		It splits each segment into 3 segments, if the orthogonal bend field
    //		of the segment contains a valid split coordinate.
    // tags:
    //		private
    
    var graph = this.getGraph();

    var iter = new ibm_ilog.graphlayout.hierarchical.HSegmentBufferedIterator(graph.getAllSegments());
    var segment;
    var link;
    var dummyNode;
    var coord;
    var fromCoord;
    var toCoord;
    var edgeDir = graph.getEdgeFlow();
    var levelDir = graph.getLevelFlow();

    while (iter.hasNext())
    {
      segment = iter.next();
      coord = segment.getOrthogonalBend();

      if (coord != /*Number.MAX_VALUE*/(3.4028235E38))
      {
        fromCoord = graph.getTranslatedCoord(segment.getFrom(), segment.getFromPoint()[levelDir], levelDir);
        toCoord = graph.getTranslatedCoord(segment.getTo(), segment.getToPoint()[levelDir], levelDir);
        link = segment.getOwnerLink();
        dummyNode = link.addDummyNodeAtGraph(segment, graph);
        dummyNode.setCoord(edgeDir, coord);
        if (segment.isReversed()) {
          dummyNode.setCoord(levelDir, fromCoord);
        }
        else {
          dummyNode.setCoord(levelDir, toCoord);
        }
        dummyNode = link.addDummyNodeAtGraph(segment, graph);
        dummyNode.setCoord(edgeDir, coord);
        if (segment.isReversed()) {
          dummyNode.setCoord(levelDir, toCoord);
        }
        else {
          dummyNode.setCoord(levelDir, fromCoord);
        }
      }
    }

    this.layoutStepPerformed();
  }

}
);
