dojo.provide('ibm_ilog.graphlayout.hierarchical.makeacyclic.HMAGraph');

///////////////////////////////////////////////////////////////////////////////
// IBM ILOG Dojo Diagrammer 1.0
///////////////////////////////////////////////////////////////////////////////

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

dojo.require('ibm_ilog.graphlayout.hierarchical.graphbase.HTBaseEdgeIterator');
dojo.require('ibm_ilog.graphlayout.hierarchical.graphbase.HTBaseGraph');
dojo.require('ibm_ilog.graphlayout.hierarchical.graphbase.HTBaseNodeIterator');
dojo.require('ibm_ilog.graphlayout.hierarchical.makeacyclic.HMAEdgeIterator');
dojo.require('ibm_ilog.graphlayout.hierarchical.makeacyclic.HMANodeIterator');
dojo.require('ibm_ilog.graphlayout.internalutil.GraphModelUtil');
dojo.require('ibm_ilog.graphlayout.internalutil.TranslateUtil');

dojo.declare('ibm_ilog.graphlayout.hierarchical.makeacyclic.HMAGraph', ibm_ilog.graphlayout.hierarchical.graphbase.HTBaseGraph, 
{
  // summary:
  //		A graph for the algorithm to make the graph acyclic. 
  // description:
  //		A graph for the algorithm to make the graph acyclic. 
  //		Additionally to the topology, this graph manages temporary removed
  //		nodes and edges, that can be easily reinserted back.   
  // tags:
  //		private
  
  _firstRemovedNode: null,
  _firstRemovedEdge: null,
  _nodeEdgeDataValid: null,

  constructor : function()
  {
    this._firstRemovedNode = null;
    this._firstRemovedEdge = null;
    this._nodeEdgeDataValid = false;
  },
  
  removeNode : function(node)
  {
    // summary:
    //		Removes a node from the graph.
    // tags:
    //		private
    
    this.inherited("removeNode", arguments, [node]);
    node.insertBefore(null, this._firstRemovedNode);
    this._firstRemovedNode = node;
  },

  removeEdge : function(edge)
  {
    // summary:
    //		Removes an edge from the graph.
    // tags:
    //		private
    
    this.inherited("removeEdge", arguments, [edge]);
    edge.insertBefore(null, this._firstRemovedEdge);
    this._firstRemovedEdge = edge;
  },

  emptyTemporaryRemovedNodes : function()
  {
    // summary:
    //		Empty the temporary removed node list.
    // tags:
    //		private
    
    this._firstRemovedNode = null;
  },

  emptyTemporaryRemovedEdges : function()
  {
    // summary:
    //		Empty the temporary removed edge list.
    // tags:
    //		private
    
    this._firstRemovedEdge = null;
  },

  restoreTemporaryRemovedNodes : function()
  {
    // summary:
    //		Reinsert all temporary removed nodes.
    // tags:
    //		private
    
    var iter = new ibm_ilog.graphlayout.hierarchical.graphbase.HTBaseNodeIterator(this._firstRemovedNode);
    var node;
    this._firstRemovedNode = null;

    while (iter.hasNext())
    {
      node = iter.nextBaseNode();
      node.remove();
      this.addNode(node);
    }
  },

  restoreTemporaryRemovedEdges : function(withSelfLoops)
  {
    // summary:
    //		Reinsert all temporary removed edge. 
    // description:
    //		Reinsert all temporary removed edge. 
    //		With a flag, we can control whether we want to add the self loops as
    //		well, or to spare them.
    // tags:
    //		private
    
    var iter = new ibm_ilog.graphlayout.hierarchical.graphbase.HTBaseEdgeIterator(this._firstRemovedEdge);
    var edge;
    this._firstRemovedEdge = null;

    while (iter.hasNext())
    {
      edge = iter.nextBaseEdge();

      if (withSelfLoops || !edge.isSelfLoop())
      {
        edge.remove();
        this.addEdge(edge);
      }
      else if (this._firstRemovedEdge == null) {
        this._firstRemovedEdge = edge;
      }
    }
  },

  getNodes : function(backwards)
  {
    // summary:
    //		Returns the nodes of the graph.
    // tags:
    //		private
    
    return new ibm_ilog.graphlayout.hierarchical.makeacyclic.HMAGraphNodeIterator(this, backwards); // ibm_ilog.graphlayout.hierarchical.makeacyclic.HMANodeIterator
  },

  getEdges : function(backwards)
  {
    // summary:
    //		Returns the edges of the graph.
    // tags:
    //		private
    
    return new ibm_ilog.graphlayout.hierarchical.makeacyclic.HMAGraphEdgeIterator(this, backwards); // ibm_ilog.graphlayout.hierarchical.makeacyclic.HMAEdgeIterator
  },

  isNodeEdgeDataValid : function()
  {
    // summary:
    //		Returns true if the node priorities are valid.
    // tags:
    //		private
    
    return this._nodeEdgeDataValid; // Boolean
  },

  clearNodeEdgeData : function()
  {
    // summary:
    //		Set all priority fields and counters of unbreakable edges of nodes
    //		to 0.
    // tags:
    //		private
    
    var iter = this.getNodes(false);

    while (iter.hasNext())
      iter.next().clearEdgeData();
    this._nodeEdgeDataValid = false;
  },

  recalcNodeEdgeData : function()
  {
    // summary:
    //		Recalculates all node priorities to be the sum of priorities of
    //		incoming and outgoing edges. 
    // description:
    //		Recalculates all node priorities to be the sum of priorities of
    //		incoming and outgoing edges. 
    //		Dito recalculates the unbreakable counters.
    // tags:
    //		private
    
    this.clearNodeEdgeData();
    var iter = this.getEdges(false);
    var edge;

    while (iter.hasNext())
    {
      edge = iter.next();
      edge.getHMASource().addOutEdgeData(edge);
      edge.getHMATarget().addInEdgeData(edge);
    }

    this._nodeEdgeDataValid = true;
  }

}
);

dojo.declare('ibm_ilog.graphlayout.hierarchical.makeacyclic.HMAGraphEdgeIterator', ibm_ilog.graphlayout.hierarchical.graphbase.HTBaseEdgeIterator, 
{
  // summary:
  //		
  // tags:
  //		private
  
  constructor : function(graph, backwards)
  {
  },
  
  next : function()
  {
    return this.nextBaseEdge(); // ibm_ilog.graphlayout.hierarchical.makeacyclic.HMAEdge
  },

  prev : function()
  {
    return this.prevBaseEdge(); // ibm_ilog.graphlayout.hierarchical.makeacyclic.HMAEdge
  }

}
);

dojo.declare('ibm_ilog.graphlayout.hierarchical.makeacyclic.HMAGraphNodeIterator', ibm_ilog.graphlayout.hierarchical.graphbase.HTBaseNodeIterator, 
{
  // summary:
  //		
  // tags:
  //		private
  
  constructor : function(graph, backwards)
  {
  },
  
  next : function()
  {
    return this.nextBaseNode(); // ibm_ilog.graphlayout.hierarchical.makeacyclic.HMANode
  },

  prev : function()
  {
    return this.prevBaseNode(); // ibm_ilog.graphlayout.hierarchical.makeacyclic.HMANode
  }

}
);
