dojo.provide('ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring');

///////////////////////////////////////////////////////////////////////////////
// IBM ILOG Dojo Diagrammer 1.0
///////////////////////////////////////////////////////////////////////////////

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

dojo.require('ibm_ilog.graphlayout.hierarchical.relpositioning.HRPNode');
dojo.require('ibm_ilog.graphlayout.internalutil.GraphModelUtil');
dojo.require('ibm_ilog.graphlayout.internalutil.TranslateUtil');

dojo.declare('ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring', null, 
{
  // summary:
  //		This class is an algorithm that colors the graph in order to decide
  //		whether a grouping is possible. 
  // description:
  //		This class is an algorithm that colors the graph in order to decide
  //		whether a grouping is possible. 
  //		Initially, all nodes are white. In order to for a group, we mark all
  //		nodes of the group black. The question is whether it is possible to
  //		restructure the graph such that all black leaves are somehow
  //		neighbored.  
  //		<p>
  //		Here, we
  //		don't do the transformation. We just color in the following way:  
  //		<ul>
  //		<li>A node that has only black children is black.</li>  
  //		<li>A node that has only white children is white.</li>  
  //		<li>A node that has black and white children is graytone.</li>  
  //		<li>A node that has a gray child is graytone.</li>  
  //		<li></li>  
  //		</ul>
  //		For the graytone, we distingish three cases. Assume the orientation is top down, because then I can easier argue about "left" and "right".  
  //		<ul>
  //		<li>If the black children or forced to be left of all white children, then the parent is blackwhite.</li>  
  //		<li>If the black children or forced to be right of all white children, then the parent is whiteblack.</li>  
  //		<li>If the black children can be either left or right, then the parent is gray.</li>  
  //		</ul>
  //		A coloring is possible, if we can arrange the black nodes such that there is no white component in between. A white component means that a node is white, gray, blackwhite or whiteblack. For instance, the following orderings of children of the same node would violate the grouping:  
  //		<ul>
  //		<li> A black node neighbored by a white node that is neighbored by a black node.</li>  
  //		<li> A black node neighbored by a gray node that is neighbored by a black node.</li>  
  //		<li> A blackwhite node whose right neighbor is black or graytone.</li>  
  //		<li> A whiteblack node whose left neighbor is black or graytone.</li>  
  //		<li> Three neighbored gray nodes. 
  //		<li>  
  //		</ul>
  //		We can think of a blackwhite node in this way: its subtree can be reordered so that the left half of the subtree becomes completely black and the right half becomes completely white. Similar, we can think of a gray node in this way: its subtree can be reordered so that it has two halfs, the one is completely black and the other its completely white, but it doesn't
  //		matter which of both halfs is left and which is right. Sketch:       
  //		 G BBW B means black     BBWWW W means white      BBW BBBWWWW
  //		BBBWWWWWW Hence, two gray children can be arranged such that the
  //		grouping is possible, in this way from left to right: Start with a
  //		gray node and make the right side black, then take all black nodes,
  //		then end with a gray node and make the left side black.  
  //		<p>
  //		The task of this algorithm is:  
  //		<ul>
  //		<li> to color the nodes</li>  
  //		<li> to decide for each node, at which side a black node could be
  //		placed.</li>  
  //		</ul>
  //		The main entry point is run(V) for the group "V" of nodes to be
  //		placed close together.  
  //		</p>
  //		<p>
  //		  
  //		</p>
  // tags:
  //		private
  
  _graph: null,
  _numMarkedLeaves: null,

  constructor : function(graph)
  {
    this.init(graph);
  },
  
  init : function(graph)
  {
    // summary:
    //		Initializes the algorithm.
    // tags:
    //		private
    
    this._graph = graph;
    this._numMarkedLeaves = 0;
  },

  run : function(nodes)
  {
    // summary:
    //		Starts all coloring. 
    // description:
    //		Starts all coloring. 
    //		This is the main entry point.  
    // nodes: Array
    //		The vector of nodes of type 'HRPNode' that need
    //		to be grouped together.
    // tags:
    //		private
    
    var allOkay = true;
    if (nodes.length <= 1) {
      return allOkay; // Boolean
    }
    this._numMarkedLeaves = nodes.length;

    var e =  new ibm_ilog.graphlayout.internalutil.ArrayIterator(nodes);

    while (e.hasNext())
      (e.next()).markForColoring();

    e =  new ibm_ilog.graphlayout.internalutil.ArrayIterator(nodes);

    while (allOkay && e.hasNext())
      allOkay = this.color(e.next());

    if (!allOkay)
    {
      e =  new ibm_ilog.graphlayout.internalutil.ArrayIterator(nodes);

      while (e.hasNext())
        (e.next()).unmarkForColoring();
    }

    return allOkay; // Boolean
  },

  color : function(node)
  {
    // summary:
    //		Calculate the colors of the input node, and propagate the color to
    //		its parents whenever possible. 
    // description:
    //		Calculate the colors of the input node, and propagate the color to
    //		its parents whenever possible. 
    //		Nodes that belong to the group to be considered are marked black,
    //		nodes that
    //		don't belong to the group are marked white. Inner nodes that have subtrees which belong only partially to the group are marked gray, if they are unordered, or blackwhite if the black subtrees must be left, or whiteblack if the white subtrees must be left. Returns true if the calculation of the colors was sucessful.
    // tags:
    //		private
    
    if (node.calcFinalColor(this._numMarkedLeaves) == false) {
      return false; // Boolean
    }
    return this.propagateColor(node); // Boolean
  },

  propagateColor : function(node)
  {
    // summary:
    //		Tries to propagate the color to the parent when the input node got
    //		its final color.
    // tags:
    //		private
    
    if (this.isGroupComplete(node)) {
      return true; // Boolean
    }

    var parent = node.getParent();

    if (parent == null) {
      return true; // Boolean
    }
    parent.updateColorInfoFromChild(node);

    if (parent.isCompletelyColored())
    {
      if (node.isNeighbored())
      {
        var color = this.getNeighborChainColor(node);

        if (color == ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.NOCOLOR) {
          return false; // Boolean
        }
        parent.setColor(color);
        return this.propagateColor(parent); // Boolean
      }

      return this.color(parent); // Boolean
    }

    return true; // Boolean
  },

  getNeighborChainColor : function(node)
  {
    // summary:
    //		Returns the color of the chain of neighors if the node belongs to
    //		such a chain. 
    // description:
    //		Returns the color of the chain of neighors if the node belongs to
    //		such a chain. 
    //		This can be called after marking all nodes of a group. It returns
    //		NOCOLOR if the neighbor chain has no unique color, which basically
    //		means that the grouping we trying to achieve is impossible. It
    //		returns BLACK also if the chain is organized in a way such that all
    //		all marked nodes are already grouped together. In this case, the
    //		color doesn't matter anymore because we won't color the parent. As
    //		side effect, it sets the index of the black side at each gray node,
    //		that is, the side in the chain where we could create a node
    //		containing all the black marked children. This is not necessary for
    //		white and black nodes, because the black side
    //		doesn't matter here, and it is not necessary for blackwhite and whiteblack nodes where the black side is automatically clear when setting the color.  
    // node: ibm_ilog.graphlayout.hierarchical.relpositioning.HRPNode
    //		A node of the neighbor chain
    // tags:
    //		private
    
    node = node.getLeftmostOrExtremeNeighbor();
    var prevNode = null;

    var indexPrevNodeToNode = 0;

    var indexNodeToPrevNode = node.getNeighborIndex(null);
    var isOrdered = node.hasOrderedNeighbors();
    var sumColor = ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.NOCOLOR;
    var color;
    var sumMarkedLeaves = 0;

    while (node != null)
    {
      color = node.getColor();
      sumMarkedLeaves += node.getNumberOfMarkedLeaves();
      indexNodeToPrevNode = node.getNeighborIndex(prevNode);

      switch (sumColor)
      {
        case ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.NOCOLOR:
          sumColor = color;
          break;
        case ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.BLACK:
          switch (color)
          {
            case ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.GRAY:
              node.setIndexOfBlackSideNeighbor(indexNodeToPrevNode);
            case ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.WHITE:
              sumColor = ((isOrdered ? ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.BLACKWHITE : ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.GRAY_EXPECT_WHITE));
              break;
            case ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.BLACKWHITE:
              sumColor = ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.BLACKWHITE;
              break;
            case ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.BLACK:
              break;
            default:
              return ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.NOCOLOR; // int
          }
          break;
        case ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.WHITE:
          switch (color)
          {
            case ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.GRAY:
              node.setIndexOfBlackSideNeighbor(1 - indexNodeToPrevNode);
            case ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.BLACK:
              sumColor = ((isOrdered ? ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.WHITEBLACK : ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.GRAY_EXPECT_BLACK));
              break;
            case ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.WHITEBLACK:
              sumColor = ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.WHITEBLACK;
              break;
            case ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.WHITE:
              break;
            default:
              return ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.NOCOLOR; // int
          }
          break;
        case ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.GRAY:
          switch (color)
          {
            case ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.BLACK:
              prevNode.setIndexOfBlackSideNeighbor(indexPrevNodeToNode);
              sumColor = ((isOrdered ? ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.WHITEBLACK : ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.GRAY_EXPECT_BLACK));
              break;
            case ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.WHITE:
              prevNode.setIndexOfBlackSideNeighbor(1 - indexPrevNodeToNode);
              sumColor = ((isOrdered ? ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.BLACKWHITE : ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.GRAY_EXPECT_WHITE));
              break;
            case ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.GRAY:
            case ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.BLACKWHITE:
              if (sumMarkedLeaves == this._numMarkedLeaves)
              {
                prevNode.setIndexOfBlackSideNeighbor(indexPrevNodeToNode);
                node.setIndexOfBlackSideNeighbor(indexNodeToPrevNode);
                return ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.GRAY; // int
              }
              else {
                return ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.NOCOLOR; // int
              }
            default:
              return ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.NOCOLOR; // int
          }
          break;
        case ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.GRAY_EXPECT_BLACK:
        case ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.WHITEBLACK:
          switch (color)
          {
            case ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.BLACK:
              sumColor = ((isOrdered ? ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.WHITEBLACK : ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.GRAY_EXPECT_BLACK));
              break;
            case ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.WHITE:
            case ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.GRAY:
            case ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.BLACKWHITE:
              if (sumMarkedLeaves == this._numMarkedLeaves)
              {
                node.setIndexOfBlackSideNeighbor(indexNodeToPrevNode);
                return ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.GRAY; // int
              }
              else {
                return ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.NOCOLOR; // int
              }
            default:
              return ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.NOCOLOR; // int
          }
          break;
        case ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.GRAY_EXPECT_WHITE:
        case ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.BLACKWHITE:
          switch (color)
          {
            case ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.WHITE:
              sumColor = ((isOrdered ? ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.BLACKWHITE : ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.GRAY_EXPECT_WHITE));
              break;
            default:
              return ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.NOCOLOR; // int
          }
          break;
        default:
          return ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.NOCOLOR; // int
      }
      prevNode = node;
      indexPrevNodeToNode = 1 - indexNodeToPrevNode;
      node = node.getNeighbor(indexPrevNodeToNode);
    }

    if (sumColor == ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.GRAY_EXPECT_WHITE || sumColor == ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.GRAY_EXPECT_BLACK) {
      return ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring.GRAY; // int
    }
    return sumColor; // int
  },

  isGroupComplete : function(node)
  {
    // summary:
    //		Returns true if the group is complete at the input node. 
    // description:
    //		Returns true if the group is complete at the input node. 
    //		That means, all nodes that belong to the group to be formed are
    //		descendent of the input node.
    // tags:
    //		private
    
    return (node.getNumberOfMarkedLeaves() == this._numMarkedLeaves); // Boolean
  }

}
);

dojo.mixin(ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring,
{
  BLACK : null,
  WHITE : null,
  GRAY : null,
  BLACKWHITE : null,
  WHITEBLACK : null,
  NOCOLOR : null,

  GRAY_EXPECT_WHITE : null,
  GRAY_EXPECT_BLACK : null,
  _fields_init : function(c)
  {
    c = c || ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring;
    
    c.BLACK = ibm_ilog.graphlayout.hierarchical.relpositioning.HRPNode.BLACK;
    c.WHITE = ibm_ilog.graphlayout.hierarchical.relpositioning.HRPNode.WHITE;
    c.GRAY = ibm_ilog.graphlayout.hierarchical.relpositioning.HRPNode.GRAY;
    c.BLACKWHITE = ibm_ilog.graphlayout.hierarchical.relpositioning.HRPNode.BLACKWHITE;
    c.WHITEBLACK = ibm_ilog.graphlayout.hierarchical.relpositioning.HRPNode.WHITEBLACK;
    c.NOCOLOR = ibm_ilog.graphlayout.hierarchical.relpositioning.HRPNode.NOCOLOR;
    c.GRAY_EXPECT_WHITE = 100;
    c.GRAY_EXPECT_BLACK = 101;
  }
  
}
);

(function(){ ibm_ilog.graphlayout.hierarchical.relpositioning.HRPColoring._fields_init(); })();
