dojo.provide('ibm_ilog.diagram.interactors.PanKeyInteractor');

dojo.require('ibm_ilog.diagram.interactors.KeyboardInteractorManager');
dojo.require('ibm_ilog.diagram.interactors.Interactor');
dojo.require('ibm_ilog.diagram.editor.undo.SimpleAction');
dojo.require('ibm_ilog.diagram.editor.undo.UndoActionList');
dojo.require('ibm_ilog.diagram.util.Geometry');

dojo.declare("ibm_ilog.diagram.interactors.PanKeyInteractor", [ibm_ilog.diagram.interactors.Interactor], {
	//summary:
	//		This Interactor manages the ViewPort Pan movement through the keyboard interaction.
	//		This Interactor have 4 connections: panLeft, panRigth, panUp, panDown.
	
	_diagram: null,
	_increment: null,
	
	initialize: function (diagram) {
	// summary:
    //		initialize the new instance.
    // diagram: ibm_ilog.diagram.widget.Diagram
    //		the associated diagram.				
		this._diagram = diagram;
		this.setIncrement(10);
		return this._initialize();
	},
	setIncrement: function(increment){
		// summary: 
		//		sets the pan increment
		// increment: integer
		//		the increment to be set
		this._increment = increment;
	},
	getIncrement: function(){
		// summary: 
		//		gets the pan increment
		return this._increment;
	},
	panUp: function(e){
		// summary:
	    //		This method pans the diagram up
	    // e: Event object
	    //		the event to be treated.
		this._pan({x:0,y:-this._increment});
	},
	panDown: function(e){
		// summary:
	    //		This method pans the diagram down
	    // e: Event object
	    //		the event to be treated.
		this._pan({x:0,y:this._increment});
	},
	panRight: function(e){
		// summary:
	    //		This method pans the diagram right
	    // e: Event object
	    //		the event to be treated.
		this._pan({x:this._increment,y:0});
	},
	panLeft: function(e){
		// summary:
	    //		This method pans the diagram left
	    // e: Event object
	    //		the event to be treated.
		this._pan({x:-this._increment,y:0});
	},
	getKeyInteractorId: function(){
		// summary:
	    //		returns the Interactor Id
		return ibm_ilog.diagram.interactors.PanKeyInteractor.KeyInteractorId;
	},
	getDefaultConnections: function () {
		// summary:
		//		returns the default event mapping
		
		return {
			panLeft: {
				hotKey: dojo.keys.LEFT_ARROW,	
				connectTo: "panLeft",
				filter: this._buildInputFilter({ctrl:false,shift:true})
			}, panRight: {
				hotKey: dojo.keys.RIGHT_ARROW,	
				connectTo: "panRight",
				filter: this._buildInputFilter({ctrl:false,shift:true})
			}, panUp: {
				hotKey: dojo.keys.UP_ARROW,	
				connectTo: "panUp",
				filter: this._buildInputFilter({ctrl:false,shift:true})
			}, panDown: {
				hotKey: dojo.keys.DOWN_ARROW,	
				connectTo: "panDown",
				filter: this._buildInputFilter({ctrl:false,shift:true})
			}
		};
	},
	_pan: function(delta){
		var v = this._diagram.getViewport();
		var g = ibm_ilog.diagram.util.Geometry;
		var viewMovement = g.mulPoint(delta, -1 / v.getZoom()); // scale delta to zoom level to get delta in model coords
		var firstViewRect = v.getViewRect();
		v.setViewRect(g.moveRect(firstViewRect, viewMovement), {instant:true});
	}
});

ibm_ilog.diagram.interactors.PanKeyInteractor.KeyInteractorId = "Pan";