/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("rmps.viewlets.internal.ContextPreferenceProvider");
	
dojo.require("com.ibm.team.dashboard.web.ui.CustomPreferenceProvider");

dojo.require("rmps.app.ui.internal.dialogs.SelectContextDialog");
dojo.require("rmps.core.util.ContextUtils");

dojo.require("dojo.i18n");
dojo.requireLocalization("rmps.viewlets.internal", "ContextPreferenceProviderMessages");
var messages = dojo.i18n.getLocalization("rmps.viewlets.internal", "ContextPreferenceProviderMessages");

var PREF_CONTEXT = "contextURI";
var ContextUtils = rmps.core.util.ContextUtils;

dojo.declare("rmps.viewlets.internal.ContextPreferenceProvider", com.ibm.team.dashboard.web.ui.CustomPreferenceProvider, {
	
	_viewlet: null,
	_contextUri: null,
	_contextCache: null,

	constructor: function(viewlet) {
		this._viewlet = viewlet;
		this._contextCache = {};
		this.cancel();
	},
	
	_cacheContext: function(contextURI) {
		if (!contextURI) {
			var def = new dojo.Deferred();
			def.callback(null);
			this.contextCache = {
				uri: null,
				context: null
			};
			return def;
		}
		var def = ContextUtils.getContextDeferred(contextURI);
		def.addCallback(this, function(context) {
			this._contextCache = {
				uri: contextURI,
				context: context
			};
		});
		def.addErrback(this, function(err) {
			this.contextCache = {
				uri: contextURI,
				context: null
			};
		});
		return def;
	},

	getLabel: function(value) {
		var message;
		if (!value) {
			message = messages.noContextSelected;
		} else if (this._contextCache.uri === value) {
			if (!this._contextCache.context) {
				return "";
			}
			message = this._contextCache.context.name;
		} else {
			this._cacheContext(value);
			// Will be called again until a value is returned.
			return null;			
		}
		
		return "<input type='text' readonly='readonly' value='" + message + "'/>";
	},

	getLoadingLabel: function() {
		return messages.loading;
	},

	createChooser: function(onOk, settings, viewlet) {
		if (!settings.scopeItemId) {
			alert(messages.noProjectSelected);
			return;
		}
		var self = this;
		var dialog = new rmps.app.ui.internal.dialogs.SelectContextDialog({
			projectId: settings.scopeItemId, 
			contextSelected: function(contextURI) {
				self._contextUri = contextURI;
				self._viewlet.setPreference(PREF_CONTEXT, contextURI);
				onOk(contextURI, PREF_CONTEXT);
			}
		});
		dialog.show();
	},

	cancel: function() {
		this._contextUri = this._viewlet.getViewletState().get(PREF_CONTEXT);
	}
	
});

}());