/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.viewlets.internal.dashboard.AssociatedDomainsInfo");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.DeferredList");
dojo.require("jazz.util.i18n");
dojo.require("net.jazz.ajax.contextRoot");
dojo.require("rmps.app.ui.internal.util.DomainRegistryUtil");
dojo.require("rmps.core.RootServices");
dojo.require("rmps.app.core.util");

dojo.require("dojo.i18n");
dojo.requireLocalization("rmps.viewlets.internal.dashboard", "ProjectContextInfoMessages");

(function(){
	
var DomainRegistryUtil = rmps.app.ui.internal.util.DomainRegistryUtil;
var Util = rmps.app.core.util;
var DOMAIN_REG_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "domainRegistry");
var DOMAINS_SETTING_PAGE_ACTION = "#action=rmps.app.settings.modeling";
	
var messages;

dojo.declare("rmps.viewlets.internal.dashboard.AssociatedDomainsInfo", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("rmps.viewlets", "internal/dashboard/templates/AssociatedDomainsInfo.html"),
	
	constructor: function(args) {
		if (!messages) {
			messages = dojo.i18n.getLocalization("rmps.viewlets.internal.dashboard", "ProjectContextInfoMessages");
		}
		this.messages = messages;
	},

	postCreate: function() {
		this._titleLink.href = DOMAINS_SETTING_PAGE_ACTION;		
		
		var def1 = new dojo.Deferred();
		var def2 = new dojo.Deferred();
		DomainRegistryUtil.isProjectActivelyManaged(def1.callback);
				
		rmps.core.client.xhrGet({
			url: DOMAIN_REG_PATH + "/domains",
			handleAs: 'json',
			headers: {
				"Accept" : "application/json"
			},
			load: function(response, ioArgs) {
				var associated = [];
				if (response && response.length) {
					associated = dojo.filter(response, function(item) {
						return item.selected;
					});
				}
				def2.callback(associated);
			}
		}, true);

		var defList = new dojo.DeferredList([def1, def2]);
		defList.addCallback(this, function(results) {
			var isManaged = results[0][1];
			var domains = results[1][1];
			this._totalCount.innerHTML = dojo.string.substitute(messages.totalCount, [domains.length]);
			this._bottom.innerHTML = ""; // Remove loading message
			
			var application = dojo.getObject("jazz.app.currentApplication");
			var projectAreas = application.config.projectAreas;
			for (var j = 0; j < domains.length; ++j) {
				for (var i = 0, p; p = projectAreas[i]; ++i) {
					if (domains[j].context === p.uri) {
						domains[j].projectName = p.name;
					}
				}
			}
			this.renderDomains(domains);	
			dojo.create("div", {innerHTML: isManaged ? messages.managed : messages.notManaged}, this._bottom);
			jazz.util.i18n.addMessage(dojo.create("div",{},this._bottom), messages.toDomainsArea)[0].href = DOMAINS_SETTING_PAGE_ACTION;
		});
	},
	
	renderDomains: function(domains) {
		for (var i = 0; i < domains.length; i++) {
			var domain = domains[i];
			dojo.create("a", {
					className: "domain",
					innerHTML: jazz.util.html.escape(domain.label),
					href: net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL+ "/web/projects/" + domain.projectName + Util.getDesignViewResourceUri(domain.id)
			}, this._contentArea);
		}
	}
});
}());