/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.viewlets.internal.resource.ResourcePopularityViewlet");

dojo.require("com.ibm.team.dashboard.web.ui.Viewlet");

dojo.require("rmps.viewlets.internal.resource.TagCloudPresentation");
dojo.require("rmps.viewlets.internal.resource.ListPresentation");

dojo.require("dojo.date.stamp");
dojo.require("dojo.i18n");
dojo.require("dojo.html");

dojo.require("rmps.core.util.DateUtils");
dojo.require("rmps.core.commenting.CommentClient");
dojo.require("rmps.core.resource.ResourceInfoClient");
dojo.require("rmps.core.error");
dojo.require("rmps.core.util.ContextUtils");
dojo.require("rmps.core.util.IRmpsConstants");
dojo.require("rmps.viewlets.internal.ContextPreferenceProvider");
dojo.require("jazz.util.i18n");
dojo.require("jazz.app.context");

dojo.requireLocalization("rmps.viewlets.internal.resource", "ResourcePopularityViewletMessages");

var PREF_PROJECT = "_scopeItem";
var CommentClient = rmps.core.commenting.CommentClient;
var ResourceInfoClient = rmps.core.resource.ResourceInfoClient;
var DateUtils = rmps.core.util.DateUtils;


//var PREF_METRIC = "metric";
var PREF_SIZE = "size";
var PREF_PRESENTATION = "presentation";
var PREF_CONTEXT_URI = "contextURI";
var PREF_DATE_RANGE = "dateRange";

var PRESENTATIONS = {
	list: "list",
	tagCloud: "tagCloud"
};

var messages = dojo.i18n.getLocalization("rmps.viewlets.internal.resource", "ResourcePopularityViewletMessages");
var IRmpsConstants = rmps.core.util.IRmpsConstants;
var ContextUtils = rmps.core.util.ContextUtils;

dojo.declare("rmps.viewlets.internal.resource.ResourcePopularityViewlet", com.ibm.team.dashboard.web.ui.Viewlet, {

	templateString: '<div class="rmps-viewlets-resource-popularity"><div class="hidden"></div><div class="loading" dojoAttachPoint="_loadingDiv">${_messages.loading}</div></div>',

	// private properties
	_messages: messages,
	_loadingDiv: null,
	_contentWidget: null,

	_errorMsgNode: null,
	_totalResults: -1,
	
	_getContextURIDef: null,
	_contextURI: null,
	_contextProvider: null,
	useJazzResourceLinks: true,	

	init: function() {
	//	this._updatePrimaryTitle();
		this._updateSecondaryTitle();
		this.refreshContext();
	},

	destroy: function() {
		this._cleanup();
		this.inherited(arguments);
	},

	_cleanup: function() {
		if (this._contentWidget) {
			this._contentWidget.destroyRecursive(true);
			delete this._contentWidget;
		}
	},

	getSize: function() {
		var size = this.getPreference(PREF_SIZE);
		return size < 0 ? 0 : size;
	},

	getStartDate: function() {
		var range = this.getPreference(PREF_DATE_RANGE);
		var date = DateUtils.rangeToDate(range);
		if (date) {
			return dojo.date.stamp.toISOString(date);
		}
		return null;
	},

	refresh: function() {
		this._cleanup();
		this._totalResults = -1;
		this._setLoading(true);
		
		// Re-retrieve context
		this.refreshContext();

//		switch (this.getPreference(PREF_METRIC)) {
//			case "mostComments":
				this._doMostComments();
//				break;
//
//			default:
//				this._setLoading(false);
//				this.getSite().doneRefresh();
//		}
	},
	
	refreshContext: function() {
		this._getContextURIDef = new dojo.Deferred(); 
		var def = this._getContextUri();
		def.addCallback(this, function(contextURI) {
			this._contextURI = contextURI;
			this._getContextURIDef.callback(contextURI);
		});
		def.addErrback(this, function() {
			// Happens for example when there is no context on a newly created project
			this._contextURI = null;
			this._getContextURIDef.callback(null);
		});
	},

	_showPresentation: function(results) {
		function comparator(a, b) {
			if (a.value == b.value) {
				var aName = a.info.shortName.toLowerCase();
				var bName = b.info.shortName.toLowerCase();
				return (aName < bName) ? -1 : ((aName > bName) ? 1 : 0);
			}
			return (a.value > b.value) ? -1 : 1;
		}
		// sort results
		results.sort(comparator);

		var presentation = this.getPreference(PREF_PRESENTATION);
		if (presentation == PRESENTATIONS.tagCloud) {
			this._contentWidget = new rmps.viewlets.internal.resource.TagCloudPresentation({results: results, getProjectPrefix: dojo.hitch(this, this.getProjectPrefix), contextURI: this._contextURI});
		} else { // if (presentation == PRESENTATIONS.list) { // default
			this._contentWidget = new rmps.viewlets.internal.resource.ListPresentation(
					{
						results: results,
						getProjectPrefix: dojo.hitch(this, this.getProjectPrefix),
						contextURI: this._contextURI,
						useJazzResourceLinks: this.useJazzResourceLinks
					});
		}
		
		dojo.addClass(this._contentWidget.domNode, "hidden");
		
		this._updateSecondaryTitle();
		this.domNode.replaceChild(this._contentWidget.domNode, this.domNode.firstChild);		
		this._contentWidget.startup();
		this._setLoading(false);
		this.getSite().doneRefresh();
	},

	getProjectPrefix: function(){
		var name;		
		var context= jazz.app.context.get();
		var scope= this.getScopeItem();
		if(scope){
			name= this.getScopeItem().name;
		}
		if(!name&& context){
			if(context){
				name= context.name;
			}
		}
		if(name){
			return this._webURL+ "/web/projects/"+ encodeURIComponent(name);
		}
		return null;
	},

	_showNoResults: function() {
		var newContent = dojo.doc.createElement("div");
		newContent.className = "message";
		if (this._errorMsgNode) {
			newContent.appendChild(this._errorMsgNode);
		} else {
			newContent.appendChild(dojo.doc.createTextNode(messages.noResults));
		}

		dojo.addClass(newContent, "hidden");
		this.domNode.replaceChild(newContent, this.domNode.firstChild);
		
		this._updateSecondaryTitle();
		this._setLoading(false);
		this.getSite().doneRefresh();
	},

	settingsChanged: function(oldSettings, newSettings) {
		this._updatePrimaryTitle();
		this._updateSecondaryTitle();
		this.inherited(arguments);
	},

	_updatePrimaryTitle: function() {
//		switch (this.getPreference(PREF_METRIC)) {
//			case "mostComments":
				this.getSite().setIcon(dojo.moduleUrl("rmps.viewlets", "graphics/icons/most_comments.gif"));
				if (this.getSite().isRemote()) {
					this.getSite().setTitle(messages.mostComments + messages.isRemote);
				}
				else {
					this.getSite().setTitle(messages.mostComments);					
				}
//				break;
//		}
	},

	_updateSecondaryTitle: function() {
		this.getSite().setSecondaryTitle(this._getSecondaryTitle());
	},

	_getSecondaryTitle: function() {
		var title = '';
		if (this.getResultCount() >= 0) {
			title += '(' + this.getResultCount() + ')';
		}
		var rangeString = DateUtils.rangeToString(this.getPreference(PREF_DATE_RANGE));
		title += ' ' + (rangeString ? rangeString : '');
		return title;
	},

	getResultCount: function() {
		return this._totalResults;
	},

	_doMostComments: function() {
		this._totalResults = -1;
		var size = this.getSize();
		var projectId = this._getProjectId();
		this._errorMsgNode = null;
		if (!projectId) {
			this._totalResults = 0;
			this._noProjectSpecified();
			this._showNoResults();
			return;
		}
		
		var deferred = CommentClient.getMostCommented(projectId, size, this.getStartDate());
		this._getContextURIDef.addCallback(this, function(contextURI) {

			deferred.addCallback(this, function(resources) {
				if (resources.length == 0) {
					this._totalResults = 0;
					this._showNoResults();
					return;
				}
	
				var resourceUris = [];
				for (var key in resources) {
					resourceUris.push(key);
				}
				
				var entries = [];
				var properties = ['shortName', 'iconUri', 'longName'];
				if (this.useJazzResourceLinks) {
					var properties = ['shortName', 'iconUri'];
				}
				
				var infoDeferred = ResourceInfoClient.getResourceInfo(resourceUris, properties, this._contextURI);
	
				infoDeferred.addCallback(this, function(infos) {
					for (var i = 0; i < infos.length; ++i) {
						var obj = {
							info: infos[i],
							value : resources[infos[i].uri]
						};
						entries.push(obj);
					}
					this._totalResults = entries.length;
					this._showPresentation(entries);
				});
	
				infoDeferred.addErrback(this, this._handleError);
			});
			deferred.addErrback(this, this._handleError);
		});
		this._getContextURIDef.addErrback(deferred.errback);
		
		return deferred;
	},

	_handleError: function(deferred, error) {
		this._totalResults = -1;
		if (deferred && deferred.ioArgs && !dojo._isDocumentOk(deferred.ioArgs.xhr)) {
			error = rmps.core.error.createXhrError(deferred.ioArgs).getGenericError();
		} else if (error) {
			error = rmps.core.error.createError(error).getGenericError();
		} else if (deferred){
			error = rmps.core.error.createError(deferred).getGenericError();
		}
		this._setLoading(false);
		this.getSite().doneRefresh();
		this.getSite().error(error);
	},

	_getProjectId: function() {
		// Check the preference first
		var id = this._site.context.getPreferences()[PREF_PROJECT];
		// If no preference then try context
		return id ? id : jazz.app.context.get().itemId;
	},

	_getProjectUri: function() {
		var id = this._getProjectId();
		if (!id) {
			return null;
		}
		return PROJECT_PATH_AREA_PATH + "/" + id;
	},

	_getContextUri: function() {
		var def = new dojo.Deferred();
		var contextURI = this.getPreference(PREF_CONTEXT_URI);
		if (contextURI) {
			def.callback(contextURI);
		} else {
			var projectId = this._getProjectId();
			// We have to check if space has been configured for the selected project and 
			// if so, then we will try to detect a proper context.
			dojo.when(ContextUtils.getProjectSpace(projectId),
					  dojo.hitch(this, function(spaceUri) {
						  if (spaceUri) {
							  ContextUtils.findCurrentContextUriAndExecute(def.callback, projectId, def.errback);
						  } else {
							  def.callback(spaceUri);
						  }
					  })
			);	
		}

		return def;
	},

	_noProjectSpecified: function() {
		if (!this._noProjectMsg) {
			// this part always shows
			this._noProjectMsg = document.createElement("span");
			this._noProjectMsg.appendChild(document.createTextNode(messages.noProjects));

			// this part only shows when dashboard in edit mode
			this._noProjectExtraMsg = document.createElement("span");
			var links = jazz.util.i18n.addMessage(this._noProjectExtraMsg, messages.editSettings);
			this.connect(links[0], "onclick", dojo.hitch(this, function(e) {
				e.preventDefault();
				this.getSite().toggleSettings();
			}));
			this._noProjectExtraMsg.className = "editModeText";
			this._noProjectMsg.appendChild(this._noProjectExtraMsg);
			if (!this.getSite().isDashboardEditing()) {
				this._noProjectExtraMsg.style.display = "none";
			}
		}
		this._errorMsgNode = this._noProjectMsg;
		this._updateSecondaryTitle();
	},
	
	_setLoading: function(loading) {
		this.getSite().setLoading(loading);
		if (loading) {
			dojo.addClass(this.domNode.firstChild, "hidden");			
			dojo.removeClass(this._loadingDiv, "hidden");
		} else {
			dojo.addClass(this._loadingDiv, "hidden");
			dojo.removeClass(this.domNode.firstChild, "hidden");
		}
	},
	
	getPreferenceProvider: function(id) {
		if (id === PREF_CONTEXT_URI) {
			if (!this._contextProvider) {
				this._contextProvider = new rmps.viewlets.internal.ContextPreferenceProvider(this);
			}
			return this._contextProvider;
		}
	},
	
	settingsCanceled: function() {
		if (this._contextProvider) {
			this._contextProvider.cancel();
		}
	}
	
});

}());