/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function(){
    dojo.provide("rmpw.text.internal.widgets.RichTextWidget");
    
    dojo.require("dijit.form._FormWidget");
    dojo.require("dijit.layout._LayoutWidget");
    dojo.require("dojo.cache");
    
    dojo.require("rmpw.text.internal.widgets.CKEditorFactory");
    
	dojo.require("rmpw.text.internal.util.Converter");
    dojo.declare("rmpw.text.internal.widgets.RichTextWidget", [dijit.form._FormValueWidget, dijit.layout._LayoutWidget], {
    
        templateString: dojo.cache("rmpw.text", "internal/widgets/templates/RichTextWidget.html"),
        _editorReady: false,
        onEditorReadyDeferred: null,
        _deferredLayout: false,
        disabled: false,
		_localValue: null,
		_holdValue: null,
		_editingReferenceValue: false,
		editorType: 'RICH_TEXT_SLIM_EDITOR',
		maxHeight: null,
		
		// Needed to enable/disable cut & paste menus when the editor is read-write/read-only
		_cutMenu: null,
		_pasteMenu: null,
        constructor: function(){
            this.onEditorReadyDeferred = new dojo.Deferred();
            this.inherited(arguments);
        },
        
        postMixInProperties: function(){
            // summary:
            //		Called after the parameters to the widget have been read-in,
            //		but before the widget template is instantiated. Especially
            //		useful to set properties that are referenced in the widget
            //		template.
            // tags:
            //		protected
            this.inherited(arguments);
        },
        
        postCreate: function(){
            // summary:
            //		Processing after the DOM fragment is created
            // description:
            //		Called after the DOM fragment has been created, but not necessarily
            //		added to the document.  Do not include any operations which rely on
            //		node dimensions or placement.
            // tags:
            //		protected
            this.inherited(arguments);
        },
        
        startup: function(){
            this.inherited(arguments);
            //NGB Inserted the setTimeout call on the actual CKEditor startup due to rendering
            //issues on generated forms if this delay isn't introduced.  Must be some issues
            //relating to DOM insertion that caused the CKEditor not to get inserted into the DOM
            //without this delay.
            setTimeout(dojo.hitch(this, this._startup), 100);
        },
        _startup: function(){
			var type = rmpw.text.internal.widgets.CKEditorFactory.EditorTypes[this.editorType];
			if (!type) {
				type = rmpw.text.internal.widgets.CKEditorFactory.EditorTypes.RICH_TEXT_SLIM_EDITOR;
			}
			
			/*
			 * While in development, there was an intermittant problem of the document
			 * editor suddenly expanding infinitely.  Traced this to a strange order in the
			 * eventing model.  maxHeight is to prevent an sudden infinite expansion of the
			 * rich text editor.
			 */  
			if(!this.maxHeight){
				var maxHeightCss = this.domNode.style.maxHeight;
				this.maxHeight = maxHeightCss ? maxHeightCss : "2560"; 
			}
			
            rmpw.text.internal.widgets.CKEditorFactory.create(this.editor, type, dojo.hitch(this, function(ckeditor){
                this.ckeditor = ckeditor;
                this.ckeditor.domNode = this.domNode;
                this.ckeditor.on("instanceReady", this._instanceReadyCallback, this);
            }), false);
        },

        _instanceReadyCallback: function(event){
            this.ckeditor.removeListener("instanceReady", this._instanceReadyCallback);
            this._editorReady = true;
            if (this.onEditorReadyDeferred) {
                this.onEditorReadyDeferred.callback(this);
            }

            if (dojo.isIE) {
            	// Need a timeout with IE
				setTimeout(this.ckeditor.resetDirty, 10);
			} else {
				this.ckeditor.resetDirty();				
			}
            this._addDirtyStateListeners();
        },
        _addDirtyStateListeners: function(){
            // summary:
            //		Add listeners to key press and certain other ckeditor events to allow
            //		propagation of dirty state to the _FormWidget API via an onChange() call.
            this.ckeditor.on("key", this._checkDirtyAfterTimeout, this);
            this.ckeditor.on("afterCommandExec", this._checkDirtyAfterTimeout, this);
			this.ckeditor.on('insertHtml', this._checkDirtyAfterTimeout, this);
			this.ckeditor.on('insertElement', this._checkDirtyAfterTimeout, this);
			this.ckeditor.on('dialogHide', this._checkDirtyAfterTimeout, this);
			this.ckeditor.on('selectionChange', this._checkDirtyAfterTimeout, this);
			this.ckeditor.on('paste', this._checkDirtyAfterTimeout, this);
        },
        _removeDirtyStateListeners: function(){
            // summary:
            //		Remove listeners to key press and certain other ckeditor events used for
            //		dirty state propagation.
            this.ckeditor.removeListener("key", this._checkDirtyAfterTimeout);
            this.ckeditor.removeListener("afterCommandExec", this._checkDirtyAfterTimeout);
			this.ckeditor.removeListener('insertHtml', this._checkDirtyAfterTimeout);
			this.ckeditor.removeListener('insertElement', this._checkDirtyAfterTimeout);
			this.ckeditor.removeListener('dialogHide', this._checkDirtyAfterTimeout);
			this.ckeditor.removeListener('selectionChange', this._checkDirtyAfterTimeout);
			this.ckeditor.removeListener('paste', this._checkDirtyAfterTimeout);
        },
        uninitialize: function(){
            this.inherited(arguments);
            if (this.ckeditor && this._editorReady) {
                this._removeDirtyStateListeners();
                this.ckeditor.destroy(true);
                this._editorReady = false;
            }
			return true;
        },
		
		resize: function(changeSize, resultSize) {
            if (!this._editorReady) {
                if (this.onEditorReadyDeferred) {
                    this.onEditorReadyDeferred.addCallback(this, function(){
                        this.resize(changeSize, resultSize);
                    });
                }
                return;
            }
			                        
            // Calculate the height
            var myDim = dojo.position(this.containerNode, true); // Get dimensions of myself    		
			var bodyHeight = document.body.offsetHeight;
            var height = bodyHeight - myDim.y - 52;	// 52px because it looks good, no method to the madness.
    		
    		// Max height might not be set, need to account for that when limiting the height.
    		if(!this.maxHeight || (this.maxHeight && height < this.maxHeight)){
    			this.containerNode.style.height = height + "px";
    		}
    		
			this.inherited(arguments);
		},
        
        layout: function(){
            if (!this._editorReady) {
                if (this.onEditorReadyDeferred) {
                    this.onEditorReadyDeferred.addCallback(this, function(){
                        this.layout();
                    });
                }
                return;
            }
			
            var cb = this._contentBox || dojo.contentBox(this.containerNode);
            if (cb.h > 0) {
                this.ckeditor.resize(cb.w, cb.h);
            }
        },
        
        _toggleDisabled: function(){
            this.set('disabled', !this.disabled);
        },
        
        _getReadOnlyAttr: function(){
            return this.get('disabled');
        },
        
        _setReadOnlyAttr: function(value){
            return this.set('disabled', value);
        },
        
        _getValueAttr: function(){
            if (this._editorReady) {
                return this._editingReferenceValue ? this._localValue : this.ckeditor.getData();
            }
            return null;
        },
        
        _setValueAttr: function(value){
            if (!this._editorReady) {
            	this._holdValue = value;
                if (this.onEditorReadyDeferred) {
                    this.onEditorReadyDeferred.addCallback(this, function(){
                        this._setValueAttr(this._holdValue);
                    });
                }
                return;
            }
            
			if (this._editingReferenceValue) {
				this._localValue = value;
				return;
			}

            this.ckeditor.setData(value, function(){
            	if (dojo.isIE){
            		// Need a timeout with IE as the callback is called before the data is ready
            		setTimeout(this.resetDirty, 10);
            	} else {
            		this.resetDirty();
            	}
            });
        },
        
		// The following two methods (as well as the _localValue and _editingReferenceValue fields) were added to support
		// reusing the same rich text editor widget in generated properties pages in DTK where the property is typed as
		// and XHTML body.  In this case the properties infrastructure uses get/setReferenceValue as well as get/setValue
		// to support reference handling.  This code could probably be revisited and tightened up.
        _getReferenceValueAttr: function(){
            if (this._editorReady) {
				var xhtmlData = this.ckeditor.getData();
                var jsonData = rmpw.text.internal.util.Converter.parseToJson(xhtmlData);
				return jsonData;
            }
            return null;
        },
        
        _setReferenceValueAttr: function(value){
			this._editingReferenceValue = true;
            if (!this._editorReady) {
                if (this.onEditorReadyDeferred) {
                    this.onEditorReadyDeferred.addCallback(this, function(){
                        this._setReferenceValueAttr(value);
                    });
                }
                return;
            }
            
			var xhtmlValue = rmpw.text.internal.util.Converter.parse(value);
            this.ckeditor.setData(xhtmlValue);
            this.ckeditor.resetDirty();
        },
        
        // We cannot use ckeditor.setReadOnly to make the editor read only, 
        // because after that we would disable also the open resource feature
        _setDisabledAttr: function(value){
            if (!this._editorReady) {
                if (this.onEditorReadyDeferred) {
                    this.onEditorReadyDeferred.addCallback(this, function(){
                        this._setDisabledAttr(value);
                    });
                }
                return;
            }
            this.disabled = !!value;
            // Turn off contentEditable.
            this.ckeditor.document.$.body.disabled = this.disabled;
            if (dojo.isIE || dojo.isFF || dojo.isWebKit || dojo.isOpera) {
                this.ckeditor.document.$.body.contentEditable = !this.disabled;
            }
            else {
                this.ckeditor.document.$.designMode = this.disabled ? "off" : "on";
            }
            
        	// when in read only mode, we need to remove the cut and paste menus as they are still enable,
            // even if the corresponding commands are disabled
            if (this.disabled){
            	// --Keep the menu objects so we can restore them when/if we go in read-write mode
            	var cutMenu = this.ckeditor.getMenuItem('cut');
            	if (cutMenu)
            		this._cutMenu = cutMenu;
            	var pasteMenu = this.ckeditor.getMenuItem('paste');
            	if (pasteMenu)
            		this._pasteMenu = pasteMenu;
            	// remove the menus from the context menu
            	this.ckeditor.removeMenuItem('cut');
            	this.ckeditor.removeMenuItem('paste');
            } else {
            	// if we are transitioning from a read-only mode we need to add back the cut and paste menus
            	if (this._cutMenu){
            		this.ckeditor.addMenuItem('cut', {
						label : this._cutMenu.label,
						command : this._cutMenu.command,
						group : 'clipboard',
						order : this._cutMenu.order
					});
            		delete this._cutMenu;
            	}
            	if (this._pasteMenu){
            		this.ckeditor.addMenuItem('paste', 	{
            			label : this._pasteMenu.label,
            			command : this._pasteMenu.command,
            			group : 'clipboard',
            			order : this._pasteMenu.order
					});
            		delete this._pasteMenu;
            	}
            }

            
            // Prevent key handling and paste commands.
            this.ckeditor[this.disabled ? 'on' : 'removeListener']('key', this._cancelCKEvent, this, null, 0);
            this.ckeditor[this.disabled ? 'on' : 'removeListener']('selectionChange', this._cancelCKEvent, this, null, 0);
            this.ckeditor[this.disabled ? 'on' : 'removeListener']('paste', this._cancelCKEvent, this, null, 0);
            this.ckeditor[this.disabled ? 'on' : 'removeListener']('pasteDialog', this._cancelCKEvent, this, null, 0);
            
            // Disable all commands in wysiwyg mode.
            var cmd, cmds = this.ckeditor._.commands, mode = this.ckeditor.mode;
            for (var name in cmds) {
        		cmd = cmds[name];
        		// we want to keep the openresource context menus in disabled mode
            	if (name === "openresource"){
        			cmd[cmd.modes[mode] ? 'enable' : 'disable']();
            	} else {
            		if (this.disabled) {
            			cmd.disable();
                		cmd.on('state', this._cancelCKEvent, this, null, 0);
           		}
            		else {
                		cmd.removeListener('state', this._cancelCKEvent, this, null, 0);
            			cmd[cmd.modes[mode] ? 'enable' : 'disable']();
            		}
            	}
            }
                                                
            // we need to hide or show the toolbar...
            var toolbar = dojo.query(".cke_top", this.domNode)[0];
            if (this.disabled) {
                this._toolbarDisplayStyle = dojo.style(toolbar, 'display');
				if (this._toolbarDisplayStyle === 'none') {
					//work around starting in disabled mode then switching to enabled mode.
					this._toolbarDisplayStyle = 'table-cell';
				}
            }
            dojo.style(toolbar, 'display', this.disabled ? 'none' : this._toolbarDisplayStyle);
        },
        _toolbarDisplayStyle: '',
        
        _cancelCKEvent: function(evt){
            evt.cancel();
        },
        
        _checkDirty: function(){
            if (this._editorReady) {
                if (this.ckeditor.checkDirty()) {
                    this.onChange();
                    this.ckeditor.resetDirty();
                }
            }
        },
        
        _checkDirtyAfterTimeout: function(){
            window.setTimeout(dojo.hitch(this, this._checkDirty), 100);
        }
    });
})();
