dojo.provide('ibm_ilog.graphlayout.internalutil.GrowableArray');

///////////////////////////////////////////////////////////////////////////////
// IBM ILOG Dojo Diagrammer 1.0
///////////////////////////////////////////////////////////////////////////////

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

dojo.require('ibm_ilog.graphlayout.internalutil.GraphModelUtil');
dojo.require('ibm_ilog.graphlayout.internalutil.TranslateUtil');
dojo.require('ibm_ilog.graphlayout.internalutil.LogResUtil');
dojo.require('ibm_ilog.graphlayout.GraphLayout');

dojo.declare('ibm_ilog.graphlayout.internalutil.GrowableArray', null, 
{
  // summary:
  //		The 'GrowableArray' class implements a growable array  of
  //		objects. 
  // description:
  //		The <code>GrowableArray</code> class implements a growable array  of
  //		objects. 
  //		Like an array, it contains components that can be  accessed using an
  //		integer index. However, the size of an <code>GrowableArray</code> can
  //		grow as needed to accommodate  adding and items after the
  //		<code>GrowableArray</code> has  been created.  
  //		<p>
  //		This class is different than java.util.Vector in the following ways: 
  //		
  //		<ul>
  //		<li>It is not synchronized</li>  
  //		<li>It never shrinks</li>  
  //		<li>It allows you to set an element at a given index, even if the
  //		previous elements are (partially) not yet set (or will never be set).
  //		In other words, it can have "holes".</li>  
  //		</ul>  
  //		</p>
  
  _elementData: null,
  _capacityIncrement: null,

  constructor : function(size, capacityIncrement)
  {
    this._elementData = ibm_ilog.graphlayout.internalutil.TranslateUtil.newArray(size, null);
    this._capacityIncrement = capacityIncrement;
  },
  
  ensureCapacity : function(minCapacity)
  {
    // summary:
    //		Increases the capacity of this array, if necessary, to ensure  that
    //		it can hold at least the number of components specified by  the
    //		minimum capacity argument. 
    // description:
    //		Increases the capacity of this array, if necessary, to ensure  that
    //		it can hold at least the number of components specified by  the
    //		minimum capacity argument. 
    //		<p>
    //		If the current capacity of this  vector is less than
    //		<tt>minCapacity</tt>, then its capacity is increased by replacing
    //		its internal data array, kept in the field  <tt>elementData</tt>,
    //		with a larger one. The size of the new data  array will be the old
    //		size plus <tt>capacityIncrement</tt>, unless  the value of
    //		<tt>capacityIncrement</tt> is nonpositive, in which  case the new
    //		capacity will be twice the old capacity; but if this new size is
    //		still smaller than <tt>minCapacity</tt>, then the  new capacity will
    //		be <tt>minCapacity</tt>.   
    //		</p>
    // minCapacity: int
    //		the desired minimum capacity.
    
    var oldCapacity = this._elementData.length;

    if (minCapacity > oldCapacity)
    {
      var oldData = this._elementData;
      var newCapacity = ((this._capacityIncrement > 0) ? (oldCapacity + this._capacityIncrement) : (oldCapacity * 2));
      if (newCapacity < minCapacity) {
        newCapacity = minCapacity;
      }
      this._elementData = ibm_ilog.graphlayout.internalutil.TranslateUtil.newArray(newCapacity, null);
      ibm_ilog.graphlayout.internalutil.TranslateUtil.arrayCopy(oldData, 0, this._elementData, 0, oldData.length);
    }
  },

  setElement : function(obj, index)
  {
    if (index > this._elementData.length - 1)
    {
      var params = [index, (this._elementData.length - 1) ];
      ibm_ilog.graphlayout.internalutil.LogResUtil.logAndThrowIllegalArgExc(ibm_ilog.graphlayout.GraphLayout, "graphlayout.expert.message.6081E", params);
    }

    this._elementData[index] = obj;
  },

  getElement : function(index)
  {
    if (index > this._elementData.length - 1)
    {
      var params = [index, (this._elementData.length - 1) ];
      ibm_ilog.graphlayout.internalutil.LogResUtil.logAndThrowIllegalArgExc(ibm_ilog.graphlayout.GraphLayout, "graphlayout.expert.message.6081E", params);
    }

    return this._elementData[index]; // Object
  },

  length : function()
  {
    return this._elementData.length; // int
  }

}
);
