dojo.provide('ibm_ilog.graphlayout.longlink.LLTerminationPoint');

///////////////////////////////////////////////////////////////////////////////
// IBM ILOG Dojo Diagrammer 1.0
///////////////////////////////////////////////////////////////////////////////

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

dojo.require('ibm_ilog.graphlayout.internalutil.GraphModelUtil');
dojo.require('ibm_ilog.graphlayout.internalutil.TranslateUtil');
dojo.require('ibm_ilog.graphlayout.internalutil.QuickSort');
dojo.require('ibm_ilog.graphlayout.Direction');

dojo.declare('ibm_ilog.graphlayout.longlink.LLTerminationPoint', null, 
{
  // summary:
  //		An auxiliary data structure to specify a point on the grid where a
  //		link can terminate, i.e. 
  // description:
  //		An auxiliary data structure to specify a point on the grid where a
  //		link can terminate, i.e. 
  //		the start or end point of a link. For both "from" and "to" node of a
  //		link, we calculate the set of possible termination points.  The
  //		termination points have a penalty and an internal order, indicating
  //		how "good" they are for this link (lower penalty == better for the
  //		link). The link starting at a termination point first runs along one
  //		segment. However, since the termination points must have a specified
  //		distance from the corresponding end node (specified by
  //		setMinStartSegmentLength or setMinEndSegmentLength), there is an area
  //		on this segment where the link cannot join the segment. This is to
  //		avoid 180 degree bends, such as WRONG:                           
  //		CORRECT: |                                                     |
  //		,-------.     |     term. point   ,-------.           term. point   |
  //		|       |-----+=========*         |       |---------------*---------+
  //		`-------'                         `-------' ^                        
  //		 ^              ^         ^ |                             \          
  //		/          | join point                        forbidden area     
  //		join point  
  // tags:
  //		private
  
  _index: null,

  _penalty: null,

  _internalOrder: null,

  _segment: null,

  _forbiddenMinIndex: null,
  _forbiddenMaxIndex: null,

  _side: null,

  constructor : function (a0, a1, a2, a3, a4, a5, a6, a7, a8)
  {
    if(arguments.length == 0)
      this._constructor_overload_ibm_ilog_graphlayout_longlink_LLTerminationPoint_0();
    else if(arguments.length == 9)
      this._constructor_overload_ibm_ilog_graphlayout_longlink_LLTerminationPoint_9(a0, a1, a2, a3, a4, a5, a6, a7, a8);
  },
  
  _constructor_overload_ibm_ilog_graphlayout_longlink_LLTerminationPoint_0 : function()
  {
    this._index = ibm_ilog.graphlayout.internalutil.TranslateUtil.newArray(2, 0);
  },
  
  _constructor_overload_ibm_ilog_graphlayout_longlink_LLTerminationPoint_9 : function(index0, index1, forbiddenMinIndex, forbiddenMaxIndex, dir0, penalty, order, side, segment)
  {
    this._index = ibm_ilog.graphlayout.internalutil.TranslateUtil.newArray(2, 0);
    this._index[dir0] = index0;
    this._index[1 - dir0] = index1;
    this._penalty = penalty;
    this._internalOrder = order;
    this._side = side;
    this._segment = segment;

    if (forbiddenMinIndex <= forbiddenMaxIndex)
    {
      this._forbiddenMinIndex = forbiddenMinIndex;
      this._forbiddenMaxIndex = forbiddenMaxIndex;
    }
    else
    {
      this._forbiddenMinIndex = forbiddenMaxIndex;
      this._forbiddenMaxIndex = forbiddenMinIndex;
    }
  },
  
  dispose : function()
  {
    // summary:
    //		The destructor.
    // tags:
    //		private
    
    this._index = null;
    this._segment = null;
  },

  copyData : function(p, offsetX, offsetY)
  {
    // summary:
    //		Copy all internal data from the input termination point. 
    // description:
    //		Copy all internal data from the input termination point. 
    //		The offsets allow to correct the grid coordinates.
    // tags:
    //		private
    
    this._index[0] = p._index[0] + offsetX;
    this._index[1] = p._index[1] + offsetY;
    this._penalty = p._penalty;
    this._internalOrder = p._internalOrder;
    this._side = p._side;
    this._segment = p._segment;
    this._forbiddenMinIndex = p._forbiddenMinIndex;
    this._forbiddenMaxIndex = p._forbiddenMaxIndex;
  },

  getIndex : function(dir)
  {
    // summary:
    //		Returns the grid index of the termination point in given direction.
    // tags:
    //		private
    
    return (this._index[dir]); // int
  },

  setIndex : function(dir, value)
  {
    // summary:
    //		Sets the grid index of the termination point in given direction.
    // tags:
    //		private
    
    this._index[dir] = value;
  },

  getPenalty : function()
  {
    // summary:
    //		Returns the penalty of the termination point. 
    // description:
    //		Returns the penalty of the termination point. 
    //		If we have several candidates where a link can start or end, we want
    //		to prefer the one with the lowest penalty.
    // tags:
    //		private
    
    return (this._penalty); // int
  },

  getOrder : function()
  {
    // summary:
    //		Returns the internal order value of the termination point. 
    // description:
    //		Returns the internal order value of the termination point. 
    //		Same purpose as the penalty, but lower priority. The penalty can be
    //		influenced by the user, but the internal order cannot. It is only
    //		used if many termination points have the same penalty.
    // tags:
    //		private
    
    return (this._internalOrder); // int
  },

  getSide : function()
  {
    // summary:
    //		Returns the side where this termination point is placed at the node.
    //		
    // description:
    //		Returns the side where this termination point is placed at the node.
    //		
    //		Valid values are Left, Top, Right, Bottom from Direction.
    // tags:
    //		private
    
    return (this._side); // int
  },

  getDeviation : function(grid, rect)
  {
    // summary:
    //		Returns the percentual deviation of the termination point from the
    //		center of the border of the input rectangle, relative to the size of
    //		the border. 
    // description:
    //		Returns the percentual deviation of the termination point from the
    //		center of the border of the input rectangle, relative to the size of
    //		the border. 
    //		For instance, if the termination point is placed at the middle of
    //		the top border, then the deviation is 0. If it is placed at the
    //		leftmost or rightmost point of the top border, the deviation is 100
    //		percent.
    // tags:
    //		private
    
    if (this._segment.isHorizontal()) {
      return (ibm_ilog.graphlayout.longlink.LLTerminationPoint.getDeviation(grid, rect, this.getIndex(1), 0)); // Number
    }
    return (ibm_ilog.graphlayout.longlink.LLTerminationPoint.getDeviation(grid, rect, this.getIndex(0), 1)); // Number
  },

  getSegment : function()
  {
    // summary:
    //		Returns the corresponding grid segment that has this termination
    //		point.
    // tags:
    //		private
    
    if (this._segment != null && this._segment.getGridLine() == null)
    {
      this._segment = null;
    }

    return (this._segment); // ibm_ilog.graphlayout.longlink.LLGridSegment
  },

  getForbiddenMinIndex : function()
  {
    // summary:
    //		Returns the lowest grid index on the segment that is forbidden to
    //		traverse in order to respect the minimal start/end segment length.
    // tags:
    //		private
    
    return (this._forbiddenMinIndex); // int
  },

  getForbiddenMaxIndex : function()
  {
    // summary:
    //		Returns the highest grid index on the segment that is forbidden to
    //		traverse in order to respect the minimal start/end segment length.
    // tags:
    //		private
    
    return (this._forbiddenMaxIndex); // int
  },

  sortAccordingToPenalty : function(array)
  {
    // summary:
    //		Sorts the array of termination points according to increasing
    //		penalty.  
    // tags:
    //		private
    
    if (array.length > 1)
    {
      var sortAlg = new ibm_ilog.graphlayout.longlink.LLTerminationPoint.TerminationPointSortAlg(array);
      sortAlg.sort(array.length);
      sortAlg.dispose();
    }
  },

  sortAccordingToSide : function(array, sidePref)
  {
    // summary:
    //		Sorts the array of termination points according to side preferences.
    //		
    // description:
    //		Sorts the array of termination points according to side preferences.
    //		
    //		sidePref is an array of four, and the sides have the following
    //		indexe in the array: top = 0, left = 1, bottom = 2, right = 3.  
    // tags:
    //		private
    
    if (array.length > 1)
    {
      var sortAlg = new ibm_ilog.graphlayout.longlink.LLTerminationPoint.TerminationPointSideSortAlg(array, sidePref);
      sortAlg.sort(array.length);
      sortAlg.dispose();
    }
  }

}
);

dojo.mixin(ibm_ilog.graphlayout.longlink.LLTerminationPoint,
{

  getDeviation : function(grid, rect, index, dir)
  {
    var deviation;

    if (dir == 0)
    {
      if (rect.height == 0.0) {
        return (0.0); // Number
      }
      deviation = grid.getGridCoord(1, index) - rect.y - 0.5 * rect.height;
      if (deviation < 0.0) {
        deviation = -deviation;
      }
      return (200.0 * deviation / rect.height); // Number
    }
    else
    {
      if (rect.width == 0.0) {
        return (0.0); // Number
      }
      deviation = grid.getGridCoord(0, index) - rect.x - 0.5 * rect.width;
      if (deviation < 0.0) {
        deviation = -deviation;
      }
      return (200.0 * deviation / rect.width); // Number
    }
  }

}
);

dojo.declare('ibm_ilog.graphlayout.longlink.LLTerminationPoint.TerminationPointSortAlg', ibm_ilog.graphlayout.internalutil.QuickSort, 
{
  _array: null,

  constructor : function(array)
  {
    this._array = array;
  },
  
  dispose : function()
  {
  },

  compare : function(loc1, loc2)
  {
    if (this._array[loc1] == this._array[loc2]) {
      return 0; // int
    }
    if (this._array[loc1] == null) {
      return (1); // int
    }
    if (this._array[loc2] == null) {
      return (-1); // int
    }
    if (this._array[loc1].getPenalty() == this._array[loc2].getPenalty()) {
      return (this._array[loc1].getOrder() - this._array[loc2].getOrder()); // int
    }
    return (this._array[loc1].getPenalty() - this._array[loc2].getPenalty()); // int
  },

  swap : function(loc1, loc2)
  {
    var h = this._array[loc1];
    this._array[loc1] = this._array[loc2];
    this._array[loc2] = h;
  }

}
);
dojo.declare('ibm_ilog.graphlayout.longlink.LLTerminationPoint.TerminationPointSideSortAlg', ibm_ilog.graphlayout.internalutil.QuickSort, 
{
  _sidePref: null,
  _array: null,

  constructor : function(array, sidePref)
  {
    this._array = array;
    this._sidePref = sidePref;
  },
  
  dispose : function()
  {
    this._sidePref = null;
  },

  compare : function(loc1, loc2)
  {
    if (this._array[loc1] == this._array[loc2]) {
      return 0; // int
    }
    if (this._array[loc1] == null) {
      return (1); // int
    }
    if (this._array[loc2] == null) {
      return (-1); // int
    }
    var sidePref1 = 0;
    var sidePref2 = 0;

    switch (this._array[loc1].getSide())
    {
      case ibm_ilog.graphlayout.Direction.TOP:
        sidePref1 = this._sidePref[0];
        break;
      case ibm_ilog.graphlayout.Direction.LEFT:
        sidePref1 = this._sidePref[1];
        break;
      case ibm_ilog.graphlayout.Direction.BOTTOM:
        sidePref1 = this._sidePref[2];
        break;
      case ibm_ilog.graphlayout.Direction.RIGHT:
        sidePref1 = this._sidePref[3];
        break;
    }
    switch (this._array[loc2].getSide())
    {
      case ibm_ilog.graphlayout.Direction.TOP:
        sidePref2 = this._sidePref[0];
        break;
      case ibm_ilog.graphlayout.Direction.LEFT:
        sidePref2 = this._sidePref[1];
        break;
      case ibm_ilog.graphlayout.Direction.BOTTOM:
        sidePref2 = this._sidePref[2];
        break;
      case ibm_ilog.graphlayout.Direction.RIGHT:
        sidePref2 = this._sidePref[3];
        break;
    }
    if (sidePref1 != sidePref2) {
      return (sidePref2 - sidePref1); // int
    }
    if (this._array[loc1].getPenalty() != this._array[loc2].getPenalty()) {
      return (this._array[loc1].getPenalty() - this._array[loc2].getPenalty()); // int
    }
    return (this._array[loc1].getOrder() - this._array[loc2].getOrder()); // int
  },

  swap : function(loc1, loc2)
  {
    var h = this._array[loc1];
    this._array[loc1] = this._array[loc2];
    this._array[loc2] = h;
  }

}
);
