dojo.provide('ibm_ilog.diagram.overview.OverviewPanInteractor');

dojo.require('ibm_ilog.diagram.overview.OverviewInteractor');
dojo.require('ibm_ilog.diagram.interactors.DragInteractor');
dojo.require('ibm_ilog.diagram.util.Geometry');
dojo.require('ibm_ilog.diagram.util.ErrorReporter');
dojo.require('ibm_ilog.diagram.util.GraphUtil');

(function() {
	var R = ibm_ilog.diagram.util.ErrorReporter;
	var g = ibm_ilog.diagram.util.Geometry,	 Rect = g.Rect, EmptyRect = g.EmptyRect, Size = g.Size, PointZero = g.PointZero, Point = g.Point;
	var gu = ibm_ilog.diagram.util.GraphUtil;
	
dojo.declare("ibm_ilog.diagram.overview.OverviewPanInteractor",[ibm_ilog.diagram.interactors.DragInteractor], {
	//
	//	summary:
	//		An OverviewPanInteractor pans the viewport when the user drags on 
	//		the overview and centers the viewport on a point when the user clicks at 
	//		that point.	//

	_dragCursor: "move",


	//
	//	_constrain: Boolean
	//		Constrains view rectangle movements to the viewport's content area
	//
	_constrain: true,
	
	setConstrain: function( /*boolean*/ b ) {
		this._constrain = b;
	},
	
	getConstrain: function() {
		return this._constrain;
	},
	
	_initialize: function() {
		this.inherited(arguments);
		this._cursorStyle = gu.getSurface(this._overview._output).rawNode.style;
	},
	
	_declareStates: function () {
		this._declareState("idle", ["fit","start","mouseenter","mouseleave"]);
		this._declareState("active", ["first", "move", "end", "cancelOnDragStart", "cancelOnSelectStart","mouseenter","mouseleave"]);
	},
	
	getDefaultConnections: function() {
		var cs = this.inherited(arguments);
		cs.fit = {
				src: this._getInitialEventSource(),
				srcEvt: "onmousedown",
				connectTo: "_fitContents",
				filter: this._buildInputFilter({shift:true,button:0}),
				gfxConnect:true
			};
		cs.mouseenter = {
				src: this._getInitialEventSource(),
				srcEvt: "onmouseenter",
				connectTo: "_hover",
				gfxConnect:true
			};
		cs.mouseleave = {
				src: this._getInitialEventSource(),
				srcEvt: "onmouseleave",
				connectTo: "_leave",
				gfxConnect:true
			};
		return cs;
	},
	
	_updateCursor: function() {
		if(this._hovered || this._state._id=="active") {
			this._showDragCursor();
		} else {
			this._hideDragCursor();
		}
	},
	
	_hover: function() {
		this._hovered = true;
		this._updateCursor();
	},
	
	_leave: function() {
		this._hovered = false;
		this._updateCursor();
	},

	_getInitialEventSource: function() {
		return this._overview.getEventSource(); 
	},
	
	_fitContents: function(e) {
		this._viewport.setViewRect(this._viewport.getContentRect());
	},

	_dragPreStart: function(e) {
		this._cursorStyle = this._cursorStyle || gu.getSurface(this._overview._output);
		var v = this._viewport;
		this.inherited(arguments);
		this._firstViewRect = v.getViewRect();
	},
    
	_dragMove: function(e) {
    this.inherited(arguments);

		// scale delta to zoom level to get delta in model coords
		var panDelta = g.mulPoint(this._totalDelta,1/this._overview.getZoom());
		
		var newViewRect = this._constrainedViewRect(g.moveRect(this._firstViewRect,panDelta));
		
		this._viewport.setViewRect( newViewRect, {instant:true} );
  },
	
	_dragEnd: function(e) {
		//
		//	summary:
		//		Terminates the current panning process.
		//
		var v = this._viewport;
		var dragged = this.hasDragged; // inherited will clear this;
		this.inherited(arguments); // clears hadDragged
		if(!dragged) {
			
			var newViewRect = this._constrainedViewRect(g.rectCenteredAt(this._viewport.getViewRect(), this.eventViewportLocation(e)));

			this._viewport.setViewRect( newViewRect );
		}
	},

  _constrainedViewRect: function( viewRect ) {
	  if(this._constrain) {
			viewRect = g.constrainRect(viewRect,g.addRect(this._viewport.getContentRect(),this._viewport.getViewRect()));
		}
	  return viewRect;
  }

});

// [AV] moved this superclass to a dojo.extend because it does not seem to work properly on dojo 1.3.2 
dojo.extend(ibm_ilog.diagram.overview.OverviewPanInteractor,ibm_ilog.diagram.overview.OverviewInteractorMixin);

})();