dojo.provide('ibm_ilog.diagram.overview.OverviewTemplateRenderer');

dojo.require('ibm_ilog.diagram.overview.OverviewRenderer');

dojo.require('ibm_ilog.diagram.util.GraphUtil');
dojo.require('ibm_ilog.diagram.templating');
dojo.require("ibm_ilog.diagram.util.Assert");
dojo.require('ibm_ilog.diagram.util.Geometry');
dojo.require('ibm_ilog.diagram.util.ErrorReporter');


(function() {
	var R = ibm_ilog.diagram.util.ErrorReporter;
	var g = ibm_ilog.diagram.util.Geometry, Rect = g.Rect, EmptyRect = g.EmptyRect, Size = g.Size, PointZero = g.PointZero, Point = g.Point;
	var CHECK = ibm_ilog.diagram.util.Assert.check, assert = ibm_ilog.diagram.util.Assert.assert;
	var gu = ibm_ilog.diagram.util.GraphUtil;

dojo.declare('ibm_ilog.diagram.overview.OverviewTemplateRenderer',ibm_ilog.diagram.overview.OverviewRenderer, {

	//
	//	summary:
	//		Uses custom templates to generate the overview image.
	//
	//	description:
	//		Iterates the given graph and reinstantiates the elements in the graph 
	//		with new templates. If no templates are specified, default ones are used.
	//	
	
	//
	//	node:  Function
	//		The node template function for the overview.
	//
	node: function(node){
		return node._template;
	},
	
	//
	//	link:  Function
	//		The link template function for the overview.
	//
	link: function(link){
		return link._template;
	},

	//
	//	subgraph:  Function
	//		The subgraph template function for the overview.
	//
	subgraph: function(subgraph){
		return subgraph._template;
	},

	//
	//	stripTexts: Boolean
	//		Optional removal of the texts from the generated shapes.
	//
	stripTexts: true,
	
	//
	//	stripBitmaps: Boolean
	//		Optional removal of the bitmaps from the generated shapes
	//
	stripBitmaps: true,

	constructor: function( kwArgs ) {
		if(kwArgs) {
			dojo.mixin(this,kwArgs);
		}
		
		this._toFunction("node");
		this._toFunction("link");
		this._toFunction("subgraph");

	},
	
	_toFunction: function(name) {
		if(typeof this[name] != "function") {
			var value = this[name];
			this[name] = function(){
				return value;
			};
		}
	},

	_cloneGroup: function(parent,g) {
		var ng = parent.createGroup();
		if(g.getTransform){
			var t = g.getTransform();
			if(t) {
				ng.setTransform(t);
			}
		}
		return ng;
	},

	_stripShapes: function(childrenArray){
		var remove = [];
		dojo.forEach(childrenArray,function(child){
			if(!child.TID) {
				if (child.shape && ((this.stripTexts && child.shape.type=="text") || (this.stripBitmaps && child.shape.type=="image") )) {
					remove.push(child);
				} else {
					if (child.children) {
						this._stripShapes(child.children);
					}
				}
			}
		},this);
		dojo.forEach(remove,function(child){
			child.getParent().remove(child);
		},this);
	},

	
	render: function( graph, /*dojox.gfx.Group*/ output ) {
		
		var sub = function(ps,ns,g,b) {
			if(g.isCollapsed()) {
				ns.clear(); // TODO: cut iteration earlier instead of removing result of collapsed subgraphs
			}
			var subrep=ns.createGroup();
			subrep = g.shallowInstantiateAt(subrep,this.subgraph(g));
            g.layoutShape(subrep,true);
			subrep.moveToBack();
      if(this.stripTexts || this.stripBitmaps) {
        this._stripShapes(subrep.children);
    }
//
//	Draft of future version of this method:
//
//			ps.remove(ns);
//			var subrep=this.cloneGroup(ps,g);
//			subrep = g.shallowInstantiateAt(subrep,this.subgraph(g));
//			subrep._graph.add(ns);
//	        g.layoutShape(subrep);
//			subrep._graph.moveToFront();
		};

		var instantiator = function(templateF) {
			return function(ps,g){
				var t = templateF(g);
				var r = g.shallowInstantiateAt(this._cloneGroup(ps,g),t);
	            g.layoutShape(r,true);
	      if(this.stripTexts || this.stripBitmaps) {
	          this._stripShapes(r.children);
	      }
				return r;
			};
		};

		gu.fold(
			graph,
			output, // state
			gu.folders({
				sub:   sub,
				node:  instantiator(this.node),
				link:  instantiator(this.link),
				cBase: null,
				accGroup: this._cloneGroup,
				accGraph: this._cloneGroup,
				accSubgraph: this._cloneGroup
			}),
			this
		);

	}
});

})();
