
dojo.provide('ibm_ilog.diagram.overview.OverviewTopLevelTemplateRenderer');

dojo.require('ibm_ilog.diagram.overview.OverviewRenderer');

dojo.require('ibm_ilog.diagram.util.GraphUtil');
dojo.require('ibm_ilog.diagram.templating');
dojo.require("ibm_ilog.diagram.util.Assert");
dojo.require('ibm_ilog.diagram.util.Geometry');
dojo.require('ibm_ilog.diagram.util.ErrorReporter');
dojo.require('ibm_ilog.diagram.templates.templates');


(function() {
	var R = ibm_ilog.diagram.util.ErrorReporter;
	var g = ibm_ilog.diagram.util.Geometry, Rect = g.Rect, EmptyRect = g.EmptyRect, Size = g.Size, PointZero = g.PointZero, Point = g.Point;
	var CHECK = ibm_ilog.diagram.util.Assert.check, assert = ibm_ilog.diagram.util.Assert.assert;
	var gu = ibm_ilog.diagram.util.GraphUtil;
	var m = dojox.gfx.matrix;
dojo.declare('ibm_ilog.diagram.overview.OverviewTopLevelTemplateRenderer',ibm_ilog.diagram.overview.OverviewRenderer, {

	//
	//	summary:
	//		Uses custom templates to generate the overview image.
	//
	//	description:
	//		Iterates the given graph and reinstantiates the elements in the graph 
	//		with new templates. If no templates are specified, default ones are used.
	//	
	
	//
	//	node:  Function
	//		The node template function for the overview.
	//
	node: ibm_ilog.diagram.templates.defaultOverviewNodeTemplate,
	
	//
	//	link:  Function
	//		The link template function for the overview.
	//
	link: ibm_ilog.diagram.templates.defaultOverviewLinkTemplate,

	//
	//	expandedSubgraph:  Function
	//		The expanded subgraph template function for the overview.
	//
	expandedSubgraph: ibm_ilog.diagram.templates.defaultOverviewExpandedSubgraphTemplate,

	//
	//	collapsedSubgraph:  Function
	//		The collapsed subgraph template function for the overview.
	//
	collapsedSubgraph: ibm_ilog.diagram.templates.defaultOverviewCollapsedSubgraphTemplate,

	//
	//	stripTexts: Boolean
	//		Optional removal of the texts from the generated shapes.
	//
	stripTexts: true,

	constructor: function( kwArgs ) {
		if(kwArgs) {
			dojo.mixin(this,kwArgs);
		}
		
		this._toFunction("node");
		this._toFunction("link");
		this._toFunction("expandedSubgraph");
		this._toFunction("collapsedSubgraph");

	},
	
	_toFunction: function(name) {
		if(typeof this[name] != "function") {
			var value = this[name];
			this[name] = function(){
				return value;
			};
		}
	},

	templateFor: function(ge) {
		return gu.dispatchClass(ge,arguments,this,{
			N:this.node,
			S:function(ge) {
				return ge.isCollapsed()?this.collapsedSubgraph():this.expandedSubgraph();
			},
      L:this.link
		});
	},
	
	renderGraphElement: function(ge) {
		
		var template = this.templateFor(ge);

		var ng = this._output.createGroup();
		
		if(ge.getTransform){
			var t = ge.getTransform();
			if(t) {
				ng.setTransform(t);
			}
		}

		var r = ge.shallowInstantiateAt(ng,template);
		
		if(!ge._isIBMDiagramLink){
			var oldBB = r.getLocalBoundingBox();
			var newBB = ge.getLocalBoundingBox();
		
			if(dojo.config.useGfxLayout) {
				r.layout(newBB);
			} else {
				r.applyLeftTransform( m.createFromRectangles( oldBB, newBB ));
			}
		} else {
    	ge.layoutShape(r);
		}
	},

	render: function( graph, /*dojox.gfx.Group*/ output ) {
		
		graph = graph || this.graph;
		
		this._output = output;
		
		dojo.forEach(graph.children,this.renderGraphElement,this);

	}
});

})();