dojo.provide('ibm_ilog.diagram.overview.OverviewZoomInteractor');

dojo.require('ibm_ilog.diagram.overview.OverviewInteractor');
dojo.require('ibm_ilog.diagram.util.ErrorReporter');
dojo.require('ibm_ilog.diagram.util.Geometry');

(function () {
	var R = ibm_ilog.diagram.util.ErrorReporter;
	var g = ibm_ilog.diagram.util.Geometry, Rect = g.Rect, EmptyRect = g.EmptyRect, Size = g.Size, PointZero = g.PointZero, Point = g.Point;

dojo.declare("ibm_ilog.diagram.overview.OverviewZoomInteractor", [ibm_ilog.diagram.overview.OverviewInteractor], {
	//
	//	summary:
	//		A ZoomInteractor controls the process of viewport panning and zooming
	//
	//	_incrementFactor: Number
	//		The factor at which the zoom is incremented at each discrete step
	//
	_incrementFactor: 1.5,



	setIncrementFactor: function ( /*Number*/ factor) {
		//
		//	summary:
		//		Sets the increment factor for zooming discrete operations
		//
		this._incrementFactor = factor;
	},

	getIncrementFactor: function () {
		//
		//	summary:
		//		Returns the increment factor for zooming discrete operations
		//
		return this._incrementFactor;
	},

	_declareStates: function () {
		this._declareState("idle", ["wheelZoom"]);
	},

	getDefaultConnections: function () {
		//
		//	summary:
		//		Returns the default event mapping
		//
		return {
			wheelZoom: {
				src: this._overview.getEventSource(),
				srcEvt: this.eventNameWheel,
				connectTo: "_wheelZoom",
				filter: this._noneFilter,
				gfxConnect: true
			}
		};
	},

	_wheelZoom: function ( /*Event*/ e) {
		//
		//	summary:
		//		Handles the wheel mouse event and dispatchs the appropriate zooming operations
		//
		var v = this._incrementFactor;

		if (this.eventWheelAmmount(e) < 0) {
			v = 1 / v;
		}

		this._viewport.setZoom(this._viewport.getZoom() * v);

		if (e.preventDefault) {
			e.preventDefault();
		}

		//	TODO: do we need this?
		e.returnValue = false;
	}

});


})();