/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function(){
dojo.provide("rmps.app.pickers.internal.CLMResourcePicker");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.layout._LayoutWidget");
dojo.require("dijit.layout.ContentPane");
dojo.require("dojox.layout.TableContainer");

dojo.require("jazz.core.loader");
dojo.require("jazz.ui.RemoteChooserResponder");
dojo.require("jazz.app.auth");

dojo.require("rmps.core.error");

dojo.require("rmps.core.projects.ProjectClient");
dojo.require("rmps.core.resource.ResourceInfoClient");
dojo.require("rmps.core.streams.StreamClient");

dojo.require("rmps.core.util.ILinksConstants");
dojo.require("rmps.core.util.TypeConstants");

dojo.require("rmpw.dsv.editor.form.ui.SelectCreatePicker");

dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants");

var ProjectClient= rmps.core.projects.ProjectClient;
var ResourceInfoClient= rmps.core.resource.ResourceInfoClient;
var StreamClient= rmps.core.streams.StreamClient;

var ILinksConstants = rmps.core.util.ILinksConstants;
var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
var TypeConstants= rmps.core.util.TypeConstants;

var QuickSearchSelection= "rmpw.dsv.editor.form.ui.QuickSearchSelection";
var ResourcesSelection= "rmpw.dsv.editor.form.ui.ResourcesSelection";
var ClipboardWidget= "rmps.ui.clipboard.ClipboardWidget";
var RMPSPicker= "rmps.app.pickers.internal.RMPSPicker";

var DEFAULT_OSLC_CONTEXT_PROPERTY = "defaultOslcContext";

/**
 * An OSLC style delegated selection dialog for selecting DM resource.
 * The picker can either be initialized via parameters in the window's URL (i.e., in a delegated UI scenario),
 * or directly via parameters passed in to the constructor.
 * In the direct instantiation case, results are returned by a provided function;
 * otherwise, the picker returns results using the OSLC Window Name or Post Message protocols.
 */
dojo.declare("rmps.app.pickers.internal.CLMResourcePicker", [dijit._Widget, dijit._Templated], {
	"class" : "rmps-app-clm-resource-picker",
		
	templateString: '<div class="rmps-app-clm-resource-picker">' +
	                    '<div class="content" data-dojo-type="dijit.layout._LayoutWidget" data-dojo-attach-point="containerNode"></div>' +
	                '</div>',

	projectId : null,
	context: null,
	changeset: null,
	typeUri : null,
	propertyUri: null,
	propertyDomainUri: null,
	documentUri: null,
	resourceUri: null,

	pickerMessages: null,

	_url: null,
	_remoteFrame: null,
	_isInternal: false,

	constructor: function(args) {
		this._url = new dojo._Url(location.href);
		var queryString = this._url.query;
		var qParams = dojo.queryToObject(queryString);

		if (qParams.project) {
			qParams.projectId = qParams.project;
			delete qParams.project;
		}

		if (qParams.diagram) {
			qParams.onlyDiagram = qParams.diagram;
			delete qParams.diagram;
		}

		if (qParams.onlyType) {
			qParams.typeUri = decodeURIComponent(qParams.onlyType);
			delete qParams.onlyType;
		}

		dojo.mixin(this, qParams);
		dojo.mixin(this, args);

		if (!this.typeUri) {
			this.typeUri = TypeConstants.DOCUMENT; 
		}

		if (!this.propertyUri) {
			this.propertyUri = ILinksConstants.DMLinkType.NAMESPACE + ProcessConstants.LinkTypes.ELABORATED_BY;
		}

		if (!this.propertyDomainUri) {
			this.propertyDomainUri = TypeConstants.DOCUMENT;
		}

		if (!this.resourceUri) {
			this.resourceUri = TypeConstants.DOCUMENT; 
		}

		if (!this.documentUri) {
			this.documentUri = this.resourceUri;
		}
	},

	postCreate: function() {
		dojo.requireLocalization("rmps.app.pickers.internal", "CLMResourcePickerMessages");
		this.pickerMessages = dojo.i18n.getLocalization("rmps.app.pickers.internal", "CLMResourcePickerMessages");

		if (this.context) {
			this._isInternal = true;
			this._createPicker();
		} else {
			// External call means the default OSLC context has to be used.
			// If no default OSCL context has been set, then default stream of the target project will be used.
			this._handleExternalCall();
		}
	},

	_handleExternalCall: function() {
		/* 
		 * Ensure we're actually authenticated.  As this is an external call, the user may not yet be authenticated.
		 */
		jazz.app.auth.loginIfNecessary();
		
		dojo.when(ProjectClient.getProjectProperty(this.projectId, DEFAULT_OSLC_CONTEXT_PROPERTY),
			dojo.hitch(this, function(oslcContextURI) {
				this.context = oslcContextURI && oslcContextURI.length > 0 ? oslcContextURI : null;
				if (this.context === null) {
					// Default OSLC context is not set, let's proceed with default stream of target project
					this._proceedWithDefaultStream(true);
				} else {
					this._createPicker();
				}
			}),
			dojo.hitch(this, function(error) {
				if (error.status === 404) {
					// Default OSLC context is not set, let's proceed with default stream of target project
					this._proceedWithDefaultStream(true);
				}
				// FIXME: Should we show error message? 
			})
		);
	},

	_proceedWithDefaultStream: function(showWarning) {
/* FIXME: show warning
		if (showWarning) {
		}
*/
		dojo.when(StreamClient.getDefaultStream(this.projectId),
				  dojo.hitch(this, function(contextURI) {
					  this.context = contextURI && contextURI.length > 0 ? contextURI : null;
					  this._createPicker();
				  })
		);
	},

	_createPicker: function() {
		this._picker = new rmpw.dsv.editor.form.ui.SelectCreatePicker({
			creationArgs:		dojo.hitch(this, function(callback) { /* Do nothing */ }),
			selectionsLabel:	this.pickerMessages.selectionsLabel,
			creationsLabel:		this.pickerMessages.creationsLabel,
			typeLabel:			this._typeLabel,
			selectionArgs:		this._getDialogSelectionsDescriptors()
		});

		this.containerNode.appendChild(this._picker.domNode);

		// We do not allow resource creation, so disable radio buttons
		this._picker._topArea.style.display = "none";
		this._picker._horizontalRule.style.display = "none";
	},

	_getDialogSelectionsDescriptors: function() {
		var descriptors = [ ];

		if (!this.onlyDiagram) {
			descriptors.push(this._getQuickSearchDialogDescriptor());
		}

		descriptors.push(this._getSearchDialogDescriptor());
		descriptors.push(this._getClipboardDialogDescriptor());

		return descriptors;
	},

	_getQuickSearchDialogDescriptor: function() {
		return {
			classConstructor: this.typeUri === TypeConstants.RESOURCE ? ResourcesSelection : QuickSearchSelection,
			args: {
				context: this.context,
				changeset: this.changeset,
				typeUri: this.typeUri,
				propertyUri: this.propertyUri,
				propertyDomainUri: this.propertyDomainUri,
				documentUri: this.documentUri,
				resourceUri: this.resourceUri,
				internalOslcRequest: this._isInternal,
				onOK: dojo.hitch(this, function(item) {
					this._populateResultResponse([{
						"rdf:resource": item.rdf_about,
						"oslc:label": item.rdfs_label || "",
						"oslc_rm:label": item.rdfs_label || ""
					}]);
				}),
				onCancel: dojo.hitch(this, function() {
					this._populateResultResponse();
				})
			},
			title: this.pickerMessages.quickSearch
		};
	},
	
	_getSearchDialogDescriptor: function() {
		return {
			classConstructor: RMPSPicker,
			args: {
				projectId: this.projectId,
				context: this.context,
				changeset: this.changeset,
				includeSuppliers: this._isInternal,
				onlyType: this.typeUri,
				onlyDiagram: this.onlyDiagram,
				onOK: dojo.hitch(this, function(selections) {
					var items = [];
					for (var index = 0; index < selections.length; index++) {
						var selection = selections[index];
						items.push({
							"rdf:resource": selection.link,
							"oslc:label": selection.shortName || "",
							"oslc_rm:label": selection.shortName || ""
						});
					}

					this._populateResultResponse(items);
				}),
				onCancel: dojo.hitch(this, function(selections) {
					this._populateResultResponse();
				})
			},
			title: this.pickerMessages.search
		};
	},

	_getClipboardDialogDescriptor: function() {
		return {
			classConstructor: ClipboardWidget,
			args: {
				projectUri: this._url.scheme + "://" + this._url.authority + net.jazz.ajax.contextRoot() + "/process/project-areas/" + this.projectId,
				baseline: this.context,
				onlyType: this.typeUri,
				readOnly: false,
				onOkButton: dojo.hitch(this, function(selections) {
					dojo.when(ResourceInfoClient.getResourceInfo(selections, ["uri","shortName"]),
							dojo.hitch(this, function(info) {
								var items = [];
								for (var index = 0; index < info.length; index++) {
									var selection = info[index];
									items.push({
										"rdf:resource": selection.uri,
										"oslc:label": selection.shortName || "",
										"oslc_rm:label": selection.shortName || ""
									});
								}
								this._populateResultResponse(items);
							}),
							dojo.hitch(this, function(error) {
								this._populateResultResponse();
							})
					);
				}),
				onClickCancelButton: dojo.hitch(this, function() {
					this._populateResultResponse();
				})
			},
			title: this.pickerMessages.clipboard
		};
	},

	_getRemoteFrameProtocol: function() {
		if (this._remoteFrame === null) {
			this._remoteFrame= new jazz.ui.RemoteChooserResponder();
		}
		return this._remoteFrame;
	},

	_populateResultResponse: function(selectedItems) {
		var response = {};

		response["oslc:message"] = "oslc.select";
		if (selectedItems && selectedItems.length > 0) {
			response["oslc:results"] = selectedItems;
			response["oslc_rm:message"] = response["oslc:message"];
			response["oslc_rm:results"] = response["oslc:results"];
		} else {
			response["oslc:results"] = "";
		}

		this._getRemoteFrameProtocol().sendResponse(dojo.toJson(response));
	},

	destroy: function() {
		if (this._picker) {
			this._picker.destroy();
			delete this._picker;
		}
		this.inherited(arguments);
	}
});
}());