/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.app.pickers.internal.MultiplePickerDialog");

dojo.require("rmps.ui.common.TabContainer");
dojo.require("jazz.app.context");
dojo.require("jazz.ui.Dialog");
dojo.require("rmps.app.pickers.internal.RMPSPicker");
dojo.require("rmps.ui.clipboard.ClipboardWidget");

dojo.requireLocalization("rmps.app.pickers", "MultiplePickerDialogMessages");

(function(){

dojo.declare("rmps.app.pickers.internal.MultiplePickerDialog", jazz.ui.Dialog, {
	messages: dojo.i18n.getLocalization("rmps.app.pickers", "MultiplePickerDialogMessages"),

	// Also see properties of jazz.ui.Dialog
	
	// mixin properties
	// Function to be called when Ok button is clicked. List of selected resource URIs are returned.
	onOkButton: null,
	// Baseline to be used in the pickers
	baseline: null,
	// Filtered list of elements for pickers which support this
	filteredList: null,
	// Type URI if the picker should be restricted to a specific type
	onlyType: null,
	// If onlyType is set, use onlyTypeLabel to set a specific label name to the type
	onlyTypeLabel: null,
	// Widget that should be added to an additional tab
	customWidget: null,
	
	readOnly: true,
	width: "1000px",
	
	_tabContainer: null,
	_clipboardWidget: null,
	_clipboardSubscribeHandle: null,
	
	postCreate: function() {
		if (!this.primaryTitle || this.primaryTitle.length === 0) {
			this.primaryTitle = this.messages.primaryTitle;
		}
		
		if (!this.filteredList) {
			this.filteredList = [];
		}
		
		var tc = this._tabContainer = new rmps.ui.common.TabContainer({
	        doLayout: false
	    });
	
		if (this.customWidget) {
			if(dojo.isArray(this.customWidget)){
				for(var i = 0, item; item = this.customWidget[i]; i++) {
					tc.addChild(item);	
				}
			}else{
				tc.addChild(this.customWidget);
			}
		}
		
		var projectId = jazz.app.context.get().itemId;
		var baseline = this.baseline || jazz.app.context.get().baseline;
		this._searchPickerWidget = new rmps.app.pickers.internal.RMPSPicker({
			title: this.messages.searchTabTitle,
			projectId: projectId,
			context: baseline,
			populateOSLCResponse: false,
			filteredList: this.filteredList,
			onClose: dojo.hitch(this, this._onSearchClose),
			onlyType: this.onlyType,
			onlyTypeLabel: this.onlyTypeLabel,
			findFragments: this.findFragments
		});
		tc.addChild(this._searchPickerWidget);
		this._clipboardWidget = new rmps.ui.clipboard.ClipboardWidget({
			title: this.messages.clipboardTabTitle,
			projectId: projectId,
			disabledList: this.filteredList,
			baseline: baseline,
			onOkButton: dojo.hitch(this, this._onOk),
			onCancelButton: dojo.hitch(this, this._onCancel),
			readOnly: false,
			onlyType: this.onlyType
		});
		tc.addChild(this._clipboardWidget);

		this._clipboardSubscribeHandle = dojo.subscribe("rmps.core.clipboard.Change", dojo.hitch(this._clipboardWidget, this._clipboardWidget.refresh));

		this.contentNode = tc.domNode;
		this.onOpen = dojo.hitch(this, this.open);
		
		this.inherited(arguments);
	},
	
	open: function() {
		this._tabContainer.startup();
		this._tabContainer.resize();
	},
	
	destroy: function() {
		if (this._tabContainer) {
			this._tabContainer.destroy();
			delete this._tabContainer;
		}

		if (this._clipboardWidget) {
			this._clipboardWidget.destroy();
			delete this._clipboardWidget;
		}
		
		if (this._searchPickerWidget) {
			this._searchPickerWidget.destroy();
			delete this._searchPickerWidget;
		}
		
		if (this._clipboardSubscribeHandle) {
			dojo.unsubscribe(this._clipboardSubscribeHandle);
			this._clipboardSubscribeHandle = null;
		}

		if (this.customWidget) {
			if(dojo.isArray(this.customWidget)){
				for(var i = 0, item; item = this.customWidget[i]; i++) {
					item.destroyRecursive();
				}
			}else{
				this.customWidget.destroyRecursive();
			}
			delete this.customWidget;
		}
				
		this.inherited(arguments);
	},
	
	_onOk: function(result) {
		if (this.onOkButton) {
			this.onOkButton(result);
		}
		this._onCancel();
	},
	
	_onCancel: function() {
		// close will call destroy as well
		this.close();
	},
	
	_onSearchClose: function(results) {
		if (!results) {
			this._onCancel();
			return;
		}
		this._onOk(this.mapResults(results));
	},
	
	/*Can be extended by subclasses to change the map returning object */
	mapResults: function(results){
		return dojo.map(results, function(result) {
			return result.link;
		});
	}
});

}());