/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.pickers.internal.RMPSPicker");
dojo.require("dojox.html");
dojo.require("rmps.app.pickers.internal.PickerController");
dojo.require("rmps.app.ui.internal.search.SearchResults");
dojo.require("rmps.app.ui.internal.search.QuickSearch");
dojo.require("dojo.string");
dojo.require("dijit.form.CheckBox");
dojo.require("dijit.form.RadioButton");
dojo.require("dijit._Widget");
dojo.require("rmps.ui.ErrorReportController");
dojo.require("jazz.app.context");
dojo.require("rmps.app.ui.internal.context.ContextSelection");
dojo.require("rmps.app.ui.internal.search.QuickSearch");
dojo.require("rmps.core.util.ContextUtils");

dojo.require("dijit._Templated");

dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("jazz.ui.RemoteChooserResponder");
dojo.require("rmps.core.changesets.ChangesetClient");

var ContextUtils= rmps.core.util.ContextUtils;
var ChangesetClient = rmps.core.changesets.ChangesetClient;
var DEFAULT_OSLC_CONTEXT_PROPERTY = "defaultOslcContext";
var ROW_CONFIG_PROP = "_tableWidgetRowConfig";

var FindInput= function(node) {
    var result = null;
    for (var index=0; index<node.childNodes.length; index++) {
        var current= node.childNodes[index];
        var tagName = current.tagName;
        if (tagName && current.tagName.toLowerCase() === "input") {
            return current;
        } else if (result) {
            break;
        } else {
            result = FindInput(current);
        }
    }
    return result;
};

/*
 * For now, we would like only to support RadioButton. We have some commented
 * code for the Picker to support nulti-selection; we could uncomment it when
 * needed
 */
dojo.declare("rmps.app.pickers.internal.RMPSPicker", [dijit._Widget, dijit._Templated], {

	"class": "root",

	messages: dojo.i18n.getLocalization("rmps.app.pickers", "RMPSPickerMessages"),
	templateString: dojo.cache("rmps.app", "pickers/internal/templates/RMPSPickerTemplate.html"),

	// private properties
	_controller: null,
	_searchResults: null,
	_quickSearch: null,
	_remoteFrame: null,
	_pickerArguments: null,
	_selectedRows: [],
	
	projectId: null,
	_selectedContext: null,
	filteredList: null,
	// restrict the picker to a certain type
	onlyType: null,
	onlyTypeLabel: null,
	// restrict the picker to diagrams
	onlyDiagram: null,

	// All external applications should not use current context
	context: null, // required parameter
	changeset: null,
	includeSuppliers: null,

	/*Set it as true here */
	populateOSLCResponse: true,

	constructor: function(params){
        if(params){
            var searchString = location.search.replace(/^\?/,'');
            this._pickerArguments = dojo.queryToObject(searchString);
            this.projectId = this._pickerArguments.project;

            // to restrict the picker to only search diagrams
            this.onlyDiagram = this._pickerArguments.diagram;

            // to restrict the picker to a given type
            this.onlyType = (this._pickerArguments.onlyType)?decodeURIComponent(this._pickerArguments.onlyType):null;

            /* Use context passed in the uri or use baseline passed as a parameter
               if null/undefined then context is either default OSLC context or default stream */
            this.context = this._pickerArguments.context ? this._pickerArguments.context : null;
            this.changeset= this._pickerArguments.changeset ? this._pickerArguments.changeset : params.changeset;
            this.findFragments = params.findFragments;
        }
        this.filteredList= [];
    },

	buildRendering: function() {
		this.inherited(arguments);
		this.containerNode = this.domNode;
	},

	postCreate: function() {
		this._controller = new rmps.app.pickers.internal.PickerController(this);
		this._quickSearch = new rmps.app.ui.internal.QuickSearch_({
			projectId  : this.projectId,
			baseline: this.context,
			changeset: this.changeset,
			includeSuppliers: this.includeSuppliers,
			onlyType: this.onlyType,
			onlyTypeLabel: this.onlyTypeLabel,
			onlyDiagram: this.onlyDiagram,
			_controller: this._controller,
			resourcesOnly: !this.onlyType && !this.findFragments
		});

		if (this._selectedContext.childNodes.length === 0) {
			if (this.changeset) {
				dojo.when(ChangesetClient.getChangeset(this.changeset), dojo.hitch(this, this._updateSelectedContext));
			} else {
				dojo.when(ContextUtils.getContextDeferred(this.context, this.projectId), dojo.hitch(this, this._updateSelectedContext));
			}
		}

		setTimeout(dojo.hitch(this, function() {
			var inputNode = FindInput(this._quickSearch.domNode);
			inputNode.focus();		
		}), 1);
		this.quickSearch.appendChild(this._quickSearch.domNode);
		
		var errorReport = new rmps.ui.ErrorReportController();
		errorReport.startup();
		this.errorReportNode.appendChild(errorReport.domNode);
	},

	_updateSelectedContext: function(contextVar) {
		this._selectedContext.appendChild(dojo.doc.createTextNode(contextVar.name));
	},

	_populateResultResponse: function(resultArray, returnMessage) {
		var newItems= [];
		var item, name, newItem;
		
		if (resultArray) {
			for (var i=0; i<resultArray.length; i++) {
				item = resultArray[i];
				name = item.shortName || "";
				newItem= { "rdf:resource": item.link, "oslc_rm:label" : name, "oslc:label" : name};
				newItems.push(newItem);
			}
		}
		var response= {
			"oslc_rm:message" : returnMessage,
			"oslc_rm:results" : newItems,
			"oslc:message" : returnMessage,
			"oslc:results" : newItems
		};

		this._getRemoteFrameProtocol().sendResponse(dojo.toJson(response));
	},

	_getRemoteFrameProtocol: function(){
		if (this._remoteFrame === null) {
			this._remoteFrame= new jazz.ui.RemoteChooserResponder();
		}
		return this._remoteFrame;
	},

	onClickOKButton: function(e) {
		if (dojo.isFunction(this.onOk)) {
			this.onOK(this._selectedRows);
			return;
		}

		this.onClose(this._selectedRows);
		if(this.populateOSLCResponse){
			this._populateResultResponse(this._selectedRows,"oslc.select");			
		}
	},
	
	onClose: function(selections){
		/*To be implemented by other clients */
	},

	onClickCancelButton: function(e) {
		if (dojo.isFunction(this.onCancel)) {
			this.onCancel(this._selectedRows);
			return;
		}

		if(this.populateOSLCResponse){
			var emptyResponse= {"oslc:message" : "oslc.select", "oslc:results" : ""};
			this._populateResultResponse(emptyResponse);			
		}
		this.onClose();
	},

	_handleWindowResize: function(event) {
		setTimeout(dojo.hitch(this, this.resize), 0);
	},

	resize: function() {
		/*Empty Block */
	},

	destroy: function() {
		if (this._searchResults) {
			this._searchResults.destroy();
			delete this._searchResults;
		}
		this.inherited(arguments);
	},

	onCheckboxChange: function(rowConfig, checked){
		if(checked) {
			this._addSelectedRow(rowConfig);
		} else {
			this._removeSelectedRow(rowConfig);
		}
		this.okCmd.disabled= this._selectedRows.length <= 0;
	},

	_addSelectedRow : function (selectedRow) {
		this._selectedRows= [selectedRow];
	},
	
	clearSelections: function (){
		this.okCmd.disabled= true;
		this._clearSelectedRows();
	},
	
	_clearSelectedRows: function (){
		this._selectedRows= [];
	},
	
	_removeSelectedRow: function (selectedRow){
		var rowIndex = this._findRow(selectedRow, this._selectedRows);
		if (rowIndex === -1){
			return;
		}
		this._selectedRows.splice(rowIndex, 1);
	},

	_findRow : function (selectedRow, selectedRowSet){
		for (var i=0; i<selectedRowSet.length; i++){
			if (selectedRowSet[i].link === selectedRow.link){
				return i;
			}
		}
		return -1;
	},
	
	showResultsForResourceInfo: function(resourceInfos, query, preserveFilter){
		this._prepareSearch(resourceInfos, preserveFilter);
		this._searchResults._showResultsForResourceInfo(resourceInfos,query, preserveFilter);
		this.searchResults.appendChild(this._searchResults.domNode);
	},
	
	hideSearchResults: function(){
		if(this._searchResults&& this._searchResults._resultTable){
			this._searchResults._resultTable.setTableData(this._searchResults._headers, []);
			this._searchResults._resultTable.update(false, 0, 0, 0, 0, null, false,null);			
		}
	},

	showSearchResults: function(queryResults, preserveFilter) {
		this._prepareSearch(queryResults, preserveFilter);
		this._searchResults.showResults(queryResults, preserveFilter, this.context);
		this.searchResults.appendChild(this._searchResults.domNode);
	},

	_prepareSearch: function(queryResults, preserveFilter){
		if (!this._searchResults) {
			this._searchResults = new rmps.app.ui.internal.search.SearchResults_({
					_quickSearch: this._quickSearch,
					filteredList: this.filteredList
					/*checkBoxes: this._selectedRows*/
					});
			this._searchResults.startup();
			this._searchResults._resultTable.onCheckboxChange = dojo.hitch(this , this.onCheckboxChange);
		}

		this._searchResults.checkBoxes= this._selectedRows;
	},

	showStatusNode: function(text) {
		this.statusNode.innerHTML = "";
		if (text !== null) {
			this.statusNode.appendChild(dojo.doc.createTextNode(text+"..."));	//$NON-NLS-1$
			this.statusNode.style.display= "inline";
		}else {
			this.statusNode.style.display= "none";
		}
	},

	hideStatusNode: function() {
		this.statusNode.innerHTML = "";
		this.statusNode.style.display= "none";
	}

});



dojo.declare("rmps.app.ui.internal.search.SearchResults_", rmps.app.ui.internal.search.SearchResults, {

	tableClass: "rmps.app.ui.internal.search._TableEx",
	_quickSearch: null,
	_machingLabel: null,
	checkBoxes: null,
	messages: dojo.i18n.getLocalization("rmps.app.pickers", "RMPSPickerMessages"),
	
	_showResultsForResourceInfo: function(resourcesInfo,query, preserveFilter){
		this._queryResult = {};
		this._queryResult.entries= resourcesInfo;
		var tableData= [];
		for(var index=0; index<resourcesInfo.length; index++){
			var resourceInfo= resourcesInfo[index];
			var INFO={
					"shortName": [],
					"longName": [],
					"displayType": [],
					"modificationTime": [{"style": { color: "#808080"}}]
			 } ;
			
			var columns= [];
			
			columns.push({checkBox: true, link: false});
			columns.push({data: query.text, link: false});
			
			for (var att in INFO) {
				var info= INFO[att];
				var val= resourceInfo[att];
				if(val instanceof Date){
					val= dojo.date.stamp.toISOString(val);
				}
				var object= {data: val, link: false};
				for (var sub in info) {
					object[sub]= info[sub];
				}
				columns.push(object);
			}
			tableData.push({
				"columns": columns,
				link: resourceInfo.uri,
				resourceUri: resourceInfo.uri
			});
		}
		this._resultTable.setTableData(this._headers, tableData);
		
		if (query.orderBy && query.orderBy.length > 1) {
			secondaryOrder = query.orderBy.slice();
			secondaryOrder.splice(0, 1);
		}
		
		this._resultTable.update(preserveFilter,
			query.page, query.pageSize,
			resourcesInfo.length, query.limit,
			query.orderBy && query.orderBy.length > 0 ? query.orderBy[0].id : null,
			query.orderBy && query.orderBy.length > 0 ? query.orderBy[0].isAscending : false,
			secondaryOrder);
		if(resourcesInfo.length<1){
			this._resultTable.messageNode.innerHTML = dojo.string.substitute(messages.tooManyResults, [Math.abs(queryResult.totalResults)]);
		}
		this._resultTable.pagingControl.domNode.style.display = "";
		this._resultTable.footerPagingControl.domNode.style.display = "";
	},

	createTableHeaders: function(parentNode) {
		var headers= this.inherited(arguments);
		headers.unshift(this.createTableHeader("checkBox", "", {filterParticipant: false, isOrderable: false}));
		return headers;
	},

	addActionsHeader: function(headers){
		/*Do Nothing */
	},

	createColumnObject: function(rowObj, columnId) {
		var result= this.inherited(arguments);

		/*We do not want by any means to enable having links at any column, as this
		 * will not work in the picker iframe */
		result.link= false;

		return result;
	},

	_checkBoxHandler: function(result, rowObj, owned, resourceUri, presentationUri) {
		result.checkBox = true;
	},

	createTable: function() {
		var table= this.inherited(arguments);
		table._quickSearch= this._quickSearch;
		return table;
	},

	setTitleNode: function(queryResult){
		if(!this._machingLabel){
			this._machingLabel= dojo.doc.createTextNode(this.messages.matchingModelElements);
			this._titleNode.appendChild(this._machingLabel);
			this._titleNode.style.fontSize= "medium";
			this._titleNode.style.fontWeight= "bold";
			this._started= true;
		}
	},

	setDescriptionNode: function(queryResult){
		dojo.html._emptyNode(this._descriptionNode);

	},

	createExplorerAction: function(toolbar, rowObj, resourceUri, presentationUri) {
		/*Do Nothing */
	},

	addButtons: function(toolbar) {
		/*Do Nothing */
	}


});


dojo.declare("rmps.app.ui.internal.search._TableEx", rmps.app.ui.internal.search._Table, {

	_quickSearch: null,
	_idToIdx: {},

		/*Only show the row if its resource uri is not in the fitlered list */
	setTableData: function(headers, tableData) {
		this._headerConfigs = headers;
		this._tableData = tableData;
	},
	
	getReorderHref: function(orderBy, isAscending, secondaryOrder) { 
		return "";
	},

	_populateColumn: function(rowConfig, tableRow, rowIdx, columnConfig, tableCell, colIdx) {
		var isChecked= false;
		dojo.forEach(this.search.checkBoxes, function(row) {
			if (row.link === rowConfig.resourceUri){
				isChecked= "checked";
				return;
			}
		});

		if (columnConfig.checkBox) {
			var checkBox = new dijit.form.RadioButton /*dijit.form.CheckBox */({
            name: "checkBox",
            value: "agreed",
            checked: isChecked,
            onChange: dojo.hitch(this ,function(event) {
                var link = rowConfig.resourceUri;
                if(link) {
//					var idx = link.lastIndexOf('#');
//					if (idx != -1) {
//						link = link.substr(0, idx);
//					}
//                  var rowItem = {
//                      name: rowConfig.columns[this._idToIdx["name"]].data,
//                      link: link
//                  };
					var rowItem = {};
					var ignoreList= ["checkBox","actions"];
					for (var i = 0; i < this._headerConfigs.length; ++i) {
						var headerConfig= this._headerConfigs[i];
						var id= headerConfig.id;
						this._idToIdx[id] = i;
						if (dojo.indexOf(ignoreList, id) === -1){
							var column= rowConfig.columns[i];
							rowItem[id]= column.data;
						}
					}
					rowItem.link= rowConfig.resourceUri;
					this.onCheckboxChange(rowItem, event);
				}
            })}, "checkBox");
			if (dojo.indexOf(this.search.filteredList, rowConfig.resourceUri)!==-1){
				checkBox.attr("disabled", true);
			}
			tableCell.appendChild(checkBox.domNode);
		} else {
			this.inherited(arguments);
		}
	},

	_handleTbodyMouseClick: function(e) {
		/*We do not allow navigation for the table links in the picker */
		return;
	},

	onCheckboxChange: function(rowConfig, checked){
		/*Do Nothing */
	},

	_constructPageSizeDropDown: function() {
		/*we do not want to allow the user to change the value of the number of the displayed entries;
		 * this is in order to avoid having any layout problems in the Resource Picker in which will added
		 * in other contents */
		this._pageSizeSelect.style.display= "none";
		this._itemsPerPage.style.display= "none";
	},

	onPreviousClick: function(e) {
		dojo.stopEvent(e);
		this.doQuery(this.pagingControl.page-1, this.pagingControl.pageSize);
	},

	onNextClick: function(e) {
		dojo.stopEvent(e);
		this.doQuery(this.pagingControl.page+1, this.pagingControl.pageSize);
	},

	doQuery: function(page, pageSize){
		var query= this.createQuery(page, pageSize);
		var queryString= query.toQueryString();
		var timestamp = new Date().getTime();
		this._quickSearch.doQuery(query);
	},

	createQuery:function(page, pageSize){
		var queryResult = this.pagingControl.table.search.getQueryResult();
		var query = dojo.clone(queryResult.query);
		query.pageSize = pageSize;
		query.page = page;
		query.token = queryResult.id;
		return query;
	},

	_filterChanged: function(filter) {
		this._filteredOutRowCount = 0;
		var i;
		if (!filter) {
			for (i = 0; i < this._rowCache.length; ++i) {
				this._rowCache[i].style.display = "";
			}
		} else {
			var shortcut = this._prevFilter && filter.indexOf(this._prevFilter) === 0;
			
			for (i = 0; i < this._rowCache.length; ++i) {
				var config = this._tableData[this._rowCache[i][ROW_CONFIG_PROP]];
				if (shortcut) {
					if (this._rowCache[i].style.display === "") {
						if (this._filterRow(filter, config)) {
							this._rowCache[i].style.display = "none";
							this._filteredOutRowCount++;
						}
					} else {
						this._filteredOutRowCount++;
					}
				} else if (this._filterRow(filter, config)) {
					this._rowCache[i].style.display = "none";
					this._filteredOutRowCount++;
				} else {
					this._rowCache[i].style.display = "";
				}
			}
		}
		this._prevFilter = filter;
		this._updateFilterMessage();
	}

});


dojo.declare("rmps.app.ui.internal.QuickSearch_", rmps.app.ui.internal.search.QuickSearch, {

    _controller: null,
	pageSize: 5,
	resourcesOnly: true,

	doQuery: function(query) {
		this._controller._pickerWidget.clearSelections();
		this._controller.search({query: query.toQueryString()});
	}
});

}());
