/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.diagrams.render.policies.GraphLayout");

dojo.require("rmpw.common.policy.Policy");
dojo.require("rmpw.diagrams.render.policies.GraphContainer");
dojo.require("rmpw.diagrams.render.graph.BaseGraphConfigurator");

/**
 * Data needed to layout the graph.  Can be configured to do auto layout
 * whenever new children are added to the graph.
 * 
 * @author Steve Shaw
 */
dojo.require("dojo.i18n");

dojo.requireLocalization("rmps.graphics.ui", "DiagramEditorMessages");

(function(){
	var messages = dojo.i18n.getLocalization("rmps.graphics.ui", "DiagramEditorMessages");
	
	dojo.declare("rmpw.diagrams.render.policies.GraphLayout", rmpw.common.policy.Policy, {
		singleton: "graphLayout",
		
		constructor: function() {
			this._type = "graph"; // default
			this._routerType = rmpw.diagrams.render.graph.BaseGraphConfigurator.prototype.RECTILINEAR_ROUTING;
			this._flowDirection = "bottom";
			this._incremental = false;
			this._firstInvocation = true;
			
			if (!this._connectAddChild) {
				this._connectAddChild = dojo.connect(this.host, "addChild", dojo.hitch(this, function(child) {
					this.revalidateLayout(child);
				}));
			}
			
			this._invalidLayout = this.host.children.length;
		},
		
		destroy: function() {
			if (this._connectAddChild) {
				dojo.disconnect(this._connectAddChild);
			}
		},
		
		setType: function(type) {
			this._type = type;
		},
		
		setFlowDirection: function(flowDirection) {
			this._flowDirection = flowDirection;
		},
		
		setRouterType: function(routerType) {
			this._routerType = routerType;
		},
		
		/*
		 * @deprecated - use setRouterType instead
		 */
		setGlobalRouter: function(routerType) {
			this.setRouterType(routerType);
		},
		
		getType: function(type) {
			return this._type;
		},
		
		getFlowDirection: function() {
			return this._flowDirection;
		},
		
		getIncrementalMode: function() {
			return this._incremental;
		},
		
		setIncrementalMode: function(incremental) {
			this._incremental = incremental;
		},
		
		isLayoutInvalid: function() {
			return this._invalidLayout > 1 || this._layoutInvoked !== undefined;
		},
		
		hideBeforeLayout: function(diagram, diagramWidget) {
			this._diagram = diagram;
			this._diagram.rootShapes[0].rawNode.style.display = 'none';
			this._diagramWidget = diagramWidget;
			this._diagramWidget.showMessage(messages.layout);
		},
		
		showAfterLayout: function(diagram, diagramWidget) {
			if (this._diagram) {
				this._diagram.rootShapes[0].rawNode.style.display = '';			
				this._diagramWidget.hideMessage();
				this._diagram = undefined;
				this._diagramWidget = undefined;
			}
		},
		
		/*
		 * Adds a child to be layed out
		 */
		revalidateLayout: function(childMapping) {
			var nodePolicy = childMapping.policies.graphNode;
			if (nodePolicy) {
				nodePolicy.getInternalGraphElement().setLayoutState("invalid");
				this._invalidLayout++;
			}
		},
		
		enforce: function() {
			this.inherited(arguments);
			
			if ((!this.host.viewer.getCommandController 
				 || !this.host.viewer.getCommandController().commandStack.isUndoRedoExecuting()) && 
				 this._type && this._invalidLayout > 1) {
				
				this._layoutInvoked = true;
				this._animationState = this.host.root.allowAnimation;
				this.host.root.allowAnimation = false;
				
				this._connectRefresh = dojo.connect(this.host.policies.graphContainer, "refresh", dojo.hitch(this, function() {
					dojo.disconnect(this._connectRefresh);
					delete this._connectRefresh;
					
					var modifyNode = function(graphNode, x, y) {
						graphNode.enforce();
						graphNode.setX(x);
						graphNode.setY(y);
						graphNode.host.invalidate();
					};
					var modifyEdge = function(graphEdge, bendpoints, sourceAnchor, targetAnchor) {
						graphEdge.enforce();
						graphEdge.setBendpoints(bendpoints);
						graphEdge.setSourceAnchor(sourceAnchor);
						graphEdge.setTargetAnchor(targetAnchor);
						graphEdge.host.invalidate();
						
						// turn off avoid obstructions for layout
						if (graphEdge.getInternalGraphElement().isAvoidingObstructions()) {
							graphEdge.getInternalGraphElement().setAvoidObstructions(false);
							graphEdge.__connectRefresh = dojo.connect(graphEdge, "refresh", dojo.hitch(this, function() {
								dojo.disconnect(graphEdge.__connectRefresh);
								delete graphEdge.__connectRefresh;
								graphEdge.getInternalGraphElement().setAvoidObstructions(true);
							}));
						}
					};
					var self = this;
					var layoutCallback = function(graph, args) {
						var ret = self.host.viewer.graphConfigurator.onCommit(graph, 
								args, modifyNode, modifyEdge);
						// connect again so that we know when the layout invocation has completed
						self._connectRefresh = dojo.connect(self.host.policies.graphContainer.graphManager, "onRefreshComplete", dojo.hitch(self, function() {
							dojo.disconnect(self._connectRefresh);
							delete self._connectRefresh;
							delete self._layoutInvoked;
							this.host.root.allowAnimation = self._animationState;
							self.showAfterLayout();
						}));
						
						return ret;
					};
					
					this.host.viewer.graphConfigurator.runGraphLayout(layoutCallback, { 
							type: this._type,
							flowDirection: this._flowDirection,
							routerType: this._routerType,
							incremental: this._firstInvocation ? false : this._incremental,
							targetController: this.host,
							resetGraphPosition: true});
					delete this._firstInvocation;
				}));
			}
			
			this._invalidLayout = 0;
		}
	});
})();