/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.sketch.policies.TextEdit");

dojo.require("rmps.core.RootServices");
dojo.require("rmps.app.core.util");
dojo.require("rmpw.common.vcs.ChangeSet");
dojo.require("rmpw.sketch.behavior.TextDialog");
dojo.require("dijit.form.Textarea");

/**
 * @author Anton McConville
 */

dojo.declare("rmpw.sketch.policies.TextEdit", rmpw.common.policy.Policy, {
	
	singleton: "textedit",
		
	myShape: null, 
	
	dialogs: null,
	
	cancel: function(){
		
		if( this.dialog ){
			this.dialog.hide();
		}
	},
	
	enter: function( value ){
				
		var renderMapping = this.host._currentMapping;
		var traces = renderMapping.traces;
		var owner = traces.stroke.getOwner();
		
		var changeSet = new rmpw.common.vcs.ChangeSet();
		var change = changeSet.createChange(owner);
		
		var bounds = renderMapping.policies.graphNode.getBounds();
		
		var newString = this.textarea.value;
		
		change.set( "text", newString );

		this.host.viewer._commandController.commandStack.repository.commit(changeSet);
		this.dialog.hide();
	},
	
	getBounds: function(){
				
		var viewport = this.host.root.viewport;
		var transform = viewport._getContentsTransform();		
		var offset = viewport.getAbsoluteCanvasOffset();
		
		var uagent = navigator.userAgent.toLowerCase();
		
		if( uagent.search( 'ipad' ) == -1 ){
		
			var bounds = this.host.renderMapping.policies.graphNode.getBounds();	
			
			bounds.x = bounds.x * transform.xx;
			bounds.y = bounds.y * transform.yy;
			bounds.width = bounds.width * transform.xx;
			bounds.height = bounds.height * transform.yy;
			
			bounds.x = offset.x + transform.dx + bounds.x;
			bounds.y = offset.y + transform.dy + bounds.y + bounds.height + 5;
			
			bounds.left = bounds.x + 'px';
			bounds.top = bounds.y + 'px';
		
		}else{
			var bounds = new Object();
			bounds.x = 0;
			bounds.y = 0;
			bounds.left = '0px';
			bounds.top = '0px';
		}
		
		return bounds;
	},
	
	updateDialog: function(){
		
		/* TODO: Consult runtime team over best way to listen for a 
		 * a change in the render mapping.
		 */
		
		setTimeout( dojo.hitch( this, 'moveDialog' ), 200 );
	},
	
	moveDialog: function(){		
				
		if( this.dialog ){
			var bounds = this.getBounds();
			var width = bounds.width;
			
			// Define a minimum width to avoid unusable sizes.
			if (width < 200) {
				width = 200;
			}
			this.dialog.setPosition( bounds.left, bounds.top, width + 'px' );
		}
	},
		
	textDialog: function(){			
		
		var viewport = this.host.root.viewport;
		var content = new dijit.layout.ContentPane();
		
		var actionBar = document.createElement( 'div' );		
		actionBar.style.backgroundColor = '#F2F2F2';
		actionBar.style.padding = '3px 5px 2px 7px';
		actionBar.style.textAlign = 'right';
		actionBar.style.borderTop = '1px solid #CDCDCD';
		actionBar.style.margin = '10px -8px -10px';	
				
		var bounds = this.getBounds();
							
		var styleString = "height:'auto';width:" + bounds.width + "px";
				
		this.dialog = new rmpw.sketch.behavior.TextDialog({
			  anchorPoint: this.surface._parent,
              title: "Add Text",
              content: content,
              style: styleString
        });
		
		this.textarea = new dijit.form.Textarea({
			name: "myarea",
			style: "overflow:auto"
		});
				
		content.containerNode.appendChild( this.textarea.domNode );
		
		var okButton = new dijit.form.Button({ 
			label: "OK",
			onClick: dojo.hitch( this, "enter", this.textarea.value ) });
		
		var cancelButton = new dijit.form.Button({ 
			label: "Cancel",
			onClick: dojo.hitch( this, "cancel" )
		});
		
		if( this.host.renderMapping.sources.svgRoot.elementId != "TextBox" ){
		
			var traces = this.host._currentMapping.traces;	
			var text = traces.text.get();
			
			if( text ){
				this.textarea.attr( 'value', text );
			}
		}
		
		okButton.domNode.style.fontSize = '6pt';
		cancelButton.domNode.style.fontSize = '6pt';
		
		actionBar.appendChild( okButton.domNode );	
		actionBar.appendChild( cancelButton.domNode );	
		
		this.dialog.containerNode.appendChild( actionBar );
		
		this.connections.push( dojo.connect(viewport, "onViewportChanged", this, function(){ this.moveDialog();  } ) ); 
	
		this.dialog.show( bounds.left, bounds.top );
	},
	
	constructor: function( ){		
		this.dialogs = [];		
		this.connections = [];
		this.myShape = this.host.renderMapping.getShape();			
		this.surface = this.host.root.surface;		
		this.connections.push( this.myShape.connect( "ondblclick", this, this.open ) );
		this.connections.push( dojo.connect( this.host.renderMapping, "destroy", this, this.cancel ) );

		
		/* Not entirely happy with how I'm deriving references here */
		
		if( this.host.renderMapping.sources.svgRoot.elementId == "TextBox" ){
			this.textDialog();
			
			var selectionController = this.host.viewer._commandController.modelingPage.selectionController;		
		}
	},
	
	destroy: function(){		
		if( this.myShape != null ){
			
			for( var c=0; c < this.connections.length; c++ ){
				this.myShape.disconnect( this.connections[c] );
			}
			
			this.connections = [];
		}
	},
	
	open: function(e){				
		this.textDialog();
		if (e) {
			dojo.stopEvent(e);
		}
	}, 
	
	enforce: function(){
		this.updateDialog();
	}
});