/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.common.policy.Policy");

/**
 * Base class for all policies.
 * 
 * @author Marc Gobeil
 */
dojo.declare("rmpw.common.policy.Policy", null, {
	constructor: function(host) {
		this.host = host;
	},
	
	destroy: function() {
		delete this.host;
	},
	
	/**
	 * Walk up the parent chain of mappings to find the first render policy
	 * stored at the given key, and return a reference to that policy.
	 */ 
	findNearestParentByKey: function(key, allowSelf) {
		var policy = null;
		var host = allowSelf ? this.host : this.host.parent;
		while(host && !policy) {
			policy = host.policies[key];
			host = host.parent;
		}
		return policy;
	},
	
	/**
	 * Look for child mapping of this key.
	 */ 
	findChildByKey: function(key) {
		for (var i=0, lg=this.host.children.length; i<lg; i++) {
			var child = this.host.children[i];
			if (child.policies[key]) {
				return child.policies[key];
			}
		}
		return null;
	},
	
	/**
	 * Enforces the policy.
	 * 
	 * Subclasses should implement.
	 */
	enforce: function() {
		// do nothing
	},
	
	/*
	 * gets the state of this policy so it can be persisted
	 */
	getState: function() {
		return {};  // no state by default
	},
	
	/*
	 * restores the state of this policy based on persisted data
	 */
	setState: function(state) {
		// do nothing
	}
});