/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.sketch.render.Ellipse");

dojo.require("rmpw.sketch.render.Primitive");
dojo.require("rmpw.sketch.serializer.SVGStyle");
dojo.require("rmpw.sketch.serializer.SVGUtil");
dojo.require("rmpw.diagrams.render.policies.GraphNode");

dojo.declare("rmpw.sketch.render.Ellipse", rmpw.sketch.render.Primitive, {
	
	setSvgEllipse: function(svgEllipse) {
		this.sources.svgRoot = svgEllipse;
	},
	
	/*
	 * @param {graphics::Group} graphicsParent
	 * 
	 */
	setGraphicsParent: function(graphicsParent) {
		var template = this.template;
		var gfxEllipse;
		
		template.group = graphicsParent.createGroup();
		
		var svgEllipse = this.sources.svgRoot;
		gfxEllipse = template.group.createEllipse();
		
		var style = new rmpw.sketch.serializer.SVGStyle();
		
		style.deriveStyle( this.sources.svgRoot );
						
		gfxEllipse.setStroke( style.getStroke() );		
		gfxEllipse.setFill( style.getFill() );
				
		this.template.gfxEllipse = gfxEllipse;
	},
	
	applyModelChanges: function() {
		this.inherited(arguments);
		var anyChanged = false;
		for(var traceKey in this.traces) {
			var trace = this.traces[traceKey];
			if(!trace.synced) {
				anyChanged = true;
			}
			trace.reset();
		}
		
		if (anyChanged) {
			var traces = this.traces;
			this.template.gfxEllipse.setShape({
				cx: traces.cx.get(),
				cy: traces.cy.get(),
				rx: traces.rx.get(),
				ry: traces.ry.get()
			});
			
			var stroke = this.template.gfxEllipse.getStroke();	
			var SVGUtil = new rmpw.sketch.serializer.SVGUtil();	
			SVGUtil.buildStroke( stroke, traces );
			this.template.gfxEllipse.setStroke(stroke);
			
			var fill = traces.fill.get();		
			
			if(fill!=null){
				var fillOpacity = traces.fillOpacity.get();			
				var fillColor = new dojo.Color( fill );			
				fillColor.a = fillOpacity;		
				this.template.gfxEllipse.setFill(fillColor);
			}
			
			var bounds = { x: traces.cx.get() - traces.rx.get(), 
					   y: traces.cy.get() - traces.ry.get(), 
					   width: 2 * traces.rx.get(), 
					   height: 2 *traces.ry.get()}; 
		
			this.showText( this.template.gfxEllipse, bounds );
		}
	},
	
	attachListeners: function() {
		this.inherited(arguments);
		
		this.connectTraces(this.sources.svgRoot, ["cx", "cy", "rx", "ry", "stroke", "strokeOpacity", "strokeWidth", "strokeDasharray", "fill", "fillOpacity"]);
	},
	
	createPolicies: function() {

		var policy = this.createPolicy(rmpw.diagrams.render.policies.GraphNode);
		policy.setFigure(this.template.group);
		policy.useDojoPosition(true);
       
		return policy;
	},	
	
	createTraces: function() {
		this.inherited(arguments);
		
		this.createTrace("cx", "svgRoot.cx");
		this.createTrace("cy", "svgRoot.cy");
		this.createTrace("rx", "svgRoot.rx");
		this.createTrace("ry", "svgRoot.ry");
		this.createTrace("strokeWidth", "svgRoot.strokeWidth");
		this.createTrace("stroke", "svgRoot.stroke");
		this.createTrace("strokeOpacity", "svgRoot.strokeOpacity");
		this.createTrace("strokeDasharray", "svgRoot.strokeDasharray");
		this.createTrace("fill", "svgRoot.fill");
		this.createTrace("fillOpacity", "svgRoot.fillOpacity");
	}
});
