/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.sketch.render.Group");

dojo.require("rmpw.diagrams.render.policies.GraphNode");
dojo.require("rmpw.diagrams.render.util.Workarounds");

dojo.require("rmpw.sketch.render.Line");
dojo.require("rmpw.sketch.render.Path");
dojo.require("rmpw.sketch.render.Ellipse");
dojo.require("rmpw.sketch.render.Rectangle");
dojo.require("rmpw.sketch.render.Text");

dojo.require("rmpw.sketch.serializer.SVGStyle");
dojo.require("rmpw.sketch.utilities.ChildBounds");
dojo.require("rmpw.common.vcs.ChangeSet");
dojo.require("rmpw.common.transform.Mapping");

dojo.declare("rmpw.sketch.render.Group", rmpw.common.transform.Mapping, {
	
	// primatives: [ "g$g", "g$path", "g$ellipse", "g$rectangle", "g$line" ],
	
	elementId: null,
	
	transform: null,
	
	/*
	 * @param {svg::GType} g
	 * 
	 */
	setG: function(g) {
		this.sources.svgRoot = g;
	},
	
	setTransform: function( transform ){
		this.transform = transform;
	},
	
	getBounds: function(){
		
		// TODO: should be done by selectable policy
		var bounds = rmpw.diagrams.render.util.Workarounds.getBoundingBox(this.template.contents);
		
		if(bounds && bounds.b) {
			bounds = {x: bounds.l, y: bounds.t, width: bounds.r - bounds.l, height: bounds.b - bounds.t};
		}
		
		return bounds;
	},
	
	/*
	 * @param {graphics::Group} graphicsParent
	 * 
	 */
	setGraphicsParent: function(graphicsParent) {
		
		this.template.figure = graphicsParent.createGroup();
		
		this.template.hitzone = this.template.figure.createRect();
		this.template.hitzone.setFill([0,0,0,0.0001]);
		
		this.template.contents = this.template.figure.createGroup();
		
		this.rootShapes.push(this.template.figure);
	},
	
	createPolicies: function() {

		var policy = this.createPolicy(rmpw.diagrams.render.policies.GraphNode);
		policy.setFigure(this.template.figure);
		policy.useDojoPosition(true);
       
		return policy;
	},	
	
	applyModelChanges: function() {
		var trace, child, added;
		
		this.inherited(arguments);
		
		trace = this.traces.svgRoot$g;
		if (! trace.synced) {
			added = trace.getAdded();
			
			for (var i = 0, gItem; gItem = added[i]; i++) {
				if (gItem.eClass == "GType") {
					child = new rmpw.sketch.render.Group();
					child.setG(gItem);
					child.setGraphicsParent(this.template.contents);
					this.addChild(child);
				}
				
				if(gItem.elementId){
				    child.elementId = gItem.elementId;
				}
			}
			trace.reset();
		}
		
		trace = this.traces.svgRoot$path;
		if (! trace.synced) {
			added = trace.getAdded();
			
			for (var i = 0, pathItem; pathItem = added[i]; i++) {
				if (pathItem.eClass == "PathType") {
					child = new rmpw.sketch.render.Path();
					child.setSvgPath(pathItem);
					child.setGraphicsParent(this.template.contents);
					this.addChild(child);
				}
			}
			trace.reset();
		}
		
		trace = this.traces.svgRoot$rect;
		if (! trace.synced) {
			added = trace.getAdded();
			
			for (var i = 0, rectItem; rectItem = added[i]; i++) {
				if (rectItem.eClass == "RectType") {
					child = new rmpw.sketch.render.Rectangle();
					child.setSvgRectangle(rectItem);
					child.setGraphicsParent(this.template.contents);
					this.addChild(child);
				}
			}
			trace.reset();
		}
		
		trace = this.traces.svgRoot$textbox;
		if (! trace.synced) {
			added = trace.getAdded();
			
			for (var i = 0, rectItem; rectItem = added[i]; i++) {
				if (rectItem.eClass == "TextBoxType") {
					child = new rmpw.sketch.render.Rectangle();
					child.setSvgRectangle(rectItem);
					child.setGraphicsParent(this.template.contents);
					this.addChild(child);
				}
			}
			trace.reset();
		}
		
		trace = this.traces.svgRoot$line;
		if (! trace.synced) {
			added = trace.getAdded();
			
			for (var i = 0, lineItem; lineItem = added[i]; i++) {
				if (lineItem.eClass == "LineType") {
					child = new rmpw.sketch.render.Line();
					child.setSvgLine(lineItem);
					child.setGraphicsParent(this.template.contents);
					this.addChild(child);
				}
			}
			trace.reset();
		}
		
		trace = this.traces.svgRoot$ellipse;
		if (! trace.synced) {
			added = trace.getAdded();
			
			for (var i = 0, ellipseItem; ellipseItem = added[i]; i++) {
				if (ellipseItem.eClass == "EllipseType") {
					child = new rmpw.sketch.render.Ellipse();
					child.setSvgEllipse(ellipseItem);
					child.setGraphicsParent(this.template.contents);
					child.groupTraces = this.traces;
					this.addChild(child);
				}
			}
			trace.reset();
		}		

		trace = this.traces.svgRoot$text;
		if (! trace.synced) {
			added = trace.getAdded();
			
			for (var i = 0, textItem; textItem = added[i]; i++) {
				if (textItem.eClass == "TextType") {
					child = new rmpw.sketch.render.Text();
					child.setSvgText(textItem);
					child.setGraphicsParent(this.template.contents);
					this.addChild(child);
				}
			}
			trace.reset();
		}
		
		trace = this.traces.transform;
		
		if(!trace.synced){			
			this.template.figure.remove( this.template.hitzone );
			trace.reset();
		}		
		
		this.template.hitzone.setShape(this.getBounds());
	},
	
	attachListeners: function() {
		this.inherited(arguments);
		
		this.connectTraces(this.sources.svgRoot, [
			"svgRoot$rect",
			"svgRoot$line",
			"svgRoot$path",
			"svgRoot$ellipse",
			"svgRoot$text",
			"svgRoot$textbox",
			"svgRoot$g",
			"transform"
		]);
		
		if(!this.sources.svgRoot.transform) {
			this.sources.svgRoot.transform = null;
		}
		
	},
	
	createTraces: function() {
		this.inherited(arguments);
		
		this.createTrace("svgRoot$rect", "svgRoot.rect", true);		
		this.createTrace("svgRoot$ellipse", "svgRoot.ellipse", true);		
		this.createTrace("svgRoot$line", "svgRoot.line", true);
		this.createTrace("svgRoot$path", "svgRoot.path", true);
		this.createTrace("svgRoot$text", "svgRoot.text", true);
		this.createTrace("svgRoot$textbox", "svgRoot.textbox", true);

		this.createTrace("svgRoot$g", "svgRoot.g", true);
		
		this.createTrace("transform", "svgRoot.transform", true);
	}
});
