/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.sketch.render.Line");

dojo.require("rmpw.sketch.serializer.SVGStyle");
dojo.require("rmpw.sketch.serializer.SVGUtil");
dojo.require("rmpw.sketch.render.Primitive");
dojo.require("rmpw.diagrams.render.policies.GraphNode");

dojo.declare("rmpw.sketch.render.Line", rmpw.sketch.render.Primitive, {
	
	setSvgLine: function(svgLine) {
		this.sources.svgRoot = svgLine;
	},
	
	setGraphicsParent: function(graphicsParent) {
		
		var template = this.template;
		var gfxEllipse;
		
		template.group = graphicsParent.createGroup();
		
		var svgLine = this.sources.svgRoot;
		var gfxLine = template.group.createLine();
		
		var style = new rmpw.sketch.serializer.SVGStyle();	
		style.deriveStyle(this.sources.svgRoot);
		gfxLine.setStroke( style.getStroke() );		
		
		// save attach points
		this.template.gfxLine = gfxLine;
	},
	
	applyModelChanges: function() {
		this.inherited(arguments);

		var anyChanged = false;
		for(var traceKey in this.traces) {
			var trace = this.traces[traceKey];
			if(!trace.synced) {
				anyChanged = true;
			}
			trace.reset();
		}
		
		if (anyChanged) {
			var traces = this.traces;
			this.template.gfxLine.setShape({
				x1: traces.x1.get(),
				y1: traces.y1.get(),
				x2: traces.x2.get(),
				y2: traces.y2.get()
			});
			
			var stroke = this.template.gfxLine.getStroke();		
			var SVGUtil = new rmpw.sketch.serializer.SVGUtil();		
			SVGUtil.buildStroke( stroke, traces );
			
			stroke.cap = "round";
			
			this.template.gfxLine.setStroke(stroke);
			var bounds = this.getBounds( traces.x1.get(), traces.y1.get(), traces.x2.get(), traces.y2.get() );
			this.showText( this.template.gfxEllipse, bounds );
		}
	},
	
	getBounds: function( x1, y1, x2, y2){

		var bounds = {};
		bounds.x = x1;
		bounds.y = y1;

		if( x1 > x2 ){ bounds.x = x2; }
		if( y1 > y2 ){ bounds.y = y2; }

		bounds.width = Math.abs(x1 - x2);
		bounds.height = Math.abs(y1 - y2);

		return bounds;
	},
	
	attachListeners: function() {
		this.inherited(arguments);	
		this.connectTraces(this.sources.svgRoot, ["x1", "y1", "x2", "y2",  "stroke", "strokeWidth", "strokeDasharray", "strokeOpacity"]);
	},
	
	createPolicies: function() {
		var policy = this.createPolicy(rmpw.diagrams.render.policies.GraphAnchorable);
		policy.setAnchorShape(this.template.gfxLine);
		
		policy = this.createPolicy(rmpw.diagrams.render.policies.GraphNode);
		policy.setFigure(this.template.group);
		policy.useDojoPosition(true);
       
		return policy;
	},	
	
	createTraces: function() {
		this.inherited(arguments);
		
		this.createTrace("x1", "svgRoot.x1");
		this.createTrace("y1", "svgRoot.y1");
		this.createTrace("x2", "svgRoot.x2");
		this.createTrace("y2", "svgRoot.y2");
		this.createTrace("style", "svgRoot.style");
		this.createTrace("strokeWidth", "svgRoot.strokeWidth");
		this.createTrace("stroke", "svgRoot.stroke");
		this.createTrace("strokeDasharray", "svgRoot.strokeDasharray");
		this.createTrace("strokeOpacity", "svgRoot.strokeOpacity");
	}
});
