/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.sketch.render.Path");

dojo.require("rmpw.sketch.render.Primitive");
dojo.require("rmpw.sketch.serializer.SVGStyle");
dojo.require("rmpw.sketch.serializer.SVGUtil");
dojo.require("dojox.gfx.path");

dojo.declare("rmpw.sketch.render.Path", rmpw.sketch.render.Primitive, {
	
	setSvgPath: function(svgRoot) {
		this.sources.svgRoot = svgRoot;
	},
	
	setGraphicsParent: function(graphicsParent) {
		var template = this.template;
		var path;
		
		template.group = graphicsParent.createGroup();
		
		template.hitZoneRect = template.group.createRect();
		template.hitZoneRect.setFill([0,0,0,0.0001]);
						
		template.path = template.group.createPath();
		
		var style = new rmpw.sketch.serializer.SVGStyle();			
		style.deriveStyle(this.sources.svgRoot);
		template.path.setStroke( style.getStroke() );		
		template.path.setFill( style.getFill() );
	},
	
	/*
	 * @param {String} value
	 * 
	 */
	setPathString: function(value) {
		if (this.template.path) {
			this.template.path.setShape({path: value});
		} else {
			this.sources.pathString = value;
		}
	},
	
	applyModelChanges: function() {
	
		var anyChanged = false;
		for(var traceKey in this.traces) {
			var trace = this.traces[traceKey];
			if(!trace.synced) {
				anyChanged = true;
			}
			trace.reset();
		}
		
		if (anyChanged) {
			var traces = this.traces;
			
			var style = traces.style.get();	
			
			var pathString = traces.pathString.get();		
			this.template.path.setShape({path: pathString});
			
			var bounds = rmpw.diagrams.render.util.Workarounds.getBoundingBox(this.template.path);				
			
			if( bounds != null ){
				this.template.hitZoneRect.setShape({x: bounds.x, y: bounds.y, width: bounds.width, height: bounds.height });
			}
			
			var stroke = this.template.path.getStroke();		
			var SVGUtil = new rmpw.sketch.serializer.SVGUtil();		
			SVGUtil.buildStroke( stroke, traces );
			stroke.cap = "round";
			this.template.path.setStroke(stroke);
			
			var fill = traces.fill.get();	
			
			if( fill ){
				var fillOpacity = traces.fillOpacity.get();			
				var fillColor = new dojo.Color( fill );			
				fillColor.a = fillOpacity;		
				this.template.path.setFill(fillColor);
			}
		
			this.showText( this.template.gfxEllipse, bounds );
		}
		
		this.inherited(arguments);
	},
	
	
	attachListeners: function() {
		this.inherited(arguments);
		
		this.connectTraces(this.sources.svgRoot, ["pathString", "style", "stroke", "strokeWidth", "strokeOpacity", "strokeDasharray", "fill", "fillOpacity"]);
	},
	
	createPolicies: function() {
		var policy = this.createPolicy(rmpw.diagrams.render.policies.GraphNode);
		policy.setFigure(this.template.group);
		policy.useDojoPosition(true);
		return policy;
	},	
	
	createTraces: function() {
		this.inherited(arguments);
		
		this.createTrace("pathString", "svgRoot.d");
		this.createTrace("style", "svgRoot.style");
		this.createTrace("strokeWidth", "svgRoot.strokeWidth");
		this.createTrace("stroke", "svgRoot.stroke");
		this.createTrace("strokeOpacity", "svgRoot.strokeOpacity");
		this.createTrace("strokeDasharray", "svgRoot.strokeDasharray");
		this.createTrace("fill", "svgRoot.fill");
		this.createTrace("fillOpacity", "svgRoot.fillOpacity");
	}
});
