/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corporation 2009, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.sketch.render.Primitive");

dojo.require("rmpw.common.transform.Mapping");

dojo.require("rmpw.sketch.serializer.SVGStyle");

dojo.require("rmpw.sketch.serializer.SVGUtil");
dojo.require("rmpw.diagrams.render.util.Workarounds");

dojo.require("rmpw.diagrams.render.policies.GraphNode");

dojo.require("dojox.html.metrics");

dojo.require("dojox.gfx.shape");

dojo.require("dojox.gfx");

dojo.declare("rmpw.sketch.render.Primitive", rmpw.common.transform.Mapping, {
	
	textTraces: [ "text", "fontSize", "fontWeight", "fontFamily", "align", "textColor", "textOpacity", "stroke", "strokeOpacity", "fontStyle"],
	
	node: null,
	
	makeTextShape: function( word, font ){
		this.node = document.createElement('div');
		var surface = dojox.gfx.createSurface( this.node, 500, 500);
		var shape = surface.createText( { x: 0, y: 0, text: word, align: "start" } );	
		shape.setFont(font);			
		return shape;	
	},
	
	splitLine: function( text, limit ){
			
		/* 
		 * Want to build sentences and paragraphs from 
		 * the building blocks. 
		 * 
		 */		
		
		var font = { family: "sans-serif", size: "12pt", weight: "bold" };
		var style = { fontSize:'12pt' };
		
		var paragraphs = text.split( '\n' );

		var space = this.makeTextShape( " ", font );
		var spaceWidth = rmpw.diagrams.render.util.Workarounds.getBoundingBox( space ).width;
		
		var phrases = [];

		for( var p = 0; p < paragraphs.length; p++ ){
		
			var words = paragraphs[p].split( ' ' );
			
			/* Treat words as building blocks. Forget that they're words - consider
			 * them shapes with definite bounds.
			 */
			
			var blocks = [];
			
			for( var w = 0; w < words.length; w++ ){
				
				var shape = this.makeTextShape( words[w], font );
			
				blocks[w] = { word: words[w],
						      bounds: rmpw.diagrams.render.util.Workarounds.getBoundingBox( shape ) };
			}
			
			var start = 0;
			var lineWidth = 0;
			var margins = limit - 40;
			
			var phrase = [];	
			
			//Word wrap if the bounding box is smaller than the phrase;
			//Iteratively build up the phrase size until it overflows
			//the bounding box.
			for( var end=0; end < blocks.length; end++ ){
				
				lineWidth = lineWidth + blocks[end].bounds.width;
				
				phrase.push( blocks[end].word );
				
				if( start != end ){
					lineWidth = lineWidth + spaceWidth;
				}
					
				//Don't wrap on the first word or there will be
				//an infinite loop here.
				if( lineWidth > margins && phrase.length > 1){		
					lineWidth=0;
					start = end;
					end=end-1;
					phrase.pop();
					phrases.push(phrase);
					phrase = [];
				}
			}
			
			phrases.push( phrase );
		}
		return phrases;
	},

	
	showText: function(target, bounds ){
		
		var traces = this.traces;
		var text = traces.text.get();
		
		var fontsize = traces.fontSize.get();
		
		if( fontsize == null ){
			fontsize = "12pt";
		}
		
		var family = traces.fontFamily.get();
		
		if( family == null ){
			family = "sans-serif";
		}
		
		var weight = traces.fontWeight.get();
		
		if( weight == null ){
			weight = "bold";
		}
		
		var style = traces.fontStyle.get();
		
		if( style == null ){
			style = "normal";
		}
		
		var font = { family: family, size: fontsize, weight: weight, style: style };
		var style = { fontSize: fontsize }
		
		var stroke = "#DC143C";		
		var fill = "#DC143C";
		
		var textColor = traces.textColor.get();
		
		if( textColor ){
			stroke = textColor;
			fill = textColor;
		}
		
		var gap = 20;

		if( this.lines ){	
			var count = this.lines.length;
			for( var l = 0; l < count; l++ ){
				this.template.group.remove( this.template[l] )
				this.template[l] = null;
			}
		}
		
		this.lines = [];
	
		if( text ){
			
			var x = bounds.x;
			var y = bounds.y;
									
			var output = this.splitLine( text, bounds.width );
			
			var start = gap * ( output.length -1 )/2;

			for( var line = 0; line < output.length; line++ ){
				
				var phrase = output[line].join(" ");

				var phraseShape = this.makeTextShape( phrase, font );		
				var phraseBounds = rmpw.diagrams.render.util.Workarounds.getBoundingBox( phraseShape );

				var xcoord = x + (bounds.width/2) - (phraseBounds.width/2);
				var ycoord = y - start + (bounds.height/2) + (gap/3) + ( line * 20 ); 
							
				var textItem = { text: phrase, align: "start" };
							
				this.template[line] = this.template.group.createText( textItem );
				
				this.template[line].setTransform( { dx: xcoord, dy: ycoord } );
				
				this.lines.push( this.template[line] );
				
				this.template[line].setFont(font);			
				this.template[line].setFill(fill);
			}
		}
	},
	
	renderText: function(){
		var fontsize = traces.fontSize.get();
		
		if( fontsize.indexOf("pt", 0) < 0 ){
			fontsize = fontsize + "pt";
		}
		
		var font = { 
			family: traces.fontFamily.get(), 
			size: fontsize, 
			weight: traces.fontWeight.get(),
			style: traces.fontStyle.get()
		}
		
		var fill = traces.fill.get();		
		
		var fOpacity = traces.fillOpacity.get();

		if( fOpacity >= 0 ){
			fill.a = traces.fillOpacity.get();
		}	
		
		var stroke = traces.stroke.get();
		
		var sOpacity = traces.strokeOpacity.get();
		
		if( sOpacity >= 0 ){
			stroke.a = sOpacity;
		}
		
		this.setText( font, fill, stroke );		
	},
	
	connectTraces: function(source, traces) {
		traces = traces.concat( this.textTraces );			
		this.inherited(arguments)	
	},
	
	createTraces: function(){
		this.inherited(arguments);
		this.createTrace("text", "svgRoot.text");	
		this.createTrace("fontSize", "svgRoot.fontSize");
		this.createTrace("fontWeight", "svgRoot.fontWeight");
		this.createTrace("fontFamily", "svgRoot.fontFamily");
		this.createTrace("align", "svgRoot.align");
		this.createTrace("textColor", "svgRoot.textColor");
		this.createTrace("textOpacity", "svgRoot.textOpacity");
		this.createTrace("stroke", "svgRoot.stroke");
		this.createTrace("fontStyle", "svgRoot.fontStyle");
		this.createTrace("strokeOpacity", "svgRoot.strokeOpacity");
	}
});