/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.sketch.render.Rectangle");

dojo.require("rmpw.sketch.render.Primitive");
dojo.require("rmpw.common.transform.Mapping");
dojo.require("rmpw.sketch.serializer.SVGStyle");
dojo.require("rmpw.sketch.serializer.SVGUtil");
dojo.require("rmpw.diagrams.render.policies.GraphNode");


dojo.declare("rmpw.sketch.render.Rectangle", rmpw.sketch.render.Primitive, {
	
	setSvgRectangle: function(svgRectangle) {
		this.sources.svgRoot = svgRectangle;
	},
	
	setGraphicsParent: function(graphicsParent) {
		var template = this.template;
		var gfxRectangle;
		
		this.graphicsParent = graphicsParent;
		
		template.group = graphicsParent.createGroup();
		
		var svgRectangle = this.sources.svgRoot;
		gfxRectangle = template.group.createRect();
		
		var style = new rmpw.sketch.serializer.SVGStyle();			
		style.deriveStyle(this.sources.svgRoot);
		gfxRectangle.setStroke( style.getStroke() );		
		gfxRectangle.setFill( style.getFill() );
		
		this.template.gfxRectangle = gfxRectangle;	
	},
	
	applyModelChanges: function() {
		this.inherited(arguments);

		var anyChanged = false;
		for(var traceKey in this.traces) {
			var trace = this.traces[traceKey];
			if(!trace.synced) {
				anyChanged = true;
			}
			trace.reset();
		}
		
		if (anyChanged) {
			var traces = this.traces;
			
			var style = traces.style.get();	

			this.template.gfxRectangle.setShape({
				x: traces.x.get(),
				y: traces.y.get(),
				width: traces.width.get(),
				height: traces.height.get(),
				style: traces.style.get()
			});

			var stroke = this.template.gfxRectangle.getStroke();		
			var SVGUtil = new rmpw.sketch.serializer.SVGUtil();		
			SVGUtil.buildStroke( stroke, traces );
			this.template.gfxRectangle.setStroke(stroke);
			
			var fill = traces.fill.get();	
			
			if( fill ){
				var fillOpacity = traces.fillOpacity.get();			
				var fillColor = new dojo.Color( fill );			
				fillColor.a = fillOpacity;		
				this.template.gfxRectangle.setFill(fillColor);
			}
			
			var bounds = { x: traces.x.get(), 
						   y: traces.y.get(), 
						   width: traces.width.get(), 
						   height: traces.height.get()}; 
			
			this.showText( this.template.gfxRectangle, bounds );
		}
	},
	
	attachListeners: function() {
		this.inherited(arguments);	
		this.connectTraces(this.sources.svgRoot, ["x", "y", "width", "height", "style", "stroke", "strokeOpacity", "strokeWidth", "strokeDasharray", "fill", "fillOpacity"]);
	},
	
	createPolicies: function() {
		var policy = this.createPolicy(rmpw.diagrams.render.policies.GraphNode);
		policy.useDojoPosition(true);	
		policy.setFigure(this.template.group);
	},	
	
	createTraces: function() {
		this.inherited(arguments);
		
		this.createTrace("x", "svgRoot.x");
		this.createTrace("y", "svgRoot.y");
		this.createTrace("width", "svgRoot.width");
		this.createTrace("height", "svgRoot.height");
		this.createTrace("style", "svgRoot.style");
		this.createTrace("strokeWidth", "svgRoot.strokeWidth");
		this.createTrace("stroke", "svgRoot.stroke");
		this.createTrace("strokeOpacity", "svgRoot.strokeOpacity");
		this.createTrace("strokeDasharray", "svgRoot.strokeDasharray");
		this.createTrace("fill", "svgRoot.fill");
		this.createTrace("fillOpacity", "svgRoot.fillOpacity");
	}
});
