/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.sketch.render.Text");

dojo.require("rmpw.common.transform.Mapping");

dojo.require("rmpw.sketch.serializer.SVGStyle");

dojo.require("rmpw.sketch.serializer.SVGUtil");
dojo.require("rmpw.diagrams.render.policies.GraphNode");

dojo.declare("rmpw.sketch.render.Text", rmpw.common.transform.Mapping, {
	
	setSvgText: function(svgText) {
		this.sources.svgRoot = svgText;
	},
	
	/*
	 * @param {graphics::Group} graphicsParent
	 * 
	 */
	setGraphicsParent: function(graphicsParent) {
		var template = this.template;
		var gfxText;
		
		template.group = graphicsParent.createGroup();
		
		var text = {x: 250, y: 50, text: "", align: "start"};
		var font = {family: "Times", size: "36pt", weight: "bold"};
		var stroke = "black";		
		var fill = "black";
		
		gfxText = this.makeText(this.template.group, text, font, fill, stroke);
		
		// gfxText = template.group.createText();
		
		var style = new rmpw.sketch.serializer.SVGStyle();
		
		style.deriveStyle( this.sources.svgRoot );
						
		gfxText.setStroke( style.getStroke() );	
		
		gfxText.setFill( style.getFill() );
				
		this.template.gfxText = gfxText;
	},
	
	makeText: function( surface, text, font, fill, stroke ){
		
		var t = surface.createText( text );
		
		if( font ){   
			t.setFont(font);
		}
		
		if( fill ){ 
			t.setFill(fill);
		}
		
		if( stroke ){
			t.setStroke(stroke);
		}
		
		return t;
	},
	
	setText: function( font, fill, stroke ){
				
		if( font ){   
			this.template.gfxText.setFont(font);
		}
		
		if( fill ){ 
			this.template.gfxText.setFill(fill);
		}
		
		if( stroke ){
			this.template.gfxText.setStroke(stroke);
		}
	},
	
	applyModelChanges: function(){
		
		var anyChanged = false;
		
		for( var traceKey in this.traces ){
			
			var trace = this.traces[traceKey];
			if(!trace.synced) {
				anyChanged = true;
			}
			trace.reset();
		}
		
		if( anyChanged ){
			
			var traces = this.traces;
			
			this.template.gfxText.setShape({
				x: traces.x.get(),
				y: traces.y.get(),
				text: traces.text.get()
			});
			
			var fontsize = traces.size.get();
			
			if( fontsize.indexOf("pt", 0) < 0 ){
				fontsize = fontsize + "pt";
			}
			
			var font = { 
				family: traces.family.get(), 
				size: fontsize, 
				weight: traces.weight.get(),
				style: traces.style.get()
			};
			
			var fill = traces.fill.get();		
			
			var fOpacity = traces.fillOpacity.get();

			if( fOpacity >= 0 ){
				fill.a = traces.fillOpacity.get();
			}	
			
			var stroke = traces.stroke.get();
			
			var sOpacity = traces.strokeOpacity.get();
			
			if( sOpacity >= 0 ){
				stroke.a = sOpacity;
			}
			
			this.setText( font, fill, stroke );			
		}
		
		this.inherited(arguments);
	},
	
	attachListeners: function() {
		this.inherited(arguments);
		
		this.connectTraces(this.sources.svgRoot, ["x", "y", "text", "size", "weight", "family", "align", "fill", "fillOpacity", "stroke", "strokeOpacity", "style"]);
	},
	
	createPolicies: function() {

		var policy = this.createPolicy(rmpw.diagrams.render.policies.GraphNode);
		policy.setFigure(this.template.group);
		policy.useDojoPosition(true);
       
		return policy;
	},	
	
	createTraces: function() {
		this.inherited(arguments);	
		this.createTrace("x", "svgRoot.x");
		this.createTrace("y", "svgRoot.y");
		this.createTrace("text", "svgRoot.text");	
		this.createTrace("size", "svgRoot.size");
		this.createTrace("weight", "svgRoot.weight");
		this.createTrace("family", "svgRoot.family");
		this.createTrace("align", "svgRoot.align");
		this.createTrace("fill", "svgRoot.fill");
		this.createTrace("fillOpacity", "svgRoot.fillOpacity");
		this.createTrace("stroke", "svgRoot.stroke");
		this.createTrace("style", "svgRoot.style");
		this.createTrace("strokeOpacity", "svgRoot.strokeOpacity");
	}
});
