/*******************************************************************************
 * Licensed Materials - Property of IBM 
 * Copyright IBM Corp. 2010, 2011. All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("rmpw.diagrams.layout.client.routers.AutomaticRouter");

dojo.require("rmpw.diagrams.layout.client.routers.AbstractRouter");
dojo.require("rmpw.diagrams.layout.client.routers.MultiValueMap");

dojo.declare("rmpw.diagrams.layout.client.routers.AutomaticRouter$HashKey", null, {
	constructor: function(conn) {
		dojo.mixin(this, new Object_0());
		this.anchor1 = conn.getSourceAnchor();
		this.anchor2 = conn.getTargetAnchor();
		this.hashCode$ = this.hashCode_autoRouterKey;
		this.equals$ = this.equals_autoRouterKey;
	},
	
	hashCode_autoRouterKey: function (){
		return this.anchor1.hashCode$() ^ this.anchor2.hashCode$();
	},
	
	equals_autoRouterKey: function (object){
		var hashKey, hkA1, hkA2, isEqual;
		isEqual = false;
		if (object != null && object.anchor1 && object.anchor2) {
			hashKey = object;
			hkA1 = hashKey.anchor1;
			hkA2 = hashKey.anchor2;
			isEqual = hkA1.equals$(this.anchor1) && hkA2.equals$(this.anchor2) || hkA1.equals$(this.anchor2) && hkA2.equals$(this.anchor1);
		}
		return isEqual;
	}
});

dojo.declare("rmpw.diagrams.layout.client.routers.AutomaticRouter", rmpw.diagrams.layout.client.routers.AbstractRouter, {
	constructor: function(nextRouter) {
		this.nextRouter = nextRouter;
		this.connections = new rmpw.diagrams.layout.client.routers.MultiValueMap();
	},
	
	getType: function() {
		return this.nextRouter.getType();
	},
	
	/*
	 * delegate to the contained router
	 */
	isOrthogonal : function() {
		return this.nextRouter.isOrthogonal();
	},
	
	route: function (conn){
		var connectionKey, connectionList, index, points_0, end, points, start, ref_0, ref;
		if (this.nextRouter)
			this.nextRouter.route(conn);
		else {
			conn.getPointList().removeAllPoints();
			points = conn.getPointList();
			points.removeAllPoints();
			ref_0 = conn.endAnchor.getReferencePoint();
			start = conn.startAnchor.getLocation(ref_0);
			ref = conn.startAnchor.getReferencePoint();
			end = conn.endAnchor.getLocation(ref);
			conn.translateToRelative(start);
			conn.translateToRelative(end);
			points.addPoint_0(start.x, start.y);
			points.addPoint_0(end.x, end.y);
			conn.setPointList(points);
		}
		if (conn.getPointList().size == 2 && !this.isFeedback(conn)) {
			points_0 = conn.getPointList();
			connectionKey = new rmpw.diagrams.layout.client.routers.AutomaticRouter$HashKey(conn);
			connectionList = this.connections.get(connectionKey);
			if (connectionList) {
				if ($indexOf(connectionList, conn, 0) != -1) {
					index = $indexOf(connectionList, conn, 0) + 1;
				}
				else {
					index = connectionList.size + 1;
					this.connections.put(connectionKey, conn);
				}
				this.handleCollision(points_0, index);
				
				// re-adjust the start and end points accordingly
				ref = points_0.getPoint(1);
				conn.translateToAbsolute(ref);
				start = conn.startAnchor.getLocation(ref);
				conn.translateToRelative(start);
				end = conn.endAnchor.getLocation(ref);
				conn.translateToRelative(end);
				points_0.setPoint(start, 0);
				points_0.setPoint(end, points_0.size - 1);
				
				conn.setPointList(points_0);
			}
			else {
				this.connections.put(connectionKey, conn);
			}
		}
	},

	getConstraint: function (connection){
		if (this.nextRouter)
			return this.nextRouter.getConstraint(connection);
		return null;
	},
	
	setConstraint: function (connection, constraint) {
		this.invalidate(connection);
		if (this.nextRouter != null)
			this.nextRouter.setConstraint(connection, constraint);
	},

	invalidate: function (conn){
		var affected, connectionKey, connectionList, i;
		if (this.nextRouter) {
			this.nextRouter.invalidate(conn);
		}
		
		if (this.isFeedback(conn) || !conn.getSourceAnchor() || !conn.getTargetAnchor())
			return;
		connectionKey = new rmpw.diagrams.layout.client.routers.AutomaticRouter$HashKey(conn);
		connectionList = this.connections.get(connectionKey);
		affected = this.connections.remove(connectionKey, conn);
		if (affected != -1) {
			for (i = affected; i < connectionList.size; ++i)
				if (conn != connectionList.array[i]) {
					connectionList.array[i].revalidate();
				}
		}
		else {
			this.connections.removeValue(conn);
		}	
	},

	remove: function (conn){
		var connectionKey, connectionList, i, index;
		if (!conn.getSourceAnchor() || !conn.getTargetAnchor())
			return;
		connectionKey = new rmpw.diagrams.layout.client.routers.AutomaticRouter$HashKey(conn);
		connectionList = this.connections.get(connectionKey);
		if (connectionList) {
			index = this.connections.remove(connectionKey, conn);
			for (i = index + 1; i < connectionList.size; ++i) {
				connectionList.array[i].revalidate();
			}
		}
		if (this.nextRouter) {
			this.nextRouter.remove(conn);
		}
	}
});