/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmpw.common.store.JfsStore");

dojo.require("dijit.Dialog");
dojo.require("rmps.core.client");
dojo.require("rmps.core.error");
dojo.require("rmpw.common.vcs.ref");
dojo.require("rmpw.common.vcs.Repository");
dojo.require("rmpw.common.vcs.ServerChangeSet");
dojo.require("rmpw.common.services.StreamService");
dojo.require("rmpw.common.services.ResourceState");
dojo.require("rmps.app.ui.internal.dialogs.ResourceInfoDialog");
dojo.require("rmps.app.ui.internal.pages.design.DesignUtil");
dojo.require("rmpw.common.util.ErrorUtil");
dojo.require("dojo.i18n");

var DesignUtil = rmps.app.ui.internal.pages.design.DesignUtil;

var JFS_STORE_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "rdfdirectory")+"/";
var SHARE_ON_SAVE_ACTION = "changeSetAction=SHARE_ON_SAVE";
var EXPLICIT_SHARE_ACTION = "changeSetAction=EXPLICIT_SHARE";

dojo.requireLocalization("rmpw.common", "JfsStoreMessages");
var messages = dojo.i18n.getLocalization("rmpw.common", "JfsStoreMessages");
var JFS_CONSTRUCTOR_PATH = JFS_STORE_PATH + "constructor";

dojo.declare("rmpw.common.store.JfsStore", null, {
	streamService : null,
	resourceState : null,
	serverSideCs : null,
	repository : null,
	locked : null,
	mode: null,
	documentUri: null,
	refResources : null,
	/**
	 * documentUri: frontside uri of the document in jfs to load
	 * 
	 * @param args
	 */
	constructor: function(args) {
		dojo.mixin(this, args);
		this.repository = new rmpw.common.vcs.Repository();
		this.serverSideCs = new rmpw.common.vcs.ServerChangeSet();

		// TODO: unused? remove?
//		this.streamService = new rmpw.common.services.StreamService(); 
		
		// TODO: remove this hack
		if(!this.documentUri) {
			this.documentUri = this.uri;
		}
		
		this._isPersisted = !!this.documentUri;
		if(this._isPersisted) {
			this.resourceState = new rmpw.common.services.ResourceState();
			dojo.connect(this.resourceState, "onLock", dojo.hitch(this, this._setLocked, true));
			dojo.connect(this.resourceState, "onUnLock", dojo.hitch(this, this._setLocked, false));
			dojo.connect(this.repository, "beforeCommit", dojo.hitch(this, this._beforeCommit));
			dojo.connect(this.repository, "afterCommit", dojo.hitch(this, this._afterCommit));
			this.locked = false;
		}
	},
	
	_afterCommit: function(changeSet) {
		if ( this.refResources!=null && this.refResources.length>0 &&
				(change=this.serverSideCs.getChangeCommand(changeSet, 
				['arrayRemove', 'delete']))!=null){
			this.update(null);
		}
	},
	
	_beforeCommit: function(changeSet) {
		// if currently saving, add a precondition to wait till done before attempting to lock
		if (this._isSaving) {
			var result = new dojo.Deferred();
			changeSet.preconditions.push(result);
			changeSet.connectAllDone = dojo.connect(this, "onAllDone", dojo.hitch(this, function() {
				if (changeSet.connectAllDone) {
					dojo.disconnect(changeSet.connectAllDone);
					delete changeSet.connectAllDone;
					
					// save done, collect and wait on locking conditions
					this._lock(changeSet);
					var lockConditions = [];
					while (changeSet.preconditions.length > 1) {
						lockConditions.push( changeSet.preconditions.pop());
					}
					var allPreconditions = new dojo.DeferredList(lockConditions, false, true, dojo.hitch(this, function() {result.reject();})); 
					allPreconditions.then(dojo.hitch(this, function() {result.resolve();}), dojo.hitch(this, function() {result.reject();}));
				}
			}));
		} else {
			this._lock(changeSet);
		}
	},
	
	_lock: function(changeSet) {
		var change;
		this.refResources = null;
		if ((change = this.serverSideCs.getChangeCommand(changeSet, ['arrayRemove', 'delete'])) != null &&
				this._getDeleteValue(change) != null) {
			var result = new dojo.Deferred(); //for additional resources
			changeSet.preconditions.push(result);
			this._lockResource(changeSet);
			var resURI = this.uri+this._getDeleteValue(change);
			var refresDef = this._getEffectedResources(this.uri, resURI);
			refresDef.then(dojo.hitch(this, function(data) {
				this.refResources = dojo.fromJson(data);
				if (this.refResources!=null && this.refResources.length > 0){
					    new rmps.app.ui.internal.dialogs.ResourceInfoDialog({
						files: this.refResources,
						updateFile: this.uri,
						uri: resURI,
						onOk: dojo.hitch(this, this._lockResources, changeSet, result)
					});
				} else {
					result.resolve();
				}
			}));
		} else {
			this._lockResource(changeSet);
		}
	},
	
	_lockResource: function(changeSet) {
		if(!this.locked) {
			if(!this.acquireLock || this.acquireLock.finished) {
				this.acquireLock = this.resourceState.lock(this.documentUri, changeSet);
			}
			changeSet.preconditions.push(this.acquireLock);
		}
	},
	
	_lockResources: function(changeSet, result) {
		this.resourceState.lockAll(this.refResources, changeSet).then( dojo.hitch(this, function(result) { 
			result.resolve();
		}, result));
	},
		
	_getDeleteValue: function(change) {
		for ( property in change ){
			if (change[property][0]!== null &&
				change[property][0]['value']!==null && 
				change[property][0]['value']['rdf_about']!==null) {
				var uri = change[property][0]['value']['rdf_about'];
				if ( uri==undefined ){
					return null;
				}
				var index;
				if ( (index=uri.indexOf('#'))>-1){
					uri = uri.substring(index+1, uri.length);
				}
				return uri;
			}
		}
	},
	
	_getEffectedResources: function (logicalURI, resourceURI){
		var args = {
				content: {
					logicalURI: logicalURI,
					resourceURI: resourceURI
				},
				headers: {
					"X-ibm-rmps-internal": "true",
					"applicationId" : jazz.app.context.get().applicationId
				},
				url: JFS_STORE_PATH+'cascadeDelete'
		};	
		return rmps.core.client.xhrGet(args, false);
	}, 
	
	_setLocked: function(locked) {
		this.locked = locked;
	},
	
	load: function() {
		var url, content, headers;
		if(this._isPersisted) {
			url = JFS_STORE_PATH;
			content = { logicalURI: this.uri };
			headers = { "Accept": "application/x-rdf+json-graph" };
		} else {
			url = JFS_CONSTRUCTOR_PATH;
			content = this.constructorArgs;
			headers = { "Accept": "application/x-rdf+json-graph" };
		}
		
		var deferred = new dojo.Deferred();
		rmps.core.client.xhrGet({
			url : url,
			content: content,
			headers: headers,
			handleAs: "text",
			load: dojo.hitch(this, function(data, ioargs) {
				if(data===null || data==''){
				    var errorMessage = dojo.string.substitute(messages.blankDocumentError,  [this.documentUri]);
					rmps.core.error.createError(errorMessage, messages.loadingAction).dispatch();
					throw new Error(errorMessage);
				}
				var modelIndex = {};
				var documents = rmpw.common.vcs.ref.fromJson(data, {
					index: modelIndex,
					idAttribute: "rdf_about",
					// Note (SD): for now loader is empty, implement if we need to support lazy loading
					loader: function() { }
				});
				var model = documents[0];
				
				if (this._connectSaveComplete) {
					dojo.disconnect(this._connectSaveComplete);
				}
				this._connectSaveComplete = dojo.connect(this, "onSaveComplete", dojo.hitch(this, function() {
					var dmcodegen_defines = model.dmcodegen_defines;
					for (var i=0, lg=dmcodegen_defines.length; i<lg; i++) {
						var subObj = dmcodegen_defines[i];
						if (subObj.rdf_about && !(subObj.rdf_about.indexOf(this.documentUri) > -1)) {
							// fix the identifier so that it's fully qualified with the document URI
							subObj.rdf_about = rmpw.common.util.IdentifierUtil.normalizeURI(this.documentUri) + subObj.rdf_about;
						}
					}
				}));
				
				// immitate legacy string key-value based namespace format
				this.namespaces = [];
				var namespaces = model.__prefixes__;
				for(var nsKey in namespaces) {
					this.namespaces.push(nsKey + "=" + namespaces[nsKey]);
				}
				
				//this.eTag = ioargs.xhr.getResponseHeader("ETag"); //uncomment once 24890 is fixed
				this.documentUri = ioargs.xhr.getResponseHeader("documentUri");
				
				var resourceName= ioargs.xhr.getResponseHeader("resourceName");
				if(resourceName){
					resourceName= decodeURIComponent(resourceName.replace(/\+/g," "));					
				}
				
				var presentationName= ioargs.xhr.getResponseHeader("resourceLongTitle");
				if(presentationName){
					presentationName= decodeURIComponent(presentationName.replace(/\+/g," "));					
				}
				
				var fragmentName = ioargs.xhr.getResponseHeader("displayFragmentName");
				if (fragmentName) {
					// it is only set if currently viewing a fragment, need to show it on tabs that are focused on the fragment
					fragmentName = decodeURIComponent(fragmentName.replace(/\+/g," "));
				}
				var fragmentTypeLabel = ioargs.xhr.getResponseHeader("fragmentTypeLabel");
				if (fragmentTypeLabel) {
					fragmentTypeLabel = decodeURIComponent(fragmentTypeLabel.replace(/\+/g, " "));
				}			
				
				deferred.callback({
					model: model,
					modelIndex: modelIndex,
					repository: this.repository,
					namespaces: this.namespaces,
					isReadOnly: (ioargs.xhr.getResponseHeader("isReadOnly") == 'true'),
					currentResourceInfo: {
						name : resourceName,
						types : ioargs.xhr.getResponseHeader("resourceType"),
						presentationName : presentationName,						
						documentUri : this.documentUri,
						iconUri : ioargs.xhr.getResponseHeader("typeIconUri"),
						fragmentName : fragmentName,
						fragmentTypeLabel : fragmentTypeLabel
					}
				});
			}),
			error: deferred.errback
		});

		return 	deferred;
	},
	
	//	TODO: unused? remove 
//	saveResource: function(changeObjects, resource, method) {
//		this.streamService.getDefaultStream().then( dojo.hitch(this, function(streamURI){
//			this._updateResource(changeObjects, resource, method, streamURI);
//		}) );
//	},
	
	update: function(monitor) {
		var task = null;
		if(monitor) {
			task = monitor.beginTask(null, 100);
		}
		
		// Layout.saveDiagram()
		var changeSets = [];
		if (this.repository.patch) {
			changeSets.push(this.repository.patch);
		}
		changeSets.push(this.repository.getStack(true));
		this.saveResource(changeSets, this.constructorArgs, dojo.hitch(this, this._onSave),  
				dojo.hitch(this, this._onDone), this._isPersisted, task, this.documentUri);
	},
	
	saveResource: function (changeSets, constructorArgs, onSave, onDone, isPersisted, task, logicalUri) {
		var changeData = new rmpw.common.vcs.ServerChangeSet({ 'changeSet': changeSets });
		var changes = changeData.getChanges();
		if (changes.length < 1) {
			// it can happen that the only change is setting a reference on a new resource on a popup form 
			// correxponding to the fragment - in that case, there is nothing to save here in regular save, 
			// just complete the save
			if(dojo.isFunction(onSave)){
				onSave(null, null, task, changeSets);					
			}			
			if(dojo.isFunction(onDone)){
				onDone();					
			}
			if (!changeset && this.resourceState){
				this.resourceState.onUnLock();
			}
			return;
		}
		var serviceUrl = JFS_STORE_PATH;
		var identifier = changes[0].rdf_about;
		if(isPersisted) {
			serviceUrl += '?logicalURI=' + encodeURIComponent(logicalUri);
		} else{
			if(identifier) {
				if(identifier != 'http://new') {
					// user has specified their own identifier, which overwrites the rdf_about of the root resource
					serviceUrl += identifier;
				}
			}
			
			// null the uri so JsonRdfEmitter will swap in the generated id,
			// rather than think we're talking about a fragment by this name
			changes[0].rdf_about = null;
		}
		
		if(constructorArgs) {
			serviceUrl += "?" + dojo.objectToQuery(constructorArgs);
		}
			
		
		// Add EXPLICIT_SHARE action if there is a changeset otherwise add SHARE_ON_SAVE action
		// TODO: Revisit if we decide to provide a way for the user to explicitly define
		// his committing mode (share on save, or explicit commit)
		var changeset = jazz.app.context.get().changeset;
		var actionSeparator = serviceUrl.indexOf("?") === -1?"?":"&";

		if(changeset){
			serviceUrl += actionSeparator + EXPLICIT_SHARE_ACTION;			
		} else {
			serviceUrl += actionSeparator + SHARE_ON_SAVE_ACTION;
		}
		
		
		//dojox.json.ref.toJson sometimes goes on infinite loop...test and replace with JsonRef	
		var jsonStr = rmpw.common.vcs.ref.toJson(changes, { idAttribute: "rdf_about" });
		var serverCallArgs = {
			url: serviceUrl,
			headers: {
				"Content-Type": "json",
				"X-ibm-rmps-internal": "true",
				"prefix" : this.namespaces
//				"ETag": this.eTag //uncomment once 24890 is fixed
			},
			handleAs: "text",
			hasBody: true,
			putData: jsonStr,
			load: dojo.hitch(this, function(response, ioArgs){
				delete this._isSaving;
				if(dojo.isFunction(onSave)){
					onSave(response, ioArgs, task, changeSets);					
				}
				if(dojo.isFunction(onDone)){
					onDone();					
				}
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				delete this._isSaving;
				if(task) {
					task.error(rmpw.common.util.ErrorUtil.getErrorMessage(error));
				} else {
					rmps.core.error.createError(error, messages.savingAction).dispatch();
					console.error(error);
				}
				if(dojo.isFunction(onDone)){
					onDone();					
				}
	        })
		};
		
		this._isSaving = true;
		if(isPersisted) {
			var deferred= rmps.core.client.xhrPut(serverCallArgs);
			// manually fire the unlock event, since resourceState doesn't actually track state
			if (!changeset && this.resourceState){
				this.resourceState.onUnLock();
			}
			return deferred;	
		} else {
			return rmps.core.client.xhrPost(serverCallArgs);
		}
	},
	
	_onSave: function(response, ioArgs, task, changeSets){
		if(!this.documentUri) {
			this.documentUri = ioArgs.xhr.getResponseHeader("logicalURI");
		}
		if(task) {
			task.done();
		}
//				this.eTag = ioargs.xhr.getResponseHeader("ETag"); uncomment once 24890 is fixed
		this.onSaveComplete();
//				return ioArgs.xhr.getResponseHeader("ETag");
	},
		
	getChanges: function(task){
		// Layout.saveDiagram()
		var changeSets = [];
		if (this.repository.patch) {
			changeSets.push(this.repository.patch);
		}
		changeSets.push(this.repository.stack);
		
		var changeData = new rmpw.common.vcs.ServerChangeSet({ 'changeSet': changeSets });
		
		var changes = changeData.getChanges();
				
		if (changes.length < 1) {
			if(task) {
				task.error("No changes detected.");
			}
			return changes;
		}
		return changes;
	},
	
	// hook for listeners
	onSaveComplete: function() {
		this.repository.resetStack();
	},
	
	_onDone: function() {
		this.onAllDone();
	},
	
	// hook for listeners -- called whether resource is successfully saved or not
	onAllDone: function() {
	},
	
	save: function(args, isExternal) {
		//dojox.json.ref.toJson sometimes goes on infinite loop...test and replace with JsonRef
		var urlArgs = {};
		urlArgs.types = args.rdf_type;
		var model = args.changeSet[0].arrayPush.dmcodegen_defines;
		if(args.rdf_about){
			urlArgs.logicalURI = args.rdf_about;	
		} else {
			urlArgs.rdf_type = model.rdf_type;
		}
		urlArgs.validate = !!args.validate;
		var resourceID = args.resourceID;
		if(resourceID) {
			model.rdf_about = "";
		} else {
			resourceID = "";
		}
		var jsonStr = rmpw.common.vcs.ref.toJson(args.changeSet, {idAttribute: "rdf_about"});
		var theStore = this;
		var postArgs = {
			headers: {
				"Content-Type": "json",
				"X-ibm-rmps-internal": "true",
				"prefix" : args.namespaces
			},
			url: JFS_STORE_PATH + resourceID + "?" + dojo.objectToQuery(urlArgs),
			handleAs: "text",
			hasBody: true,
			postData: jsonStr,
			load: dojo.hitch(this, function(response, ioArgs){
				var logicalURI = ioArgs.xhr.getResponseHeader("logicalURI");
				if(!isExternal) {
					this.documentUri = logicalURI;
				}
				
				//this.eTag = ioArgs.xhr.getResponseHeader("ETag");//uncomment once 24890 is fixed
				return logicalURI;
			}),
			error: function(error, ioargs) {
				 var isValidationError = false;
				 if ( error.responseText!=null ){
					 var subStatus = rmpw.common.util.ErrorUtil.getTextBetween(error.responseText, '<err:errorSubStatus', '</err:errorSubStatus>');
					 if (subStatus === 'INVALID') {
						 isValidationError = true;
					 }
				 }
                //TODO: code below is untested
                var msg = theStore._getErrorMessage(error);
				rmps.core.error.createError(isValidationError ? messages.validationError : messages.serverError, messages.savingAction )
				    .details(msg)
				    .dispatch();
	        }				
		};
		return rmps.core.client.xhrPost(postArgs,  /* sendProjectId */ true);
	},

	_getPrefixString: function(namespaces) {
		if (namespaces == undefined ){
			return undefined;
		}
		var ns = '';
		for (var property in namespaces) {
			if (namespaces.hasOwnProperty(property)) {
				if (ns == '') {
					ns += namespaces[property] + '=' + property;
				}else {
					ns += ',' + namespaces[property] + '=' + property;
				}
			}
		}
		return ns;
	}
});
})();