/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.diagrams.edit.tools.DragPan");

dojo.require("rmpw.common.policy.Policy");
dojo.require("dojox.gfx.matrix");
dojo.require("dojox.gfx.fx");
dojo.require("dojo.fx.easing");
dojo.require("rmpw.diagrams.edit.util.Utils");

/**
 * Pans the diagram when a mouse drag event isn't caught by anything else first.
 * 
 * @author Marc Gobeil
 */
dojo.declare("rmpw.diagrams.edit.tools.DragPan", rmpw.common.policy.Policy, {
	singleton: "dragPan",
	
	constructor: function() {
		this.viewport = this.host.root.viewport;
		this.container = this.viewport.container;
		this.diagramLayer = this.host.diagramLayer;
		
		this._connectListeners();
	},
	
	_connectListeners: function() {
		this.dragPanListener = dojo.connect(this.container, "onmousedown", this, "_onMouseDown");
		this.touchPanListener = dojo.connect(this.container, "touchstart", this, "_onMouseDown");
	},
	
	allowImageDragPan: function(b) {
		this._allowImageDragPan = b;
	},
	
	// TODO: the diagram loader is clearing all policies on load, so this gets destroyed right after creation
//	destroy: function() {
//		dojo.disconnect(this.dragPanListener);
//	},

	_onMouseDown: function(e) {
		this.start = {x: e.clientX, y:e.clientY};
		
		// cancel any previous move that we apparently lost track of (didn't get mouseup)
		// for example, can happen if a breakpoint is hit or exception occurs between mousedown and mouseup
		if(this.viewport.isPanning()) {
			this._onMouseUp();
			return;
		}
		
		// don't interfere with text input and buttons
		dojo.fixEvent(e);
		var isImage = e.target.tagName === "image" && !this._allowImageDragPan;
		if( e.target.tagName !== "INPUT" && e.target.tagName !== "TEXTAREA" 
			&& !isImage && rmpw.diagrams.edit.util.Utils.singleHit(e) ){
		
			var eOrig = e; 
            e = e.touches ? e.touches[0] : e;
			
			// initialize mouse tracking
			this._lastX = e.clientX;
			this._lastY = e.clientY;
			
			if( eOrig.touches ){
				this.dragEvents = [	
				    dojo.connect(document, "touchmove", this, "_onMouseMove"), 
				    dojo.connect(document, "touchend",   this, "_onMouseUp")
				];
			}else{
				// wait for drag events
				this.dragEvents = [
					dojo.connect(document, "onmousemove", this, "_onMouseMove"),
					dojo.connect(document, "onmouseup",   this, "_onMouseUp"),
					
					// cancel text selection and text dragging
					dojo.connect(document, "ondragstart",   dojo, "stopEvent"),
					dojo.connect(document, "onselectstart", dojo, "stopEvent")
				];
			}
			
			dojo.stopEvent( eOrig );
		}
	},
	
	_onMouseMove: function(e) {
		// some browsers get into state where they toss a move right after a down
		// don't pan if user is dragging a commenting tool
        if( rmpw.diagrams.edit.util.Utils.singleHit(e) && !this.diagramLayer.commenting
				&& (!this.start || this.start.x!==e.clientX && this.start.y!==e.clientY)) {
				
			// do first-move initialization
			if(!this.viewport.isPanning()) {
				dojo.style(this.container, "cursor", "move");
				this.viewport.beginPan();
			}
			
			e.preventDefault(); 
			
			var eOrig = e; 
            e = e.touches ? e.touches[0] : e;
			
			// find position delta
			var dx = e.clientX - this._lastX;
			var dy = e.clientY - this._lastY;
			
			// pan the diagram thru outline to keep outline shape in sync
			if (this.diagramLayer.outline) {
				this.diagramLayer.outline.pan(dx,dy);
			} else {
				this.viewport.pan(dx, dy);
			}
			
			// keep track of coordinates for next delta calculation
			this._lastX = e.clientX;
			this._lastY = e.clientY;
			
			dojo.stopEvent( eOrig );
		}
	},
	
	_onMouseUp: function() {
		
		// release mouse capture
		dojo.forEach(this.dragEvents, dojo.disconnect);
				
		// do last-move cleanup
		if(this.viewport.isPanning()) {
			dojo.style(this.container, "cursor", "auto");
			this.viewport.endPan();
		}
	}
});