/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmpw.dsv.editor.form.ui.ChangeMixin");

dojo.require("rmpw.common.store.JfsResource");
dojo.require("rmpw.common.vcs.JfsResourceChangeSet");
dojo.require("rmpw.common.vcs.Repository");
dojo.require("rmpw.common.vcs.ChangeSet");
dojo.require("rmpw.common.vcs.ChangeSetWrapper");
dojo.require("rmpw.common.util.ChangeUtil");

/**
 * Used as a mixin for different classes to set the default change, and change set.
 * 
 */
dojo.declare("rmpw.dsv.editor.form.ui.ChangeMixin", null, {
	jfsResource: null,
	target: null,
	changeSet: null,
	repository: null,
	namespaces: null,
	
	addTypes: function(types) {
		if(!types){
			return;
		}
		
		var target= this.getTarget();
		if(!target){
			return;
		}
		if(!dojo.isArray(types)){
			types= [types];
		}
		if(!target["rdf_type"]){
			target["rdf_type"]= [];
		}
		target["rdf_type"]= target["rdf_type"].concat(types);
	},
	
	getTypes: function(){
		var target= this.getTarget();
		if(!target){
			return [];
		};
		var types= target["rdf_type"];
		if(!types){
			return [];
		}
		return dojo.map(types, function(entry){return entry.rdf_about});
	},
	
	setJfsResource: function(jfsStore){
		this.jfsResource = new rmpw.common.store.JfsResource(jfsStore);
	},
	
	setJfsResourceExisting: function(jfsResource){
		this.jfsResource = jfsResource;
	},		
	
	getJfsResource: function(jfsStore){
		return this.jfsResource;
	},
	
	setNamespaces: function(namespaces){
		this.namespaces= namespaces;	
	},
	
	getNamespaces: function(){
		return this.namespaces;
	},
	
	setChangeDetails: function(target, repository, isDiagramPopup){
		this.setRepository(repository);
		this.setTarget(target, isDiagramPopup);
	},
		
	setRepository: function(repository){
		if(!repository){
			repository = new rmpw.common.vcs.Repository();
		}
		this.repository = repository;
	},
	
	getRepository: function(){
		return this.repository;
	},
	
	isImmediate: function() {
		if (this.immediate !== undefined) {
			return this.immediate;
		}
		
		if (this._resource) {
			this.immediate = this.target == this._resource.formObject;
		}
		
		return this.immediate;
	},
	
    createChange: function(property, value) {
    	var change = this.createUncommittedChange(property, value);
    	
    	// Change is undefined in case no actual change is there.
    	// Only commit change right away if this is the top level form.
    	if(change && this.isImmediate()) {	
			this.getRepository().commit(this.changeSet);    		
    	}
    },
    
    createUncommittedChange: function(property, value) {
		var currentValue = this.target[property];
		
		if(rmpw.common.util.ChangeUtil.valueEqual(currentValue, value)){
    		return;
    	}
		
		// special case for identifier - since it may be set to the short version in the form and we don't want
		// the form to get dirtied when it's fixed
		if (property == "rdf_about" && this.isIdentifierFixed) {
			return;
		}
		
    	var change= this.changeSet.createChange(this.target)
    	
    	//Required for cases such as when a text box accepted value is set to integer, as in this case,
    	// the value will be NaN. So, mak sure to detect this case, and also avoid setting values like
    	// empty string, false boolean, or number "0" to undefined.
    	// It is better to put this check in this method as it is used by all places, and having
    	// a possiblity to set a property to "NaN" will created a unprocessable json to be sent to the 
    	// server, and causes the whole update to fail. 
    	if(typeof value!=="boolean"&& !value&& !dojo.isString(value)&& value!== 0){
    		value= undefined;
    	}
    	
    	change.set(property, value);
    	
    	return change;
    },
    
    applyChange: function(invoker, immediate){
    	//No need to apply redundant check on the invoker being a function. This is the responsability for the callers
    	if(this.immediate || immediate){
    		var changeSet = new rmpw.common.vcs.ChangeSet();
	    	invoker(changeSet.createChange(this.target));
	    	this.getRepository().commit(changeSet);	
    	}else{
	    	invoker(this.changeSet.createChange(this.target));
    	}
    },
    
    applyJfsResourcesChange: function(resourcesSet){
		/*resourceSet must be an instance of array. check for arrays to be done by callers */    	
    	this.getRepository().commit(new rmpw.common.vcs.JfsResourceChangeSet(this.getJfsResource(), resourcesSet));	
    },
    
    getTarget: function(){
    	return this.target;
    },
    
    get: function(property){
    	return this.getTarget()[property];
    },
    
    setTarget: function(target){
    	if(this.target){
    		return; //one time setter
    	}
    	this.target = target || {};
		
    	if (!this.isImmediate()) {
    		this.changeSet = new rmpw.common.vcs.ChangeSet(this.target);
    	} else {
	    	this.changeSet = new rmpw.common.vcs.ChangeSetWrapper(this.target);
    	}
    }
});

})();