/*******************************************************************************
 * Licensed Materials - Property of IBM Copyright IBM Corp. 2011, 2012. All
 * Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
(function() {
dojo.provide("rmpw.dsv.editor.form.ui.CreateResource");

dojo.require("jazz.ui.Dialog");
dojo.require("rmpw.dsv.editor.form.ui.ModelValidationUtil");
dojo.require("rmpw.dsv.editor.form.ui.CreateDataType");
dojo.require("rmpw.common.util.FormsLauncher");
dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.requireLocalization("rmpw.dsv.editor.form.ui", "CreateResourceMessages");
var createResourceMessages = dojo.i18n.getLocalization("rmpw.dsv.editor.form.ui", "CreateResourceMessages");

/**
 * 
 * //FIXME: Must extend an instance of _Widget
 * 
 * Create Resource from a form dialog.
 * 
 * @author Sameh Ayoub
 */
dojo.declare("rmpw.dsv.editor.form.ui.CreateResource", null, {

	listeners: null,
	featureId: null,
	constructor: function() {
		this._sources = {};
		this.listeners = [];
	},
	
	getObjectTypes: function(args) {
		var objectTypes = [];
		if (args.target && args.target.rdf_type) {
			for (var i=0, lg=args.target.rdf_type.length; i<lg; i++) {
				objectTypes.push(args.target.rdf_type[i].rdf_about);
			}
		}
		else if (args.parent.typeUri) {
			objectTypes.push(args.parent.typeUri);
		}
		
		return objectTypes;
	},
	
	isEditMode: function(args) {
		if (args.target && args.target.rdf_about) {
			return true;
		}
		
		return false;
	},
	
	createResource: function(typeName, typeForm, defaultUri, defaultLabel, parent, featureId, target, isFragment){
		this._isEditing = this.isEditMode({parent: parent, target: target});
		this.typeName = dojo.string.substitute(this._isEditing ? createResourceMessages.editType : createResourceMessages.createType, [typeName]);
		
		rmpw.common.util.FormsLauncher.loadForm({ 
				formName: typeForm,
				objectTypes: this.getObjectTypes({parent: parent, target: target}),
				readOnly: parent.form ? parent.form._resource.page.isReadOnly : false,
				standAlone: false,
				editor: parent.form ? parent.form._resource.editor : parent.host.root.definition.uri,
				create: !target|| !target.rdf_about
			}, 
			dojo.hitch(this, dojo.hitch(this, this._show, typeForm, parent, defaultUri, featureId, target, isFragment)),
			function(error){rmps.core.error.create(error).dispatch();}
		);
	},
	
	_show: function(typeForm, parent, defaultUri, featureId, target, isFragment) {
	    var rdfTypes = this.getObjectTypes({parent: parent, target: target}),
	       template,
	       unique;
	    
		this._form = new rmpw.dsf.codegen.properties.forms[typeForm](rdfTypes);
		this._form.isFragment = isFragment;
		// we can create other documents from here, set JfsResource
		if (parent.form) { 
			this._form.setJfsResourceExisting(parent.form.getJfsResource());
		}	
		
		
		
		//FIXME: setResource must be removed. It's redundant data
		if(this._form._setResource){
			this._form._setResource({
				documentUri: parent.form ? parent.form._resource.documentUri : parent.host.root.diagram.getDefaultSource().rdf_about,
				namespaces: parent.form ? parent.form._resource.namespaces : [],
				objectTypes: rdfTypes,
				modelIndex: rmps.ui.editor.PageHelper.getInstance().getActiveEditorInput().resourceInfo.modelIndex,
				page: parent.form ? parent.form._resource.page : {},
				editor: parent.form ? parent.form._resource.editor : parent.host.root.definition.uri,
				formObject: parent.form ? parent.form._resource.formObject : parent.host.root.diagram.getDefaultSource(),
				id: target ? target.rdf_about : null
			});
			
			//TODO: owl_oneOf should have its own template, with the section below being part of its logic
			//adding a One Of is only enabled once the resource has been saved 
			//(i.e when this._form._resource.id stores the resource's URI)
			if( rmpw.dsv.editor.form.ui.ModelValidationUtil.isUri(this._form._resource.id) ){
			    template = this._form.template;
                for(unique in template){
                    if(template.hasOwnProperty(unique) && template[unique].label === "owl_oneOf"){
                        template[unique].addCreationActions("resource saved");
                    }
                }
			}
		}
		
		//TODO: Must provide a setter instead
		this._form.featureId= featureId;
		
		if (this._isEditing){
			this._form[this._form.setterNames[typeForm]](target);	
			this._completeShow(typeForm, parent, defaultUri, featureId, target);
		}else{
			var self = this;
			rmps.core.client.xhrGet({
				url: rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "rdfdirectory") + "/constructor",
				content: { rdf_type: rdfTypes },
				headers: { "Accept": "application/x-rdf+json-graph" },
				handleAs: "text",
				load: function(data, ioargs) {
					
					// process json for initial model
					if(data === null || data == '') {
						rmps.core.error.create("Failed to load initial model").dispatch();
						return;
					}
					
					var documents = rmpw.common.vcs.ref.fromJson(data, { idAttribute: "rdf_about" });
					var model = documents[0];
					// TODO: shouldn't have to assign this, constructor service should provide correct uri
					model.rdf_about = self.defaultUri;
					self._form[self._form.setterNames[typeForm]](model);
					self._form.immediate= true;
					self._completeShow(typeForm, parent, defaultUri, featureId, model);
				}
			});
		}
	},
	
	_completeShow: function(typeForm, parent, defaultUri, featureId, target) {
		if (parent.host) {
			// assume diagram pop-up
			this._form.setChangeDetails(target, parent.host.root.repository, true);
			parent._commandController= {
					commandStack: parent.host.root.getCommandController().commandStack
				};
		}
		else if (parent.typeUri && parent.typeLabel) {
			if (this._isEditing) {
				this._form.setChangeDetails(target, parent.form.repository, false);
			}
			else {
				this._form.setChangeDetails(/*Change with null repository, as it is used only to track changes but does not commit */);
			}
			parent._commandController= {
					commandStack: this._form.repository
				};
		}
		
		this._form._setRoot(parent);
		
		this._form.startup();
		var form = this._form.template.form ? this._form.template.form : this._form;		
        this.selectTypeDialog = new jazz.ui.Dialog({
    		primaryTitle: this.typeName,
            secondaryTitle: "",
            width: "650px",
            onClose: dojo.hitch(this, this.onClose),
            contentNode: form.domNode
        });
        this._form.domNode.style.padding= "6px";
		this.listeners.push(dojo.connect(this._form, "onSubmit", this, "add"));
		this.listeners.push(dojo.connect(this._form, "onCancel", this, "onCancel"));	
	},
	
	add: function() {
		this.onSubmit(this._form.getTarget());
		this.selectTypeDialog.close();
		this._form.setChangeDetails();
	},
	
	onCancel: function(){
		this.selectTypeDialog.close();
	},
	
	onSubmit: function(createdResource){
		/*To be called by other places */
	},
	
	onClose: function() {
		dojo.forEach(this.listeners, dojo.disconnect);
		this.listeners = [];
	}
});
})();