/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.dsv.editor.behavior.ui.EditorPage");

dojo.require("dijit.TitlePane");
dojo.require("dijit.layout._LayoutWidget");
dojo.require("dijit.layout.BorderContainer");
dojo.require("dojox.json.ref");

dojo.require("rmpw.dsv.editor.behavior.EcoreModel");
dojo.require("rmpw.dsv.editor.behavior.ui.Toolbar");
dojo.require("rmpw.dsv.editor.behavior.ui.NewModelPane");
dojo.require("rmpw.dsv.editor.behavior.ui.ExistingModelPane");
dojo.require("rmpw.dsv.editor.behavior.ui.BundleEditor");

/**
 * Page wrapper widget for the behavior model editor.
 * 
 * @author Marc Gobeil
 */
dojo.declare("rmpw.dsv.editor.behavior.ui.EditorPage", dijit.layout.BorderContainer, {
	postCreate: function(){
		this.inherited(arguments);
		var ActionRegistry = net.jazz.ajax.ui.PlatformUI.getWorkbench().getActionRegistry();
		ActionRegistry.registerAction("rmpw.dsv.editor.behavior.startup", this, "startup"); 

		// listen for resizes
		this.resizer = jazz.util.ResizeNotifier.register({
			context: this,
			callback: this._onResize,
			force: true    	        
		});

		this.models = {};
		
		// TODO (MG): prevent race condition with loading a behavior bundle
    	// load tools
    	dojo.xhrGet({
			url: net.jazz.ajax._contextRoot + "/resource/itemPath/Model/API/Edit Model",
			handleAs: "text",
			load: dojo.hitch(this, function(data) {
				this.models.behavior = new rmpw.dsv.editor.behavior.EcoreModel({
					ePackage: dojox.json.ref.fromJson(data)[0]
				});
			}),
			error: function(error) {
				console.error(error);
			}
		});		
		
    	// load tools
    	dojo.xhrGet({
			url: net.jazz.ajax._contextRoot + "/resource/itemPath/Model/API/Edit Tool Model",
			handleAs: "text",
			load: dojo.hitch(this, function(data) {
				this.models.tools = new rmpw.dsv.editor.behavior.EcoreModel({
					ePackage: dojox.json.ref.fromJson(data)[0]
				});
			}),
			error: function(error) {
				console.error(error);
			}
		});

    	// load actions
    	dojo.xhrGet({
			url: net.jazz.ajax._contextRoot + "/resource/itemPath/Model/API/Edit Action Model",
			handleAs: "text",
			load: dojo.hitch(this, function(data) {
				var actionModel = new rmpw.dsv.editor.behavior.EcoreModel({
					ePackage: dojox.json.ref.fromJson(data)[0]
				});
				this.models.actions = dojo.filter(actionModel.getSubClassesOf("Action"), function(eClassifier) {
					return !eClassifier["abstract"]; // abstract is a reserved word
				});				
			}),
			error: function(error) {
				console.error(error);
			}
		});
    	
    	// load action bindings
    	dojo.xhrGet({
			url: net.jazz.ajax._contextRoot + "/resource/itemPath/Model/API/Edit Action Binding Model",
			handleAs: "text",
			load: dojo.hitch(this, function(data) {
				var actionModel = new rmpw.dsv.editor.behavior.EcoreModel({
					ePackage: dojox.json.ref.fromJson(data)[0]
				});
				this.models.actionBindings = dojo.filter(actionModel.getSubClassesOf("Action"), function(eClassifier) {
					return !eClassifier["abstract"]; // abstract is a reserved word
				});				
			}),
			error: function(error) {
				console.error(error);
			}
		});    	
	},
    
    startup: function() {
    	this.inherited(arguments);

    	var toolbar = new rmpw.dsv.editor.behavior.ui.Toolbar({
    		region: "top",
    		style: "height: 20px",
    		page: this
    	});
    	this.addChild(toolbar);
    },
    
    _onResize: function() {
    	var size = dojo.contentBox(dojo.byId('content-cell'));
    	this.resize(size);
    }
});