/*******************************************************************************
 * Licensed Materials - Property of IBM Copyright IBM Corp. 2012. All
 * Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
(function(){
dojo.provide("rmpw.dsv.editor.form.ui.FileReferencesActionsTable");

dojo.require("rmpw.dsv.editor.form.ui.ReferencesActionsTable");
dojo.require("rmps.app.ui.internal.pages.design.DesignUtil");
dojo.require("rmps.core.util.IRmpsConstants");

//Load async modules
var FileUploader= "rmpw.dsv.editor.form.ui.FileUploader";
var imageLocationLookup = net.jazz.ajax._contextRoot + "/fileUpload/locationFor?resourceUri=";

/* 
 * Advanced ReferencesActionsTable to support file/image upload. Used in two cases:
 * - For file upload form that allows the file upload. In this case, isFileUploadForm is true.
 * - For forms opened from a File or Image reference property. 
 */
dojo.declare("rmpw.dsv.editor.form.ui.FileReferencesActionsTable", rmpw.dsv.editor.form.ui.ReferencesActionsTable, {
	
	isImage: null,
	isFileUploadForm: null,
	fileUploaderMessages: null,
	
	postMixInProperties: function(){
		this.inherited(arguments);
		
		
		//FIXME: Should have a message file for this class only
		dojo.requireLocalization("rmpw.dsv.editor.form.ui", "FileUploaderMessages");
		this.fileUploaderMessages = dojo.i18n.getLocalization("rmpw.dsv.editor.form.ui", "FileUploaderMessages");
	},
	
	isSelectCreatePicker: function() {
		return this.form.getJfsResource();
	},
	
	isCreatePicker: function() {
		// it is never the case for image selection to just be able to create new images
		return false;
	},
	
	getPresentationUri: function(about){
		return rmps.app.ui.internal.pages.design.DesignUtil.getFileUri(about|| this.getFocusReference());
	},
	
	getDialogSecondaryTitle: function(){
		return "";
	},
	
	getActionDescriptors: function(){
		if(this.isFileUploadForm){
			return [{actionCallback: dojo.hitch(this, this.editRefrence), 
					args: {imgClass: 'edit', 
					       title: this.getFocusReference() ? this.fileUploaderMessages.openReplaceFile : this.fileUploaderMessages.upload}}];
		}
		return this.inherited(arguments);
	},
	
	getSelectTypeClassConstructor: function(callback){
		if(!this.isFileUploadForm){
			return this.inherited(arguments);
		}
		jazz.core.loader.batch_load_async([FileUploader], function() {
			callback(dojo.getObject(FileUploader));
		});
	},
		
	getDialogTitle: function(){
		if(!this.isFileUploadForm){
			return this.inherited(arguments);
		}
		if (this.getFocusReference()) {
			return this.readOnly? this.fileUploaderMessages.openFile: this.fileUploaderMessages.openReplaceFile;
		} else {
			return this.isImage? this.fileUploaderMessages.uploadImage: this.fileUploaderMessages.uploadFile;
		}
	},
	
	getSelectTypeArguments: function(dialogGetter, row){
		if(!this.isFileUploadForm){
			return this.inherited(arguments);
		}
		return {
			isImage: this.isImage,
			frontSideUri: this.getFocusReference(),
			onOK: dojo.hitch(this, function(uri, label){
				//The way we save new images/files is by uploading the file/image, and then commit and save the current ontology right away.
				//So, if the current editor is not dirty yet, this means that the resource is still unlocked; otherwise, it is locked.
				//If the file/image is unlocked, we have to be carefull with the current sequnce of event
				//1) When making a change to update the label, this will go through a deferred call (as it is the first time)
				//2) When making a change to update the uri field, this will also go through another deferred call (the first deferred is not answered)
				//3) And then the resource state call for locking the current resource will come. This call is also deferred. We can get into a situation that
				//   the first deferred call is answered and then the resource is unlocked and then the next deferred call is answered, or it could be in any different
				//   order. So, we have to make sure to set our listenrs as follows:
				// If the editor is not dirty. Then, we put a listener on the method "afterCommit". Make sure that it will be called twice ( a call for eahc call).
				// Then, we have to check and see if the editor is not dirty (as na indication that the model was not orginally locked). If it is not dirty, then, we
				// we have to put a listener on the resource state after lock method. And after that, we are fine to save and referesh the new ontology
				// If the editor is dirty. Then, we will only need to listen to the two calls for afterCommit, and then save an continue.
				// For the existing image/file changes, we follow the exact procedure, but the difference here is in behvaior. We only update the thumbnail, and
				// do not close the dialog
				//A flag to deetermine if the first commit call is invoked
				if (!this.form.featureId) {
					// when !this.form.featureId it means that this is not a popup image document whose purpose is 
					// to set the parent form's reference (e.g. Icon URI)
					if (!this.getFocusReference()) {
						var tried = false;
						var onCommit = this.connect(this.repository, "afterCommit", function(change){
							//Used to indicate if the second field has been updated as well, so we make sure that the target is updated with both values
							//when tried is true
							if (tried) {
								var doSave = dojo.hitch(this, function(){
									this.disconnect(onCommit);
									// save the form so we don't lose the reference to the uploaded file								
									this.form._resource.page.editor.save();
									dialogGetter().close();
								});
								//If editor is not dirty, then, we have to make sure that we will not save till the resource is locked, so, we need
								//to listen to the resource state till it becomes unlocked
								var resourceState = this.form._resource.page.editor.editorInput.store.resourceState;
								if (!this.form._resource.page.editor.isDirty()) {
									var onLock = this.connect(this.form._resource.page.editor.editorInput.store.resourceState, "onLock", dojo.hitch(this, function(change){
										this.disconnect(onCommit);
										//Now, we can save. Both fields are updated, and the resource state is locked
										doSave();
									}));
								}
								else {
									//Save directly. Both fields are saved, and resource was already locked
									doSave();
								}
								
							}
							
							tried = true;
						});
					}
				}
				//TODO: Should be revisited with 26815: Changing values and saving in one form editor does not update the contents of another form editor
				this.form.createChange("rdfs_label", label);
				this.form.createChange("dcterms_title", label);
					
				dojo.publish("dsf-widget-update-" + this.form.id + "rdfs_label", [label]);
				dojo.publish("dsf-widget-update-" + this.form.id + "dcterms_title", [label]);
				
				this.onSetReference({target: uri, row: row});
				if (this.form.featureId) {
					dialogGetter().close();
				}
			}),
			onCancel: function(){dialogGetter().close()},
			isReadOnly: this.readOnly,
			parentForm: this.form,
			onOk: dojo.hitch(this, this.onSubmit)	
		};
	},
	
	getTargetRdfAbout: function(logicalUri, target) {
		if (target.dmfile_location) {
			return target.dmfile_location;
		}
		return this.inherited(arguments);
	},
	
	onSetReference: function(args){
		if (args.target && args.target.rdf_about && args.target.rdf_about.indexOf("fileUpload") === -1) {
			// target points to the image document, we need image location - go get it
			var self = this;
			var lookupArgs = {
				url: imageLocationLookup + args.target.rdf_about,
				load: dojo.hitch(this, function(location, ioargs){
					args.target.rdf_about = location ? location : "fileUpload"; // shouldn't happen that location is not returned
					self.onSetReference(args);
				})
			};
			rmps.core.client.xhrGet(lookupArgs, false);
		} else {
			this.inherited(arguments);
		}
	}
	
});

})();