/*******************************************************************************
 * Licensed Materials - Property of IBM Copyright IBM Corp. 2012. All
 * Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/

/**
 * Utility object for domain validation.
 * 
 * @author Wayne Diu
 */
(function(){
	
	dojo.provide("rmpw.dsv.editor.form.ui.ModelValidationUtil");

	dojo.require("dojox.json.ref");
	dojo.requireLocalization("rmpw.dsv.editor.form.ui", "ValidationMessages");

	var messages = dojo.i18n.getLocalization("rmpw.dsv.editor.form.ui", "ValidationMessages");
	var util = rmpw.dsv.editor.form.ui.ModelValidationUtil;
	var RDF_DIRECTORY_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "modelvalidation") + "/prevalidate";
	var activeDlg; //warning: not safe for multiple dialogs (split into separate class if desired)
	
	dojo.mixin(util, {
		
		isUri: function(string) {
        	var regexp = /(ftp|http|https):\/\/(\w+:{0,1}\w*@)?(\S+)(:[0-9]+)?(\/|\/([\w#!:.?+=&%@!\-\/]))?/;
        	return regexp.test(string);
    	},
	
		validate : function(ontology) {

			var theUrl = RDF_DIRECTORY_PATH;
			var baseline = jazz.app.context.get().baseline;
			if (baseline) {
				theUrl += "?rmps.context=" + encodeURIComponent(baseline);
			}
			
			var args = {
				headers: {
					"X-ibm-rmps-internal": "true",
					"Content-Type": "json"						
				},
				
				handleAs: "json",
				url: theUrl,
				postData :  dojox.json.ref.toJson(ontology),
				hasBody: true,
				load: dojo.hitch(this, function(response, ioArgs) {
					return response;
				})
			};
			
			return jazz.client.xhrPost(args);
		},
	
		showValidationError : function(msg) {
			var root =  new dijit.layout.BorderContainer({
				region: "center",
				style: "margin-left: 16px; height:100%;"
			});
			this.root = root;
			
			var dialogMessageNode = dojo.create("div", { innerHTML: "<p>" + msg + "</p>" });
			root.domNode.appendChild(dialogMessageNode);
			
			var buttonArea = new dijit.layout.BorderContainer({
				region: "bottom",
				style: "height: 30px; width: 100%;"
			});
			
			root.addChild(buttonArea); 			

			var okButton = new dijit.form.Button({
				spanLabel: true, 
				label: "OK",
				onClick: dojo.hitch(this, function(response, ioArgs) {
					this.activeDlg.close();
				}),
				layoutAlign: "right"
			});
			buttonArea.addChild(okButton);
			
			this.activeDlg = new jazz.ui.Dialog({
				primaryTitle: messages.validationError,
	            destroyContents: true,
				contentNode: root.domNode
			});
		},
		
		showValidationCode : function(code) {
			this.showValidationError(this.getMessageForCode(code));
		},
		
		getMessageForCode : function(code) {
			var stringResult = "";
			if (code.indexOf("INVALID_IDENTIFIER") > -1) {
				stringResult += messages.invalidIdentifier;
			}
			if (code.indexOf("DESCRIBE_EXISTS") > -1) {
				if (stringResult.length > 0) {
					stringResult += "\n";
				}
				stringResult += messages.validationDescribeExists;
			}
			if (code.indexOf("SELECT_EXISTS") > -1) {
				if (stringResult.length > 0) {
					stringResult += "\n";
				}
				stringResult += messages.validationSelectExists;
			}
			if (code.indexOf("NSPREFIX_EXISTS") > -1) {
				if (stringResult.length > 0) {
					stringResult += "\n";
				}
				stringResult += messages.validationNsPrefixExists;
			}
			
			//if stringResult.length is 0, this is an unrecognized code
			return (stringResult.length == 0) ? messages.validationError : stringResult;
		},
		
		closeDialog : function (){
			if (this.activeDlg) {
				this.activeDlg.close();
				this.activeDlg = null;
			}
		},
		
		getAlreadyExistsMessage : function(id) {
			return dojo.string.substitute(messages.alreadyExists, [id]);
		},

		/*
		 * Called prior to saving. If a validation error is present
		 * which will prevent save from completing then clients should
		 * populate push the error reason (String|Error) into the errors array.
		 * Client may also push a Deferred object to defer the validation. This allows
		 * the client to communicate with the server to perform validation. To add a validation
		 * failure the deferred must either perform an error call back or push errors into
		 * the errors array.
		 */
		validatePopup: function(form, /*Array*/errors) {
			if (form && form.template) {
				var validationFailed = false;
				
				//ideally, we would use form.validate(); and form.isValid();
				//however, the getDependents() are not populated since the top level
				//form is used only as a placeholder
				for (var key in form.template) {
					var item = form.template[key];
					if (item instanceof dijit.form.ValidationTextBox) {
						item._hasBeenBlurred = true;
						if (!item.validate()) {
							//in order to highlight all fields, do not return now
							validationFailed = true;
						}
					}
				}
				
				if (validationFailed) {
					//consider adding specific invalid details message to this message
					errors.push(messages.pleaseCorrect);
				}
				//for now, only creation supports server side validate
				if (form.onValidate) {
					return form.onValidate();
				}
			}
		}		
		
	});
    
})(); // end function()
