/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.dsv.editor.domain.ui.ModelingPage");

dojo.require("rmpw.jazz.ModelingPage");
dojo.require("rmpw.dsv.editor.domain.ui.Configuration");
dojo.require("rmpw.dsv.editor.domain.ui.DiagramWidget");
dojo.require("rmpw.jazz.ui.DrawingTools");

(function(){
	var controls = rmpw.jazz.ui.Controls.items;
    dojo.declare("rmpw.dsv.editor.domain.ui.ModelingPage", [ dijit._Widget, dijit._Templated ],	{

		tools: [{ name: controls.ZOOM, display: true, enabled: true },
				{ name: controls.UNDO, display: true, enabled: true },
				{ name: controls.REDO, display: true, enabled: true }],
   	
     	 templateString: "<div dojoAttachPoint='containerNode'>" + 
     	 					"<div dojoAttachPoint='canvas'/>" +
     	 					"<div dojoAttachPoint='viewableSketches'/>" +
     	 					"<div dojoAttachPoint='editableSketches'/>" +
     	 		         "</div>",
     	 		         
         handler: null,
        
    	 prefix: "rmpw.dsv.editor.domain",
		
		 startup : function() {
			
    		this.inherited(arguments);
    		
    		var resourceInfo = this.editor.editorInput.resourceInfo
			if( resourceInfo ){
				this.fetchDiagram(resourceInfo);
			}
    	 },
    	 
    	 fetchDiagram: function(resourceInfo) {
	 		this.viewer.diagramLayer.viewer.repository = resourceInfo.repository;
	 		this.viewer.diagramLayer.viewer.definition = this.definition;
			var diagramStore = new this.configuration.store( this.viewer.diagramLayer.viewer );
			diagramStore.fetch(resourceInfo, this.definition, this.viewer);
    	 },
    	 
    	 /* Hook for being tab content in Design page */
    	 
    	 doSave: function(/*rmps.core.TaskMonitor*/monitor) {
			this.layout.saveDiagram();
    	 },
    	 
    	 /* Hook for being tab content in Design page */
		
    	 doRefresh: function() {
			this.layout.loadDiagramContent(this.editor.editorInput.resourceInfo.uri);
    	 },
		
    	 initializeToolBar: function(
				/* jazz.ui.SimpleToolbar */ toolbar,
				/*function*/setDirty,
				/*function*/enableRefresh) {
			/*
			 * Hook for being tab content in Design page
			 * @param tooblar
			 * 					is passed so extra buttons can be added if required
			 * @param setDirty(bool?) 
			 * 					is a function that should be called when the state of the
			 * 					Diagram is dirtied or un-dirtied.
			 * @param enableRefresh(boolean)
			 * 					is a function to call to allow the refresh button to be
			 * 					enabled or disabled
			 */
			
//			this.layout["_setDirty"] = setDirty;
//			this.layout["enableRefresh"] = enableRefresh;
    	 },
		
		/*
		* Called prior to saving. If a validation error is present
		* which will prevent save from completing then clients should
		* populate push the error reason (String|Error) into the errors array.
		* Client may also push a Deferred object to defer the validation. This allows
		* the client to communicate with the server to perform validation. To add a validation
		* failure the deferred must either perform an error call back or push errors into
		* the errors array.
		*/
		validate: function(/*Array*/errors) {
		
		},
		
	    initializeDiagram: function(){
	    	 	    	 
	    	 this.configuration = new rmpw.dsv.editor.domain.ui.Configuration(),
	     	 this.commentConfig = new rmps.graphics.ui.Configuration();
	     	 this.selectionConfig = new rmps.graphics.ui.Configuration();
	     	
	 	     dojo.addClass( this.domNode, 'rmps-graphics-diagram-layout' );
	 	    
	 	     this.domNode.style.overflow = "hidden";

	     	 this.configuration.canvas = this.canvas; 
	     	 this.commentConfig.canvas = this.editableSketches; 
	     	 this.selectionConfig.canvas = this.viewableSketches;  	
	     	
	     	 this.viewer = new rmpw.dsv.editor.domain.ui.DiagramWidget({
	 			configuration: this.configuration,
	 			commentConfig: this.commentConfig,
	 			selectionConfig: this.selectionConfig,
	 			commentingWidget: this.editor.commentingWidget,
	 			editor: this,
	 			resourceInfo: this.editor.editorInput.resourceInfo,
				tools : this.tools
	 		 });
	     	 
	     	this.selectionController = this.viewer.sketchLayer.viewer.getSelectionController();
		
	    	this.commandController = this.viewer.sketchLayer.viewer._commandController;	
//			this.commandController.modelingPage = this.viewer.toolsManager;
			
			this.resetOutlineListeners = dojo.connect( this.viewer.diagramLayer.viewer, "setDiagram", this, "setDiagram" );    	
	     },

		setDiagram: function(){
		},
		
	     postCreate: function() {

	    	 this.initializeDiagram();
	    	 
	    	 var s = dojo.marginBox( this.domNode );
	    	 this.containerNode.appendChild( this.viewer.domNode );
	    	 
	    	 this.viewer.startup();    		    	 
	     },
		
	     resize: function(size){
	    	 	    	 
	    	 if( size != null ){

	    		 var rootSize = {};
	 	 		
	 	 		 if (size.w) {
	 	 			rootSize.w = size.w;
	 	 		 }
	 	 		 
	 	 		 if (size.h) {
	 	 			this.domNode.style.height = size.h + "px";
	 	 			rootSize.h = size.h;
	 	 		 }
	 	 		 
	 	 		 if ((!size.w || size.w>0) && (!size.h || size.h>0)) {
	 	 			 this.viewer.resize( size );
	 	 		 }
	    	 }
	     }
    });
    
})();