/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.dsv.editor.behavior.ui.NewModelPane");

dojo.require("dijit.form.Form");
dojo.require("dijit.form.ComboBox");

dojo.require("dojo.data.ItemFileReadStore");
dojo.require("dojox.json.ref");

dojo.require("rmpw.dsv.editor.behavior.ui.ControllerEditor");

/**
 * Widget for editing the root of a behavior model that extends a
 * render mapping with interactive components.
 * 
 * @author Marc Gobeil
 */
dojo.declare("rmpw.dsv.editor.behavior.ui.NewModelPane", dijit.form.Form, {
	widgetsInTemplate: true,
	templateString:
		"<div dojoAttachPoint='containerNode'>" +
			"<table>" +
				"<tr>" +
					"<td>Visualization</td>" +
					"<td><input dojoAttachPoint='renderSelector'/></td>" +
				"</tr>" +
				"<tr>" +
					"<td>Inject Defaults</td>" +
					"<td><input dojoType='dijit.form.CheckBox' type='radio' name='defaults' value='true' disabled='true'/></td>" +
				"</tr>" +
				"<tr>" +
					"<td>&nbsp;</td>" +
					"<td><button dojotype='dijit.form.Button' type='submit'>Create</button></td>" +
				"</tr>" +
			"</table>" +
		"</div>",
	
	postCreate: function() {
		var renderMappingStore = new dojo.data.ItemFileReadStore({
			url: "resource/itemCol/Model?domain=qvtr",
			clearOnClose: true,
			urlPreventCache: true
		});
		 
		var renderSelector = new dijit.form.ComboBox({
			store: renderMappingStore,
	        searchAttr: "path",
	        name: "renderMappingPath"
		}, this.renderSelector);
	},
		
	onSubmit: function(e) {
		this.onRequest();
		
		var form = this.attr("value");
		var generateDefaults = form.defaults && form.defaults[0];
		
		if(generateDefaults) {
			// TODO (MG): implement
		} else {
			dojo.xhrGet({
				url: net.jazz.ajax._contextRoot + "/resource/itemPath/Model" + form.renderMappingPath,
				handleAs: "text",
				load: dojo.hitch(this, function(data) {
					// parse render model
					var transform = dojox.json.ref.fromJson(data)[0];
					
					// Initialize default behavior model
					// TODO: have option to generate a better one on the server
					var bundle = {
						renderMapping: transform,
						controller: []
					};
					var relations = transform.rule;
					for(var i = 0, relation; relation = relations[i]; i++) {
						bundle.controller.push({
							shape: relation,
							actions: [],
							tools: [],
							actionBindings: []
						});
					}
					
					this.onLoad(bundle);
				}),
				error: function(error) {
					console.error(error);
				}
			});
		}
	},
	
	resize: function() {
		// dummy to satisfy parent
	}
});