/*******************************************************************************
 * Licensed Materials - Property of IBM Copyright IBM Corp. 2011, 2012. All
 * Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
(function(){
dojo.provide("rmpw.dsv.editor.form.ui.TableChangesMixin");

dojo.require("jazz.ui.ResourceLink");
dojo.require("rmps.ui.CollapsibleSection");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("rmps.ui.ActionsTable");
dojo.require("rmps.core.util.TypeConstants");
dojo.require("rmpw.dsv.editor.form.ui.CreateResource");
dojo.require("rmps.core.resource.ResourceInfoClient");
dojo.require("rmps.app.ui.internal.pages.design.DesignUtil");
dojo.require("rmpw.common.util.FormsLauncher");
dojo.require("rmps.app.core.util");
dojo.require("rmpw.common.util.IdentifierUtil");
dojo.require("rmpw.dsv.editor.form.ui.ModelValidationUtil");
dojo.require("dojox.json.ref");
dojo.require("rmps.core.resource.ResourceInfoClient");

var TypeConstants= rmps.core.util.TypeConstants;
var DesignUtil= rmps.app.ui.internal.pages.design.DesignUtil;
var ResourceInfoClient= rmps.core.resource.ResourceInfoClient;
var ModelValidationUtil= rmpw.dsv.editor.form.ui.ModelValidationUtil;
var InfoClient = rmps.core.resource.ResourceInfoClient;


//Load async modules
var SelectCreatePicker= "rmpw.dsv.editor.form.ui.SelectCreatePicker";
var CreatePicker= "rmpw.dsv.editor.form.ui.CreatePicker";
var SelectionsPicker= "rmps.app.pickers.internal.SelectionsPicker";
var ClipboardWidget= "rmps.ui.clipboard.ClipboardWidget";
var RMPSPicker= "rmps.app.pickers.internal.RMPSPicker";
var QuickSearchSelection= "rmpw.dsv.editor.form.ui.QuickSearchSelection";
var ResourcesSelection= "rmpw.dsv.editor.form.ui.ResourcesSelection";

var DEFAULT_DIALOG_WIDTH= "700px";
var LOCALE_FORMAT_OPTIONS = {selector: "date", fullYear: true};
/**
 * Designed as a mixin object for table creations (in ListSection, and ReferencesActionTable, etc)
 */
dojo.declare("rmpw.dsv.editor.form.ui.TableChangesMixin", dijit._Widget, {
	
	//mixin
	form: null,
	featureName: null,
	featureId: null,
	typeUri: null,
	propertyUri: null,
	propertyDomainUri: null,
	isFragment: null,
	typeLabel: null,
	typeForm: null,
	dataTypeProperty: null,
	redirectName: null,
	uniqueNum: null,
	readOnly: null,
	section: null,
	isDocument: null,
	onSubmitListener: null,
	table: null,
	tableChangesMixinMessages: null,
	resolvedLinks: null,
	
	//private
	_updateHrefs: null,
	_connectAfterCommit: null,
	
	constructor: function(args) {
		this.featureName = args.featureName;
		this.typeUri = args.typeUri;
		this.propertyUri = args.propertyUri;
		this.propertyDomainUri = args.propertyDomainUri;
		this.typeLabel = args.typeLabel;
		this.typeForm = args.typeForm; // the form to create new resource of this type
		this.redirectName = args.redirectName;
		this.isFragment = args.isFragment;
		this.dataTypeProperty = this.typeForm == "CreateDataTypeProperty";
		this.uniqueNum = 1;
		this.isDocument= args.isDocument;
		this._updateHrefs= {};		
		this.readOnly= args.readOnly;
		this.resolvedLinks = [];
	},
	
	postMixInProperties: function() {
		if (this.definableTypes && dojo.isString(this.definableTypes)) {
			this.definableTypes = dojo.eval(this.definableTypes);
		}
		
		dojo.requireLocalization("rmpw.dsv.editor.form.ui", "TableChangesMixinMessages");
		this.tableChangesMixinMessages = dojo.i18n.getLocalization("rmpw.dsv.editor.form.ui", "TableChangesMixinMessages");
		this.inherited(arguments);
	},
	
	buildRendering: function(){
		this.inherited(arguments);
		dojo.addClass(this.domNode, "rmpw-dsv-editor-form-ui-actions-table");
	},
	
	_createTableContents: function(){
		var args= {
			showHeaders: false
		};
		this.setTableArguments(args);
		
		/*Make sure to clear previous table contents */
		if(this.table&& !this.table._beingDestroyed){
			this.table.destroyRecursive();
		}
		dojo.empty(this.tableContainer);
		
		this.table= new rmps.ui.ActionsTable(args);
		this.tableContainer.appendChild(this.table.domNode);
		
		this.table.fillTable(this.getData());
		
		//Required to set the initial model
		this.form.createChange(this.featureId, this.object);
		
		if (this.isAutoSet) {
			this.onAutoSet();
		}
		
		if (this.dependentProperties) {
			var depPropArray = dojox.json.ref.fromJson(this.dependentProperties);
			if (depPropArray && depPropArray.length > 0) {
				this.__connectSetRepository = dojo.connect(this.form, "setRepository", dojo.hitch(this, function() {
					dojo.disconnect(this.__connectSetRepository);
					delete this.__connectSetRepository;
					
					this.__connectApplyChange = dojo.connect(this.form, "applyChange", dojo.hitch(this, function() {
						for (var i=0, lg=depPropArray.length; i<lg; i++) {
							if (this.form.changeSet.isChanged(depPropArray[i])) {
								this._clearContents();
								break;
							}
						}
					}));
					
					this.__connectAfterCommit = dojo.connect(this.form.getRepository(), "afterCommit", dojo.hitch(this, function(changeSet) {
						for (var i=0, lg=depPropArray.length; i<lg; i++) {
							if (changeSet.isChanged(depPropArray[i])) {
								this._clearContents();
								break;
							}
						}
					}));
					
				}));
			}
		}
		 
		this.onFillTable();
		this.table.startup();
	},
	
	_clearContents: function() {
		dojo.disconnect(this.__connectApplyChange);
		delete this.__connectApplyChange;
		
		dojo.disconnect(this.__connectAfterCommit);
		delete this.__connectAfterCommit;
		
		// reset the control contents
		ModelValidationUtil.showValidationError(this.tableChangesMixinMessages.propertyScopeChange);
		
		var changeSet = new rmpw.common.vcs.ChangeSet();
    	var change= this.form.changeSet.createChange(this.form.target)
    	change.set(this.featureName, undefined);
    	this.form.getRepository().commit(changeSet);	
    	
    	if(this.table&& !this.table._beingDestroyed){
			this.table.destroyRecursive();
		}
	},
	
	setTableArguments: function(args){
		dojo.mixin(args, {
			trim: jazz.ui.StyledTable.TrimStyles.SECONDARY_HEADLESS,
			columns: this.getColumnDescriptors(),
			actionDescriptors: this.readOnly?null:this.getActionDescriptors()
		});
	},
	
	getData: function(){
		return [];
	},
	
	setValue: function(object, document, repository, feature) {
		this.object= object;
		this.document = document;
		this.repository = repository;
		this.featureId= feature;
		this._createTableContents();
	},
		
	getColumnDescriptors: function(){
		return [
			{label: this.tableChangesMixinMessages.nameHeader, fieldName: "name", creator: dojo.hitch(this, this.createNameCell),width: "100%"}
		]
	},
	
	getActionDescriptors: function(){
		return [];
	},
	
	getLabel: function(object, guess) {
		if (dojo.isString(object)) {
			return object;
		}
		
		if (object.rdfs_label && dojo.isString(object.rdfs_label)) {
			return object.rdfs_label;
		}
		
		if (object.dcterms_title && dojo.isString(object.dcterms_title)) {
			return object.dcterms_title;
		}
		
		// otherwise - send the $ref back
		if (object.$ref) {
			return object.$ref;
		}
		
		if (object instanceof Date){
			return dojo.date.locale.format(object, LOCALE_FORMAT_OPTIONS);
		}
		
		if (guess) {
			// didn't find rdfs_label or dcterms_title - so make best guess
			for (var prop in object) {
				if(prop.substr(0, 2) != '__' && prop != "rdf_about") {  // ignore internal properties
					if (object[prop] && dojo.isString(object[prop])) {
						return object[prop];
					}
				}
			}
		}
		
		return "";
	},
					
	createNameCell: function(labelCell, tableRow, object){
		var about,label;
		
		if(object === undefined || object === null){
			about = object;
			label = this.tableChangesMixinMessages.emptyLabel;
		}else if(dojo.isObject(object)){
			if (this.redirectName && object[this.redirectName]) {
				if (dojo.isArray(object[this.redirectName])) {
					// in cases of lack of "canDefine" property for fragments (non-dtk resources only) cardinality restriction 
					// will not be picked up in the metamodel, and we end up with a collection
					about = object[this.redirectName][0].rdf_about;
					label = this.getLabel(object[this.redirectName][0], false);
				} else {
					about = object[this.redirectName].rdf_about;
					label = this.getLabel(object[this.redirectName], false);
				}
			}else {
				about = object.rdf_about;
				label = this.getLabel(object, false);
				
				/*Required for the restriction, annomous classes cases, etc */
				if(!label){
					var isNotUrl= !new dojo._Url(about).authority;
					
					var rdf_type = object.rdf_type;
					
					if(rdf_type){
						label= "<";
						if(!dojo.isArray(rdf_type)){
							rdf_type= [rdf_type];
						}
						
						var typesLength= rdf_type.length;
						var properties= [];
					
						var currentTypeAbout;
						for(var typeIndex = 0, type; type = rdf_type[typeIndex]; typeIndex++) {
							currentTypeAbout= type.rdf_about;
							if(!currentTypeAbout){
								continue;
							}
							var typeLabel= this.getLabel(type, true);
							if(typeLabel&& dojo.indexOf(type, typeLabel)<0){
								properties.push(typeLabel);
							}
						}
						
						//If about is undefined, then it is a new resource to be created, so avoid having an about value for it
						//Assing the value of the type about if the current about defined value is not a url
						if(currentTypeAbout&& !about&& isNotUrl){
							about= currentTypeAbout;
						}
						
						if(properties.length<1){
							properties.push(this.typeLabel);
						}
						if(this.isDocument){
							properties.push(this.tableChangesMixinMessages.documentLabel);
						}
						
						var propertiesLength= properties.length;
						for(var propertyIndex = 0, propertyLabel; propertyLabel = properties[propertyIndex]; propertyIndex++) {
							label += propertyLabel;	
							if(propertyIndex< propertiesLength-1){
								label += " , ";	
							}	
						}	
						
						label+= "> ";
						
						// add extra information about sub properties if they exist
						var count = 0;
						for (var prop in object) {
							if(prop.substr(0, 2) != '__' && prop != "rdf_about") { 
								var objLabel = this.getLabel(object[prop], true);
								if (objLabel) {
									if (count > 0) {
										label += ", ";
									}
									label += prop + ": " + objLabel;
									count++;
									
									if (count > 3) {
										break; // only display first 3 property values
									}
								}
							}
						}
					}
				}
			}

		}else if( dojo.isString(object) && object.match(/\d+-\d+-\d+T\d+:\d+(:\d+)?Z/) ){ 
			// /\d+-\d+-\d+T\d+:\d+(:\d+)?Z/  matches the ISO Date String format
			about = dojo.date.locale.format(new Date(object), LOCALE_FORMAT_OPTIONS);
			label = dojo.date.locale.format(new Date(object), LOCALE_FORMAT_OPTIONS);
		}else{
			about = object;
			label = object;
		}
		
		if (this.dataTypeProperty){
			labelCell.appendChild(dojo.doc.createTextNode(label?label: about));	
			return labelCell;
		}else{
			//If about is undefined, this means the field is not set at all, so do not bother listening to its change
			if(!about && (!object || !object.$ref)){
				labelCell.appendChild(dojo.doc.createTextNode(label||about|| this.tableChangesMixinMessages.emptyLabel));
				return;
			} 
			
			if (!about) {
				about = object.$ref;
			}
			if (!new dojo._Url(about).scheme) {
				if(!dojo.isObject(object)){
					//Make sure to avoid updating the url if the object is not an instance of object. What does this mean? It means that
					//It is literal value (string, int, etc)
					labelCell.appendChild(dojo.doc.createTextNode(label));
					return labelCell;
				}
				// for new resources always add them as simple text until the document is saved.  Otherwise, the link
				// is invalid since the data doesn't exist on disk
				labelCell.appendChild(dojo.doc.createTextNode(label?label: about));	
								
				//Updating urls for document elements has different handling
				if(!this.isDocument&& this.form._resource&& this.form._resource.documentUri){
					this._updateHrefs[about]= {
						cell: labelCell,
						label: label,
						href: rmpw.common.util.IdentifierUtil.normalizeURI(this.form._resource.documentUri) + about
					};
					
					// listen to save and convert to link
					if (!this._connectAfterCommit) {
						this._connectAfterCommit = this.connect(this.form._resource.page.editor, "onSaveComplete", dojo.hitch(this, function() {
							this.disconnect(this._connectAfterCommit);
							delete this._connectAfterCommit;
							
							for(var att in this._updateHrefs){
								var current= this._updateHrefs[att];
								var cell= current.cell;
								dojo.empty(cell);
								this._createLinkNameCell(cell, current.href, current.label);
								delete this._updateHrefs[att];
							}
						}));
					}					
				}
			}
			else {
				this._createLinkNameCell(labelCell, about, label);
			}
		}
		//FIXME: Using the TableContainer in forms generation did not give me enough options to make sure that the cell will not be expanded for the lengthy labels
		labelCell.style.maxWidth= "40px";
        return labelCell;
	},
	
	_createLinkNameCell: function(labelCell, about, label){
		var resourceLabel= label||about|| this.tableChangesMixinMessages.emptyLabel;
		var resourceLink = new jazz.ui.ResourceLink({
			uri: this.getUri(about),
			presentationUri: this.getPresentationUri(about),
			lazyFetch : true,
			label: jazz.util.html.escape(resourceLabel),
		});
		
		this.connect(this, "uninitialize", function() {
			resourceLink.destroyRecursive();
		});
		
		labelCell.appendChild(resourceLink.domNode);
		resourceLink.startup();
	
		/*
		 *  Update the link on hover, will try to prevent a full-page refresh for links within the same project area.
		 *  There is a race condition with this solution however, where the user might click too fast while the resolution
		 *  is in flight.  For this case, the behaviour will be the same as it was before; a full-page refresh.
		 */
		dojo.connect(resourceLink.domNode, "onmouseover", dojo.hitch(this, function(){
			if(undefined === this.resolvedLinks[about] || null == this.resolvedLinks[about]){
				this.resolvedLinks[about] = InfoClient._getResourceInfos(about, "resourceContext", null, null);
			}
			
			// Wait for a resolved value.
			dojo.when(this.resolvedLinks[about], dojo.hitch(this, function(info){
				var resolvedProjectUri = info[0].resourceContext;
				if(resolvedProjectUri && resolvedProjectUri.indexOf("dm/process/project-areas/") >= 0){
					var projectAreaUuid = resolvedProjectUri.split("dm/process/project-areas/")[1];
					if(projectAreaUuid && projectAreaUuid === jazz.app.context.get().itemId){
						var newUri = "#action=rmps.app.design.edit&resource=" + encodeURIComponent(about);
						var newResourceLink = new jazz.ui.ResourceLink({
							uri: resourceLink.uri,
							presentationUri: newUri,
							lazyFetch : resourceLink.lazyFetch,
							label: resourceLink.label
						});
						
						// Make sure we destroy the old link.
						this.connect(this, "uninitialize", function() {
							newResourceLink.destroyRecursive();
						});
						
						// Replace and delete the old node.
						dojo.place(newResourceLink.domNode, resourceLink.domNode, "replace");
						resourceLink.destroyRecursive();
					}
				}
			}));
		}));
		
		return labelCell;
	},
	
	getUri: function(about){
		return this._buildBaseUri("rendering", about);
	},
	
	getPresentationUri: function(about){
		return this._buildBaseUri("logical", about);
	},
	
	_buildBaseUri: function(segment, resource){
		var baseUri = net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL + '/' + segment + '?' + rmps.core.util.IRmpsConstants.BASELINE + "=" + encodeURIComponent(jazz.app.context.get().baseline);
		if(jazz.app.context.get().changeset){
			baseUri += '&' + rmps.core.util.IRmpsConstants.CHANGESET + "=" + encodeURIComponent(jazz.app.context.get().changeset);
		}
		var resourceParam = (segment === "rendering")?"&resourceUri=":"&resource=";		
		var normalized= this._getNormalizedAboutValue(resource);
		baseUri+= normalized?resourceParam+encodeURIComponent(normalized):resourceParam+encodeURIComponent(resource);
		return baseUri;
	},
	
	_getNormalizedAboutValue: function(about){
		if(!about){
			return null;
		}
		if(about.indexOf(net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL)){
			return about;
		}
		var query= new dojo._Url(about).query;
		
		if(query){	//Safe check
			var object= dojo.queryToObject(new dojo._Url(about).query);
			for(var att in object){
				return about;
			}
		}
		return null;
	},
	
	/*
	 * @param {null} object
	 */
	addNewItem: function(object) {
		var row= this.table.addRow(object);
		this.form.resize({});
		return row;
	},
		
	addRefrence: function(args) {
		this.useSelectDialog(args);
	},
	
	useSelectDialog: function(row){
		this.getSelectTypeClassConstructor(dojo.hitch(this, function(clazz){
			var dialog;
			
			var instnace= new clazz(this.getSelectTypeArguments(function(){return dialog;},row));
			dialog = new jazz.ui.Dialog({
				contentNode: instnace.domNode,primaryTitle: this.getDialogTitle(), 
				secondaryTitle:this.getDialogSecondaryTitle(), destroyContents: true, width: DEFAULT_DIALOG_WIDTH
			});
			
			//Required for the enter key events. As when we use enter key to close the dialog, in that case, the last focus will be the add anchor, and then the dialog API
			//will make a call to focus this anchor, and thus, this will open the dialog again.
			dialog._lastFocus= this.domNode;
			
			instnace.startup();
		}));
	},
	
	getDialogTitle: function(){
		return dojo.string.substitute(this.tableChangesMixinMessages.addType, [this.typeLabel]);
	},
	
	getDialogSecondaryTitle: function(){
		//Null Implemntation. Can be extended by sub-classes
		return "";
	},
	
	/**********************************************************Dialog Descriptors (Start)****************************************************************************** */
	/**Can be extended to add further descritpros */
	getDialogSelectionsDescriptors: function(getDialog, row){
		return [this.getQuickSearchDialogDescriptor(getDialog, row), this.getSearchDialogDescriptor(getDialog, row), this.getClipboardDialogDescriptor(getDialog, row)];
	},
	
	getQuickSearchDialogDescriptor: function(dialogGetter, row){
		return {classConstructor: this.typeUri=== TypeConstants.RESOURCE? ResourcesSelection:QuickSearchSelection,
				args: {
					typeUri: this.typeUri,
					definableTypes: this.definableTypes,
					propertyUri: this.propertyUri,
					propertyDomainUri: this.propertyDomainUri,
					documentUri: this.form._resource ? this.form._resource.documentUri : null,
					resourceUri: this.form._resource ? this.form._resource.id : this.form.target.rdf_about,	
					isConstrained: this.isConstrained,
					defaultValue: this.getFocusReference(),
					onOK: dojo.hitch(this, function(item){
						this.onSetReference({target: item, row: row});
						dialogGetter().close();}
				),
				onCancel: function(){dialogGetter().close()},
			},
			title: this.tableChangesMixinMessages.quickSearch
		}
	},
	
	getSearchDialogDescriptor: function(dialogGetter, row){
		return 	{classConstructor: RMPSPicker,
				args: {
					projectId: jazz.app.context.get().itemId,
					context: jazz.app.context.get().baseline,
					onlyType: this.typeUri,
					onClose: dojo.hitch(this, function(selections){
						for(var index = 0, selection; selection = selections[index]; index++) {
							this.onSetReference({target: {rdfs_label: selection.shortName, rdf_about: selection.link}, row: row});
						}
						dialogGetter().close();
					}),
				},
				title: this.tableChangesMixinMessages.search
			};
	},
	
	getClipboardDialogDescriptor: function(dialogGetter, row){
		return 	{classConstructor: ClipboardWidget,
				args: {
					projectId: jazz.app.context.get().itemId,
					baseline: jazz.app.context.get().baseline,
					onOkButton: dojo.hitch(this, function(selections){
						//Close the dialog first to avoid mutlipe clicks, as the call for resource info can take a moment
						dialogGetter().close();
						
						ResourceInfoClient.getResourceInfo(selections, ["uri","shortName"]).then(dojo.hitch(this, function(info) {
							for(var index = 0, selection; selection = info[index]; index++) {
								this.onSetReference({target: {rdfs_label: selection.shortName, rdf_about: selection.uri}, row: row});
							}
						}));
					}),
					onClickCancelButton: function(){dialogGetter().close()},
					readOnly: false,
					onlyType: this.typeUri
				},
				title: this.tableChangesMixinMessages.clipboard
			};
	},
	/***********************************************************Dialog Descriptors (End)****************************************************************************** */
	
	isCreatePicker: function() {
		return (this.isDocument || this.isFragment || (this.definableTypes && (this.definableTypes.length > 0))) && !this.canSelect;
	},
	
	isSelectCreatePicker: function() {
		return (this.isDocument || this.isFragment || (this.definableTypes && (this.definableTypes.length > 0))) && this.canSelect;
	},
	
	/*
	 *Can be extended by sub-classes 
	*/
	getSelectTypeClassConstructor: function(callback){
		if (this.isSelectCreatePicker()){
			return jazz.core.loader.batch_load_async([SelectCreatePicker], function() {
				callback(dojo.getObject(SelectCreatePicker));
			});		
		}
		else if (this.isCreatePicker()) {
			return jazz.core.loader.batch_load_async([CreatePicker], function() {
				callback(dojo.getObject(CreatePicker));
			});	
		}
		
		// Select picker only
		return jazz.core.loader.batch_load_async([SelectionsPicker], function() {
				callback(dojo.getObject(SelectionsPicker));
			});
	},
	
	/*
	 *Can be extended by sub-classes 
	*/
	getSelectTypeArguments: function(dialogGetter, row){
		if (this.isSelectCreatePicker()) {
			return {creationArgs: dojo.hitch(this, function(callback){
					this.createCreationContents(callback, dialogGetter, row);
				}),
				selectionsLabel: this.tableChangesMixinMessages.selectionsLabel,
				creationsLabel: this.tableChangesMixinMessages.creationsLabel,
				typeLabel: this.typeLabel,
				selectionArgs: this.getDialogSelectionsDescriptors(dialogGetter, row)
			};
		}
		else if (this.isCreatePicker()) {
			return {creationArgs: dojo.hitch(this, function(callback){
					this.createCreationContents(callback, dialogGetter, row);
				}),
				creationsLabel: this.tableChangesMixinMessages.creationsLabel,
				typeLabel: this.typeLabel
			};
		}

		// Select picker only
		return {argsList: this.getDialogSelectionsDescriptors(dialogGetter, row),
			selectionsLabel: this.tableChangesMixinMessages.selectionsLabel
		};
	},
	
	//FIXME: This method can be enhanced
	//Can be extended by sub classes
	createCreationContents: function(callback, dialogGetter, row) {
		var whereLabel = "OPTIONAL { ?value rdfs:label ?rdfs_label ." +
			"       FILTER (lang(?rdfs_label) = '"+ dojo.locale +"') }" +
			"OPTIONAL { ?value rdfs:label ?rdfs_label ." +
			"       FILTER (langMatches(lang(?rdfs_label) , '')) }" +
			"OPTIONAL { ?value rdfs:label ?rdfs_label }";
		
		var types = this.definableTypes;
		if (!types) {
			types = [this.typeUri];
		}
		
		var whereDefinableTypes = "";
		for (var i=0, lg=types.length; i<lg; i++) {
			var whereLine = "?value rdfs:subClassOf* <"+ types[i] +">."+
							"?value rdf:type owl:Class .";
			if (this.isConstrained) {
				whereLine += "?value rdf:type dmcore:AnonymousClass . ";
			}
			
			if (i==0) {
				whereDefinableTypes += "{ " + whereLine + " }";
			}
			else {
				whereDefinableTypes += "UNION { " + whereLine + " }";
			}
		}
		
		var whereAllTypes = "?value rdfs:subClassOf* <"+ this.typeUri +">."+
			"?value rdf:type owl:Class ."+
			"?value rdf:type ?classType ."+ whereLabel;
		
		var whereClause = whereAllTypes;
		if (this.isDocument) {
			whereClause = whereAllTypes;
		}
		else if (this.definableTypes && this.definableTypes.length > 0) {
			whereClause = whereDefinableTypes;
		}
		
		var self = this;
		// use simple select type dialog, till we have the search dialog works 
		rmps.core.client.xhrGet({
			url: net.jazz.ajax._contextRoot + "/modelvalidation/values",
			handleAs: 'json',
			headers: {
				"Accept" : "application/json",
				"X-ibm-rmps-internal" : "true"
			},
			content: {
				resourceUri: (this.form && this.form._resource.id) ? this.form._resource.id : (this.form ? this.form._resource.documentUri : null),
				adjustedRangeUri: this.typeUri,
				propertyUri: "http://www.w3.org/1999/02/22-rdf-syntax-ns#type",
				adjustedDomainUri: this.typeUri,
				whereClause: whereClause
			},
			load: function(data, ioArgs) {
				if(!data){
					return;
				}
				
				var types = [];
				for (var i=0, lg=data.length; i<lg; i++) {
                    types.push(self.getTypeObject(data[i]));
				}
				
				//FIXME: We can use construct statement in the SPARQL query, but this will require changes in the query service to accept the construct
				//body. And since this query is a temp solution till we get our enhancmenets for queries in, we can just use this quick mapping way.
				var items= dojo.map(types, function(entry){return {title: entry.rdfs_label, identifier: entry.id, tooltip: entry.label, 
				classConstructor: function(type, callback){
					// TODO: exploit common code with rmpw.common.store.JfsStore
					// request initial model (may contain default values)
					rmps.core.client.xhrGet({
						url: rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "rdfdirectory") + "/constructor",
						content: { rdf_type: type },
						headers: { "Accept": "application/x-rdf+json-graph" },
						handleAs: "text",
						load: function(data, ioargs) {
							
							// process json for initial model
							if(!data) {
								rmps.core.error.create("Failed to load initial model").dispatch();
								return;
							}
							
							var documents = self.processFromJSON(data);
							var model = documents[0];
							
							// TODO: shouldn't have to assign this, constructor service should provide correct uri
							model.rdf_about = self.defaultUri;
							
							var objectTypes= dojo.map(model.rdf_type, function(type) { return type.rdf_about; });
							// create form for it
							rmpw.common.util.FormsLauncher.loadForm({
									formName: self.typeForm,
									objectTypes: objectTypes,
									readOnly: self.form._resource.page.isReadOnly,			//FIXME: Anti-pattern to have this long list of setters.
									standAlone: false,
									editor: self.isDocument ? null : self.form._resource.editor,	//FIXME: Anti-pattern to have this long list of setters.
									create: true
								}, 
								function(signature){
									var creationForm = new rmpw.dsf.codegen.properties.forms[signature.formName]();
									//TODO: Must provide a setter instead
									creationForm.featureId = self.featureId;
									// ensure that the resources created from this popup form will be added to the JfsResource 
									// which keeps track of all new resources created from pop-up forms
									creationForm.setJfsResourceExisting(self.form.getJfsResource());
																		
									var method = creationForm.setterNames[self.typeForm];
									creationForm[method](model);
									
									var resource = self.form._resource;
									resource.objectTypes = objectTypes;
									
									creationForm.setChangeDetails();
									creationForm._setResource({
										documentUri: resource.documentUri,
										id: resource.id,
										namespaces: resource.namespaces,
										objectTypes: objectTypes,
										page: resource.page,
										repository: creationForm.repository,
										editor: resource.editor,
										formObject: resource.formObject,
										
										//FIXME: Find a better way instead of passing this information over and over everytime 
										modelIndex: self.form._resource.modelIndex
									});
									
									var rdf_about= model.rdf_about;
									if(!rdf_about&& !self.isDocument){	// leave defaultUri null, trigger autogen on server (if document)
										rdf_about = self.uniqueNum + rmpw.common.util.IdentifierUtil.guid();
									}
									
									creationForm.createChange("rdf_about", rdf_about);
									creationForm.immediate= true;
									creationForm._setRoot(self);
									
									self.connect(creationForm, "onSubmit", function(){
										var errors = [];
										ModelValidationUtil.validatePopup(creationForm, errors);
										if (errors.length > 0) {
											ModelValidationUtil.showValidationError(errors[0]);
										} else {
											var repository= creationForm.getRepository();
											var changeSets = [];
											if (repository.patch) {
												changeSets.push(repository.patch);
											}
											changeSets.push(repository.stack);
											
											self.onSetReference({
													target: creationForm.getTarget(), 
													namespaces: creationForm.getNamespaces(), 
													types: creationForm.getTypes(),
													row: row,
													item: changeSets
											});
											dialogGetter().close();
										}
									});
									self.connect(creationForm, "onCancel", function(){dialogGetter().close();});
									
									callback(creationForm);
								}
							);
						}
					});
				}};});
				if(items.length < 1){
					return;
				}
				callback(items);
			},
			error: function(error) {return rmps.core.error.createError(error).dispatch();}
		}, true);
    },
    
    processFromJSON: function( /*string*/ data ) {
        var result = rmpw.common.vcs.ref.fromJson(data, { idAttribute: "rdf_about" }),
        rdfType = result[0].rdf_type,
        i;
            
        //remove "rdf_type: Document", and all "defines" for "owl_oneOf" relationships
        //TODO: encapsulate the nuances of "owl_oneOf" into its own class (ListSectionIndividual.js?)
        if(this.featureId === "owl_oneOf") { 
          for(i = rdfType.length - 1; i >= 0; i -= 1){ 
            if(rdfType[i].rdf_about === "http://jazz.net/ns/dm/core#Document"){
                rdfType.splice(i,1);
            }
          }
          delete result[0].dmcodegen_defines;
        }
        
        return result;
    },
    
	removeRefrence: function(dataEntry, currentRow, currentActionsCell) {
		this.form.applyChange(dojo.hitch(this, function(change){
			change[this.featureId==="dmcodegen_defines"?"removeArray":"removeReferenceArray"](this.featureId, dataEntry);
			this.onRemoveReference(dataEntry, currentRow, currentActionsCell, change);		
		}));
		this.form.resize({});
	},
	
	getCollection: function() {
		return dojo.map(this.table._rowCache._rowsInformation, function(entry) {
			return entry.getDataEntry()});
	},
	
	newReference: function(currentTarget, tr, td) {
		var dialog = new rmpw.dsv.editor.form.ui.CreateResource();
		var defaultLabel = this.typeLabel.replace(/ /g, "") + this.uniqueNum;
		var defaultUri = defaultLabel + rmpw.common.util.IdentifierUtil.guid();   
		dialog.createResource(this.typeLabel, this.typeForm, null, defaultLabel, this, this.featureId, currentTarget, true);
		
		var onSubmitListener = this.connect(dialog, "onSubmit", function(target, types, namespaces) {
			this.uniqueNum++;
			this.disconnect(onSubmitListener);
			if(currentTarget && target && tr){
				// edit scenario - changes have already been committed
				this.table.updateRowCell(tr, "name", target);
			}else{
				if(this.dataTypeProperty){
					this.addNewItem(target[this.featureId]);
					this.form.createChange(this.featureId, this.getCollection());
				} else {
					if(!target.rdf_about) {
						target.rdf_about = defaultUri;
					}	
					this.addNewItem(target);
					this._addNewReference(target, namespaces);
				}
			}
			this.form.resize({});
			this.uniqueNum++;
		});
		//FIXME: potential memory leaks in the dialog after closing it
	},
	
	_addNewReference: function(item, namespaces, types, target, row){
		if(!item){
			return;
		}
		this.setRowReference(item, namespaces, row, types, target);
	},
	
	setRowReference: function(target, namespaces, row, types, item){
		if(this.isDocument){
			this._createNewReference(target, namespaces, row, types, item);
			return;
		}
		this._onAddNewReference(target, row, types, item);
	},
	
	_createNewReference: function(target, namespaces, row, types, changes){
		if(!target){
			this._onAddNewReference(null, row);
			return;
		}
		if(new dojo._Url(target.rdf_about).scheme){	//Means the resource is already created, so just the push the reference
			this._onAddNewReference(target, row);
			return;
		}
		
		var onResourceCreation= dojo.hitch(this, function(target, row, logicalUri){
			var object= {
				rdf_about: this.getTargetRdfAbout(logicalUri, target),
				rdfs_label: target.rdfs_label|| target.dcterms_title|| ""
			};
			target.rdf_about = logicalUri;
 			this.table.updateRowCell(row, "name", object);
 			// Indicate that we need the change to be saved right away in all cases, as this happens after Save is clicked
			this._onAddNewReference(object, row, null, null, true);
			// in case of setting a reference on the popup form, transfer the change from the local form's 
			// repository into the jfsResource so it can be properly saved later on 
			if (this.form.jfsResource.jfsStore.repository != this.form.repository) {
				var changeSets = [];
				// just grab the last change - the one that was just made
				changeSets.push(this.form.getRepository().getCurrentRevision());
				this.form.jfsResource.addResourceUpdate(changeSets);	
			}
		},target, row);
		
		this.form.applyJfsResourcesChange([{
			changes: changes,
			types: types|| target.rdf_type,
			namespaces:  namespaces|| this.form.getNamespaces(),
			identifier: target.rdf_about,
			target: target,
			onResourceCreation: onResourceCreation
		}]);
	},
	
	getTargetRdfAbout: function(logicalUri, target) {
		// returns the value to be stored for the new reference; by default, it is uri of the newly created resource
		return logicalUri;
	},
	
	handleMoveUp: function(dataEntry, currentRow, currentActionsCell){
		if(this.table.moveRowUp(currentRow)){
			return;
		}
		var index= this.table.getRowIndex(currentRow);
		this.form.applyChange(dojo.hitch(this, function(change){
			change.arrayMoveElement(this.featureId, index, index+1);
			this.onMoveUp(dataEntry, currentRow, currentActionsCell,change);
		}));
	},
	
	handleMoveDown: function(dataEntry, currentRow, currentActionsCell){
		if(!this.table.moveRowDown(currentRow)){
			return;
		}
		var index= this.table.getRowIndex(currentRow);
		this.form.applyChange(dojo.hitch(this, function(change){
			change.arrayMoveElement(this.featureId, index, index-1);
			this.onMoveDown(dataEntry, currentRow, currentActionsCell,change);
		}));
	},
	
	_onAddNewReference: function(target, node, types, item, immediate){
		this.form.applyChange(dojo.hitch(this, function(change){
			this.addReference(change, target);
		}), immediate);
		if(this.form._resource&& this.form._resource.modelIndex){
			this.form._resource.modelIndex[rmpw.common.util.IdentifierUtil.normalizeURI(this.form._resource.documentUri) + target.rdf_about] = target;			
		}
		
		// if it's a new object - add to the defines collection
		if (this.featureId != "dmcodegen_defines" && target && !rmpw.common.util.IdentifierUtil.isIdentifierFixed(target.rdf_about) && !this.isDocument ) {
			var rootObj = rmpw.common.util.IdentifierUtil.getRootObject(this);
			if (rootObj && rootObj.dmcodegen_defines && !rmpw.common.util.IdentifierUtil.exists(rootObj.dmcodegen_defines, target.rdf_about)) {
				rootObj.dmcodegen_defines.push(target);
			}
		}
		
		this.onAddNewReference(target, node, types, item);
	},
	
	addReference: function(change, target){
		if(this.isDocument){
			change.pushReference(this.featureId||"dmcodegen_defines", target);				
		}else{
			change.push(this.featureId||"dmcodegen_defines", target);	
		}
	},
	
	getTypeObject : function (object, labelMustExist) {
		if(dojo.isString(object)){
			return {"label" : object, "rdfs_label" : object,"rdf_about" : object};
		}
		//Assume that uri is always defined, as it is a positive UI failure to show
		//undefined lablel as a way to indicate issues.
		var uri= object.value || object.uriImage || object.rdf_about || "";		//Required to prevent id NPEs one fetching items
		var label= object.rdfs_label|| "";
		if (uri.length > 0) {
			if (label.length > 0 && label != uri) {
				label += ": " + uri;
			}
			else {
				// check if label must exist and if so return null if no label - we don't want users
				// to select anonymous class as an example.  They should always be created inline.
				if (labelMustExist && ModelValidationUtil.isUri(uri)) {
					return null;
				}
				
				label = uri;
			}
		}
		
		if(uri == "http://new") {	
			uri = this.uniqueNum + rmpw.common.util.IdentifierUtil.guid();
		}
		
		newObj = dojo.mixin({}, object);
		newObj = dojo.mixin(newObj, {"label" : label,
				"id" : uri,
				"rdfs_label" : object.rdfs_label|| "",
				"rdf_about"	 : uri
			});
		
		return newObj;
	},
	
	/** 
	* args: {
	*				target: target, 
	*				namespaces: namespaces, 
	*				row: row, 
	*				types: types, 
	*				item: item
	*			} 
	*/
	onSetReference: function(args){
		/*To be inherited by sub classes */
	},
	
	getFocusReference: function(item, namespaces){
		/*To be inherited by sub classes */
	},
	
	onRemoveReference: function(dataEntry, currentRow, currentActionsCell, change){
		/*To be inherited by sub classes */
	},
	
	onMoveUp: function(dataEntry, currentRow, currentActionsCell,change){
		/*To be inherited by sub classes */
	},
	
	onMoveDown: function(dataEntry, currentRow, currentActionsCell,change){
		/*To be inherited by sub classes */
	},
	
	onFillTable: function(){
		/*To be inherited by sub classes */
	},
	
	onAddNewReference: function(target, node, types, item){
		/*To be inherited by sub classes */
	},
	
	checkReload: function() {
		if (this.form._resource.page && this.isReload && this.form._resource.page.setReload) {
			this.form._resource.page.setReload();
		}
	},
	
	onAutoSet: function(){
		/*To be inherited by sub classes */
	},
});

})();