/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.dsv.editor.behavior.ui.Toolbar");

dojo.require("dijit._Contained");
dojo.require("dijit.Toolbar");
dojo.require("dijit.form.DropDownButton");
dojo.require("dijit.TooltipDialog");
dojo.require("dojox.json.ref");

dojo.require("rmpw.dsv.editor.behavior.ui.NewModelPane");

/**
 * Toolbar for creating and loading behavior models.
 * 
 * @author Marc Gobeil
 */
dojo.declare("rmpw.dsv.editor.behavior.ui.Toolbar", [dijit.Toolbar, dijit._Contained], {
	postCreate: function() {
		this.inherited(arguments);
		
		var dialog, button;
		
		// add New menu
		dialog = new dijit.TooltipDialog();
		dialog.attr("content", new rmpw.dsv.editor.behavior.ui.NewModelPane({
			onRequest: dojo.hitch(this, this._onRequestBehaviorModel),
    		onLoad: dojo.hitch(this, this._onLoadBehaviorModel)
    	}));
		button = new dijit.form.DropDownButton({ label: "New", dropDown: dialog });
		this.addChild(button);
		this.newButton = button;

		// add Load menu
		dialog = new dijit.TooltipDialog();
		dialog.attr("content", new rmpw.dsv.editor.behavior.ui.ExistingModelPane({
			onRequest: dojo.hitch(this, this._onRequestBehaviorModel),
    		onLoad: dojo.hitch(this, this._onLoadBehaviorModel)
    	}));
		button = new dijit.form.DropDownButton({ label: "Load", dropDown: dialog });
		this.addChild(button);
		this.loadButton = button;
		
		// add save button
		button = new dijit.form.Button({ label: "Save", onClick: dojo.hitch(this, this._onSave) });
		this.addChild(button);
	},
	
    _onRequestBehaviorModel: function() {
		if(this.editor) {
			this.editor.destroy();
		}
		// TODO (MG): do better than just closing everything that may have been opened
		this.newButton.closeDropDown();
		this.loadButton.closeDropDown();
		// TODO (MG): add loading feedback
    },
    
    _onLoadBehaviorModel: function(bundle, path) {
		// WORKAROUND (MG): resolve eClass references to metamodel
    	// TODO (MG): come up with a true, general solution for multi-model support
    	var behaviorModel = this.page.models.behavior;
    	var toolsModel = this.page.models.tools;
    	bundle.eClass = behaviorModel.getClassByName(bundle.eClass);
    	
    	var controllers = bundle.controllers;
    	for(var i = 0, controller; controller = controllers[i]; i++) {
    		controller.eClass = behaviorModel.getClassByName(controller.eClass);
    		
    		var tools = controller.tools;
    		if(tools) {
	    		for(var j = 0, tool; tool = tools[j]; j++) {
	    			tool.eClass = toolsModel.getClassByName(tool.eClass);
	    		}
    		}
    	}
    	// END WORKAROUND

    	var page = this.getParent();
    	
		this.editor = new rmpw.dsv.editor.behavior.ui.BundleEditor({
			region: "center",
			bundle: bundle,
			metamodel: page.models
		});
		page.addChild(this.editor);
		
		// keep reference for save
		this.bundle = bundle;
		this.path = path;
    },
    
    _onSave: function() {
    	// WORKAROUND (MG): stub off external model references
    	// TODO (MG): come up with a true, general solution for multi-model support
		var eClass = this.bundle.eClass;
		this.bundle.__eClass = eClass;
		this.bundle.eClass = { $ref: eClass.ePackage.nsURI + "#//" + eClass.name };
		
    	var controllers = this.bundle.controllers;
    	for(var i = 0, controller; controller = controllers[i]; i++) {
    		// controller's type
    		var eClass = controller.eClass;
    		controller.__eClass = eClass;
    		controller.eClass = { $ref: eClass.ePackage.nsURI + "#//" + eClass.name };
    		
        	// shapes/relations in the render mapping 
    		var shape = controller.shape;
    		controller.__shape = shape;
    		controller.shape = { $ref: shape.transformation.nsURI + "#//" + shape.name };
    		
        	// references to meta model
    		var tools = controller.tools;
    		if(tools) {
	    		for(var j = 0, tool; tool = tools[j]; j++) {
		    		var eClass = tool.eClass;
		    		tool.__eClass = eClass;
		    		tool.eClass = { $ref: eClass.ePackage.nsURI + "#//" + eClass.name };
	    		}
    		}
    	}
    	// END WORKAROUND
    	
    	
    	// send data
    	dojo.xhrPut({
			url: "resource/itemPath/Model" + this.path,
			putData: dojox.json.ref.toJson(this.bundle)
		});
    	
    	
    	// WORKAROUND (MG): restore references
    	this.bundle.eClass = this.bundle.__eClass;
    	delete this.bundle.__eClass;
    	for(var i = 0, controller; controller = controllers[i]; i++) {
    		controller.eClass = controller.__eClass;
    		delete controller.__eClass;
    		
    		controller.shape = controller.__shape;
    		delete controller.__shape;
    		
    		if(controller.tools) {
	    		for(var j = 0, tool; tool = controller.tools[j]; j++) {
	    			tool.eClass = tool.__eClass;
		    		delete tool.__eClass;
	    		}
    		}
    	}
    	// END WORKAROUND
    }
});