/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("rmps.app.ui.internal.LinkTypeEditorInput");

dojo.require("rmps.ui.editor.EditorInput");
dojo.require("dojox.encoding.digests.MD5");

dojo.require("rmps.core.process.SecurityClient");
dojo.require("rmps.app.ui.ProjectLinkingEditorUtil");
dojo.require("rmps.app.ui.internal.editors.views.link.LinkClient");
dojo.require("dojo.DeferredList"); 

dojo.require("dojo.i18n");
dojo.requireLocalization("rmps.app.ui.internal", "ProjectLinkingEditorInputMessages");
var messages= dojo.i18n.getLocalization("rmps.app.ui.internal", "ProjectLinkingEditorInputMessages");

var ProcessConstants = rmps.app.ui.ProjectLinkingEditorUtil.process;
var SecurityClient = rmps.core.process.SecurityClient;
var ProjectLinkingEditorUtil= rmps.app.ui.ProjectLinkingEditorUtil;

dojo.declare("rmps.app.ui.internal.LinkTypeEditorInput", rmps.ui.editor.EditorInput, {

	linkTypeUri: null,
	linkType: null,
	_presentationName: null,


	constructor: function(linkTypeUri, mediator) {
		this.linkTypeUri = linkTypeUri;
		this.mediator= mediator;
	},

	load: function() {
		var deferreds= [];
		
		var operationsDeferred= SecurityClient.isActionAllowed(
			ProcessConstants.Operations.LINK_TYPE,
			dojo.getObject("jazz.app.currentApplication").context,
			[
				ProcessConstants.Actions.CREATE,
				ProcessConstants.Actions.MODIFY,
				ProcessConstants.Actions.DELETE
		]);
		
		deferreds.push(operationsDeferred);
		
		operationsDeferred.addCallback(this, function(report) {
			this.operationReport = report;
			return this;
		});
		
		var linkTypesDeferred= rmps.app.ui.internal.editors.views.link.LinkClient.getAllLinkTypes(jazz.app.context.get().itemId,{orderBy:"title"}).addCallback(this, function(result){
			this.linkTypes= result.userDefined.concat(result.systemDefined);
			for(var index=0; index<this.linkTypes.length; index++){
   				var current= this.linkTypes[index];
   				if(current.about==this.linkTypeUri){
   					this.linkType= current;
   					break;
   				}
   			}
   			if(!this.linkType){
   				console.error("Could not find the link type speicfied in the url. So, we start creating a new link");
   				this.linkTypeArg= "newLinkType";
   			}else{
   				this._presentationName= this.linkType.title;
   				this.linkTypeArg= this.linkType.about; 
   			}
		});
		deferreds.push(linkTypesDeferred);
		return new dojo.DeferredList(deferreds);
	},

	isLoaded: function() {
		return this.operationReport != null;
	},

	unload: function() {
		delete this.operationReport;
	},

	getId: function() {
		return dojox.encoding.digests.MD5(this.declaredClass + ',' + this.linkTypeUri);
	},
	
	getPresentationName: function() {
		if(this.linkTypeArg== "newLinkType"||!this.linkType||!this.linkType.title&&
				!this.linkType.about){
			return messages.newLinkTypeTitle;	
		}
		var presentationName= "["; 
		presentationName+= this.linkType.title+ "] ";
		presentationName+= this.linkType.about;
		return presentationName;
	},

	setName: function(value) {
		this._inputName= value;
	},
	
	getName: function() {
		return this._inputName;
	},

	getBasePresentationUri: function()  {
		return ProjectLinkingEditorUtil.baseUrl;
	}
});

})();
