/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function(){
 
dojo.provide("rmps.app.ui.internal.dialogs.SelectContextDialog");

dojo.require("rmps.app.pickers.internal.BaselinePicker");
dojo.require("rmps.app.pickers.internal.StreamPicker");

dojo.require("jazz.ui.Dialog");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("dojo.i18n");
dojo.requireLocalization("rmps.app.ui.internal.dialogs", "SelectContextDialogMessages");
var messages= dojo.i18n.getLocalization("rmps.app.ui.internal.dialogs", "SelectContextDialogMessages");

/**
 * A dialog for choosing an existing context (stream or baseline) on the server.  This dialog
 * just *returns* the selected context; it is not meant to be used to actually set a new context.
 */
dojo.declare("rmps.app.ui.internal.dialogs.SelectContextDialog", [], {

	//mixin parameters
	projectId: null, //String
	contextSelected: null, //function; called when a context is selected, and takes the selected context URL as a parameter
	
	_dialog: null, //the actual dialog widget
	
	constructor: function(args) {
		//we must explicitly call mixin, since this is not a subclass of Widget
		dojo.mixin(this, args);
	},

	/**
	 * Opens the dialog.
	 */
	show:function() {
		this._dialog = new jazz.ui.Dialog({
			contentNode: new rmps.app.ui.internal.dialogs._SelectContextDialogContent({ 
				handleResults : dojo.hitch(this, this._handleResults), 
				projectId: this.projectId,
			}).domNode,
			primaryTitle: messages.dialogTitle
		});
	},
	
	/**
	 * Called when results are returned from the picker.  ContextUrl may be null.
	 */
	_handleResults: function(contextUrl) {
		this._closeDialog();
		
		if (contextUrl != null) {
			this.contextSelected(contextUrl);
		}
		
	},
		
	/**
	 * Closes the dialog, if it is open.
	 */
	_closeDialog: function() {
		if (this._dialog) {
			this._dialog.close();
		}
	}
	
});

dojo.declare("rmps.app.ui.internal.dialogs._SelectContextDialogContent", [dijit._Widget, dijit._Templated], {

	templateString: '<div>' +
						'<input type="radio" name="type" checked="true" dojoAttachPoint="_workspaceRadio" dojoAttachEvent="onclick:_typeSelected" id="workspace"/>' +
						'<label for="workspace">${messages.workspaceLabel}</label><br/>' +
						'<input type="radio" name="type" dojoAttachPoint="_snapshotRadio" dojoAttachEvent="onclick:_typeSelected" id="snapshot"/>' +
						'<label for="snapshot">${messages.snapshotLabel}</label><br/>' +
						'<div dojoAttachPoint="_pickerContent"></div>' +
					'</div>',
					
	messages: messages, //local copy for use in template string
					
	//mixin
	projectId: null,					
					
	_workspacePicker: null,				
	_snapshotPicker: null,

	/**
	 * Overridden from superclass.
	 */					
	postCreate: function() {
		//workspace picker is selected by default
		this._initWorkspacePicker();
		this._setPicker(this._workspacePicker);
	},
	
	/**
	 * Displays the given picker widget as the dialog contents.
	 */
	_setPicker: function(picker) {
		//first, clear out the old picker contents
		dojo.empty(this._pickerContent);
		//now, add the new child
		this._pickerContent.appendChild(picker.domNode);
	},
	
	_cancelSelected: function() {
		this.handleResults(null);
	},
	
	_contextSelected: function(contextObject) {
		this.handleResults(contextObject.uri);
	},
	
	/**
	 * Called when the radio button to toggle the context type is selected.
	 */
	_typeSelected: function(event) {
		if (event.currentTarget === this._workspaceRadio) {
			this._initWorkspacePicker();
			this._setPicker(this._workspacePicker);
		} else if (event.currentTarget === this._snapshotRadio) {
			this._initSnapshotPicker();
			this._setPicker(this._snapshotPicker);
		}
	},
	
	/**
	 * Instantiates the workspace picker and saves it to a field.
	 */
	_initWorkspacePicker: function() {
		if (!this._workspacePicker) {
			this._workspacePicker = new rmps.app.pickers.internal.StreamPicker({
				projectId: this.projectId, 
				onOk: dojo.hitch(this, this._contextSelected),
				onCancel: dojo.hitch(this, this._cancelSelected)
			});
		}
	},
	
	/**
	 * Instantiates the snapshot picker and saves it to a field.
	 */
	_initSnapshotPicker: function() {
		if (!this._snapshotPicker) {
			this._snapshotPicker = new rmps.app.pickers.internal.BaselinePicker({
				projectId: this.projectId, 
				onOk: dojo.hitch(this, this._contextSelected),
				onCancel: dojo.hitch(this, this._cancelSelected)
			});
		}	
	}

});
 
})();