/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function(){

dojo.provide("rmps.app.ui.internal.editors.ConfigureSpaceEditor");

dojo.require("rmps.app.ui.internal.editors.BaseEditor");

dojo.require("rmps.core.projects.ProjectClient");
dojo.require("rmps.core.util.ContextUtils");

dojo.require("jazz.ui.MessageBox");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

var ProjectClient = rmps.core.projects.ProjectClient;
var ContextUtils = rmps.core.util.ContextUtils;

var UA_KEY = "com.ibm.xtools.rmps.web.app.page_ConfigureSpaceEditor001";

dojo.require("dojo.i18n");
dojo.requireLocalization("rmps.app.ui.internal.editors", "ConfigureSpaceEditorMessages");
var messages= dojo.i18n.getLocalization("rmps.app.ui.internal.editors", "ConfigureSpaceEditorMessages");
var DIRTY_GROUP_ID = "rmps.app.ui.internal.editors.ConfigureSpaceEditor.dirty";

/**
 * Editor for viewing and setting the VVC Space of a project.
 */
dojo.declare("rmps.app.ui.internal.editors.ConfigureSpaceEditor", [rmps.app.ui.internal.editors.BaseEditor], {

	"class": "rmps-app-editors-configure-space",
	isRefreshable: false, //override from superclass
	spaceSelectionChoice: null,
	existingSpacesCombo: null,
	selectedSpace: null,
	spaces: [],
	uaKey: UA_KEY,

	postCreate: function() {
		this.inherited(arguments);
		// this.showLoading();
		this.initSpaceSelection();
	},

	startup: function() {
		this.inherited(arguments);
		
		if (!this.selectedSpace) {
			this.addDirty(this, DIRTY_GROUP_ID);
		}
	},

	initSpaceSelection: function() {
		this.spaceSelectionChoice = new rmps.app.ui.internal.editors._ConfigureSpaceEditorContent({ parent: this });
		var mainContainer = dojo.create('div', {"class" : "main"}, this.containerNode);

		this.existingSpacesCombo = dojo.create("select");
		this.spaceSelectionChoice._spaceComboContainer.appendChild(this.existingSpacesCombo);
		mainContainer.appendChild(this.spaceSelectionChoice.domNode);

		var dpls = [];
		dpls.push(ContextUtils.getProjectSpace());
		dpls.push(this.getAllSpaces());

		var dl = new dojo.DeferredList(dpls);
		dl.then(dojo.hitch(this, function(result) {
			var currentSpace = result[0][1];
			this.spaces = result[1][1];

			this.spaces.sort(this.compareSpaces);

			for (var space in this.spaces) {
				var _space = this.spaces[space];
				if (currentSpace === _space.uri) {
					this.selectedSpace = space;
				}
				var item = dojo.create("option", null, this.existingSpacesCombo);
				var itemText = dojo.doc.createTextNode(_space.title);
				item.appendChild(itemText);
			}

			// Start off disabled
			this.disableAll();

			if (!this.selectedSpace) {
				this.spaceSelectionChoice._c1.disabled=false;
				this.spaceSelectionChoice._c2.disabled=false;
				this.spaceSelectionChoice.t1.disabled = false;
				this.spaceSelectionChoice.t2.disabled = false;
				this.spaceSelectionChoice._c1.checked = true;

				// use project area name as the suggested name for the new space
				this.spaceSelectionChoice.t1.value = jazz.app.context.get().name;
			} else {
				this.spaceSelectionChoice._c2.checked = true;
				this.existingSpacesCombo.selectedIndex = this.selectedSpace;
			}
		}));
	},

	compareSpaces: function(a, b) {
		return a.title.localeCompare(b.title);
	},

	getAllSpaces: function() {
		var args = {
				url: net.jazz.ajax.contextRoot() + "/queryvvc/allspaces",
				headers: {
					"Accept" : "application/json"
				},
				handleAs: "json",
				sync: true,
				load: function(response, ioArgs) {
					var spaces = [];
					for (var uri in response) {
						var r = {uri: uri, title: response[uri]};
						spaces.push(r);
					}
					return spaces;
				}
		};
		return jazz.client.xhrGet(args);
	},

	_doSave: function() {
		// check for name collision with existing spaces
		if (this.spaceSelectionChoice._c1.checked) {
			var spaceName = this.spaceSelectionChoice.t1.value;
			if (spaceName && spaceName.length > 0) {
				// trim leading and trailing spaces
				spaceName = spaceName.replace(/^\s\s*/, '').replace(/\s\s*$/, '');
				for (var i in this.spaces) {
					var space = this.spaces[i];
					if (space.title === spaceName) {
						this.showMessage({
							type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
							summary: messages.duplicateName
						});
						this.saveButton.setEnabled(true);
						return;
					}
				}
			}
		}

		this.inherited(arguments);
	},

	/**
	 *Overridden from superclass.  Saves the value of the selected space
	 */
	doSave: function(taskMonitor) {
		var retVal;
		if (this.spaceSelectionChoice._c2.checked) {
			retVal = this.saveExistingSpace(taskMonitor);
		} else {
			retVal = this.saveNewSpace(taskMonitor);
		}

		retVal.addCallback(this, function() {
			if (retVal.ioArgs.xhr.status === 201) {
				this.showSaveOk();
				this.disableAll();
				// clear dirty flag first before reload
				this._clearDirty();

				// we need to refresh the page
				var defautWorkspaceUri = retVal.ioArgs.xhr.getResponseHeader("Location");
				if (defautWorkspaceUri) {
					var currentURL = location.search;
					if (currentURL && currentURL.length > 1) {
						currentURL += "&";
					}
					currentURL += "rmps.context=" + encodeURIComponent(defautWorkspaceUri);
					location.search = currentURL;
				} else {
					window.location.reload();
				}
			}
		});
		
		var task = taskMonitor.beginTask(taskMonitor.name, 1);
		task.handleDeferredWork(deferred, 1);
	},

	saveNewSpace: function(taskMonitor) {
		var query = {};
		query.projectId = jazz.app.context.get().itemId;
		var __spaceName = this.spaceSelectionChoice.t1.value;
		if (__spaceName && __spaceName !== "") {
			query.spaceName = __spaceName;
		}
		var __spaceDescription = this.spaceSelectionChoice.t2.value;
		if (__spaceDescription && __spaceDescription !== "") {
			query.spaceDescription = __spaceDescription;
		}
		var link = net.jazz.ajax.contextRoot() + "/queryvvc/setnewspace?" + dojo.objectToQuery(query);

		var args = {
				url: link,
				error: dojo.hitch(this, function(e) {
					taskMonitor._error(null, e);
				})
		};
		return jazz.client.xhrPost(args);
	},

	saveExistingSpace: function(taskMonitor) {
		var link = net.jazz.ajax.contextRoot() + "/queryvvc/setexistingspace?projectId=" + escape(jazz.app.context.get().itemId);
		this.selectedSpace = this.existingSpacesCombo.selectedIndex;
		var __spaceUri = this.spaces[this.selectedSpace].uri;
		link += "&spaceUri=" + escape(__spaceUri);

		var args = {
				url: link,
				error: dojo.hitch(this, function(e) {
					taskMonitor._error(null, e);
				})
		};
		return jazz.client.xhrPost(args);
	},

	disableAll: function() {
		this.spaceSelectionChoice._c1.disabled = true;
		this.spaceSelectionChoice._c2.disabled = true;
		this.spaceSelectionChoice.t1.disabled = true;
		this.spaceSelectionChoice.t2.disabled = true;
		this.existingSpacesCombo.disabled = true;
	}
});

dojo.declare("rmps.app.ui.internal.editors._ConfigureSpaceEditorContent", [dijit._Widget, dijit._Templated], {
	messages : messages,

	templateString: '<div class="main">' + 
					  '<form>' + 
					    '<div class="choiceSection">' +
					      '<label title="${messages.spaceChoiceDescription}">${messages.spaceChoiceDescription}</label>' +
					      '<div class="choice">' +
					        '<input id="${id}_c1" type="radio" name="typeChoice" value="1" dojoAttachPoint="_c1" dojoAttachEvent="onclick: _toggleChoice"/>' +
					        '<label for="${id}_c1">${messages.newSpaceChoice}</label>' +
					        '<div class="subsection">' +
					          '<span class="field">${messages.spaceName}</span><br/><input id="${id}_t1" type="text" size="100" dojoAttachPoint="t1" dojoAttachEvent="onchange: _onValueChange"/><br/><br/>' +
					          '<span class="field">${messages.spaceDescription}</span><br/><input id="${id}_t2" type="text" size="100" dojoAttachPoint="t2" dojoAttachEvent="onchange: _onValueChange"/><br/>' +
					        '</div>' +
					      '</div>' +
					      '<div class="choice">' +
					        '<input id="${id}_c2" type="radio" name="typeChoice" value="2" dojoAttachPoint="_c2" dojoAttachEvent="onclick: _toggleChoice"/>' +
					        '<label for="${id}_c2">${messages.existingSpaceChoice}</label>' +
					        '<div class="subsection" dojoAttachPoint="_spaceComboContainer"/>' +
					      '</div>' +
					    '</div>' +
					  '</form>' +
					'</div>',

	parent: null,

	postCreate: function() {
		this.inherited(arguments);
	},

	setMode: function(newSpace) {
		if (newSpace) {
			this._c1.checked=true;	
		} else {
			this._c2.checked=true;
		}
	},

	_toggleChoice: function(e) {
		if (!this.parent.selectedSpace) {
			this.parent.addDirty(this, DIRTY_GROUP_ID);
		}
		if (e.target.value === 1) {
			this.t1.disabled = false;
			this.t2.disabled = false;
			this.parent.existingSpacesCombo.disabled = true;
		} else {
			this.t1.disabled = true;
			this.t2.disabled = true;
			this.parent.existingSpacesCombo.disabled = false;
		}
	},

	_onValueChange: function(e) {
		this.parent.addDirty(this, DIRTY_GROUP_ID);
	}
});

})();

