/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("rmps.app.ui.internal.editors.DomainsManagementEditor");

	dojo.require("rmps.app.ui.internal.editors.BaseEditor");
	
	dojo.require("dojo.i18n");
	dojo.requireLocalization("rmps.app.ui.internal.editors", "DomainsEditorMessages");
	
	var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.editors", "DomainsEditorMessages");
	var DIRTY_GROUP_ID = "rmps.app.ui.internal.editors.DomainsManagementEditor.dirty";
	var DOMAIN_REG_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "domainRegistry");
	
	dojo.require("rmps.app.ui.internal.editors.views.DataManagementSection");
	dojo.require("rmps.core.client");
	dojo.require("rmps.core.util.ContextUtils");
	
	dojo.require("rmps.app.ui.internal.util.DomainRegistryUtil");
	var domainUtil = rmps.app.ui.internal.util.DomainRegistryUtil;
	
	dojo.require("dojox.grid.EnhancedGrid");
	dojo.require("dojox.grid.enhanced.plugins.IndirectSelection");
	dojo.require("dojox.data.JsonRestStore");    
	
	dojo.require("dojo.DeferredList");
	
	var UA_KEY = "com.ibm.xtools.web.app.page_DomainsManagement001";
	dojo.require("com.ibm.team.vvc.shared.web.ui.context.ContextUtils");
	dojo.require("rmps.app.ui.internal.dialogs.SelectDomainVersionDialog");
	  
	/*
	 * private function for enabling disabling all rows in the grid (since there doesn't seem to be grid api for this...
	 */
	var enableGrid = function(grid, enabled) {
		for (var i = 0; i < grid.rowCount; ++i) {
			grid.rowSelectCell.setDisabled(i, !enabled);
		}
	};
	
	dojo.declare("rmps.app.ui.internal.editors.DomainsManagementEditor", rmps.app.ui.internal.editors.BaseEditor, {

		"class": "rmps-app-editors-domains-management-definition",
		isRefreshable: false,
		uaKey: UA_KEY,
		
		postCreate: function() {
			this.inherited(arguments);
			this.showLoading();

			this.__isReady_dmSection = new dojo.Deferred();
			this.__isReady_projectType = new dojo.Deferred();
			this.__isReady = new dojo.DeferredList([this.__isReady_dmSection, this.__isReady_projectType]);
			
			// When both parts are initialized, we finish our initialization
			this.__isReady.addCallback(dojo.hitch(this, function() {
				this._setEnablementOfProjectTypeSection();	
			}));
			
			// Create Data Management Section
			this.dmSection = new rmps.app.ui.internal.editors.views.DataManagementSection({ editor: this });
			this.containerNode.appendChild(this.dmSection.domNode);

			// Create Domain Associations
			this.initDomainAssocaitions();
		},
		
		startup: function() {
			this.inherited(arguments);
			
			// Wait for the Data Management section to get its result before continuing
			var handle = dojo.subscribe(this.dmSection.dataReady, dojo.hitch(this, function(message) {
				dojo.unsubscribe(handle);
				if (message && message.error) {
					// Determining Import or DM mode failed.
					
					this.showMessage({
						type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
						errorObject: message.error,
						closable: true
					});
					
					enableGrid(this.grid, false); // disable the grid too
				} else {
					this.__isReady_dmSection.callback();
				 }
			}));

			this.dmSection.startup();
			this.grid.startup();
			this.grid.selection.clickSelectEvent = dojo.hitch(this, this.clickSelectEvent);
		},
		
		clickSelectEvent: function(e) {
			// When the user clicks on the row domain labels, the default
			// behaviour in base class is doing the delect all first and then
			// toggle the selection of the clicked on row. But we preferred not
			// to lose all the current selection in this case, So we call
			// the clickSelect method in base class with control key flag as
			// true when the shift key flag is false.
			var inCtrlKey = dojo.isCopyKey(e);
			if (e.shiftKey === false) {
				inCtrlKey = true;
			}
			this.grid.selection.clickSelect(e.rowIndex, inCtrlKey, e.shiftKey);
		},
		
		_setEnablementOfProjectTypeSection: function() {
			domainUtil.isCurrentProjectDSP(dojo.hitch(this, function(isDsp, isSystem, canCreateDsp) {
				
				this.projectTypeChoice.setMode(isDsp);
				
				// Users can adjust this property iff
				// Not a System DSP
				// Not a DM Managed
				var isDMManaged = this.dmSection.isDMManagedMode();
				
				var canChangeManagementMode = !isSystem && !isDMManaged; 
				this.dmSection.setEnabled(canChangeManagementMode);
				
				// The grid can be edited iff we're DM managed and not a DSP
				var canEditGrid = !isDsp && isDMManaged;
				this.projectTypeChoice.setEnabled(canCreateDsp && canEditGrid);
				enableGrid(this.grid, canEditGrid);
				
				if (isSystem) {
					this.showMessage({
						type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
						summary: messages.systemDSP,
						closable: true
					});
				} else {
					this.hideMessage();
				}
			}));
		},
		
		initDomainAssocaitions: function() {

	        this.projectTypeChoice = new rmps.app.ui.internal.editors.DataManagementEditor._ChoiceSection({ parent : this });
	        var domainsContainer = dojo.create('div', {"class" : "domains"}, this.containerNode);
	        
	        domainsContainer.appendChild(this.projectTypeChoice.domNode);
	        
			var self = this;
			var restService = function(query, queryOptions) {
				
				var url = DOMAIN_REG_PATH + "/domains";
				
				var def = rmps.core.client.xhrGet({
					url: url,
					handleAs: 'json',
					headers: {
						"Accept" : "application/json"
					},
					load: dojo.hitch(this, function(response, ioArgs) {
						self._regETag = ioArgs.xhr.getResponseHeader("ETag");
						
						self._originalSelection = [];
						
						if (response && response.length) {
							for (var i in response) {
								var sel = response[i].selected;
								self.grid.selection.setSelected(i, sel);
								
								// Store indexes of original selection
								if (sel) {
									self._originalSelection[i] = true;
								}
							}	
						}
						
				        // listen for changes to checkboxes
				        dojo.connect(self.grid.selection, 'onSelected', self, '_selectionChanged');
				        dojo.connect(self.grid.selection, 'onDeselected', self, '_selectionChanged');
				        self.__isReady_projectType.callback();
						return response;
					})
				}, true);
				return def;
			};
			
			
			var store = new dojox.data.JsonRestStore({ service: restService, target : DOMAIN_REG_PATH });
			
	        var gridDiv = dojo.create('div', null, domainsContainer);
	        
	        // create a new grid:
	        var grid = new dojox.grid.EnhancedGrid({
	            store: store,
	            structure: [{
		            field: 'label',
		            name: messages.domains,
		            width: 'auto'

		        }],
	            width:"auto",
	            plugins: {
	                indirectSelection: {
	                    name: messages.applied,
	                    width: "70px",
	                    styles: "text-align: center;",
	                    checkedText: "",
	                    unCheckedText: ""
	                }
	            },
	            canSort: function() {return false;}
	        }, gridDiv);
	        
	        dojo.addClass(grid.domNode, "grid");
	        dojo.addClass(grid.domNode, "domainGrid");
	       
	        this.grid = grid;
		},
		
		_selectionChanged: function(message) {
			/*
			 * Called when the user selects or unselects a domain
			 */
			
			var selected = this.grid.selection.selected;
			var original = this._originalSelection;
			
			this._newItemSelected = false;
			var dirty = false;
			for (var i = 0, len = selected.length; i < len; ++i) {
				// Using == here because I want undefined to equate to false
				if (selected[i] != original[i]) {
					dirty = true;
					if (!original[i]) {
						this._newItemSelected = true;
					}
				}
			}
			if (dirty === true) {
				this.addDirty(this.grid, DIRTY_GROUP_ID);
			} else {
				this.removeDirty(this.grid, DIRTY_GROUP_ID);
			}
		},
		
		validate: function(/*Array*/errors) {
			this.dmSection.validate(errors);
		},
		
		_doSave: function() {
			// make sure the project is associated with a configuration space first
			var contextUri = rmps.core.util.ContextUtils.getCurrentContextURI();
			if (!contextUri) {
				this.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary: messages.noSpace,
					closable: true
				});
				return;
			}
			
			this.inherited(arguments);
		},
		
		doSave: function(taskMonitor) {
			// Note
			// If we're externally managed, then domain changes can't be made
			// If we're dm managed, then we can't switch back to external

			// First check if the change is going from External To DM managed
			if (this.dmSection.isDirty) {
				// if this is the change, then we know domain association changes aren't applicable
				var def = this.dmSection.doSave(taskMonitor);
				def.addCallback(dojo.hitch(this, function() {
					this._setEnablementOfProjectTypeSection();
				}));
				return def;
			}
			
			var url = DOMAIN_REG_PATH;
			if (this.projectTypeChoice._c1.checked) {
				// We're setting this to be a DSP
				url += "?isDsp=true";
			} else {
				// If we're here, we need to check what domains have changed.
				var selected = this.grid.selection.selected; // array of selected indices
				var data = [];
				for (var index in selected) {
					var item = this.grid.getItem(index);
					data.push({ uri: item.id });
				}
			}
			
			var deferred = rmps.core.client.xhrPut({
				url: url,
				putData: dojo.toJson(data),
				handleAs: 'json',
				headers: {
					"If-Match" : this._regETag,
					"Accept" : "application/json",
					"Content-Type" : "application/json"
				},
				load: dojo.hitch(this, function(data, ioArgs) {
					// Update latest ETag
					this._regETag = ioArgs.xhr.getResponseHeader("ETag");
					// Update our 'original' selection so we know when we're dirty
					this._originalSelection = dojo.clone(selected);
					return data;
				})
			}, true);
			
			var task = taskMonitor.beginTask(messages.task_saveConfiguration, 1);
			task.handleDeferredWork(deferred, 1);
			this._askUserToPickSnapshot = this._newItemSelected;
			return deferred;
		},
		
		onSaveComplete: function() {
			// Changing from Externally to Actively managed, can affect the Design menu (creation types for DSP or 'import' action, etc),
			// as well as changing the associate domains.  Therefore always refresh.
			if (this._askUserToPickSnapshot) {
				this._askUserToPickSnapshot = false;
				this.showSaveOk();
				var args = { reloadAfterClose: true };
				var dialog = rmps.app.ui.internal.dialogs.SelectDomainVersionDialog(args);
				dialog.show();
			} else {
				location.reload(true);
			}
		},
		
		getWorkspaceDependencyLink: function() {
			var dmContextUri = rmps.core.util.ContextUtils.getCurrentContextURI();
			var isSnapshotContext = rmps.core.util.ContextUtils.isSnapshot(dmContextUri);
			var context = com.ibm.team.vvc.shared.web.ui.context.ContextUtils.getCurrentContext();
			if (context && context.uri) {
				var link = context.uri;
				var i = link.indexOf("://");
				if (i != -1) {
					i = link.indexOf("/", i+3);
					if (i != -1) {
						i = link.indexOf("/", i+1);
						if (i != -1) {
							link = link.substring(0, i+1) + "web?";
							var args = {};
							args.operation = "open";
							args.contextUri = context.uri;
							args.showTab = "dependencies";
							args.clientInterface = this._getCallbackUri();
							link += dojo.objectToQuery(args);
							if (isSnapshotContext) {
								link += "#action=SnapshotEditor";
							} else {
								link += "#action=WorkspaceEditor";
							}
							return link;
						}
					}
				}
			}
		},
		
		_getCallbackUri: function() {
			var args = {};
			args.context = window.location.href;
			args.contextClientName = messages.productName;
			return net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL + "/queryvvc/vvccallback?" + dojo.objectToQuery(args);
		},
		
		_setDomainVersions: function() {
			if (this.isDirty()) {
				this.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
					summary: messages.mustSaveBeforeSetVersions,
					closable: true
				});	
				return;
			}
			var dialog = rmps.app.ui.internal.dialogs.SelectDomainVersionDialog();
			dialog.show();
		}
		
	});
	
	dojo.declare("rmps.app.ui.internal.editors.DataManagementEditor._ChoiceSection", [dijit._Widget, dijit._Templated], {
		
		messages: messages,
	
		
		_mode: null,
		_originalMode: null,
	
		templateString: '<div class="domains"><form><div class="choiceSection" >' +
				'<label title="${messages.choiceDescription}">${messages.choiceDescription}</label>' +
			'<div class="choice">' +
				'<input id="${id}_c1" type="radio" lable="test" name="typeChoice" value="1" dojoAttachPoint="_c1" dojoAttachEvent="onclick: _toggleChoice"/>' +
				'<label for="${id}_c1">${messages.choice_dsp}</label>' +
			'</div>' +
				'<div class="choice">' +
					'<input id="${id}_c2" type="radio" name="typeChoice" value="2" dojoAttachPoint="_c2" dojoAttachEvent="onclick: _toggleChoice"/>' +
					'<label for="${id}_c2">${messages.choice_availableDomains}</label>' +
					'<span style="float: right;"><a dojoAttachPoint="_workspaceEditorLink">${messages.pick_versions}</a></span>' +
					'<span style="float: right;"><a dojoAttachPoint="_snapshotEditorLink">${messages.view_versions}</a></span>' +
				'</div>' +
			'</div></form></div>',
	
		parent: null,
		
		postCreate: function() {
			this.inherited(arguments);
			
			// Start off disabled
			this._c1.disabled=true;
			this._c2.disabled=true;
			var contextUri = rmps.core.util.ContextUtils.getCurrentContextURI();
			var isSnapshotContext = rmps.core.util.ContextUtils.isSnapshot(contextUri);
			var link = this.parent.getWorkspaceDependencyLink();
			if (link) {
				var removeNode;
				var showNode;
				if (isSnapshotContext) {
					removeNode = this._workspaceEditorLink;
					showNode = this._snapshotEditorLink;
				} else {
					removeNode = this._snapshotEditorLink;
					showNode = this._workspaceEditorLink;
				}
				showNode.style.cursor='pointer'; 
				dojo.connect(showNode, "onclick", this, this.onSetDomainVersion);
				removeNode.parentNode.removeChild(removeNode);
			} else {
				// hide the links for the case that the configuration space has not been set for the current project.
				this._workspaceEditorLink.parentNode.removeChild(this._workspaceEditorLink);
				this._snapshotEditorLink.parentNode.removeChild(this._snapshotEditorLink);
			}
		},
		
		setMode: function(isDSP) {
			if (isDSP) {
				this._c1.checked=true;
			} else {
				this._c2.checked=true;
			}
		},
		
		setEnabled: function(enabled) {
			this._c1.disabled = !enabled;
			this._c2.disabled = !enabled;
		},
		
		onSetDomainVersion: function() {
			this.parent._setDomainVersions();
		},
		
		_toggleChoice: function(e) {
			this.onSetDomainVersion();
			if (e.target.value == 1) {
				this.parent.addDirty(this, DIRTY_GROUP_ID);	
				enableGrid(this.parent.grid, false);
			} else {
				this.parent.removeDirty(this, DIRTY_GROUP_ID);
				enableGrid(this.parent.grid, true);
			}
		}
	});

	dojo.declare("rmps.app.ui.internal.editors.DataManagementEditor._PickVersionReminderDialog", [dijit._Widget, dijit._Templated], {
		messages: messages,
		widgetsInTemplate: true,
		_pickWorkspaceDependencyLink: null,
		
		templateString: '<div dojoAttachPoint="_dialogContent">' +
				'<div>${messages.asking_pick_versions}</div><br>'+
				'<table align=right>' +
					'<tr>' +
						'<td><button dojoAttachPoint="_yesButton">${messages.yes_label}</button></td>' +
						'<td><button dojoAttachPoint="_noButton">${messages.no_label}</button></td>' +
					'</tr>' +
				'</table>' +
			'</div>',
		
		postCreate: function() {
			this.dialog = new jazz.ui.Dialog({
				contentNode: this._dialogContent,
				primaryTitle: messages.asking_pick_versions_title,
				width: "400px",
				isClosable: true
			});
			this.connect(this._yesButton, "onclick",  this._yesPressed);
			this.connect(this._noButton, "onclick",  this._noPressed);
			this._yesButton.focus();
		},
		
		setWorkspaceDependencyLink: function(link) {
			this._pickWorkspaceDependencyLink = link;
		},
		
		close: function() {
			this.dialog.close();
		},
		
		_yesPressed: function() {
			this.close();
			if (this._pickWorkspaceDependencyLink) {
				window.location.href = this._pickWorkspaceDependencyLink;
			}
		},
		
		_noPressed: function() {
			this.close();
			location.reload();
		},
	});
	
})();
