/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("rmps.app.ui.internal.editors.views.DataManagementSection");

	dojo.require("dijit._Widget");
	dojo.require("rmps.core.group.GroupClient");
	dojo.require("dojo.i18n");
	dojo.requireLocalization("rmps.app.ui.internal.editors", "DataManagementEditorMessages");

	var DIRTY_GROUP_ID = "rmps.app.ui.internal.editors.views.DataManagementSection.dirty";
	var GroupClient = rmps.core.group.GroupClient;
	var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.editors", "DataManagementEditorMessages");
	
	dojo.require("rmps.app.ui.internal.util.DomainRegistryUtil");
	
	dojo.declare("rmps.app.ui.internal.editors.views.DataManagementSection", dijit._Widget , {

		"class": "rmps-app-editors-data-management-definition",
		_dmManaged : false, // DM managed mode if enabled. otherwise, import is enabled.
		_projectId : null,
		editor: null, // enclosing editor
		isDirty: false,
		
		dataReady: "Data_Ready", // event to publish
		
		postCreate: function() {
			this.inherited(arguments);
			this._projectId = this.editor.getEditorInput().projectId;
			this._createSections();
		},
		
		startup: function() {
			this.inherited(arguments);
			this.choiceSection.startup();
			this.initDataManaged();
		},
		
		initDataManaged: function () {
			rmps.app.ui.internal.util.DomainRegistryUtil.isProjectActivelyManaged(
				dojo.hitch(this, function(isDmManaged, error) {
					var dataToPublish = {};

					if (error) {
						dataToPublish.error = error.error;
						rmps.core.error.createXhrError(error.ioArgs).dispatch();
					}

					this._dmManaged = isDmManaged;
					this.choiceSection.setMode(isDmManaged);

					// Tell the world we're ready
					dojo.publish(this.dataReady, [dataToPublish]);
				})
			);
		},
		
		_createSections: function() {
			dojo.forEach(dijit.findWidgets(this.domNode), function(widget) { 
				if (widget.destroyRecursive) {
					widget.destroyRecursive();
				}
			});
			this.choiceSection = new rmps.app.ui.internal.editors.views.DataManagementEditor._ChoiceSection({parent: this});
			this.choiceSection.setEnabled(false);
			this.domNode.appendChild(this.choiceSection.domNode);
		},
		
		isDMManagedMode: function() {
			/* Return true if we're in DM Managed mode */
			return this._dmManaged;
		},
		
		setEnabled: function(enabled) {
			this.choiceSection.setEnabled(enabled);
		},
		
		/*
		 * Called prior to saving. If a validation error is present
		 * which will prevent save from completing then clients should
		 * populate push the error reason (String|Error) into the errors array.
		 * Client may also push a Deferred object to defer the validation. This allows
		 * the client to communicate with the server to perform validation. To add a validation
		 * failure the deferred must either perform an error call back or push errors into
		 * the errors array.
		 */
		validate: function(/*Array*/errors) {
			if (this.isDirty && !confirm(messages.confirmToDMManagedMode)) {
				errors.push(messages.cancelled);
			}	
		},
		
		doSave: function(taskMonitor) {
			var deferred = GroupClient.setEditable(this._projectId, true);
			deferred.addCallback(this, function() {
				this._dmManaged=true;
				this.isDirty=false;
				this.setEnabled(false);
			});
			deferred.addErrback(this, function(error) {
				if (error.status === 403) {
					return new Error(messages.error_save_permission);
				} else {
					return new Error(messages.error_save);
				}
			});
			var task = taskMonitor.beginTask(messages.task_saveConfiguration, 1);
			task.handleDeferredWork(deferred, 1);
			return deferred;
		},		
		
		_updateDirty: function(originalValue, newValue, dirtyKey) {
			var isEqual = originalValue == newValue;
			if (!isEqual && dojo.isArray(newValue) && originalValue && newValue && originalValue.length === newValue.length) {
				isEqual = true; // empty arrays
				isEqual = dojo.every(newValue, function(item, idx) {
					return item == originalValue[idx];
				});
			}
			if (isEqual) {
				this.isDirty = false;
				this.editor.removeDirty(dirtyKey, DIRTY_GROUP_ID);
				this.editor.hideMessage();
			} else {
				this.isDirty = true;
				this.editor.addDirty(dirtyKey, DIRTY_GROUP_ID);
				this.editor.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
					summary: messages.info_save_first,
					closable: false
				});
			}
		}
		
	});	
	
	dojo.declare("rmps.app.ui.internal.editors.views.DataManagementEditor._ChoiceSection", [dijit._Widget, dijit._Templated], {
	
		messages: messages,
		
		_mode: null,
		_originalMode: null,
	
		templateString: '<form><div class="choiceSection">' +
				'<label title="${messages.choiceDescriptionDesc}">${messages.choiceDescription}</label>' +
				'<div class="choice">' +
					'<input id="${id}_c1" type="radio" lable="test" name="typeChoice" value="1" dojoAttachPoint="_c1" dojoAttachEvent="onclick: _toggleChoice"/>' +
					'<label for="${id}_c1">${messages.choice_externallyManagedMode}</label>' +
				'</div>' +
				'<div class="choice">' +
					'<input id="${id}_c2" type="radio" name="typeChoice" value="2" dojoAttachPoint="_c2" dojoAttachEvent="onclick: _toggleChoice"/>' +
					'<label for="${id}_c2">${messages.choice_dmManagedMode}</label>' +
				'</div>' +
			'</div></form>',
	
		parent: null,
		setMode: null, /* function setup during postCreate - can be called only once*/
		
		postCreate: function() {
			this.inherited(arguments);
			
			// Start off disabled
			this._c1.disabled=true;
			this._c2.disabled=true;
			
			var self = this;
			this.setMode = (function() {
				var firstCall = true;
				return function(isDmManaged) {
					if (firstCall) {
						firstCall = false; // disallow further calls
						self._mode = isDmManaged;
						self._originalMode = dojo.clone(isDmManaged);
						if (isDmManaged) {
							self._c2.checked=true;
						} else {
							self._c1.checked=true;
						}
					}
				};
			})();
			
		},
		
		setEnabled: function(enabled) {
			this._c1.disabled = !enabled;
			this._c2.disabled = !enabled;
		},
		
		_toggleChoice: function(e) {
			this._mode = e.target.value == 2;
			this.parent._updateDirty(this._originalMode, this._mode, this);
		}
	});

	
})();
