/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.pages.SettingsPage");

dojo.require("dijit._Widget");

dojo.require("rmps.ui.editor.EditorRegistry");
dojo.require("rmps.ui.editor.AbstractPage");
dojo.require("rmps.app.ui.internal.pages.SettingsPageController");
dojo.require("rmps.app.ui.internal.ProjectLinkingEditorInput");
dojo.require("rmps.app.ui.internal.ExplorerFilterEditorInput");
dojo.require("rmps.app.ui.internal.DomainsManagementEditorInput");
dojo.require("rmps.app.ui.internal.OslcContextEditorInput");
dojo.require("rmps.app.ui.internal.ConfigureSpaceEditorInput");
dojo.require("rmps.app.ui.internal.LinkTypeEditorInput");
dojo.require("jazz.ui.SideNavigationLayout");
dojo.require("net.jazz.ajax.ui.ErrorPage");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("rmps.core.error");
dojo.require("jazz.app.context");

dojo.require("dojo.i18n");

dojo.requireLocalization("rmps.app.ui.internal.pages", "SettingsPageMessages");

var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var PAGE_ID = "rmps.app.settings";
var PAGE_REMOVAL_ID = "com.ibm.xtools.rmps.jfs.web.removeSettingsPage";
var ACTION_NAMES = [
	"_default",
	"projectManagement",
	"explorer",
	"modeling",
	"oslcContext",
	"configureSpace"
];

dojo.declare("rmps.app.ui.internal.pages.SettingsPage", rmps.ui.editor.AbstractPage, {

	"class": "rmps-app-page",

	messages: null,

	_activeLinkTypeEditorInputs: null,

	// private properties
	_controller: null,
	_sideNav: null,
	_status: null,
	_resizeHandle: null,

	_statusSection: null,
	_projectLinkingSection: null,
	_explorerFilterSection: null,
	_domainsManagementSection: null,
	_versioningSection: null,
	_highlightedNode: null,
	_explorerFilterEditorInput: null,
	_domainsManagementEditorInput: null,
	_configureSpaceEditorInput: null,

	constructor: function(){
		this._activeLinkTypeEditorInputs= [];
	},

	postCreate: function() {
		this.messages= dojo.i18n.getLocalization("rmps.app.ui.internal.pages", "SettingsPageMessages");
		this.containerNode = this.domNode;

		var actionRegistry = PlatformUI.getWorkbench().getActionRegistry();

		this.mediator= new rmps.app.ui.internal.pages.SettingsPage.Mediator_();

		var projectId = jazz.app.context.get().itemId;
		this._projectLinkingEditorInput = new rmps.app.ui.internal.ProjectLinkingEditorInput(projectId, this.mediator);
		this._explorerFilterEditorInput = new rmps.app.ui.internal.ExplorerFilterEditorInput(projectId);
		this._domainsManagementEditorInput = new rmps.app.ui.internal.DomainsManagementEditorInput(projectId);
		this._oslcContextEditorInput = new rmps.app.ui.internal.OslcContextEditorInput(projectId);
		this._configureSpaceEditorInput = new rmps.app.ui.internal.ConfigureSpaceEditorInput(projectId);

		// register editors
		rmps.ui.editor.EditorRegistry.register("rmps.app.ui.internal.editors.ProjectLinkingEditor", function(editorInput) {
			return editorInput instanceof rmps.app.ui.internal.ProjectLinkingEditorInput;
		});
		rmps.ui.editor.EditorRegistry.register("rmps.app.ui.internal.editors.ExplorerFilterEditor", function(editorInput) {
			return editorInput instanceof rmps.app.ui.internal.ExplorerFilterEditorInput;
		});

		rmps.ui.editor.EditorRegistry.register("rmps.app.ui.internal.editors.DomainsManagementEditor", function(editorInput) {
			return editorInput instanceof rmps.app.ui.internal.DomainsManagementEditorInput;
		});

		rmps.ui.editor.EditorRegistry.register("rmps.app.ui.internal.editors.link.LinkTypeEditor", function(editorInput) {
			return editorInput instanceof rmps.app.ui.internal.LinkTypeEditorInput;
		});

		rmps.ui.editor.EditorRegistry.register("rmps.app.ui.internal.editors.DefaultOslcContextEditor", function(editorInput) {
			return editorInput instanceof rmps.app.ui.internal.OslcContextEditorInput;
		});

		rmps.ui.editor.EditorRegistry.register("rmps.app.ui.internal.editors.ConfigureSpaceEditor", function(editorInput) {
			return editorInput instanceof rmps.app.ui.internal.ConfigureSpaceEditorInput;
		});

		// create controller and register actions
		this._controller = new rmps.app.ui.internal.pages.SettingsPageController(this);

		this.inherited(arguments);
	},

	startup: function() {
		var editorView = this.getEditorView();
		if (editorView) {
			this._sideNav.setMainContent(editorView.domNode);
		}
	},

	subscribeActions: function() {
		this.subscribeAllActions(this._controller);
	},

	registerActions: function() {
		this.registerActionsList(ACTION_NAMES);
	},

	createPageContents: function(){
		// init side navigation
		this._sideNav = new jazz.ui.SideNavigationLayout();

		var section = this._sideNav.addNavigationSection(dojo.doc.createElement("div"), this.messages.linkManagement, true, false);
		var explorersection = this._sideNav.addNavigationSection(dojo.doc.createElement("div"), this.messages.explorerSettings, true, false);
		var modelingsection = this._sideNav.addNavigationSection(dojo.doc.createElement("div"), this.messages.modelingSettings, true, false);
		var oslcSection = this._sideNav.addNavigationSection(dojo.doc.createElement("div"), this.messages.oslcSettings, true, false);
		var versioningSection = this._sideNav.addNavigationSection(dojo.doc.createElement("div"), this.messages.versioningSettings, true, false);

		this._projectLinkingSection = section.addSectionRow(
				this._projectLinkingEditorInput.getName(),
				this._projectLinkingEditorInput.getPresentationUri(),
				this._projectLinkingEditorInput.getIconUri(), null);
		this._explorerFilterSection = explorersection.addSectionRow(
				this._explorerFilterEditorInput.getName(),
				this._explorerFilterEditorInput.getPresentationUri(),
				this._explorerFilterEditorInput.getIconUri(), null);
		this._domainsManagementSection = modelingsection.addSectionRow(
				this._domainsManagementEditorInput.getName(),
				this._domainsManagementEditorInput.getPresentationUri(),
				this._domainsManagementEditorInput.getIconUri(), null);
		this._oslcContextSection = oslcSection.addSectionRow(
				this._oslcContextEditorInput.getName(),
				this._oslcContextEditorInput.getPresentationUri(),
				null, null);
		this._versioningSection = versioningSection.addSectionRow(
				this._configureSpaceEditorInput.getName(),
				this._configureSpaceEditorInput.getPresentationUri(),
				null, null);

		this.addChild(this._sideNav);
		this.inherited(arguments);
	},

	getPageId: function(){
		return PAGE_ID;
	},

	getPageRemovalId: function(){
		return PAGE_REMOVAL_ID;
	},

	showProjectManagement: function(args) {
		this._highlightNode(this._projectLinkingSection);
		this.pageHelper.openEditor(this._projectLinkingEditorInput, args);
	},

	createProjectManagement: function(args) {
		this._highlightNode(this._projectLinkingSection);

		var input= this._activeLinkTypeEditorInputs[args.linkType];

		var call= dojo.hitch(this, function(){
			input = new rmps.app.ui.internal.LinkTypeEditorInput(args.linkType, this.mediator);
			this._activeLinkTypeEditorInputs[args.linkType]= input;
		});

		if (!input) {
			call();
		} else if (input.linkTypeUri !== args.linkType) {
			/*If input.linkTypeUri!= args.linkType this means that the editor input value URI was updated in it editor. This is
			 * because of updating the link type URI value or creating new link type. So, we have to make sure that when trying
			 * to re-open the editor (esspecially for the creating new link type) to avoid getting the previously created/edited 
			 * link type with the same URI*/
			this.pageHelper.closeEditorInput(input, true, true);

			/*Why do we make this call? This is in order to make sure that getId will get the actual editor input value
			 * that was set recently, so it would be in sync with the current active list */
			input.linkTypeUri= args.linkType;
			this.pageHelper.closeEditorInput(input, true, true);
			call();
		}

		this.pageHelper.openEditor(input, args);
	},

	showExplorerSettings: function(args) {
		this._highlightNode(this._explorerFilterSection);
		this.pageHelper.openEditor(this._explorerFilterEditorInput, args);
	},

	showModeling: function(args) {
		this._highlightNode(this._domainsManagementSection);
		this.pageHelper.openEditor(this._domainsManagementEditorInput, args);
	},

	showOslcContext: function() {
		this._highlightNode(this._oslcContextSection);
		this.pageHelper.openEditor(this._oslcContextEditorInput);
	},

	showConfigureSpace: function() {
		this._highlightNode(this._versioningSection);
		this.pageHelper.openEditor(this._configureSpaceEditorInput);
	},

	_highlightNode: function(node){
		if (this._highlightedNode !== node) {
			if (this._highlightedNode) {
				dojo.removeClass(this._highlightedNode, "selected");
			}
			dojo.addClass(node, "selected");
			this._highlightedNode = node;
		}
	}
});

dojo.declare("rmps.app.ui.internal.pages.SettingsPage.Mediator_", null, {
	notifiers: null,

	constructor: function(){
		this.notifiers= [];
	},

	addNotifier: function(func, identifier){
		/*Checking for func being function or not is the caller responsability */
		this.notifiers[identifier](func);
	},

	removeNotifier: function(identifier){
		delete this.notifiers[identifier];
	},

	execute: function(identifier){
		this.notifiers[identifier](arguments)(func);
	}
});

})();
