/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){
dojo.provide("rmps.app.ui.internal.pages.design.widgets.DomainLazyTreeGrid");

dojo.require("dojox.grid.LazyTreeGrid");

dojo.require("rmps.ui.explorer.ExplorerUtilities");
var ExplorerUtil = rmps.ui.explorer.ExplorerUtilities;

dojo.declare("rmps.app.ui.internal.pages.design.widgets.DomainLazyTreeGrid", dojox.grid.LazyTreeGrid, {
	_saveState: function() {},
	_preventCache: (new Date()).getTime(),
	"class" : "rmps-app-pages-design-treeGrid",
	_resourceLinks: null,
	_buttonInx: 0,
	
	constructor: function() {
		this._resourceLinks = [];
		this.toolbars = [];
		this.needLink = [];
	},
	
	destroyResourceLinks: function() {
		dojo.forEach(this._resourceLinks, function(link) {
			if (link) {
				link.destroy();
			}
		});
	},
	
	
	destroy: function() {
		this.destroyResourceLinks();
		this.inherited(arguments);
		if (this._setFocusIndexHandle) {
			dojo.disconnect(this._setFocusIndexHandle);
		}
	},
	
	postCreate: function() {
		this.inherited(arguments);

		this._setFocusIndexHandle = dojo.connect(this.focus, "setFocusIndex", this, "_setFocusIndex" );

		dojo.forEach(this.views.views, function(view){
			this.connect(view, "onAfterRow", dojo.hitch(this, "_onAfterRow", view.index));
		}, this);
	},
	
	canSort: function(){
		return false;
	},
	
	_fetchItems: function(idx, onBegin, onComplete, onError){
		if(this._pending_requests[this.reqQueue[idx].startRowIdx]){
			return;
		}
		this.showMessage(this.loadingMessage);
		var level = this.reqQueue[idx].startTreePath.split("/").length - 1;
		this._pending_requests[this.reqQueue[idx].startRowIdx] = true;
		var root_children = this.model && this.model.root && this.model.root.children; 
		if(level === 0 && !root_children){
			this.store.fetch({
				start: parseInt(this.reqQueue[idx].startTreePath, 10),
				startRowIdx: this.reqQueue[idx].startRowIdx,
				count: this.reqQueue[idx].count,
				query: this.query,
				sort: this.getSortProps(),
				queryOptions: this.queryOptions,
				onBegin: onBegin,
				onComplete: onComplete,
				onError: onError
			});
		}else{
			var startTreePath = this.reqQueue[idx].startTreePath;
			var parentTreePath = startTreePath.substring(0, startTreePath.lastIndexOf("/"));
			var startIdx = startTreePath.substring(startTreePath.lastIndexOf("/") + 1);
			var parentItem = root_children? this.treeModel.root : this.cache.getItemByTreePath(parentTreePath);
			if(!parentItem){
				throw new Error("Lazy loading TreeGrid on fetch error:");
			}
			var parentId = this.store.getIdentity(parentItem);
			var queryObj = {
				start: parseInt(startIdx, 10),
				startRowIdx: this.reqQueue[idx].startRowIdx,
				count: this.reqQueue[idx].count,
				parentId: parentId,
				sort: this.getSortProps()
			};
			var _this = this;
			var onFetchComplete = function(){
				if(arguments.length === 1){
					onComplete.apply(_this, [arguments[0], queryObj]);
				}else{
					onComplete.apply(_this, arguments);
				}
			};
			if (onBegin && root_children) {
				onBegin(root_children.length, queryObj);
			}
			this.treeModel.getChildren(parentItem, onFetchComplete, onError, queryObj);
		}
	},
	
	_addItem: function(item, index, noUpdate, dontUpdateRoot) {
		if (arguments.length === 3) {
			// dontUpdateRoot is not set. Set it true.
			this._addItem(item, index, noUpdate, true);
		} else {
			this.inherited(arguments);	
		}
	},
	
	// we have to replace modify the storeId with the treepath to make it unique,
	// since the same item may appear in multiple places in the grid. 
	_getUniqueId: function (index, item) {
		var treePath = this.cache.getTreePathByRowIndex(index);
		return treePath + "/" + this.store.getIdentity(item);
	},
	
	
	_createItem: function (item, index){
		var idty = this._getUniqueId(index, item);
		var o = this._by_idty[idty] = { idty: idty, item: item };
		return o;
	}, 
	
	_onExpandoComplete: function(childItems, request, size){
		var parentTreePath = this.cache.getTreePathByRowIndex(this.expandoRowIndex);
		if(size && !isNaN(parseInt(size, 10))){
			size = parseInt(size, 10);
		}else{
			size = childItems.length;
		}
		var i, j = 0, len = this._by_idx.length;
		for(i = this.expandoRowIndex + 1; j < size; i++, j++){
			this.cache.insertItem(i, {
				item: null,
				treePath: parentTreePath + "/" + j,
				expandoStatus: false
			});
		}
		this.updateRowCount(this.cache.items.length);
		
		this.cache.updateCache(this.expandoRowIndex, {childrenNum: size});
		for(i = 0; i < size; i++){
			this.cache.updateCache(this.expandoRowIndex + 1 + i, {item: childItems[i]});
		}
		// insert NULL to grid._by_idx
		for(i = 0; i < size; i++){
			this._by_idx.splice(this.expandoRowIndex + 1 + i, 0, null);
		}
		// insert the fetched items to grid._by_idx
		for(i = 0; i < Math.min(size, this.rowsPerPage); i++){
			// tma: Changes start here
			var idty = this._getUniqueId(this.expandoRowIndex + 1 + i, childItems[i]);
			// tma: Changes end here
			this._by_idty[idty] = { idty: idty, item: childItems[i] };
			this._by_idx.splice(this.expandoRowIndex + 1 + i, 1, this._by_idty[idty]);
		}
		
		this._updateChangedRows(this.expandoRowIndex + 1);
		this.toggleLoadingClass(false);
		this.stateChangeNode = null;
		if(this._loading){
			this._loading = false;
		}
		// correct focus
		this.focus._delayedCellFocus();
		
		//---------------------------------------------------------------------------------------------------------------------------------------------------------
		if (this.explorer.expandDeferred) {
			this.explorer.expandDeferred.callback();
		}
		//----------------------------------------------------------------------------------------------------------------------------------------------------------
	},
	
	_onFetchComplete: function(items, request, size) {
		this.inherited(arguments);
		if (request.startRowIdx === 0) {
			this.explorer.restoreLastState();
		}
	},
	
	expandoFetch: function(rowIndex, open) {
		this.inherited(arguments);
		if (!open) {
			if (this.explorer.expandDeferred) {
				this.explorer.expandDeferred.callback();
			}			
		}
	},
	
	
	createLink: function(inRowNode, inRowIndex) {
		var _this = this;
		var resourceLinks = this._resourceLinks;
		var row = this._by_idx[inRowIndex];
		var item = row ? row.item : null;
		if (item) {
			var uri = ExplorerUtil.getResourceURIFromNodeID(this.model.getIdentity(item));
			dojo.query("a.resource-link-attach-anchor", inRowNode).forEach(
					function(n){
						if (item.isEllipsis) {
							n.onclick = dojo.hitch(_this, function(){
								this.onEllipsisClick(inRowIndex);
								return false;
							});
							n.href = "#";
						} else {
							if (resourceLinks[inRowIndex]) {
								resourceLinks[inRowIndex].destroy();
							}
							resourceLinks[inRowIndex] = ExplorerUtil.createResourceLink(uri, n);
						}
			});			
		}
	},
	
	rowHovered: function(inRowNode, inRowIndex) {
		var toolbar = this.toolbars[inRowIndex];
			if (toolbar && !toolbar._populated) {
				var model =  this.treeModel;
				var datanode = this.getItem(inRowIndex);
				var uri = ExplorerUtil.getResourceURIFromNodeID(model.store.getIdentity(datanode));
				this.explorer.populateToolbar(toolbar, inRowIndex, datanode, uri);
				toolbar._populated = true;
				
			}	
			if (this.needLink[inRowIndex]) {
				this.createLink(inRowNode, inRowIndex);
				this.needLink[inRowIndex] = false;
			}
	},
	
	_onAfterRow: function(viewIndex, inRowIndex, cells, inRowNode) {
		dojo.connect(inRowNode, "onmouseenter", dojo.hitch(this, function(event){ 
					this.rowHovered(inRowNode, inRowIndex);
				})
		);

	},
	
	// accessibility -- if user tabs to first column, show toolbar
	_setFocusIndex: function(inRowIndex, inCellIndex) {
		if (inCellIndex == 0 || inCellIndex == 2) {
			var inRowNode = this.getRowNode(inRowIndex);
			this.rowHovered(inRowNode, inRowIndex);	
			var toolbar = this.toolbars[inRowIndex];			
			var buttons = toolbar.getChildren();
			if (buttons.length > 0) {
				// select row
				this.selection.deselectAll();
				this.selection.select(inRowIndex);
				
				// show toolbar
				if (inCellIndex === 0) {
					dojo.addClass(toolbar.domNode, "visibleToolbar");
					
					// remember toolbar for key actions
					this._toolbar = toolbar;
					this._buttons = buttons;
					this._enabledButtons = [];
					for (var i=0; i<this._buttons.length; i++) {
						if (this._buttons[i].enabled) {
							this._enabledButtons.push(i);
						}
					}
					if (this._buttonInx > this._enabledButtons.length-1) {
						this._buttonInx = this._enabledButtons.length-1;
					}
					this._refreshButtons();
				} else {
					// remember row if user presses enter
					this._openLinkOnRow = inRowIndex;
				}
			}
		}

	},
	
	// accessibility-- if toolbar is shown because of tabbing, process left/right/enter keys to highlight/click on buttons
	onKeyDown: function(e) {
		var processed = false;
		if ( !(e.altKey || e.metaKey)) {
			var dk = dojo.keys;
			if (this._buttons) {
				switch(e.keyCode){
					case dk.LEFT_ARROW:
						if (this._buttonInx>0) {
							processed = true;
							this._buttonInx--;
							this._refreshButtons();
						}
						break;
					case dk.RIGHT_ARROW:
						if (this._buttonInx < this._enabledButtons.length-1) {
							processed = true;
							this._buttonInx++;
							this._refreshButtons();
						}
						break;
					case dk.ENTER:
						this._buttons[this._enabledButtons[this._buttonInx]].onClick();
						processed = true;
						break;
				}
			}
			if (this._openLinkOnRow) {
				switch(e.keyCode){
					case dk.ENTER:
						if (e.shiftKey) {
							var uri = this._resourceLinks[this._openLinkOnRow].presentationUri;
							location.href =uri;
						} else {
							processed = true;							
						}
						break;
					default:
						processed = e.shiftKey;
						break;
				}
			}
		}
		if (processed) {
			dojo.stopEvent(e);
		} else {
			if (this._buttons) {
				for (var i=0; i<this._enabledButtons.length; i++) {
					this._buttons[this._enabledButtons[i]]._setEnabledAttr(true);
				}
				delete this._buttons;
				if (this._toolbar.domNode) {
					dojo.removeClass(this._toolbar.domNode, "visibleToolbar");
				}
			}
			delete this._openLinkOnRow;
			this._noResize = true;
			this.inherited(arguments);
			this._noResize = false;
		}
	},
	
	sizeChange: function(){
		// no grid resize on key down event-- messes up accessibility selected cell (DOJO bug)
		if (!this._noResize) {
			this.inherited(arguments);
		}
	},
	
	// accessibility-- use enabled state to indicate which button is active
	_refreshButtons: function() {
		for (var i=0; i<this._enabledButtons.length; i++) {
			this._buttons[this._enabledButtons[i]]._setEnabledAttr(i===this._buttonInx);
		}
	},
	
	onEllipsisClick: function() {
		this.explorer.getOwner().refresh();
	},
	
	createParentEllipsis: function(item) {
		var cache = this.cache.items;

		var tp,info, i;
		
		if (item.treePath) {
			tp = new dojox.grid.TreePath(item.treePath, this);
			info = this.cache.getInfoByItem(tp.item());			
		} else {
			info = this.cache.getInfoByItem(item);
			tp = new dojox.grid.TreePath(info.treePath, this);
		}
		
		
		if (tp && tp.level > 1) {
			var parent_tp = tp.parent();
			var grand_parent_tp = parent_tp.parent();
			var parent_info = parent_tp && this.cache.getInfoByItem(parent_tp.item());
			var grand_info = grand_parent_tp && this.cache.getInfoByItem(grand_parent_tp.item());
			var children = grand_info && grand_info.item.children;
			if (parent_info) {
				for (i = 0; i < cache.length; i++) {
					if (!this.isChildOfTreePath(grand_info.treePath, cache[i].treePath) && !this.isChildOfTreePath(cache[i].treePath, grand_info.treePath)) {
						this._onDelete(cache[i].item);
					}
				}
				for (i = 0; i < cache.length; i++) {
					if (!this.isChildOfTreePath(grand_info.treePath,cache[i].treePath) || grand_info.treePath === cache[i].treePath) {
						this._by_idx.splice(i, 1);
						this.cache.items.splice(i, 1);
						i--;
					}
				}
				this.updateRowCount(this.cache.items.length);

				var ellipsis = this.explorer._createEllipsisItem(tp.level-1, item.item.__parent, []);
				ellipsis.children = children;
				var idx = 0;
				var new_treepath = "0";
				this.cache.insertItem(idx,
						{
							item: ellipsis,
							treepath: "0", 
							chlidrenNum: children.length,
							expandoStatus: true
						});
				
				var idty = ellipsis.ellipsisUUID;
				this._by_idty[idty] = { idty: idty, item: ellipsis };
				this._by_idx.splice(0, 0, this._by_idty[idty]);
				
				for (i = 0; i < children.length; i++) {
					var id = this.model.store.getIdentity(children[i]);
					var child_info = this.cache.getInfoByItem(children[i]);
					var child_path = new_treepath + "/" + i;
					var child_idx = idx+i+1;
					if (child_info && child_info.treePath) {
						this.replaceTreePath(child_info.treePath, child_path, child_info.rowIdx);
					}
				}
				this.updateRowCount(this.cache.items.length);
				this._updateChangedRows(0);
			}
		}
	},
	
	isChildOfTreePath: function(parent, child) {
		return (parent.length <= child.length) &&
		(child.substr(0, parent.length) === parent);
	},
	
	isChildOf: function(parent, child) {
		return (parent.treePath.length < child.treePath.length) &&
			(child.treePath.substr(0, parent.treePath.length) === parent.treePath);
	},
	
	createSiblingEllipsis: function(item, bUpdate) {		
		var tp,info;
		
		if (item.treePath) {
			tp = new dojox.grid.TreePath(item.treePath, this);
			info = this.cache.getInfoByItem(tp.item());			
		} else {
			info = this.cache.getInfoByItem(item);
			tp = new dojox.grid.TreePath(info.treePath, this);
		}

		var parent_tp = tp.parent();
		if (!parent_tp) {
			return;
		}
		var parent_item = parent_tp.item();
		var siblings = parent_item.children;
		
		var parent_info = this.cache.getInfoByItem(parent_item);
		if (siblings.length > 2) {
			var new_children = [tp.item()];
			var ellipsis = this.explorer._createEllipsisItem(siblings.length-1, parent_item, []);
			for (var i = 0; i < siblings.length; i++)
			{
				if (siblings[i] !== tp.item()) {
					this._onDelete(siblings[i]);
				} 
			}
			info = this.cache.getInfoByItem(tp.item());
			var idx = info.rowIdx+1;
			this.replaceTreePath(info.treePath, parent_tp.toString() + "/0",  info.rowIdx);
			while (idx < this.cache.items.length && this.isChildOf(this.cache.items[info.rowIdx], this.cache.items[idx])) {
				idx++;
			}
			this.cache.insertItem(idx, {item: ellipsis,
				treePath: parent_tp.toString() + "/" + new_children.length, 
				expandoStatus: false});
			this.cache.updateCache(parent_info.rowIdx, {childrenNum: 2});
			this.updateRowCount(this.cache.items.length);

			var idty = ellipsis.ellipsisUUID;
			this._by_idty[idty] = { idty: idty, item: ellipsis };
			this._by_idx.splice(idx, 0, this._by_idty[idty]);
			new_children.push(ellipsis);
			parent_item.children = new_children;
			if (bUpdate) {
				this._updateChangedRows(0 /*info.rowIdx*/);
			}
			
		}
	},
	
	focusRow: function(rowIdx) {
		var item = this.cache.items[rowIdx];
		var original_item = item;
		var treePath, parent_tp, parent_item, info;

		this.collapseIrrelevantNodes(item.treePath);		
		
		do {
			treePath = new dojox.grid.TreePath(item.treePath, this);
			this.createSiblingEllipsis(item);
			parent_tp = treePath.parent();
			parent_item = parent_tp && parent_tp.item();
			info = parent_item && this.cache.getInfoByItem(parent_item);
			item = info && this.cache.items[info.rowIdx];
		} while (item);
		this.createParentEllipsis(original_item);
		this._updateChangedRows(0);
		var new_info = this.cache.getInfoByItem(original_item.item);
		if ( ("rowIdx" in new_info) && !new_info.expandoStatus) {
			this.explorer.toggleExpandRow(new_info.rowIdx);
		}
	},
	
	
	collapseIrrelevantNodes: function(treePath){
		var cache = this.cache.items;
		for (var i = 0; i < cache.length; i++) {
			if (cache[i].expandoStatus && !this.isChildOfTreePath(cache[i].treePath, treePath)) {
				this.explorer.toggleExpandRow(i);
			}
		}
	},
	
	startsWith: function(str, prefix) {
		return str.length >= prefix.length && str.substr(0, prefix.length) === prefix;
	},
	
	replaceString: function(where, old_s, new_s) {
		return new_s + where.substring(old_s.length, old_s.length + where.length - old_s.length);
	},
	
	replaceTreePath: function(oldPath, newPath, startIdx) {
		var cache = this.cache.items;
		for (var i = startIdx; i < cache.length; i++) {
			if (this.startsWith(cache[i].treePath, oldPath)) {
				cache[i].treePath = this.replaceString(cache[i].treePath, oldPath, newPath);
			} else {
				break;
			}
				
		}
		
	}
	

});

dojo.extend(dojox.grid._LazyExpando, {
	onToggle: function(event, bNotUser){
		// Summary
		//		Function for expand/collapse row
		if (!bNotUser) {
			this.view.grid.explorer.saveAfterExpand();
		}
		
		this.setOpen(!this.view.grid.cache.getExpandoStatusByRowIndex(this.rowIdx));
		if (event) {
			try{
				dojo.stopEvent(event);
			}catch(e){}
		}
	}
});

}());