/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.util.DomainRegistryUtil");

dojo.require("rmps.core.RootServices");
dojo.require("rmps.core.client");
dojo.require("jazz.app.context");
dojo.require("rmps.core.group.GroupClient");
dojo.require("rmps.core.user.UserClient");
var GroupClient = rmps.core.group.GroupClient;

var DOMAIN_REG_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "domainRegistry");
var userUri;
var DESIGN_SERVICE = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "design");

rmps.app.ui.internal.util.DomainRegistryUtil = {
		
		RECENTLY_SELECTED_RESOURCES: "dm-resources-create-type",
		MAX_SELECTION_HISTORY: 10,
		UNSET_CONTEXT: "UNSET",
		
		isCurrentProjectDSP: function (callback) {
			// callback: function(isDsp: bool , isSystem : bool, eTag : string)
			//    param isDsp - true if the current project area is a DSP, false otherwise
			//    param isSystem - true if isDsp is true and its a system managed DSP
			//    param canCreateDsp - internal (flag says no user dsp exists)
			var url = DOMAIN_REG_PATH + "/dsp";
			
			var def = rmps.core.client.xhrHead({
				url: url,
				handleAs: 'json',
				headers: {
					"Accept" : "application/json"
				},
				load: function(response, ioArgs) {
					return ioArgs.xhr.getResponseHeader("Project-Area-Type");
				}
			}, true /* add project id */);
			
			def.addCallback(function(data){
				var status = parseInt(data, 10);
				if (isNaN(status)) {
					console.error("Unrecognizable response from server: " + data);
					callback(false, false);
					return;
				}
				
				// 600 = non-dsp
				// 601 - non-dsp, can create
				// 602 = dsp
				// 603 = system dsp
				var isDsp = status > 601;
				var isSystem = status === 603;
				var canCreateDsp = status === 601;
				
				// Developer back door to enable Setting as DSP
				if (!isDsp) {
					var params = dojo.queryToObject(window.location.href);
					if (params.createDsp) {
						canCreateDsp = true;
					}
				}
				
				callback(isDsp, isSystem, canCreateDsp);
			});
		},
		
		getCreateableURIs: function (callback, groupBy) {
			var content = {};
			if (userUri === undefined) {
				userUri = jazz.app.currentApplication.config.user ? rmps.core.user.UserClient.getUserUri(jazz.app.currentApplication.config.user.userId) : null;
			}
			if (userUri) {
				content.userURI = userUri;
			}
	
			rmps.core.client.xhrGet({
				url: DESIGN_SERVICE + "/actions",
				handleAs: 'json',
				headers: {
					"Accept" : "application/json",
					"Content-Type" : "application/json"
				},
				content: content
			}, true /* send project Id */).addCallback(function(data) {
				return callback(data); // save for later
			});	
		},
		
		getAssociatedDomains: function (callback, groupBy) {
			/*
			 * callback: function(uris : [string])
			 * 		param uris  - an array of domain uris associated with the current context
			 */
			
			var projectId = jazz.app.context.get().itemId;
			
			var url = DOMAIN_REG_PATH + "/domains/" + projectId;
			
			var def = rmps.core.client.xhrGet({
				url: url,
				handleAs: 'json',
				headers: {
					"Accept" : "application/json"
				},
				load: function(response, ioArgs) {
					return response;
				}
			}, true /* add project id */);
			def.addCallback(function(data, groupBy) {
				callback(data);
			});
		},
		
		isProjectActivelyManaged: function (callback) {
			/*
			 * Test the current project area's management mode to determine if we are externally or actively managed
			 * 
			 * callback: function(isActivelyManaged : boolean) 
			 * 		@param isActivelyManaged  true if actively managed, false if externally (import) managed
			 */
			var deferredGroupId = GroupClient.getGroupId(jazz.app.context.get().itemId, null, true);
			deferredGroupId.addBoth(this, function(response){
				if ( response instanceof Error) {
					if (response.status && (response.status === 410 || response.status === 404)) {
						// if can't find groupId, project area has not been 
						// activated yet, so we are externally managed
						callback(false);
					}else {
						/*
						 * Return any error info.  Most callers should not need to worry about this data.
						 */
						callback(false, {
							error: response,
							ioArgs: deferredGroupId.ioArgs});
					}
				} else {
					var groupId = response;
					var deferredGroupData = GroupClient.getGroup(groupId, null);
					deferredGroupData.addBoth(this, function(response) {
						if ( response instanceof Error) {
							/*
							 * Return any error info.  Most callers should not need to worry about this data.
							 */
							callback(false, {
								error: response,
								ioArgs: deferredGroupId.ioArgs});
						} else {
							var group = response[0];
							callback(group.groupEditable);
						}
					});
				}
			});	
		},
		
		loadRecentSelections: function(args) {
			var value = dojo.cookie(this.RECENTLY_SELECTED_RESOURCES);
			var cookies= value?dojo.fromJson(value): {};
			if(!cookies){
				return [];
			}
			
			if(!args){
				args= {};
			}
			
			var context= jazz.app.context.get();
			var projectId= context.itemId;
			var projectCookie= cookies[projectId];
			if(!projectCookie){
				return [];
			}
			
			if(args.all){
				return projectCookie;
			}
			
			var currentCookie= projectCookie[context.baseline||this.UNSET_CONTEXT];
			
			if(!currentCookie|| !dojo.isObject(currentCookie)){
				return [];
			}
			
			var userCookies= currentCookie[jazz.app.currentApplication.config.user.userId];
			var values= dojo.isArray(userCookies)?userCookies: [];
			
			//Return in reverse order so we receive the list based on the last selected items
			return values.reverse();
		},
		
		/** 
		 *Example for cookies Json file:
		 *		dm-resources-create-type= {
		 *			_WmduwX41EeGTjOQsKsIvpw: {
		 *						https://localhost:9444/dm/streams/9: {
		 *							test: [
		 *								"http://ibm.com/owl/library#Library",
		 *								"http://jazz.net/ns/dm/sketcher#Sketch",
		 *								"http://jazz.net/ns/dm/folder#Folder"
		 *							]
		 *						}
		 *			}
		 *		}
		 * 
		 * max: It is used to limit the number of the selected items history to be accordng to specific length. So, if the items exceeded this number, then we splice the first items
		 * 
		 */
		addCreationSelection: function(selection, max) {
			if(!max){
				max= this.MAX_SELECTION_HISTORY;
			}
			var value = dojo.cookie(this.RECENTLY_SELECTED_RESOURCES);
			var cookies= value?dojo.fromJson(value): {};
			
			var cuurrentContext= jazz.app.context.get().baseline||this.UNSET_CONTEXT;
			var current= cookies[jazz.app.context.get().itemId];
			
			var currentUser;
			if(!current){
				current={};
				current[cuurrentContext]= {};
				current[cuurrentContext][jazz.app.currentApplication.config.user.userId]=[selection];
			}else{
				var forCurrentContext= current[cuurrentContext];
				//Check if it is undefined (means a user's cookies for that context have not been set yet)
				//Check if it is not an object (means a user cookies for that context have been corrupted)
				//Check if it is not an object (means a user cookies for that context have been corrupted and set to an array)
				if(!forCurrentContext|| !dojo.isObject(forCurrentContext)|| dojo.isArray(forCurrentContext)){
					current[cuurrentContext]= {};
					current[cuurrentContext][jazz.app.currentApplication.config.user.userId]=[selection];
				}else{
					currentUser= current[cuurrentContext][jazz.app.currentApplication.config.user.userId];
					if(!currentUser|| !dojo.isArray(currentUser)){
						currentUser= current[cuurrentContext][jazz.app.currentApplication.config.user.userId]= [];
					}
					var indexOf= dojo.indexOf(currentUser, selection);
					if(indexOf>-1){
						currentUser.splice(indexOf, 1);
					}
					currentUser.push(selection);
					current[cuurrentContext][jazz.app.currentApplication.config.user.userId]= currentUser;
				}
			}
			
			if(max&& currentUser){
				var difference= currentUser.length-max;
				if(difference>0){
					currentUser.splice(0, difference);
					current[cuurrentContext][jazz.app.currentApplication.config.user.userId]= currentUser;
				}
			}
			cookies[jazz.app.context.get().itemId]= current;
			dojo.cookie(this.RECENTLY_SELECTED_RESOURCES, dojo.toJson(cookies), {expires: 90});
		}
};

})();