/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/

dojo.provide("com.ibm.xtools.rmps.setup.ui.internal.wizard.FinalizeStep");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.FinalizeStep");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.xtools.rmps.setup.ui.internal.wizard", "FinalizeStepMessages");

var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;

(function() {
dojo.declare("com.ibm.xtools.rmps.setup.ui.internal.wizard.FinalizeStep", com.ibm.team.repository.web.ui.internal.wizard.FinalizeStep, {
	
	userSyncCompleted: false,
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.xtools.rmps.setup.ui.internal.wizard", "FinalizeStepMessages");
		this.stepTitle = messages.stepTitle;
	},

	postCreate: function() {
		this.inherited(arguments);
		this.nowSetCompletedState(false);

		// Create DOM for the individual finalize operations
		this.domApplyLicense = this._addOperationDiv(this.messages.finalizeApplyLicense, this.messages.finalizeApplyLicenseDescription);
		this.domImportProcessTemplate = this._addOperationDiv(this.messages.finalizeImportProcessTemplate, this.messages.finalizeImportProcessTemplateDescription);
		this.domConfigureVVC = this._addOperationDiv(this.messages.finalizeConfigureVVC, this.messages.finalizeConfigureVVCDescription);
		this.domProvision = this._addOperationDiv(this.messages.finalizeProvision, this.messages.finalizeProvisionDescription);
		this.domStorageAreasCreation = this._addOperationDiv(this.messages.finalizeStorageAreasCreation, this.messages.finalizeStorageAreasCreationDescription);
		this.domProjectAreasCreationCore = this._addOperationDiv(this.messages.projectAreasCreationCore, this.messages.projectAreasCreationCoreDescription);
		this.domProjectAreasCreationOther = this._addOperationDiv(this.messages.projectAreasCreationOther, this.messages.projectAreasCreationOtherDescription);
		this.domProjectAreasCreationUser = this._addOperationDiv(this.messages.projectAreasCreationUser, this.messages.projectAreasCreationUserDescription);
		this.domDeploySystemLinkTypes = this._addOperationDiv(this.messages.deploySystemLinkTypes, this.messages.deploySystemLinkTypesDescription);
	},
	
	isFinalizationNeeded: function() {
		var self = this;
		var isPopupBlockerEnabled = this.detectPopupBlocker();
		if (isPopupBlockerEnabled) {
		    // A popup blocker was detected, we cannot continue
			self.header.showMessage({type: "ERROR", summary: self.messages.loginPopupBlocked });
			self.nowSetCompletedState(false);
		} else {
			var dFinalize = this.finalizationIsNeeded();
			dFinalize.addCallback(function(response){
				if(response === "true") {
					//Finalize needed
					var srh = new ServiceResponseHandler(self, "setJTSUrl", "handleAppNotRegistered");
					ServerStatusClient.getConfiguredJazzTeamServerURI(srh);
				} else {
					self.header.showMessage({type: "OK", summary: self.messages.noFinalizeNeeded });
					self.nowSetCompletedState(true);
					dojo.addClass(self.progressSection, "noDisplay");
				}
			});
			dFinalize.addErrback(function(errObject){
				//Allow the user to try to finalize since we don't know if they have already done so.
				self._handleError(self.messages.finalizeCheckError, errObject);
				self.startButton.disabled = false;
			});
		}
	},
	
	showCompletedMessage: function() {
		// overridden
		// userSync calls this when it is successful
		// Note: it may partially be successful but thats still ok...
		this.applyInternalLicense();
	},
	
	_setCompletedState: function(state) {
		// overridden
		// userSync calls this when it completes.
		// We dont want to do anything here.
	},
	
	nowSetCompletedState: function(state) {
		this.isCompleted = state;
		this.triggerButtonStateUpdate();
	},
	
	nowShowCompletedMessage: function() {
		var buttonLabel = this.useFinishButton() === true ? this.messages.finish : this.messages.next;
		this.header.showMessage({
			type: "OK",
			summary: dojo.string.substitute(this.messages.allComplete, [ buttonLabel ])
		});
		this.nowSetCompletedState(true);
	},
	
	applyInternalLicense: function() {
		var self = this;
		var domNode = this.domApplyLicense;
		var deferred = new dojo.Deferred();
		dojo.addClass(domNode, "loading");
		jazz.client.xhrPost({
			url: net.jazz.ajax._contextRoot + '/configure?action=applyLicense',
			headers: {
				"X-ibm-rmps-internal": "true"
			},
			load: function(response) {
				deferred.callback(response);
			},
			error: function(errObject) {
				deferred.errback(errObject);
			}
		});	
		deferred.addCallback(function(response){
			dojo.removeClass(domNode, "loading");
			dojo.addClass(domNode, "ok");
			self.importProcessTemplate();
		});
		deferred.addErrback(function(errObject){
			self._showInlineError(domNode, self.messages.licenseError, errObject);
		});
	},
	
	importProcessTemplate: function() {
		var self = this;
		var domNode = this.domImportProcessTemplate;
		var deferred = new dojo.Deferred();
		dojo.addClass(domNode, "loading");
		jazz.client.xhrPost({
			url: net.jazz.ajax._contextRoot + '/configure?action=importProcessTemplates',
			headers: {
				"X-ibm-rmps-internal": "true"
			},
			load: function(response) {
				deferred.callback(response);
			},
			error: function(errObject) {
				deferred.errback(errObject);
			}
		});	
		deferred.addCallback(function(response){
			dojo.removeClass(domNode, "loading");
			dojo.addClass(domNode, "ok");
			self.configureVVC();
		});
		deferred.addErrback(function(errObject){
			self._showInlineError(domNode, self.messages.operationError, errObject);
		});
	},
	
	configureVVC: function() {
		var self = this;
		var domNode = this.domConfigureVVC;
		var deferred = new dojo.Deferred();
		dojo.addClass(domNode, "loading");
		jazz.client.xhrPost({
			url: net.jazz.ajax._contextRoot + '/configure?action=configureVVC',
			headers: {
				"X-ibm-rmps-internal": "true"
			},
			load: function(response) {
				deferred.callback(response);
			},
			error: function(errObject) {
				deferred.errback(errObject);
			}
		});	
		deferred.addCallback(function(response){
			dojo.removeClass(domNode, "loading");
			dojo.addClass(domNode, "ok");
			self.provision();
		});
		deferred.addErrback(function(errObject){
			self._showInlineError(domNode, self.messages.operationError, errObject);
		});
	},
	
	provision: function() {
		var self = this;
		var domNode = this.domProvision;
		var deferred = new dojo.Deferred();
		dojo.addClass(domNode, "loading");
		jazz.client.xhrPost({
			url: net.jazz.ajax._contextRoot + '/configure?action=provision',
			headers: {
				"X-ibm-rmps-internal": "true"
			},
			load: function(response) {
				deferred.callback(response);
			},
			error: function(errObject) {
				deferred.errback(errObject);
			}
		});	
		deferred.addCallback(function(response){
			dojo.removeClass(domNode, "loading");
			dojo.addClass(domNode, "ok");
			self.storageAreasCreation();
		});
		deferred.addErrback(function(errObject){
			self._showInlineError(domNode, self.messages.operationError, errObject);
		});
	},
	
	storageAreasCreation: function() {
		var self = this;
		var domNode = this.domStorageAreasCreation;
		var deferred = new dojo.Deferred();
		dojo.addClass(domNode, "loading");
		jazz.client.xhrPost({
			url: net.jazz.ajax._contextRoot + '/dminstall?recreate=false',
			headers: {
				"X-ibm-rmps-internal": "true"
			},
			load: function(response) {
				deferred.callback(response);
			},
			error: function(errObject) {
				deferred.errback(errObject);
			}
		});	
		deferred.addCallback(function(response){
			dojo.removeClass(domNode, "loading");
			dojo.addClass(domNode, "ok");
			self.projectAreasCreationCore();
		});
		deferred.addErrback(function(errObject){
			self._showInlineError(domNode, self.messages.operationError, errObject);
		});
	},
	
	projectAreasCreationCore: function() {
		var self = this;
		var domNode = this.domProjectAreasCreationCore;
		var deferred = new dojo.Deferred();
		dojo.addClass(domNode, "loading");
		jazz.client.xhrPost({
			url: net.jazz.ajax._contextRoot + '/dtkConfigure?action=installCoreSystemDomain',
			headers: {
				"X-ibm-rmps-internal": "true"
			},
			load: function(response) {
				deferred.callback(response);
			},
			error: function(errObject) {
				deferred.errback(errObject);
			}
		});	
		deferred.addCallback(function(response){
			dojo.removeClass(domNode, "loading");
			dojo.addClass(domNode, "ok");
			self.projectAreasCreationOther();
		});
		deferred.addErrback(function(errObject){
			self._showInlineError(domNode, self.messages.operationError, errObject);
		});
	},
	
	projectAreasCreationOther: function() {
		var self = this;
		var domNode = this.domProjectAreasCreationOther;
		var deferred = new dojo.Deferred();
		dojo.addClass(domNode, "loading");
		jazz.client.xhrPost({
			url: net.jazz.ajax._contextRoot + '/dtkConfigure?action=installOtherSystemDomains',
			headers: {
				"X-ibm-rmps-internal": "true"
			},
			load: function(response) {
				deferred.callback(response);
			},
			error: function(errObject) {
				deferred.errback(errObject);
			}
		});	
		deferred.addCallback(function(response){
			dojo.removeClass(domNode, "loading");
			dojo.addClass(domNode, "ok");
			self.projectAreasCreationUser();
		});
		deferred.addErrback(function(errObject){
			self._showInlineError(domNode, self.messages.operationError, errObject);
		});
	},
	
	projectAreasCreationUser: function() {
		var self = this;
		var domNode = this.domProjectAreasCreationUser;
		var deferred = new dojo.Deferred();
		dojo.addClass(domNode, "loading");
		jazz.client.xhrPost({
			url: net.jazz.ajax._contextRoot + '/dtkConfigure?action=installUserDomain',
			headers: {
				"X-ibm-rmps-internal": "true"
			},
			load: function(response) {
				deferred.callback(response);
			},
			error: function(errObject) {
				deferred.errback(errObject);
			}
		});	
		deferred.addCallback(function(response){
			dojo.removeClass(domNode, "loading");
			dojo.addClass(domNode, "ok");
			self.deploySystemLinkTypes();
		});
		deferred.addErrback(function(errObject){
			self._showInlineError(domNode, self.messages.operationError, errObject);
		});
	},
	
	deploySystemLinkTypes: function() {
		var self = this;
		var domNode = this.domDeploySystemLinkTypes;
		var deferred = new dojo.Deferred();
		dojo.addClass(domNode, "loading");
		jazz.client.xhrPost({
			url: net.jazz.ajax._contextRoot + '/dtkConfigure?action=deploySystemLinkTypes',
			headers: {
				"X-ibm-rmps-internal": "true"
			},
			load: function(response) {
				deferred.callback(response);
			},
			error: function(errObject) {
				deferred.errback(errObject);
			}
		});	
		deferred.addCallback(function(response){
			dojo.removeClass(domNode, "loading");
			dojo.addClass(domNode, "ok");
			self.nowSetCompletedState(true);
			self.nowShowCompletedMessage();
		});
		deferred.addErrback(function(errObject){
			self._showInlineError(domNode, self.messages.operationError, errObject);
		});
	},
	
	_showInlineError: function(node, message, errObject) {
		this.inherited(arguments);
		this.nowSetCompletedState(false);
	},	
	
	_handleError: function(message, response) {
		this.inherited(arguments);
		this.nowSetCompletedState(false);
	},
	
	detectPopupBlocker: function() {
	    // return true if there is a popup blocker enabled for this server
		/*global window */
		var myTestWindow = window.open("about:blank","","directories=no,height=100,width=100,menubar=no,resizable=no,scrollbars=no,status=no,titlebar=no,top=0,location=no");
		var isPopupBlockerEnabled = true;
		if (myTestWindow) {
			myTestWindow.close();
			isPopupBlockerEnabled = false;
		}
		return isPopupBlockerEnabled;
	},

});

})();
