dojo.provide('ibm_ilog.diagram.util.BidiUtil');

dojo.require('ibm_ilog.diagram.util.BidiEngine'); 


(function(){
	var gfx = dojox.gfx;

    var gu = ibm_ilog.diagram.util.BidiUtil;
 
    var findDirAndRTLChar = function(text){
    	var len = text.length;
    	var c, ret = {dir: "ltr", findChar: false};
    	var isDirFound = false;
    	for (var i = 0; i<len; i++){
			c = text.charCodeAt(i);
			if((c >= 0x05d0 && c <= 0x065f ||
				c >= 0x066a && c <= 0x06ef ||
				c >= 0x06fa && c <= 0x07ff ||
				c >= 0xfb1d && c <= 0xfdff ||
				c >= 0xfe70 && c <= 0xfefc)){
					ret.findChar = true;
					if (!isDirFound){
						ret.dir = "rtl";
					}
					return ret;
			}
			if((c > 64 && c < 91) || (c > 96 && c < 123)){
				if (!isDirFound){
					ret.dir = "ltr";
					isDirFound = true;
				}
			}	        		
    	}
    	return ret;
    };
    
    var checkSourceDir = function(sh) {            	        	
    	var parent = this.getParent();
    	var text = sh.text;
    	if (sh.textDir.toLowerCase() == "auto" && parent && 
    			parent instanceof ibm_ilog.diagram.util.MultilineText) {
    		var firstChild = parent.children[0];
    		if (firstChild !== this){
    			if (firstChild.getShape().__beforeBidi)
    				text = firstChild.getShape().__beforeBidi;
    			else if(firstChild.getShape().text)
    				text = firstChild.getShape().text;
    		}	
    	}
    	var d = gu.checkDir(text);
    	return d;
 
	};
	
	var convertText = function(text, targetDir, sourceDir, hasBidiChar) {
	    var LRM = '\u200E', LRE = '\u202A', RLE = '\u202B', PDF = '\u202C';
		var RLM = '\u200f';
			   
	   if (ibm_ilog.diagram.isSvgWeb) {
			if (targetDir == "rtl"){
				text = ibm_ilog.diagram.util.BidiEngine.bidiTransform(text, "IRNNN", "ILNNN");
			}
		}
		else if(ibm_ilog.diagram.isSilverlight) {
			if (targetDir == "rtl"){
				text = ibm_ilog.diagram.util.BidiEngine.bidiTransform(text, "IRNNN", "VLYSN");
			}
			else {
                text = ibm_ilog.diagram.util.BidiEngine.bidiTransform(text, "ILNNN", "VLYSN");				
			}
		}
		else if(ibm_ilog.diagram.isCanvas) {
			if (targetDir == "rtl"){
				text = RLE + text + PDF;
			}
			else {
				text = LRE + text + PDF;
			}
		}
		else if(ibm_ilog.diagram.isVml){ 
            if (targetDir != sourceDir){
				if (targetDir == "rtl") {
					if (!hasBidiChar)
						text = ibm_ilog.diagram.util.BidiEngine.bidiTransform(text, "IRNNN", "ILNNN");
					else
						text = RLM + text;
				}
				else
					text = LRM + text;
			}
		}
		else if(ibm_ilog.diagram.isSvg){
			if (dojo.isFF){
				if (targetDir == "rtl"){
					text = ibm_ilog.diagram.util.BidiEngine.bidiTransform(text, "IRYNN", "VLNNN"); 					
				}
				else {
					text = ibm_ilog.diagram.util.BidiEngine.bidiTransform(text, "ILYNN", "VLNNN");					
				}							
			}
			else if(dojo.isChrome){
				text = LRM + (targetDir == "rtl"? RLE : LRE) + text + PDF;
			}
		}	
		return text;
	};
		
	gu.checkDir = function(/*String*/text) {
	//
    //	summary:
    //		Returns "ltr" or "rtl" text direction depending on first strong RTL character in the text
    //	
    
   	    var len = text.length;
	    var c;
	    for(var i = 0;i < len;i++){
		    c = text.charCodeAt(i);
		    if((c >= 0x05d0 && c <= 0x065f ||
			    c >= 0x066a && c <= 0x06ef ||
			    c >= 0x06fa && c <= 0x07ff ||
			    c >= 0xfb1d && c <= 0xfdff ||
			    c >= 0xfe70 && c <= 0xfefc)){
				    return "rtl";
		    }
		    if((c > 64 && c < 91) || (c > 96 && c < 123)){
			    return "ltr";
		    }
	    }
	    return "ltr";   //return here some default for neutrals	
    };
    		
	gu.bidiFormat = function(/*Shape*/sh) { 
	//
    //	summary:
    //		apply BidiFormat on the text of the shape, according to the text direction
    //   sh - Shape
    //	
                       
		if (sh.textDir && sh.text && sh.text.length > 1) {
			var sourceDir = "ltr", targetDir = sh.textDir;
			var sourceText = sh.text, targetText = sh.text;
			var hasBidiChar = false;
			var parent = this.getParent();
			if (parent && parent instanceof ibm_ilog.diagram.util.MultilineText){
				targetText = parent.getShape().text;
			}			
			if (ibm_ilog.diagram.isVml){
				sh.text = sh.text.replace(/[\u200E\u200F]/, "");
			    sourceText = sh.text;
				var obj = findDirAndRTLChar(sourceText); 
				sourceDir = obj.dir;
				hasBidiChar = obj.findChar;
				if (targetDir == "auto"){
					if (sourceText == targetText)
						targetDir = sourceDir;
					else
						targetDir = gu.checkDir(targetText);
				}
			}
			else if(targetDir == "auto")
				targetDir = gu.checkDir(targetText);
				
			if (ibm_ilog.diagram.isSvg && dojo.isWebKit){ 
			    if (targetDir == "rtl"){
					var node = this.getNode();
					node.setAttribute("unicode-bidi","embed");
					node.setAttribute("direction", "rtl");
				}
				else if (targetDir == "ltr"){
					var node = this.getNode();
					node.setAttribute("unicode-bidi","embed");
					node.setAttribute("direction", "ltr");
				}
			}
			else
			   sh.text = convertText(sh.text, targetDir, sourceDir, hasBidiChar);		
		}
		return sh;			
	};

    gu.complExp = function(/*Shape*/sh) {
    //
    //	summary:
    //		Returns Shape with text converted according to complex expression type
    //	
    //   sh - Shape
    //
        var ce = sh.ceType; //complex expression type,  email: "<>@.,;" or delim[<separators>]
        var textDir = "ltr", delim = "", textout = "", sourcedir = "";
        
        if (ce.match(/delim\[/) != null)
            delim = ce.substr(6, ce.length-7);
        else if (ce == "email")
            delim = "<>@.,;"
        else
            return sh;
            
  
        if (delim != ""){
            var array = "";
            dojo.forEach(sh.text, function(ch, i){
                if (delim.indexOf(ch) > -1) {
                    if (array != ""){
                        if(ibm_ilog.diagram.isVml){
                            if (array.charAt(0) == '\u200E')
	                            sourcedir = "ltr";
                            else
                                sourcedir = gu.checkDir(array);
                        }    
                        textout += convertText(array, textDir, sourcedir);
                    }     
                    textout += ch;
                    if(!ibm_ilog.diagram.isVml && !ibm_ilog.diagram.isSilverlight)
                        textout += '\u200E'; //LRM
                    array = "";
                }else    
                    array += ch;
            }); 
            if (array != ""){
                if(ibm_ilog.diagram.isVml){
                    if (array.charAt(0) == '\u200E')
                        sourcedir = "ltr";
                    else
                        sourcedir = gu.checkDir(array);
                }
                textout += convertText(array, textDir, sourcedir);
            }    
        }  
        sh.text = textout;           
        return sh;
    };

    })();
