/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| Copyright IBM Corp. 2009, 2010.  All Rights Reserved.                  |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */

(function() {

dojo.provide("rmpw.sketch.utilities.ChildBuilder");

dojo.require("dojox.gfx.path");

dojo.declare("rmpw.sketch.utilities.ChildBuilder", null, {
	
	build: function( parent, item ){
		var type = item.eClass;
		// take out the "Type" postfix and lowercase it
		type = type.substring(0, type.length - 4).toLowerCase();
		this._build[type].call(this, parent, item );
	},
	
	/* TODO: Add handling for these types. */
	//		"circle"
	//		"PolylineType":	
	//		"ImageType":				    	
	//		"TextType":
	
	_build: {
		g: function( parent, item ){
			dojo.forEach(["g", "rect", "path", "line", "ellipse"], function(type) {
				if(type in item) {
					var typeList = item[type];
					var buildFunction = this._build[type];
					for( var child, i = 0; child = typeList[i]; i++ ) {
						buildFunction.call(this, parent, child );
					}
				}
			}, this);
		},
		
		line: function( parent, item ){
			var child = new rmpw.sketch.render.Line();
			child.setSvgLine(item);
			child.setGraphicsParent(parent.template.contents);
			parent.addChild(child);
			child.elementId = "Line";
		},
	
		ellipse: function( parent, item ){		
			var child = new rmpw.sketch.render.Ellipse();
			child.setSvgEllipse(item);
			child.setGraphicsParent(parent.template.contents);
			parent.addChild(child);
			child.elementId = "Oval";
		},
		
		text: function( parent, item ){		
			var child = new rmpw.sketch.render.Text();
			child.setSvgText(item);
			child.setGraphicsParent(parent.template.contents);
			parent.addChild(child);
			child.elementId = "Text";
		},
	
		rect: function( parent, item ){		
			var child = new rmpw.sketch.render.Rectangle();
			child.setSvgRectangle(item);
			child.setGraphicsParent(parent.template.contents);
			parent.addChild(child);
			child.elementId = "Rectangle";
		},
	
		path: function( parent, item ){
			var child = new rmpw.sketch.render.Path();
			child.setSvgPath(item);
			child.setGraphicsParent(parent.template.contents);
			parent.addChild(child);
			child.elementId = "Path";
		}
	}
});

})();