/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.common.vcs.JfsResourceChangeSet");

dojo.require("rmpw.common.vcs.ChangeSet");

/* 
 *  Used to create new JFS resources (of doucment typs mostly). So, when we are working
 *  on our current ontology changes, we can create other ontologies for different documents, and then
 *  get notified with their created logical URIS.
 *
 *  Example:
 *     var resourceSetArray= [{
 * 			{
 *				changes: createObject,
 *				namespaces:  resourceNamespaces,
 *				identifier: identifier,
 *				onResourceCreation: function(){}	//A callback function to be invoked when the resource is created
 *			}
 * 	   }];
 *     var jfsResourceChangeSet= new rmpw.common.vcs.JfsResourceChangeSet(myJfsResource, resourceSetArray);
 * 	   
 * 	  //We may add extra resources to our resourceSetArray by calling for jfsResourceChangeSet.addResource(newResource) etc;
 *	  
 *    this.repository.commit(jfsResourceChangeSet)
 */
dojo.declare("rmpw.common.vcs.JfsResourceChangeSet", rmpw.common.vcs.ChangeSet, {
	
	jfsResource: null,
	resourceIdentifiers: null,
	resources: null,
	
	constructor: function(jfsResource, resources) {
		this.jfsResource = jfsResource;
		this.resources= resources;
	},
	
	addResource: function(resource){
		this.resources.push(resource);
	},
	
	removeResource: function(resource){
		var index= dojo.indexOf(this.resources, resource);
		if(index>-1){
			this.resources.splice(index, 1);
			return true;			
		}
		return false;
	},
	
	addAll: function(){
		if (!this.jfsResource) {
			return;
		}		
		for(var index = 0, resource; resource = this.resources[index]; index++) {
			this.jfsResource.addResource(resource.changes, resource.types, resource.namespaces, resource.identifier,resource.target, resource.onResourceCreation);
		}
	},
	
	removeAll: function(){
		for(var index = 0, resource; resource = this.resources[index]; index++) {
			this.jfsResource.removeResource(resource.identifier);
		}
	},
	
	commit: function() {
		this.inherited(arguments);
		this.addAll();
	},
	
	rollback: function() {
		this.inherited(arguments);
		
		/*Get rid of all added resources when rollback */
		this.removeAll();
	}
});