dojo.provide('ibm_ilog.diagram.widget.ZoomHudComponent');

dojo.require('ibm_ilog.diagram.widget.HudComponent');

(function () {
	var R = ibm_ilog.diagram.util.ErrorReporter
	var g = ibm_ilog.diagram.util.Geometry, Rect = g.Rect, EmptyRect = g.EmptyRect, Size = g.Size, PointZero = g.PointZero, Point = g.Point;

	dojo.declare('ibm_ilog.diagram.widget.ZoomHudComponent', ibm_ilog.diagram.widget.HudComponent, {
		//
		//	summary:
		//		A viewport component that displays the current zoom factor.
		//

		//
		//	_currentZoom:
		//		The current zoom factor being displayed at the component
		//
		_currentZoom: 0,

		constructor: function (viewport) {
			this._shape.setTransform(dojox.gfx.matrix.translate(ibm_ilog.diagram._invisibleLocation, ibm_ilog.diagram._invisibleLocation));
			this._shape.createRect({
				width: 30,
				height: 20
			}).setFill([0, 0, 0, 0.5]);
			this._shape.createText({
				x: 15,
				y: 15,
				align: "middle",
				text: "x" + ibm_ilog.diagram.util._round(this._zoom, 1)
			}).setFont({
				size: "10pt",
				family: "sans-serif"
			}).setStroke("white").setFill("white");
			this._updateLayout();
			this._updateValue();

			dojo.connect(viewport, "onLayoutHud", this, "_updateLayout");
			dojo.connect(viewport, "onViewRectChanged", this, "_updateValue");
			//dojo.connect(viewport,"onClientSizeChanged",this,"_updateLayout");
		},

		_enable: function () {
			//
			//	summary:
			//			Enables the component and displays it.
			//
			this.inherited(arguments);
			this._updateLayout();
			this._updateValue();
		},

		_updateValue: function () {
			//
			//	summary:
			//		Updates the value displayed at the component
			//
			if (this._enabled && this._viewport.isHudEnabled() && (this._viewport.getZoom() !== this._currentZoom)) {
				this._currentZoom = this._viewport.getZoom();
				this._shape.children[1].setShape({
					text: "x" + ibm_ilog.diagram.util._round(this._currentZoom, 1)
				});
			}
		},

		_updateLayout: function () {
			//
			//	summary:
			//		Corrects the position of the component after a resize of the viewport.
			//
			if (this._enabled && this._viewport.isHudEnabled()) {
				var hudSize = this._viewport.getHudSize();
				var location;
				if(!this._viewport.isRtl()) {
					location = Point(hudSize.width - 30, 0);
				} else {
					location = Point(0,0);
				}
				this._shape.setTransform(dojox.gfx.matrix.translate(location));
			}
		}
	});

})();