dojo.provide("ibm_ilog.diagram.widget._Diagram");

dojo.require('ibm_ilog.diagram.interactors.MoveInteractor');
dojo.require('ibm_ilog.diagram.interactors.MarqueeSelectionInteractor');
dojo.require('ibm_ilog.diagram.interactors.WheelPanInteractor');
dojo.require('ibm_ilog.diagram.interactors.AdornerFocusInteractor');
dojo.require('ibm_ilog.diagram.interactors.FocusInteractor');

dojo.require('ibm_ilog.diagram.interactors.KeyboardInteractorManager');
dojo.require('ibm_ilog.diagram.interactors.Navigator');
dojo.require('ibm_ilog.diagram.interactors.SelectionKeyInteractor');
dojo.require('ibm_ilog.diagram.interactors.MoveKeyInteractor');
dojo.require('ibm_ilog.diagram.interactors.PanKeyInteractor');
dojo.require('ibm_ilog.diagram.interactors.ZoomKeyInteractor');
dojo.require('ibm_ilog.diagram.interactors.SwitchModeInteractor');

dojo.require('ibm_ilog.diagram.adorners.Adorner');
dojo.require('ibm_ilog.diagram.adorners.LinkAdorner');
dojo.require('ibm_ilog.diagram.adorners.AnnotationAdorner');

dojo.require("dojo.parser"); // for compatibility...

dojo.require("dijit.Tooltip");
dojo.require("dojo.number"); // for correct rounding

// This class contains the Diagram code that is conditionnally loaded
// only on desktop (non-mobile) platforms.

(function() {
	var gu = ibm_ilog.diagram.util.GraphUtil;
	//var R = ibm_ilog.diagram.util.ErrorReporter;

	var _defaultWaiLabelFunction = function(ge) {
		if (ge.getLabel) {
			return ge.getLabel();
		} else {
			return ge.toString();
		}
	};
	
	var _defaultWaiDescriptionFunction = function(ge) {
		if (ge.getLabel) {
			return ge.getLabel();
		} else {
			return ge.toString();
		}
	};
	
	// Extender object to add functionality on desktop (i.e., non-mobile) platforms:
	
	ibm_ilog.diagram.widget._Diagram = {
		
		_isDiagramExtended: true,
			
		// =======================================================================
		//
		// INTERACTORS:
		//
		
		_initializeInteractors: function() {
			//
			//	summary:
			//		Initializes the interactors used by the Diagram
			//
			//	tags:
			//		protected
			//
			
			this._interactors.add("pan", new ibm_ilog.diagram.interactors.PanInteractor().initialize(this));
			this._interactors.add("select", new ibm_ilog.diagram.interactors.SelectionInteractor().initialize(this._graph, this.getSelection(), this._viewport.getBackgroundShape()));
			this._interactors.add("zoom", new ibm_ilog.diagram.interactors.ZoomInteractor().initialize(this));
			
			this._interactors.add("move", new ibm_ilog.diagram.interactors.MoveInteractor().initialize(this._selection, this.getViewport(), {
				doMove: this.moveFunction,
				isMovable: this.isMovableFunction
			}));
			this._interactors.add("marquee", s = new ibm_ilog.diagram.interactors.MarqueeSelectionInteractor().initialize(this.getViewport(), this.getSelection()));
			this._interactors.add("wheelPan", new ibm_ilog.diagram.interactors.WheelPanInteractor().initialize(this));
			this._interactors.add("adorners", new ibm_ilog.diagram.interactors.AdornerFocusInteractor().initialize(this));
			this._interactors.add("focus", new ibm_ilog.diagram.interactors.FocusInteractor().initialize(this));
			
			this._setupInteractors(["pan", "select", "zoom", "move", "marquee", "wheelPan", "adorners", "focus"]);
			
			dojo.connect(this._graph, "onLayoutAnimationStart", this, function() {
				// TODO focusedElement also
				this.setNodeAdornersFocus(null);
				if (this._getNodeAdornersFocusAttr()) {
					this._setNodeAdornersFocusAttr(false);
					
					var h = dojo.connect(this._graph, "onLayoutAnimationEnd", this, function() {
						dojo.disconnect(h);
						this._setNodeAdornersFocusAttr(true);
					});
				}
			});
		},
		
		enterStandardMode: function() {
			//
			//	summary:
			//		TODO TBD. Probably to be renamed also
			//    	
			this._interactors.switchTo("pan", "select", "zoom", "move", "marquee", "wheelPan", "adorners", "focus");
		},	
		
		//
		//  move: [markup] boolean
		//      enable or disable moving entities
		//
		move: false,
		
		_setMoveAttr: function( /*Boolean*/b) {
			if (!this._viewportReady) 
				// at creation time, interactors are initialized in the postCreate stage.
				// Since customSetter are called before postCreate(), skip it in this case.
				return; 
			var old = this._getMoveAttr();
			if (old !== b) {
				this.move = b;
				this._interactors.get("move").setEnabled(b);
				this._firePropertyChanged('move', old, b);
			}
			return this;
		},
		
		_getMoveAttr: function() {
			return this._viewportReady ? this._interactors.get("move").getEnabled() : false;
		},
		
		// =======================================================================
		//
		// KEY INTERACTORS:
		//
		_keyInteractors: null,
		
		_initializeKeyInteractors: function() {
			//
			//	tags:
			//		private
			//
			this._keyInteractors = new ibm_ilog.diagram.interactors.KeyboardInteractorManager(this);
			
			this._keyInteractors.add(ibm_ilog.diagram.interactors.Navigator.KeyInteractorId, new ibm_ilog.diagram.interactors.Navigator().initialize(this));
			this._keyInteractors.add(ibm_ilog.diagram.interactors.SelectionKeyInteractor.KeyInteractorId, new ibm_ilog.diagram.interactors.SelectionKeyInteractor().initialize(this));
			this._keyInteractors.add(ibm_ilog.diagram.interactors.PanKeyInteractor.KeyInteractorId, new ibm_ilog.diagram.interactors.PanKeyInteractor().initialize(this));
			this._keyInteractors.add(ibm_ilog.diagram.interactors.ZoomKeyInteractor.KeyInteractorId, new ibm_ilog.diagram.interactors.ZoomKeyInteractor().initialize(this));
			this._keyInteractors.add(ibm_ilog.diagram.interactors.MoveKeyInteractor.KeyInteractorId, new ibm_ilog.diagram.interactors.MoveKeyInteractor().initialize(this));
			var edition = new ibm_ilog.diagram.interactors.SwitchModeInteractor();
			this._keyInteractors.add(ibm_ilog.diagram.interactors.SwitchModeInteractor.KeyInteractorId, edition.initialize(this));
			edition.enterNavigation();
			this._keyInteractors.activate();
		},
		
		getKeyInteractor: function(id) {
			return this._keyInteractors.get(id);
		},
	
		assignKeyInteractorInputs: function(iid, assignments) {
			this.getKeyInteractor(iid).assignInputs(assignments);
		},
		
		getKeyboardEventSource: function() {
			var wai = this._getWaiNode(this._graph);
			return wai || this.focusNode || this.domNode;
		},
		
		// =======================================================================
		//
		//	Adorners
		//
	
		_initializeAdorners: function() {
			var v = this.getViewport();
			
			this._adornerTemplates = ibm_ilog.diagram.templates.defaultAdornerTemplates.clone();
			
			v.whenLoaded(dojo.hitch(this, function() {
				this._adornerLayer = v.getOverlayLayer().createGroup();
			}));
		},
		
		
		_createAdorners: function() {
			this._focusedNodeAdorner = this.createAdorner(ibm_ilog.diagram.adorners.Adorner, this._adornerTemplates, "NodeFocusAdorner");
			
			this._focusedLinkAdorner = this.createAdorner(ibm_ilog.diagram.adorners.LinkAdorner, this._adornerTemplates, "LinkFocusAdorner");
	
		},
		
		// =======================================================================
		//
		//	WAI-ARIA / A11Y
		//
		
		tabIndex: "-1",
		
		//
		//	disableGraphWai: boolean
		//		Disables the infrastructure used for assigning WAI roles. (Initialization-only attribute).
		//        
		disableGraphWai: false,
		
		//
		//	enableGraphWaiDescribedBy: boolean
		//		Used to enable description for wai structure
		//
		enableGraphWaiDescribedBy: false,
		
		//
		//	waiNodeLabelFunction: Function?
		//		This function is used to compute the WAI label text for a node.
		//
		waiNodeLabelFunction: _defaultWaiLabelFunction,
		
		//
		//	waiLinkLabelFunction: Function?
		//		This function is used to compute the WAI label text for a link.
		//
		waiLinkLabelFunction: _defaultWaiLabelFunction,
		
		//
		//	waiSubgraphLabelFunction: Function?
		//		This function is used to compute the WAI label text for a subgraph.
		//
		waiSubgraphLabelFunction: _defaultWaiLabelFunction,
		
		//	waiGraphLabelFunction: Function?
		//		This function is used to compute the WAI label for the graph.
		//
		waiGraphLabelFunction: function(graph) {
			return "Graph";
		},
		
		
		
		//
		//	waiNodeRoleFunction: Function?
		//		This function is used to compute the WAI role for a node.
		//
		waiNodeRoleFunction: function(ge) {
			return "listitem";
		},
		
		//
		//	waiLinkRoleFunction: Function?
		//		This function is used to compute the WAI role for a link.
		//
		waiLinkRoleFunction: function(ge) {
			return "listitem";
		},
		
		//
		//	waiSubgraphRoleFunction: Function?
		//		This function is used to compute the WAI role for a subgraph.
		//
		waiSubgraphRoleFunction: function(ge) {
			return "group";
		},
		
		//	waiGraphRoleFunction: Function?
		//		This function is used to compute the WAI role for the graph.
		//
		waiGraphRoleFunction: function(graph) {
			return "group";
		},
		
		
		
		//
		//	waiNodeDescriptionFunction: Function?
		//		This function is used to compute the WAI description text for a node.
		//
		waiNodeDescriptionFunction: _defaultWaiDescriptionFunction,
		
		//
		//	waiLinkDescriptionFunction: Function?
		//		This function is used to compute the WAI description text for a link.
		//
		waiLinkDescriptionFunction: _defaultWaiDescriptionFunction,
		
		//
		//	waiSubgraphDescriptionFunction: Function?
		//		This function is used to compute the WAI description text for a subgraph.
		//
		waiSubgraphDescriptionFunction: _defaultWaiDescriptionFunction,
		
		//	waiGraphDescriptionFunction: Function?
		//		This function is used to compute the WAI description for the graph.
		//
		waiGraphDescriptionFunction: function(graph) {
			return "";
		},
		
		
		
		_bindWaiNode: function( /*ibm_ilog.diagram.GraphElement|ibm_ilog.diagram.Graph*/ge, /*div*/ waiNode, /*label*/ waiDesc) {
			//
			//	summary:	
			//		Adds private WAI information to graph element and the WAI node. 
			//
			//	tags:
			//		private
			//
			ge.__diagramWaiNode = waiNode;
			ge.__diagramWaiDesc = waiDesc;
			waiNode.__graphElement = ge;
		},
		
		_onWaiFocus: function(e) {
			//
			//	summary:
			//		Occurs when a WAI node has received focus browser focus.
			//		Used for debugging for now.
			//
			//	tags:
			//		private
			//
		},
		
		_onWaiBlur: function(e) {
			//
			//	summary:
			//		Occurs when a WAI node has lost focus browser focus.
			//		Used for debugging for now.
			//
			//	tags:
			//		private
			//
		},
		
		_waiInvisibleStyle: {
			position: 'absolute',
			left: '0px',
			top: '0px',
			width: '100%',
			height: '100%',
			overflow:"visible"
		},
		
		_initializeGraphWai: function() {
			//
			//	summary:
			//		Initializes the infrastructure to support WAI screen reading on the graph elements.
			//
			//	tags:
			//		private
			//
			
			// when the widget receives focus, forward to appropiate WAI node 
			this._connections.push(dojo.connect(this.domNode, "onfocus", this, "_onDiagramFocus"));
			
			// create root node at a hidden position
			var waiRoot = dojo.create('div', {style:{
				position: 'absolute',
				left: '0px',
				top: '0px',
				width: '100%',
				height: '100%',
				overflow: "visible",
				'z-index': "-2"
			}}, this.domNode, "first");
			
			this._createWaiNode(this._graph, this.id, waiRoot, this.waiGraphRoleFunction(this._graph));
			
			this._graph.__diagramWaiNode.setAttribute('tabIndex','0');
			
			dijit.setWaiState(this._graph.__diagramWaiNode, "label", this.waiGraphLabelFunction(this._graph));
			if (this.enableGraphWaiDescribedBy) {
				this._graph.__diagramWaiDesc.innerHTML = this.waiGraphDescriptionFunction(this._graph);
			}
			
			// connect to onfocus/onblur
			if (!dojo.isIE) {
				waiRoot.addEventListener("focus", dojo.hitch(this, "_onWaiFocus"), true /*capturing phase*/);
				waiRoot.addEventListener("blur", dojo.hitch(this, "_onWaiBlur"), true /*capturing phase*/);
			} else {
				// use focusin focusout which DOES bubble up (IE does NOT support capturing phase)
				this._connections.push(dojo.connect(waiRoot, "focusin", dojo.hitch(this, "_onWaiFocus")));
				this._connections.push(dojo.connect(waiRoot, "focusout", dojo.hitch(this, "_onWaiBlur")));
			}
			
			this._connections.push(dojo.connect(this, "onNodeCreated", this, "_registerNodeWai"));
			this._connections.push(dojo.connect(this, "onSubgraphCreated", this, "_registerSubgraphWai"));
			this._connections.push(dojo.connect(this, "onLinkCreated", this, "_registerLinkWai"));
			
			this._connections.push(dojo.connect(this, "onFocusedElementChanged", this, "_updateElementWaiFocus"));
			//
			this._connections.push(dojo.connect(this.getSelection(), "onSelectionChanged", this, "_updateSelectedState"));
			
			this._subscriptions.push(dojo.subscribe('/ibm_diagram/GraphElement/beforeDisposing', this, "_destroyWaiNode"));
			this._subscriptions.push(dojo.subscribe('/ibm_diagram/GraphElement/refreshWaiLabel', this, "_updateWaiLabel"));
			
			this._subscriptions.push(dojo.subscribe('/ibm_diagram/GraphElement/labelChanged', this, "_updateWaiLabel"));
			
			this._subscriptions.push(dojo.subscribe("/ibm_diagram/Subgraph/collapsedChanged", this, "_updateSubgraphCollapsed"));
		},
		
		_getWaiNode: function(e) {
			//
			//	summary:
			//		Returns the WAI node associated with a graph element.
			//
			//	tags:
			//		private
			//
			return e.__diagramWaiNode;
		},
		
		_onDiagramFocus: function(subgraph) {
			//
			//	summary:
			//		Used when the widget receives HTML focus. Forwards it to the current focused element.
			//
			//	tags:
			//		private
			//
			//	TODO: check this is needed (maybe tabTndex is always enough).
			//
			var wainode = this._getWaiNode(this._focusedElement|| this._graph)
			if(wainode) {
				dijit.focus(wainode);
			}
		},
		
		_updateSubgraphCollapsed: function(subgraph) {
			//
			//	summary:
			//		Used when a subgraph collapsed state has changed. Updates the corresponding WAI property. 
			//
			//	tags:
			//		private
			//
			var wainode = this._getWaiNode(subgraph);
			if(wainode) {
				dijit.setWaiState(wainode, "expanded", !subgraph.isCollapsed());
			}
		},
		
		_updateSelectedState: function(selected, deselected) {
			//
			//	summary:
			//		Used a graph element is selected or deselected. Updates the correspondig WAI attribute.
			//
			//	tags:
			//		private
			//
			var selection = this.getSelection();
			dojo.forEach(deselected, function(e) {
				var ge = selection._asGraphElement(e);
				if (ge) {
					var wai = this._getWaiNode(ge);
					if (wai) {
						dijit.setWaiState(wai, "selected", false);
					} else {
						// R.warn("MissingWaiNode", e);
					}
				}
			}, this);
			dojo.forEach(selected, function(e) {
				var ge = selection._asGraphElement(e);
				if (ge) {
					var wai = this._getWaiNode(ge);
					if (wai) {
						dijit.setWaiState(wai, "selected", true);
					} else {
						// R.warn("MissingWaiNode", e);
					}
				}
			}, this);
		},
		
		_updateElementWaiFocus: function(old, current) {
			//
			//	summary:
			// 		Change wai focus to node associated with the current focused element position
			//
			//	tags:
			//		private
			//
			var oldWai = (old && this._getWaiNode(old)) || this._getWaiNode(this._graph);
			var curWai = (current && this._getWaiNode(current)) || this._getWaiNode(this._graph);

			//	TODO: [av] try aria-activedescendant instead?

			oldWai.setAttribute("tabIndex", "-1");
			curWai.setAttribute("tabIndex", "0");

			dijit.focus(curWai);
		},

		_htmlRefocus: function() {
			//
			//	summary:
			//		Set the HTML focus into the appropiate component based on the current widget state.
			//
			var current = this.getFocusedElement();  
			var curWai = (current && this._getWaiNode(current)) || this._getWaiNode(this._graph);
			dijit.focus(curWai);
		},
		
		_getFunctionTriad: function(pre, post) {
			//
			//	summary:
			// 		helper. returns a {N:L:S:} for use with GraphUtil.dispatchClass().
			//
			//	tags:
			//		private
			//
			if (pre && pre.length) {
				return {
					N: this[pre + "Node" + post + "Function"],
					L: this[pre + "Link" + post + "Function"],
					S: this[pre + "Subgraph" + post + "Function"]
				};
			} else {
				return {
					N: this["node" + post + "Function"],
					L: this["link" + post + "Function"],
					S: this["subgraph" + post + "Function"]
				};
			}
		},
		
		_updateWaiDescription: function(ge) {
			//
			//	summary:
			// 		Regenerates the WAI description text of a graph element.
			//
			//	tags:
			//		private
			//
			if (ge && ge.__diagramWaiDescription) {
				ge.__diagramWaiDescription.innerHTML = gu.dispatchClass(ge, arguments, this, this._getFunctionTriad("wai", "Description"));
			}
		},
		
		_updateWaiLabel: function(ge) {
			//
			//	summary:
			// 		Regenerates the WAI label text of a graph element.
			//
			//	tags:
			//		private
			//
			if (ge && ge.__diagramWaiNode) {
				dijit.setWaiState(ge.__diagramWaiNode, 'label', gu.dispatchClass(ge, arguments, this, this._getFunctionTriad("wai", "Label")));
			}
		},
		
		_createWaiNode: function(element, id, waiParent, role) {
			//
			//	summary:
			//		create node div for WAI
			//
			//	tags:
			//		private
			//
			var waiNodeId = id + "_waiNode";
			var waiLabelId = id + "_waiLabel";
			var waiDescId = id + "_waiDesc";
			
			var atts = dojo.mixin({
				id: waiNodeId,
				tabIndex: -1
			}, {style:this._waiInvisibleStyle});
			
			var geWaiNode = dojo.create("div", atts, waiParent);
			var geWaiDesc;
			if (this.enableGraphWaiDescribedBy) {
				geWaiDesc = dojo.create("label", dojo.mixin({
					id: waiDescId,
					'for': waiNodeId
				}, {style:this._waiInvisibleStyle}), waiParent);
			}
			
			dijit.setWaiRole(geWaiNode, role);
			
			if (this.enableGraphWaiDescribedBy) {
				dijit.setWaiState(geWaiNode, 'describedBy', waiDescId);
			}
			
			this._bindWaiNode(element, geWaiNode, geWaiDesc);
		},
		
		_registerGraphElementWai: function(ge, role) {
			//
			//	summary:
			//		Register a div node for Wai roles for a recently created Graph Element
			//
			//	tags:
			//		private
			//
			
			// get div parent of parent
			
			var container = ge.getParent(), parentWaiNode;
			
			if (!container || !container.__diagramWaiNode) {
				// R.warn("MissingWaiNode", container);
				return;
			}
			
			parentWaiNode = container.__diagramWaiNode;
			
			// create node div for WAI
			this._createWaiNode(ge, ge.getId(), parentWaiNode, role);
			
			this._updateWaiLabel(ge);
			this._updateWaiDescription(ge);
		},
		
		_destroyWaiNode: function(ge) {
			if (ge.__diagramWaiLabel) {
				dojo.destroy(ge.__diagramWaiLabel);
			}
			if (ge.__diagramWaiNode) {
				dojo.destroy(ge.__diagramWaiNode);
			}
		},
		
		_registerLinkWai: function(link) {
			//
			//	summary:
			//		Register a div node for Wai roles for a recently created Link
			//
			//	tags:
			//		private
			//
			
			this._registerGraphElementWai(link, this.waiLinkRoleFunction());
			
		},
		
		_registerNodeWai: function(node) {
			//
			//	summary:
			//		Register a div node for Wai roles for a recently created Node
			//
			//	tags:
			//		private
			//
			
			this._registerGraphElementWai(node, this.waiNodeRoleFunction());
			
		},
		
		_registerSubgraphWai: function(subgraph) {
			//
			//	summary:
			//		Register a div node for Wai roles for a recently created Subgraph
			//
			//	tags:
			//		private
			//
			
			this._registerGraphElementWai(subgraph, this.waiSubgraphRoleFunction());
			
			this._updateSubgraphCollapsed(subgraph);
			subgraph.getGraph().__diagramWaiNode = subgraph.__diagramWaiNode;
			
		}
	};

	// Add _Diagram features to Diagram if not on mobile.
	// (done here or in Diagram.js, depending on the load order - layer or not)
	if(dojo.config._notDiagramForMobile &&
			ibm_ilog.diagram.widget._Diagram &&
			ibm_ilog.diagram.widget.Diagram &&
			ibm_ilog.diagram.widget.Diagram.prototype &&
			!ibm_ilog.diagram.widget.Diagram.prototype._isDiagramExtended)
		dojo.extend(ibm_ilog.diagram.widget.Diagram, ibm_ilog.diagram.widget._Diagram);
})();
