/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.diagrams.edit.actions.AddReference");

dojo.require("rmpw.common.policy.Policy");

/**
 * @author Marc Gobeil
 */
dojo.declare("rmpw.diagrams.edit.actions.AddReference", rmpw.diagrams.edit.actions.Action, {

	run: function(e) {
		var source = this.host.renderMapping.getSourceOwnerByPath(this._referencePath);
		var sourcePropertyName = this._referencePath.substring(this._referencePath.lastIndexOf('.') + 1);
			
		var target;
		if(this._itemPath) {
			target = e.targetController.renderMapping.getSourceByPath(this._itemPath);
		}
		else {
			// TODO:
			// - exploit commonality with AddRelationship.isEnabled
			// - throw a warning if there are more is more than one source available,
			// since assumption not held, but might work anyway
			target = e.targetController.renderMapping.getDefaultSource();
		}

		// TODO: should lookup the command based on sourceObject.eClass, sourceProperty,
		// and optionally targetObject.eClass if backreference is defined
		var command = this.host.root.getCommandPackage().select({
			commandId: "rmpw.common.command.AddReference",
			typeId: dojo.map(source.rdf_type, function (type) { return type.rdf_about; }),
			connectionPath: this._connectionPath
		});
		
		this.waiting(e, true);
		this._connectAddChild = dojo.connect(this.host.renderMapping, "addChild", dojo.hitch(this, function(child) {
			dojo.disconnect(this._connectAddChild);
			delete this._connectAddChild;
			if (child.__editController.policies.selectable) {
				child.__editController.policies.selectable.selectOnUpdate();
			}
			this.waiting(e, false);
		}));
		
		var cs = this.host.root.getCommandController().commandStack;
		var bret = cs.execute(command, { sourceObject: source, targetObject: target });
		if (!bret) {
			dojo.disconnect(this._connectAddChild);
			delete this._connectAddChild;
			this.waiting(e, false);
		}
		
	},
	
	addCreationFeedback: function(editController, pt) {
	},
	
	isEnabled: function(e) {
		var validTarget = false;
		var potentialTarget = e.targetController.renderMapping.getDefaultSource();
		if(this.host.renderMapping.isTypeOf(potentialTarget, this._targetType)) {
			validTarget = true;
		}
		return validTarget;
	},
	
	setReferencePath: function(path) {
		this._referencePath = path;
	},
	
	setConnectionPath: function(path) {
		this._connectionPath = path;
	},

	setItemPath: function(path) {
		this._itemPath = path;
	},
	
	setTargetType: function(type) {
		this._targetType = type;
	}
	
});